/*
	3D character sample code

*/

//#include <afxdao.h>
#include <windows.h>
#include <stdio.h>
#include <conio.h>
#include <math.h>
#include <limits.h>



#include "3dstate.h"

#define STEP 1
#define ANGLES 1


DWORD load_object(char *file_name, char *bmp_file, char *cfg_file, char *token);
DWORD  get_next_animation_sequence(DWORD obj);
void put_demi_tag_object(DWORD base_obj, DWORD box_obj, char *tag_name);
void write_frame_number(char *str, DWORD obj, int x, int y);
void switch_skin(char **skin_names, int num_of_skins, DWORD lower_obj, DWORD upper_obj, DWORD head_obj);
void help_message();
void set_animation_sequence(DWORD obj, char *sequence_name);

int APIENTRY WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR     lpCmdLine,  int nCmdShow)
{

	//All the function that starts with STATE_ are part of the 3DSTATE 3D Engine 
	//To read more about a specif function look for it at the 3dstate.h file
	//The rest of the function are local functions implemented in this file (e.g. load_object() )
	//or WIN32 function such as GetAsyncKeyState() these functions have nthing to do with 3DSTATE
	//you can read about them in Microsoft MSDN archive
	
	
	//skins
	int num_of_skins;
	//The skins folder contain all the skins for the images. Any new images added to this folder will automatically be used
	//Gets the names of all the images in the folder (you can use any other function you like to read the file names)
	char **all_skins_names=STATE_utilities_get_file_names("model\\skins\\", NO, &num_of_skins);

	//Load the 3 parts of the body and the weapon
	//The implementation of the load_object function is below
	DWORD lower_obj=load_object("model\\Lower.md3", "model\\skins\\skin1.jpg","model\\animation.cfg", "LEGS_");	
	DWORD upper_obj=load_object("model\\Upper.md3", "model\\skins\\skin1.jpg","model\\animation.cfg", "TORSO_");	
	DWORD head_obj=load_object("model\\Head.md3", "model\\skins\\skin1.jpg",NULL, NULL);	
	DWORD weapon_obj=load_object("model\\railgun.md3", NULL,NULL, NULL);	


	//connect the parts of the uman model
	STATE_object_md3_connect_models_automatic(lower_obj, upper_obj);
	STATE_object_md3_connect_models_automatic(upper_obj, head_obj);
	STATE_object_md3_connect_models_automatic(upper_obj, weapon_obj);
	
	//set the lower body to walking sequence and an upper body to the Gesture sequence
	set_animation_sequence(upper_obj, "GESTURE");
	set_animation_sequence(lower_obj, "WALK");


	STATE_engine_maximize_default_rendering_window();

	STATE_engine_set_background_color(0,0,0);

	DWORD camera=STATE_camera_get_default_camera();
	STATE_camera_set_location(camera, 180, 0, 0);
	STATE_camera_point_at(camera, 0,0,0);


	STATE_engine_show_frames_per_second_rate(INFORMATION_ALL);

	//creating the help layer
	DWORD help_bmp=STATE_bitmap_load("help.jpg", -1);
	STATE_layer_create(65, 2, 98, 45, help_bmp, DISABLE_BLENDING);

	


	double direction[3]={-1,-1,0};
	DWORD light2=STATE_light2_create_directional_light("my_light", direction);

	//loop till the Escape key is pressed. GetAsyncKeyState() is a WI32 function
	//In a windows environment it is consider bad programming but we do it just for simplicity
	//A better way is to create a timer or to use OnIdle event
	while( (GetAsyncKeyState(VK_ESCAPE)&1) ==0 ) {

		//This is the most important function that does all the work
		//This is the function that generates all the graphics.
		//It takes the view seen from the given camera and put it on the given window - STATE_engine_render(window,camera);
		STATE_engine_render(NULL,camera);
		
		//write the yellow text on the screen
		write_frame_number("Lower part sequence (Press L to change)", lower_obj, 0,50);
		write_frame_number("Upper part sequence (Press H to change)", upper_obj, 0,60);

		//move the object forward and backward when the Up/Down arrow keys are pressed
		//STEP is a constant defined above
		if(GetAsyncKeyState(VK_UP)<0)	STATE_camera_move(camera, WORLD_SPACE, STEP,0,0);
		if(GetAsyncKeyState(VK_DOWN)<0) STATE_camera_move(camera, WORLD_SPACE,-STEP,0,0);

		DWORD obj=lower_obj;
		
		//Turn the object (the default rotation). ANGLES is a constant defined above
		STATE_object_rotate_z(obj, (double)ANGLES/2,OBJECT_SPACE);
		
		//Turn the object faster when the Left/Write arrow keys are pressed
		if(GetAsyncKeyState(VK_LEFT)<0) STATE_object_rotate_z(obj, ANGLES,OBJECT_SPACE);
		if(GetAsyncKeyState(VK_RIGHT )<0) STATE_object_rotate_z(obj, -ANGLES*2,OBJECT_SPACE);

		//Switch the skin
		if(GetAsyncKeyState('S')&1)
			switch_skin(all_skins_names, num_of_skins,lower_obj, upper_obj, head_obj);

		if(GetAsyncKeyState(VK_F1)&1)
			help_message();


		//Turn just the head (if keys 9,0 are pressed)
		if(GetAsyncKeyState('0')<0) STATE_object_rotate_z(head_obj, ANGLES,OBJECT_SPACE);
		if(GetAsyncKeyState('9' )<0) STATE_object_rotate_z(head_obj, -ANGLES,OBJECT_SPACE);
		
		//Toggle weapon
		if(GetAsyncKeyState('W')&1) {
			if(STATE_object_is_enabled(weapon_obj)==YES)
				STATE_object_disable(weapon_obj);
			else
				STATE_object_enable(weapon_obj);
		}
 
		
		//Get the next animation sequence for the upper body
		if(GetAsyncKeyState('H')&1) get_next_animation_sequence(upper_obj);


		//Get the next animation sequence for the legs
		//and if there is a maching upper bod sequence then set it as well
		if(GetAsyncKeyState('L')&1) {
			DWORD lower_current_sequence=get_next_animation_sequence(lower_obj);
			DWORD upper_animation=STATE_object_get_3D_animation(upper_obj);
			
			//If there is a matching upper body sequence we want to set it too
			char *lower_seq_name=STATE_3D_sequence_get_name(lower_current_sequence);
			DWORD upper_seq=STATE_3D_sequence_get_using_name(upper_animation, lower_seq_name);
			if(upper_seq!=NULL)
				STATE_object_set_3D_sequence(upper_obj, upper_seq, 350);
		}


	}

	STATE_engine_close();

	return(0);
}

//Called when F1 is pressed
void help_message()
{
	MessageBox(NULL, "Use the following keys:\nF1 - get this help screen\nS - to change the Skin\nL - To change the Legs animation\nH - to change the upper body Hand animation\nW - to add/remove the weapon\n9,0 Turn the head\nArrow keys to zoom in/out\n\nEnjoy !\nThe 3DSTATE Team", "F1-HELP", MB_ICONINFORMATION);
}



//load a 3D object into the engine
// These objects will be later rendered (painted on the screen) when STATE_engine_render() is called
DWORD load_object(char *file_name, char *bmp_file, char *cfg_file, char *token)
{

	DWORD obj;
	if(cfg_file!=NULL)
		obj=STATE_object_md3_create_from_file(file_name, cfg_file, token);
	else
		obj=STATE_object_create_from_file(file_name);


	if(obj==NULL)
		return(NULL);



	if(bmp_file==NULL)
		return(obj);

	DWORD bmp=STATE_bitmap_load(bmp_file, -1);
	
	if(bmp==NULL)
		return(obj);

	STATE_object_set_bitmap(obj, bmp, 0);


	return(obj);

}


DWORD  get_next_animation_sequence(DWORD obj)
{
	DWORD animation = STATE_object_get_3D_animation(obj);
	if (animation == NULL)
		return(NULL);

	DWORD current_sequence=STATE_object_get_3D_sequence(obj);

	if(current_sequence==NULL)
		current_sequence=STATE_3D_sequence_get_first(animation);


	current_sequence=STATE_3D_sequence_get_next(current_sequence);
	if(current_sequence==NULL)
		current_sequence=STATE_3D_sequence_get_first(animation);

	STATE_object_set_3D_sequence(obj, current_sequence, 350);



	return(current_sequence);
}



//Write a text above the object. This text is attached to the object
//This function is not used in this sample
void object_set_text(DWORD obj, char *txt)
{
    DWORD ariel10 = STATE_font_create("Ariel", 10, FONT_CREATE_DEFAULT);
	double box[2][3];
    STATE_object_get_bounding_box(obj, box);
    double object_height = box[1][2] - box[0][2];
	double text_offset[3]={ 0, 0, object_height * 0.75 };
    //add some text to the player
    STATE_object_add_text(obj, ariel10, txt, 255, 0, 0, FONT_WRITE_CENTERED_X | FONT_WRITE_CENTERED_Y, 3, text_offset);


}
        

//The yellow text ...
void write_frame_number(char *str, DWORD obj, int x, int y)
{
	char *name="error";
	int frame=-1;
	DWORD seq=STATE_object_get_3D_sequence(obj);
	if(seq!=NULL) {
		name=STATE_3D_sequence_get_name(seq);	
		frame=(int)STATE_3D_sequence_get_current_frame(seq);	
	}

	char text[1000];
	sprintf(text, "%s: %s %ld", str, name, frame);
	DWORD ariel10=STATE_font_create("Ariel", 10, FONT_CREATE_DEFAULT);
	STATE_font_write_text(ariel10, text, x,y, 255, 255, 255, 120, FONT_WRITE_DEFAULT, INT_MAX);
}

//Set a new skin to the object
//Since our model is made from three parts (legs, upper body and head) we need
//to set the skin to all the three parts
void switch_skin(char **skin_names, int num_of_skins, DWORD obj1, DWORD obj2, DWORD obj3)
{
	static int skin_index=0;

	if(skin_names==NULL)
		return;

	if(num_of_skins<=0)
		return;

	skin_index++;
	if(skin_index>=num_of_skins)
		skin_index=0;



	DWORD bmp=STATE_bitmap_load(skin_names[skin_index], -1);
	if(bmp!=NULL) {
	
		if(obj1!=NULL)
			STATE_object_set_bitmap(obj1, bmp, 0);
	
		if(obj2!=NULL)
			STATE_object_set_bitmap(obj2, bmp, 0);

		if(obj3!=NULL)
			STATE_object_set_bitmap(obj3, bmp, 0);

	}
		


}

//Set a specific animation sequence
void set_animation_sequence(DWORD obj, char *sequence_name)
{
	DWORD animation = STATE_object_get_3D_animation(obj);
	if (animation == NULL)
		return;

	DWORD seq=STATE_3D_sequence_get_using_name(animation, sequence_name);
	if(seq==NULL)
		return;

	STATE_object_set_3D_sequence(obj, seq,500);
	
	
}