/*
                                                                                                                                
                                              3D                                                                                
      STATE3DS      TATE3DSTATE3D         STATE3DSTAT    E3DSTATE3DSTATE3D      STATE      3DSTATE3DSTATE3DS  TATE3DSTATE3DST   
    ATE3DSTATE3     DSTATE3DSTATE3D      STATE3DSTATE3   DSTATE3DSTATE3DST     ATE3DST     ATE3DSTATE3DSTATE  3DSTATE3DSTATE3   
    DSTAT  E3DST    ATE3D   STATE3DS    TATE3     DSTAT        E3DSTA         TE3DSTATE         3DSTATE       3DSTA             
     TE     3DST    ATE3D      STATE3   DSTAT      E3DS        TATE3          DSTATE3DS          TATE3        DSTA              
          TE3DST    ATE3D       STATE   3DSTATE3D              STATE         3DSTA TE3DS         TATE3        DSTAT             
        E3DSTA      TE3DS        TATE    3DSTATE3DSTA          TE3DS        TATE3  DSTAT         E3DST        ATE3DSTATE3DSTA   
        TE3DSTAT    E3DST        ATE3D     STATE3DSTATE        3DSTA        TE3DS   TATE3        DSTAT        E3DSTATE3DSTATE   
            3DSTA   TE3DS       TATE3          DSTATE3DS       TATE3       DSTATE3DSTATE3D       STATE        3DSTA             
             TE3D   STATE       3DSTA   TE3D       STATE       3DSTA       TE3DSTATE3DSTATE      3DSTA        TE3D              
   STATE    3DSTA   TE3DS      TATE3D   STATE      3DSTA       TE3DS      TATE3DSTATE3DSTAT      E3DST        ATE3              
    DSTATE3DSTATE   3DSTATE3DSTATE3D    STATE3DSTATE3DST       ATE3D      STATE        3DSTA     TE3DS        TATE3DSTATE3DSTA  
     TE3DSTATE3     DSTATE3DSTATE3       DSTATE3DSTATE         3DSTA     TE3DS         TATE3D    STATE        3DSTATE3DSTATE3D  
       STATE3       DSTATE3DSTAT           E3DSTATE3D          DSTAT     E3DS           TATE3    DSTAT        TE3DSTATE3DSTATE  
                                                                                                                                


	3DSTATE.h

	STATE 3D Engine API
	
	Version 7.47


	
	NOTE: There are many functions in this API. You will probably need
		  only a few of them. You can write a great game using only about 20
		  functions from this API. We suggest that you first look at some
		  sample code and only when you feel comfortable, take a deeper 
		  look at this API.
		  

	
	  
		
	The main idea
	-------------
	With one command you load a three dimensional world model
	into the engine. With one command you create a camera
	for taking pictures from inside the world. 
	Now all you have to do is call this function
	STATE_engine_render(HWND hwnd, DWORD camera)
	and voila !!! you have the picture taken using the camera
	pasted on the given window !!!!
	now if you modify the camera location or direction
	and call the rendering function again you get movement !!!

		
	Naming convention.
	1- All the functions of this API start with STATE_
		example: STATE_camera_create(char *camera_name);

	2- The second word is the name of the group of functions.
		examples:
			STATE_camera_set_location(x,y,x);
			STATE_engine_increase_atmospheric_effect();


	There are several groups off API commands
	1- the engine API
		all function starting with STATE_engine_
		controls things like picture quality, atmospheric effect etc ...

	2- the camera API
		all function starting with STATE_camera_
		controls properties of the camera like
		zoom location direction etc...

	3- The object API
		all function starting with STATE_object_
		In your world you can define objects like an airplane,
		a boy, a runner, a bird etc ...
		this api allows you to control their behavior.

	The Other APIs are:
	The Polygon API, the Point API, the group API,
	the Track API, the Bitmap API, the Math API, the Utilities API,
	the Animation API, the profiler API, the background API and more.
	You can find more information at the beginning of each API.


	


	

*/

#ifndef _3DSTATE_H
#define _3DSTATE_H


#include <windows.h>

#ifdef __cplusplus
extern "C" {
#endif



/*
#if defined(__BORLANDC__)
	#define STATE_WINAPI WINAPI
#else
	#define STATE_WINAPI 
#endif
*/




#ifdef STATE_STATIC_LINK
#define STATE_WINAPI
#else
#define STATE_WINAPI WINAPI
#endif


#define OK              0
#define VR_ERROR -1


//Note that YES/NO ON/OFF and TRUE/FALSE are all the same.
#define YES 1  
#define NO 0

#define ON 1
#define OFF 0


#define WORLD_SPACE	0

#define OBJECT_SPACE (WORLD_SPACE+1)

#ifndef CAMERA_SPACE
#define CAMERA_SPACE (WORLD_SPACE+2)
#endif

#define PLAYER_CONTROL		1
#define COMPUTER_CONTROL	2
	
// used for STATE_camera_save()
#define CAMERA_DESCRIPTOR_SIZE 53  //Number of DWORDs (212 bytes)

#define FORBID_DIRECTX	

#define EDGE_NOT_SEEN		1
#define EDGE_FULLY_SEEN		2
#define EDGE_PARTIALLY_SEEN	3	
	
// Automatic movement constants == "Chasing"
// --------------------------
// In the wld file we have the following lines that are relevant
// to the Automatic movement.
// TRACK: track_name track_offset //chase name could be NO_TRACK if ...
// SPEED: track_speed    // track speed is also used in CHASE_PRECISE and CHASE_FLEXIBLE
						 // in those cases it is the distance behind the chased object
// CHASE: dynamic_object_to_chase  chase_offset
// CHASE_TYPE: chase_type tracking_softness  //tracking_softness is a number between 0 and 1.

// Below are the values that could appear in chase_type.
// relevant fields: none
// The NO_CHASE Const Must be 0.
#define	NO_CHASE							0 // doesn't move at all. Control is through software

// relevant fields: dynamic_object_to_chase, chase_offset
#define	CHASE_LOCATION						2 // trace only location. Only the line "CHASE: object_to_chase x y z " is relevant
											  // The three numbers are the offset from the object being chased
// relevant fields: dynamic_object_to_chase, chase_offset, chase_distance
#define	CHASE_PRECISE    					3 //  trace location and cord system. The track is ignored

	// relevant fields: dynamic_object_to_chase, chase_offset, chase_distance, tracking_softness
#define	CHASE_FLEXIBLE   					4 // trace location and direction with a delay

// not available yet
#define	CHASE_ON_TRACK   					(5+CHASE_TRACK_FAMILY) // chase the object to be chased but doesn't go out of the track line

// not available yet
#define	CHASE_INSIDE_TRACK_BOUNDING_BOX		6 // chase the object to be chased but doesn't go out from the track bounding box

// relevant fields: track_name, track_offset, speed, tracking_softness
#define	CHASE_TRACK   						(7+CHASE_TRACK_FAMILY) // just advance on the track according to the speed with out chasing anyone

// relevant fields: track_name, track_offset, speed, tracking_softness
#define	CHASE_TRACK_AIRPLANE				(8+CHASE_TRACK_FAMILY) // like CHASE_TRACK just with an airplane movements

//Not a chase type but a chase family
//Makes it possible to do things like if(chase_type & CHASE_TRACK_FAMILY) ...
#define CHASE_TRACK_FAMILY				128

#define CHASE_PHYSICS						9	//Moves the object according to the physics params (that is
												// speed,friction, elasticity, max_speed)
	
// Orientation values
#define ORIENTATION_UNKNOWN 0
#define ORIENTATION_TOP		1
#define ORIENTATION_BOTTOM	2
#define ORIENTATION_FRONT	4
#define ORIENTATION_BACK	8

#define NO_TRACK			0


#define X_AXIS 0
#define Y_AXIS 1
#define Z_AXIS 2

#ifdef STATE_EXPORTS
#define STATE_IMPORT __declspec(dllexport)
#else
#define STATE_IMPORT __declspec(dllimport)
#endif

#ifdef STATE_STATIC_LINK
#define STATE_IMPORT 
#endif 

#ifdef STATE_STATIC_LINK
#define STATE_stdcall
#else
#define STATE_stdcall WINAPI
#endif
	
//--------------------------------------------------------//
//=========== T H E    E N G I N E   A P I ===============//
//--------------------------------------------------------//
	
// This API controls things like picture quality, atmospheric effect etc ...
	
// world_mode options (for function STATE_engine_load_world() )
#define USER_DEFINED_BEHAVIOR 0
#define AUTO_BEHAVIOR01	1   //JAPPLIC_ID
#define AUTO_BEHAVIOR02	2	//CORE_APPLIC // NOTE AUTO_BEHAVIOR02 is obsolete
#define EDITOR_MODE	4
#define DONT_USE_ZBUFFER 8	//This option doesn't  works together with  EDITOR_MODE. When zbuffer is off
							// rendering will be faster but less accurate (things that are supposed to be hidden will sometime appear
							// depending on the structure of the used world). If zbuffer is on the rendering will
							// be accurate though slower. We suggest that you try both possibilities and choose what's best
							// The default is zbuffer ON.

////The resample flags cause every bitmap that is loaded to be automatically resampled to a power of two dimensions
//Note that since version 3.0 all the bitmaps that are not in the power of two are automatically resampled (with RESAMPLE_CLOSEST).
#ifndef RESAMPLE_UP
	#define RESAMPLE_UP			16 //The new width and height will always be bigger than or equal than the original dimension of the bitmap
	#define RESAMPLE_CLOSEST	32 //The new dimensions will be the closest power of two numbers to the original dimensions
	#define RESAMPLE_DOWN		64 //The new width and height will always be smaller than or equal than the original dimension of the bitmap
#endif

#define ENABLE_BRIGHT_LIGHT 2048 //Relevant only when 3D card is used.
#define ENABLE_VERY_BRIGHT_LIGHT 4096
#define CREATE_BSP_FOR_DYNAMIC_OBJECTS 8192
#define NO_BSP_FOR_STATIC_WORLD 16384
#define	DONT_CREATE_ADDITIONAL_BITMAPS 32768
#define	DONT_LOAD_BITMAPS	65536

#define OUTPUT_RENDER_LOCATIONS 131072 //The location from where render is done (the location of the camera)
									  //Will be output to a file named visited_points.txt
									  //This file is used with STATE_engine_create_speed_database()



#define BUILD_BSP_FAST	262144//Usually builds the BSP faster. You should check the log file to
						//if you get more polygons added or maybe less.
#define BUILD_BSP_WITH_LESS_POLYGONS  524288//Usually adds less polygons tyo the world in the buildup of the bsp tree 
									 //though this is not always the case. Usually it will also take more time to create.

#define BUILD_BSP_WITH_MINIMUM_POLYGONS 786432//Beware, using this parameter will increase the duration of the BSP Tree
										//construction significantly. If your world is big then it
										//might take several hours. So run your program with this parameter
										//and pray that by the time you get beack to work on the day after, the computer
										//will finish loading the world


// For checking if a handle is legal (for example a camera handle) you can do one of the two:
// 1- call the function STATE_camera_is_camera(handle, "my_test");
// 2- Use the macro IS_CAMERA. The advantage of the macro is that you can 
// replace it with an empty macro on your release program.
// See more details next to the declaration of function STATE_camera_is_camera()
#define         IS_CAMERA(camera_handle,function_name)			STATE_camera_is_camera(camera_handle,function_name)
#define         IS_OBJECT(object_handle,function_name)			STATE_object_is_object(object_handle,function_name)			
#define         IS_GROUP(group_handle,function_name)			STATE_group_is_group(group_handle,function_name)
#define         IS_TRACK(track_handle,function_name)			STATE_track_is_track(track_handle,function_name)
#define         IS_POLYGON(polygon_handle,function_name)		STATE_polygon_is_polygon(polygon_handle,function_name)
#define         IS_LIGHT(light_handle,function_name)			STATE_light_is_light(light_handle,function_name)
#define         IS_LIGHT2(light2_handle,function_name)			STATE_light2_is_light(light2_handle,function_name)
#define         IS_POINT(point_handle,function_name)			STATE_point_is_point(point_handle,function_name)
#define         IS_BITMAP(bitmap_handle,function_name)			STATE_bitmap_is_bitmap(bitmap_handle,function_name)
#define         IS_ANIMATION(animation_handle,function_name)	STATE_animation_is_animation(animation_handle,function_name)
#define         IS_BACKGROUND(background_handle,function_name)	STATE_background_is_background(background_handle,function_name)
#define         IS_3D_ANIMATION(animation3d_handle,function_name)	STATE_3D_animation_is_3D_animation(animation3d_handle,function_name)
#define         IS_3D_SEQUENCE(sequence3d_handle,function_name)	STATE_3D_sequence_is_3D_sequence(sequence3d_handle,function_name)
#define         IS_MESH(mesh_handle,function_name)				STATE_mesh_is_mesh(mesh_handle,function_name)
#define         IS_EFFECT(effect_handle,function_name)			STATE_effect_is_effect(effect_handle,function_name)
#define         IS_TEXT(text_handle,function_name)				STATE_text_is_text(text_handle,function_name)
#define         IS_FONT(font_handle,function_name)				STATE_font_is_font(font_handle,function_name)

// This function puts the image seen from the camera on the given window
// The rendering operation is not automatic after every API function
// This way we can save lots of CPU time by calling a group of API function
// followed by only one call to STATE_engine_render(). (This function takes lots of
// CPU time while all the other are very fast to execute)
STATE_IMPORT int  STATE_WINAPI STATE_engine_render(HWND hwnd, DWORD camera);

//exactly like STATE_engine_render() only easier to use
// if you are working with MFC
//STATE_IMPORT void  STATE_WINAPI STATE_engine_render_on_cwnd(void *pointer_to_CWnd_object, void *camera);

// returns (OK / VR_ERROR)
// It is important to check the return value
// Arguments:
// world_file_name: the name of the file to be loaded 
//					examples: "my_world.wld" , "my_world.raw" , "C:\\worlds\\my_world.x"
// world_directory_path: relevant only if  INCLUDE_FILE is used
//						 (INCLUDE_FILE is used in the world file if our world consist of several files)
//						  using a different world_directory_path we can load the same
//						world with different models library
//						or we can use one models library for several worlds
//						examples: "", "C:\\models\\chaires\\plastic", "C:\\models\\chaires\\plastic"
// bitmaps_directory_path:
//						using a different bitmaps_directory_path we can load the same
//						world with different bitmaps library
//						or we can use one bitmap library for several worlds
//						examples: "", "C:\\bitmaps1", "C:\\bitmaps2", "bitmaps"
// world_mode options: 
//		USER_DEFINED_BEHAVIOR 
//		AUTO_BEHAVIOR01			Automatically moves the camera or an object according the arrows key
//		AUTO_BEHAVIOR02			Obsolete
//		EDITOR_MODE				Dont move objects automatically. make it possible to use all the group API
//
//		DONT_USE_ZBUFFER 
//						This option doesn't  work together with  EDITOR_MODE. When zbuffer is off, the
//						rendering will be faster but could be less accurate (things that are supposed to be hidden will sometime appear
//						depending on the structure of the used world). If zbuffer is on, the rendering will sometime
//						more accurate though slower. We suggest that you try both possibilities and choose what's best
//						The default is zbuffer ON. The changes in speed are not so relevant when using a 3D card (harware rendering)
//
//		The resample flags cause every bitmap that is loaded to be automatically resampled to a power of two dimensions
//		RESAMPLE_UP			 The new width and height will always be bigger than or equal than the original dimension of the bitmap
//		RESAMPLE_CLOSEST	 The new dimensions will be the closest power of two numbers to the original dimensions
//		RESAMPLE_DOWN		 The new width and height will always be smaller than or equal than the original dimension of the bitmap
//
//		ENABLE_BRIGHT_LIGHT				Relevant only when 3D card is used. Makes it possible to create brighter lighting on textures
//		ENABLE_VERY_BRIGHT_LIGHT 
//		DONT_CREATE_ADDITIONAL_BITMAPS	When a bitmap is in jpg format the engine convert it into a bitmap and leave the bitmap on the disk so that the next
//										time that this world is loaded will be faster. This flag will tell the engine not to create additional bitmaps.
//										Instead of using this flag one can simply delete all the unneeded bitmaps before 
//										releasing the application.		
//
//		DONT_LOAD_BITMAPS				If this flag is given then the world will be loaded without its bitmaps.
//										Bitmaps can be loaded later using STATE_bitmap_load(). To retrieve the names of the bitmaps use STATE_polygon_get_bitmap_name()
//
//		Constants for BSP data structure creation
//		----------------------------------------
			//Some background about BSP Trees
			//---------------------------------
			//The BSP (Binary Space Partition) is an internal data structure inside the engine
			//that is used in Viewer Mode (the default mode) in the process of rendering the world and for collision detection.
			//There is nothing that you need to know about this internal data structure. Its existence and operation are 
			//hide inside the engine. Your only encounter with the BSP tree is when you wait for the engine to create it
			//while you load your world in the first time. When the world is loaded at the first time 
			//the engine creates a file called world_name.bsp . When you release the world you should include
			//this file to save the engine from regenerating it. Feel free to omit this file from
			// your release version if the creation process is fast (less than 1 second)

			//New parameters to control the bsp construction speed and number of add polygons 
			//------------------------------------------------------------------------------
			//In the past the construction of a bsp tree was a very long process 
			//(it could even reach 12 hours and more !!!) though now it should be very fast
			//about one second. If however this process takes too long (you can see the duration in the log file)
			//or it creates too many polygons in the way then you could try and load the world with the constants
			//below. You should try all the three constants because sometimes the one that supposed to give fewer polygons
			//gives more and vice versa. PLEASE NOTE THAT YOU MUST DELETE THE BSP FILE (IF EXSITS) OTHERWISE THE BUILD_BSP CONSTANTS
			//WILL HAVE NO EFFECT (because if the bsp file exists the engine takes it instead of building a new one).
			//You can read in the log file how well the constants affected the construction of the bsp tree.

			//Tips on how to minimize the number of polygons that are added when the bsp tree is constructed
			//------------------------------------------------------------------------------------------------
			//1) When building your world (using World Builder or other programs)
			// Dont make polygons intersect one each other. For example, if you put a tree on the ground
			// and the bottom of the tree polygons intersect the ground then it will force the engine to split
			//the tree into two polygons. In order to avoid this, don't put the tree on the ground manually, instead
			//use World Builder tools for doing dropping an object on the ground.
			//
			//2) If you have implemented tip "1" and tried to load your world with each one of the three parameters above 
			//	 and still you too many polygons are added in the process then
			//	 you should try to call STATE_engine_set_bsp_precision() before loading the world.


//		BUILD_BSP_FAST					//Usually builds the BSP faster. You should check the log file to
//										//if you get more polygons added or maybe less.
//		BUILD_BSP_WITH_LESS_POLYGONS	//Usually adds less polygons tyo the world in the buildup of the bsp tree 
//										//though this is not always the case. Usually it will also take more time to create.
//
//		BUILD_BSP_WITH_MINIMUM_POLYGONS //Beware, using this parameter will increase the duration of the BSP Tree
										//construction significantly. If your world is big then it
										//might take several hours. So run your program with this parameter
										//and pray that by the time you get beack to work on the day after the computer
										//will finish loading the world

//
//		CREATE_BSP_FOR_DYNAMIC_OBJECTS	This used to be the default. The default was changed since a BSP tree might add polygons to the world

//		NO_BSP_FOR_STATIC_WORLD			The world might have less polygons but collision detection will be slower. Usually it is not recommended to use this flag.




// Use the OR operation to combine several options ,for example STATE_engine_load_world("my_world.3dstate", NULL, "Bitmaps", USER_DEFINED_BEHAVIOR | EDITOR_MODE); //this will load the world in editor mode
//
//Example:
//
// A)
//		int rc=STATE_engine_load_world("my_world.3dstate", NULL, "Bitmaps", USER_DEFINED_BEHAVIOR);
//		if(rc!=OK) {
//			MessageBox("Failed loading world");
//			//lets exit the program
//			STATE_engine_close();
//			exit(-1);
//		}
//
// B)
//		//starting the engine with an empty world
//		STATE_engine_load_world(NULL, NULL, NULL, USER_DEFINED_BEHAVIOR);
STATE_IMPORT int STATE_WINAPI STATE_engine_load_world(char *world_file_name, char *world_directory_path, char *bitmaps_directory_path, int world_mode);

// The loaded world will be add to the existing world
// the given position indicates where the origin (note that the origin is 0,0,0 and not the center of the world) 
// of the added world will be located
// returns a handle to the group that encapsulates the added world
// if an error occurred it will return NULL
// It is important to check the return value
// Note that usually you will call STATE_group_set_location(added_world_grp, center_position);
// after calling STATE_engine_add_world() to move the center of the added world
// to its place. Not calling STATE_group_set_location() will put the origin
// of the added world in the location as given by the position argument. Note that this is different from  moving the center of the world to 
// the given position
//
// The function is relevant only for EDITOR_MODE. For VIEWER_MODE (the default mode)
// use STATE_object_create_from_file() instead.
STATE_IMPORT DWORD STATE_WINAPI STATE_engine_add_world(char *world_file_name, char *world_directory_path, char *bitmaps_directory_path, double position[3]);

//The same as STATE_engine_add_world() the only difference
//is that using this command the engine will also load general settings as the fog color and intensity.
DWORD STATE_WINAPI STATE_engine_add_world_with_settings(char *world_file_name, char *world_directory_path, char *bitmaps_directory_path, double position[3]);

// returns YES if no world is loaded into the engine
// otherwise returns NO
STATE_IMPORT int STATE_WINAPI STATE_engine_is_engine_empty(void);

/*

// This function is not in use anymore instead 
// When doing STATE_engine_load_world() for editor mode do an OR operation for example USER_DEFINED_BEHAVIOR | EDITOR_MODE
//return OK or VR_ERROR
//The editor mode is slower but it enables you to
// edit the world in real-time
// It will only effect the next world that will be loaded
// 
STATE_IMPORT int STATE_WINAPI STATE_engine_set_editor_mode(void);


// This function is not in use anymore instead 
// Viewer mode is the default mode. Each time 
// we do STATE_engine_load_world() and we dont use the EDITOT_MODE flag it 
// will load the world in viewer mode.
//
STATE_IMPORT int STATE_WINAPI STATE_engine_set_viewer_mode(void);
 */

STATE_IMPORT int STATE_WINAPI STATE_engine_is_editor_mode(void);



//The log window is the small window with the progress bar that appears 
//while the engine is loading. The reason it is called the Log Window (and not 
//the progress window for example ...) is that in the first versions of the engine this window
//was a lot bigger and it included some of the engine log as now appear in the STATE.log file
//
// Hides the log window
STATE_IMPORT void  STATE_WINAPI STATE_engine_hide_log_window(void);

//Makes the log window visible (The default)
STATE_IMPORT void  STATE_WINAPI STATE_engine_set_log_window_visible(void);

//Returns YES or NO. YES if the log window is visible. 
STATE_IMPORT int   STATE_WINAPI STATE_engine_is_log_window_visible(void);

// On the log window there are two fields that has to do with the progress bar
// one field represent the current progress and the other the target number
// that the progress should reach.
STATE_IMPORT int   STATE_WINAPI STATE_engine_get_log_window_progress(void);
STATE_IMPORT int   STATE_WINAPI STATE_engine_get_log_window_target(void);
STATE_IMPORT void  STATE_WINAPI STATE_engine_set_log_window_progress(int value);
STATE_IMPORT void  STATE_WINAPI STATE_engine_set_log_window_progress_text(char *text);
STATE_IMPORT void  STATE_WINAPI STATE_engine_set_log_window_target(int value);
STATE_IMPORT void  STATE_WINAPI STATE_engine_log_window_minimize(void);

//Add text to the log window
STATE_IMPORT void  STATE_WINAPI STATE_engine_log_window_output(char *text);

//Delete the old text and put the new text instead.
STATE_IMPORT void  STATE_WINAPI STATE_engine_log_window_set_text(char *new_text);

//Set the title of the log window
STATE_IMPORT void  STATE_WINAPI STATE_engine_log_window_set_title(char *new_caption);

//Obsolete
STATE_IMPORT HWND  STATE_WINAPI STATE_engine_log_window_get_hwnd(void);

// Delete all the text in 3DSTATE log files
// There are two log files. 3dstate.log and error.log.
STATE_IMPORT void  STATE_WINAPI STATE_engine_clear_STATE_log_files(void);

//Write the given text into the 3dstate.log file and the error.log. 
STATE_IMPORT void  STATE_WINAPI STATE_engine_write_to_log_file(char *text);

// (x,y) is a point in the rendering window
// usually we get this point from the events like OnLButtonDown()
// returns OK if a polygon was marked
// returns VR_ERROR if there is no polygon in that spot
STATE_IMPORT int  STATE_WINAPI STATE_engine_mark_polygon_at_point(int x, int y);

// *** Very useful. one of the most important functions
// Gets a two dimensional point (x,y) which is a point in the rendered window
// Usually we get this point from events like OnLButtonDown()
// The function returns the corresponding three dimensional point
// in the 3D world. Note that if the 2D point that is given is located where there is no 
// polygon then the function will return VR_ERROR. This is because the point
// in 3D is in infinity ... in that case the 3d point retrieved (result[] )
// will hold a point in that direction that is one million units away
// from the camera.
//
// Arguments:
//	x,y   -  A two dimensional point in the rendered window (client area)
//  result - a point x,y,z in the 3D world which corresponds to the given 2D point.
//			 When calling the function to content of this argument is not relevant.
//			 The function calculate the 3D point and returns it through this argument.
//
//  selected_object_handle - If the pixel X,Y (the 2D point) on the rendered window is taken from
//							 a dynamic object then this argument will hold the handle to this object.
//			 
//
//  selected_polygon_handle - a handle to a polygon that corresponds with the given 2D point.

// Return Value: OK or VR_ERROR
//
// See also STATE_engine_2D_point_to_3D_point_on_plane(), STATE_engine_2D_point_to_3D_point_on_sphere()
//			STATE_polygon_get_bitmap_pixel_rgba()
//
//
//	Remark:
//		Note that this function doesn't take a camera handle as one of its parameters.
//		Note also that the result of the operation is dependent on the current camera that is used.
//		The engine will use the current camera (The camera that was used in the last render)
//		In most of the cases this is the right thing to do. In the case when you want
//		to use a different camera for the calculation then use STATE_camera_set_current()
//		Before calling this function.
//
// Example:
//		int x=10, y=20;
//		double result[3];
//		DWORD selected_object_handle;
//		DWORD selected_polygon_handle;
//		int return_code;
//		return_code=STATE_engine_2D_point_to_3D(x,y,result, &selected_object_handle, &selected_polygon_handle);

STATE_IMPORT int  STATE_WINAPI STATE_engine_2D_point_to_3D(int x, int y, double result[3], DWORD *selected_object_handle, DWORD *selected_polygon_handle);

// This function is similar to STATE_engine_2D_point_to_3D() only that

// For the cases that we just need to know the direction in the world that our mouse (2d point)
// is pointing it is enough to call STATE_engine_2D_point_to_3D_point_on_sphere()
// If however we need to know the object or the polygon that is pointed by the 2D point then
// we should call STATE_engine_2D_point_to_3D() instead.
// Note that STATE_engine_2D_point_to_3D_point_on_sphere() is much faster then STATE_engine_2D_point_to_3D()
// An example:
//		A player in a game holds a shotgun. when the player moves the mouse we want to set the
//		direction of the shotgun accordingly. To do so it is enough to call STATE_engine_2D_point_to_3D_point_on_sphere()
//		and call the slower function STATE_engine_2D_point_to_3D() only when the player make a shot (only then we will need to know
//		what object / polygon in the world got hit.
STATE_IMPORT int  STATE_WINAPI STATE_engine_2D_point_to_3D_point_on_sphere(int x, int y, double sphere_radius,double p3d[3]) ;

// The function associates a 2D point on the rendered window and a plane in the 3D world.
// This function is useful for example when dragging an object with the mouse.
// Once we click on the object we can get the polygon that was clicked using STATE_engine_2D_point_to_3D()
// Now when dragging the object the mouse could move outside the object and then we wouldn't be able to use
// STATE_engine_2D_point_to_3D(). By using this function we tell the engine that the 2D point
// is related to a specific polygon even if the 2d coords are outside this polygon.
// Detailes:
// Gets a two dimensional point (x,y) which is a point in the rendered window
// Usually we get this point from events like OnLButtonDown()
// The function returns the corresponding three dimensional point
// in the 3D world that is on the given plane. 
//
// Arguments:
//	x,y   -  A two dimensional point in the rendered window
//  p3D - a point x,y,z in the 3D world which corresponds to the given 2D point and is located on the given plane.
//			 When calling the function to content of this argument is not relevant.
//			 The function calculate the 3D point and returns it through this argument.
//
//  polygons_plane - A plane in the 3D World (doesnt have to be a plane of any specific polygon)
//					 The result p3D point will always be on this plane.
//
// Return Value: OK or VR_ERROR
//
// See also STATE_engine_2D_point_to_3D_point(), STATE_polygon_get_plane()
//
// Example:
//		int x=10, y=20;
//		double result[3];
//		the_plane[0]=10; //This is a random plane. Instead we could have used STATE_polygon_get_plane()
//		the_plane[1]=11;
//		the_plane[2]=12;
//		the_plane[3]=14;
//		int return_code;
//		return_code=STATE_engine_2D_point_to_3D(x,y, the_plane ,result);
STATE_IMPORT int  STATE_WINAPI STATE_engine_2D_point_to_3D_point_on_plane(int x, int y, double polygons_plane[4],double p3d[3]);


// Gets a point in the world and returns the 
// matching point on the screen.
// Arguments:
//	p2D
//		On call p2D[2] has no meaning
//		on exit it will contain the 2D point.
//
//	p3D
//		on call p3D[3] should be the 3D point in the world.
//
// Return value:
// returns -1 if an error occurred. p2D contains 0,0
// returns 0 if every thing is OK. p2D contains a valid result
// returns 1 if the point is out of the window. p2D contains a valid result.
// returns 2 if the point is behind the camera. p2D contains a valid result though  not accurate
// returns 3 if both: behind the camera and out of the window. p2D contains a valid result though  not accurate
//
//	Remark:
//		Note that this function doesn't take a camera handle as one of its parameters.
//		Note also that the result of the operation is dependent on the current camera that is used.
//		The engine will use the current camera (The camera that was used in the last render)
//		In most of the cases this is the right thing to do. In the case when you want
//		to use a different camera for the calculation then use STATE_camera_set_current()
//		Before calling this function.
//
STATE_IMPORT int  STATE_WINAPI STATE_engine_3D_point_to_2D(double p3D[3], int p2D[2]);


//The engine will not use bitmaps. All the polygon will be drawn using color fill.
//This rendering mode does not have any effect when using hardware rendering.
//Using this function will make rendering a lot faster.
// On the whole this function is not very useful
//See also STATE_engine_set_color_fill_rendering_mode(), STATE_engine_set_normal_rendering_mode()
STATE_IMPORT void  STATE_WINAPI STATE_engine_set_color_fill_rendering_mode(void);

//The engine will not use bitmaps. Only the wire frame of the polygons will be drawn.
//This rendering mode does not have any effect when using hardware rendering.
//Using this function will make rendering a lot faster.
// On the whole this function is not very useful
//See also STATE_engine_set_color_fill_rendering_mode(), STATE_engine_set_normal_rendering_mode()
STATE_IMPORT void  STATE_WINAPI STATE_engine_set_wire_frame_rendering_mode(void);

//Reset the engine to its normal way of rendering (the default)
//See also STATE_engine_set_wire_frame_rendering_mode(), STATE_engine_set_color_fill_rendering_mode()
STATE_IMPORT void  STATE_WINAPI STATE_engine_set_normal_rendering_mode(void);


//Obsolete
//STATE_IMPORT void  STATE_WINAPI STATE_engine_toggle_rendering_mode(void);

//Obsolete
//STATE_IMPORT void  STATE_WINAPI STATE_engine_toggle_wire_frame_flag(void);


//About the brightness functions
//--------------------------------
//
//We dont recommend using these functions any more since they work differently in software 
//and hardware rendering.
//Use the brightness demo in the SDK to see how the change in brightness effect
//the rendered image. Note that the effect is different when using hardware
//rendering. We recommend to use these functions only if your application
//is not planed to run with hardware rendering.

//Obsolete
STATE_IMPORT void  STATE_WINAPI STATE_engine_set_default_brightness(void);

//Obsolete
STATE_IMPORT void  STATE_WINAPI STATE_engine_increase_brightness(void);

//Obsolete
STATE_IMPORT void  STATE_WINAPI STATE_engine_decrease_brightness(void);

//Obsolete
// Zero means according to what is set in the input world file.
// a positive number means make brighter. a negative number means darker
// values should be in the range -64 till +64
STATE_IMPORT void  STATE_WINAPI STATE_engine_set_brightness(int value);

//Obsolete
STATE_IMPORT int  STATE_WINAPI STATE_engine_get_brightness(void);

//Obsolete. Use STATE_engine_set_fog() instead.
//Increases the fog effect.
STATE_IMPORT void  STATE_WINAPI STATE_engine_increase_atmospheric_effect_intensity(void);

//Obsolete. Use STATE_engine_set_fog() instead.
//Decreases the fog effect.
STATE_IMPORT void  STATE_WINAPI STATE_engine_decrease_atmospheric_effect_intensity(void);

//Obsolete. Use STATE_engine_set_fog() instead.
//Set the fog intensity to the engine default.
STATE_IMPORT void  STATE_WINAPI STATE_engine_set_default_atmospheric_effect_intensity(void);

//Obsolete. Use STATE_engine_set_fog() instead.
//Returns the current fog intensity.
// 1 means according to what is set in the input world file.
// 2 means the atmospheric effect intensity is doubled.
// 0.5 means atmospheric effect intensity is half.
// 0 means that atmospheric effect is off.
STATE_IMPORT double  STATE_WINAPI STATE_engine_get_atmospheric_effect_intensity(void);

// Obsolete. Use STATE_engine_set_fog() instead.
// Set the fog intensity. 
// Using fog is one of the easiest ways to increase your game speed.
// The more intensed fog, the higher FPS (frames per seconds) you should expect.
// Note however that if you do not use fog at all or use an unnoticeable intensity of fog
// then it is better to shut off fog entirely (by calling STATE_engine_set_atmospheric_effect_intensity(0) )
//
//	Parameters:
//
// fog_intensity
//		1 means according to what is set in the input world file.
//		2 means the atmospheric effect intensity is doubled.
//		0.5 means atmospheric effect intensity is half.
//		0 means that atmospheric effect is off.
//
//		Note that if you test this function and suddenly you don't anything in your world that
//		usually means that you are using a too strong fog intensity. Try calling with a smaller value
//		till you get the desired effect.
STATE_IMPORT void  STATE_WINAPI STATE_engine_set_atmospheric_effect_intensity(double fog_intensity);

//Obsolete. Use STATE_engine_set_fog() instead.
//Changes the color of the fog between some pre selected values.
STATE_IMPORT void  STATE_WINAPI STATE_engine_toggle_atmospheric_effect(void);

//Obsolete. Use STATE_engine_set_fog() instead.
// Set the color of the fog.
// Even if you don't use fog, you might want to use this function
// to reduce visible cracks seen between adjacent polygons. For that matter one should use a color that is as close as possible
// to the dominate world in the world
//
// This function also set automatically the background color to the fog color (see STATE_engine_set_background_color() )
//
// Here are some tips for getting natural results:
// 1) Choose a color that is very close to the dominant color in your world.
//	  for example if your world is a desert then your fog color (atmosphere color) should
//	  lean towards the color of sand.
//
// 2) Choose colors where the red==green==blue .Shades of gray look best
//	  even if you want to have a blue fog color then don't use 0,0,255 
//	  this will be very unnatural ... Colors such as (180,200,220) where the blue is a little bit
//	  higher than the rest, will give a much better effect.
//	  Always try:
//	  Black - (0,0,0)
//	  Gray - (128,128,128)  or any shade of gray
//
// See also STATE_engine_set_background_color()
STATE_IMPORT void  STATE_WINAPI STATE_engine_set_atmospheric_effect(int red, int green, int blue);

//Obsolete. Use STATE_engine_set_fog() instead.
//Returns the current fog color
STATE_IMPORT void  STATE_WINAPI STATE_engine_get_atmospheric_effect(int *red, int *green, int *blue);

//Sets the color of the background. The background is the color of the window on which
//the rendering is done. If the rendered image does not cover the whole window then the 
//background color will be seen in the area where there are no polygons.
STATE_IMPORT void  STATE_WINAPI STATE_engine_set_background_color(int red, int green, int blue);

//Returns the current background color. For more information see STATE_engine_set_background_color()
STATE_IMPORT void  STATE_WINAPI STATE_engine_get_background_color(int *red, int *green, int *blue);

//This function will not influance hardware rendering
// Toggles between automatic perspective correction and
// no perspective correction(fastest).
STATE_IMPORT void  STATE_WINAPI STATE_engine_toggle_automatic_perspective_correction(void);

//This function will not effect hardware rendering
// Cancels automatic mode and toggle between several accuracy modes.
STATE_IMPORT void  STATE_WINAPI STATE_engine_toggle_perspective_correction_accuracy(void);

//The following functions (color accuracy) are not extremely useful, they can be used to
//speed up software rendering and has little effect when doing hardware rendering.
//The engine has the capability to render far object using color fill instead of using
//the polygon bitmap. This can speed up software rendering. When using color fill instead
//of bitmap it will reduce the "color accuracy"
//
//Set the distance from the viewer that is needed so that the engine will use
//color fill instead of using the bitmap of the polygon to its default value.
STATE_IMPORT void  STATE_WINAPI STATE_engine_set_default_far_objects_color_accuracy(void);


//Increase the distance from the viewer that is needed so that the engine will use
//color fill instead of using the bitmap of the polygon.
//See also STATE_engine_set_default_far_objects_color_accuracy()
STATE_IMPORT void  STATE_WINAPI STATE_engine_increase_far_objects_color_accuracy(void);

//Decrease the distance from the viewer that is needed so that the engine will use
//color fill instead of using the bitmap of the polygon.
//See also STATE_engine_set_default_far_objects_color_accuracy()
STATE_IMPORT void  STATE_WINAPI STATE_engine_decrease_far_objects_color_accuracy(void);


// Returns a number in the range [0, 31]
//See also STATE_engine_set_default_far_objects_color_accuracy()
STATE_IMPORT int  STATE_WINAPI STATE_engine_get_far_objects_color_accuracy(void);

//The value should be in the range [0, 31].
//For more information please see STATE_engine_set_default_far_objects_color_accuracy()
STATE_IMPORT void  STATE_WINAPI STATE_engine_set_far_objects_color_accuracy(int value);

// Obsolete. Not used anymore
//The engine can ignore polygons that are far away in order to speed up rendering.
//For example if our character in a game enters a building we could use
//this function to tell the engine that it doesnt have to work on far polygons
//that are outside the building ...
//When using fog the engine automatically ignores polygon that are totaly inside the
//fog.
STATE_IMPORT void  STATE_WINAPI STATE_engine_set_default_culling_depth(void);

// Obsolete. Not used anymore
//Increases the distance from the viewer in which the engine ignores polygons.
STATE_IMPORT void  STATE_WINAPI STATE_engine_increase_culling_depth(void);

// Obsolete. Not used anymore
//Decreases the distance from the viewer in which the engine ignores polygons.
STATE_IMPORT void  STATE_WINAPI STATE_engine_decrease_culling_depth(void);

// Obsolete. Not used anymore
//Returns a number that represent the distance from the viwer in which 
//the engine starts to ignore polygons.
//The given value is in the range 0-31
STATE_IMPORT int  STATE_WINAPI STATE_engine_get_culling_depth(void);

// Obsolete. Not used anymore
// The given value should be in the range [0 - Infinity ]
// Tipical values are 15, 20 ,30
STATE_IMPORT void  STATE_WINAPI STATE_engine_set_culling_depth(int value);

//This function is very important. It checks whether it is possible to move 
// between one point to another in the 3D world without intersecting any polygon.
// If the movement is possible it returns YES else returns NO
// If movement is not possible it returns the blocking polygon 
// and the intersection point.
// Note that the normal of the blocking surface is N=( plane[0], plane[1], plane[2])
// If the return value is NO and blocking_object==NULL
// it means that what blocks us is the static part of the world
// Note that the function will be faster if the distance between the start_location and the end_location is smaller
// Important NOTE:
// Polygons have only one side. 
// So there is a question what to do when we want to
// go through a polygon that is not visible to the camera
// (the camera sees only its back )
// The solution that we chose is that: (can be changed using STATE_engine_ignore_back_faced_polygons_for_collision_detection() )
// In Viewer (STATE_load_world() without EDITOR_MODE) mode we dont care if you see the polygon or not
// you can never go through it !
// In Editor mode we apply the following.
	// If the current camera sees the polygon then it is like
	// a solid wall (nothing can go through the wall and nothing can come from the wall)
	// If the current camera doesn't see the polygon then it is as if
	// the polygon doesn't exist at all.
	// If you didnt get it, we will try again.
	// say A is a point in space in the front of a polygon and B is a point on the back side 
	// of that polygon (The front is the side of the normal where you see the vertex in counterclockwise order)
	// then if the location of the camera is in the side of A then
	// one cant move not from A to B nor form B to A.
	// if the current camera is in the side of B
	// then it is possible to move both from A to B and from
	// B to A.
	// Note that the result is insensitive to the direction
	// of movement ( start_point to end_point or the other way around)
	// IMPORTANT:
	// If you want to move the camera from A to B. Lets say there is a one sided polygon
	// on the way that doesn't allow the movement. Now watch this one, there is a big
	// difference between the following options
	// I- moving the camera to B and then asking STATE_engine_is_movement_possible()
	// II- asking STATE_engine_is_movement_possible() while we are in A
	// In the first case we would get YES (error)
	// In the second case we would get the correct answer - NO
	// The reason for that is that if the camera is at point B then from point
	// B the polygon is not seen (we see only its back side which is invisible)
	// and if the obstacle is not seen then the movement is allowed !
//
// Arguments:
//
//	start_location:
//		The x,y,z coordinate if the starting point
//
//	end_location:
//		The x,y,z coordinate if the end point
//
//  intersected_polygon: If the line starting from start_point intersects a polygon in the world then
//						 the handle to this polygon will be returned through this argument
//
//  intersection: The x,y,z  coordinate of the intersection point.
//
//  blocking_object: If the intersected_polygon belongs to a dynamic object (see the STATE_object API) then
//					 the handle to this object will be returned through this argument.
//
//
//	Remark:
//		Note that this function doesn't take a camera handle as one of its parameters.
//		Note also that the result of the operation is dependent on the current camera that is used.
//		The engine will use the current camera (The camera that was used in the last render)
//		In most of the cases this is the right thing to do. In the case when you want
//		to use a different camera for the calculation then use STATE_camera_set_current()
//		Before calling this function.
//
//
//
// Example:
//		double start_location[3]={10,0,0};
//		double end_location[3]={10,0,1000};
//		double intersection[3];
//		DWORD intersected_polygon;
//		DWORD blocking_object;
//		int rc;
//		rc=STATE_engine_is_movement_possible(start_location, end_location, &intersected_polygon, intersection, &blocking_object);
//		if(rc==NO) .... //Collision
//	    else ....   //no collision
//
// See also: STATE_engine_ignore_back_faced_polygons_for_collision_detection(),
//			 STATE_engine_ignore_dynamic_objects_for_collision_detection()
//
STATE_IMPORT int STATE_WINAPI STATE_engine_is_movement_possible(double start_location[3], double end_location[3], DWORD *intersected_polygon, double intersection[3], DWORD *blocking_object);

// This function is only relevant for the way collision detection functions work (mainly STATE_engine_is_movement_possible() ). 
// By default STATE_engine_is_movement_possible() acts as follows: 
//		VIEWER_MODE (the default mode): does not ignore polygons that are back-facing the camera (if a polygon is back facing the camera it could still cause collision and become the blocking polygon when calling STATE_engine_is_movement_possible() )
//		EDITOR_MODE : Polygons back facing the camera are ignored and do not participate in the collision detection.
// To change these defaults one can use this function.
//
// Among the collision detection functions that are influenced by STATE_engine_ignore_dynamic_objects_for_collision_detection() are: 
//		STATE_engine\object_is_movement_possible(), STATE_engine_2D_point_to_3D(), STATE_engine_get_object_at_point_2D().
//		STATE_engine_get_polygon_at_point_2D(), STATE_object_drop_down() and other functions that is sensing the shape of the world
//		and that its result is dependent on the shape of the world
//
// Example:
//
//		//In editor mode this will change the default setting and will cause STATE_engine_is_movement_possible() to
//		//consider all the polygons for collision purposes. This might be useful if you are writing a game in EDITOR_MODE
//		STATE_engine_ignore_back_faced_polygons_for_collision_detection(NO);
//
// See also: STATE_engine_ignore_dynamic_objects_for_collision_detection()
STATE_IMPORT void STATE_WINAPI STATE_engine_ignore_back_faced_polygons_for_collision_detection(int YES_or_NO);


// This function is only relevant for the way collision detection functions work. 
// The function is used for making collision detection checks that exclude all dynamic objects
// For example, we might use STATE_engine_is_movement_possible() to measure the altitude of an aeroplane to the ground
// (We take the plane location and the same point just with a very low Z value to be used as a point below the plane
// then we call STATE_engine_is_movement_possible() and check collision between those two points)
// For the matter of this check we do not care for the dynamic objects in the world and above all we don't want 
// to get wrong altitude measurment if there is any dynamic object between the our plane and the ground)
// Calling STATE_engine\object_is_movement_possible() after calling STATE_engine_ignore_dynamic_objects_for_collision_detection(YES)
// Will have the same effect as the following commands
//
//		disable_all_objects_that_are_not_currently_disabled()
//		STATE_engine_is_movement_possible()
//		enable_objects_that_were_disabled() //restore the enabled objects
//
// The default is NO, which means that all the non-disabled objects are included in collision detection tests
//
// Among the collision detection functions that are influenced by STATE_engine_ignore_dynamic_objects_for_collision_detection() are: 
//		STATE_engine\object_is_movement_possible(), STATE_engine_2D_point_to_3D(), STATE_engine_get_object_at_point_2D().
//		STATE_engine_get_polygon_at_point_2D(), STATE_object_drop_down() and other functions that is sensing the shape of the world
//		and that its result is dependent on the shape of the world
//
// Return value:
//		return the previous state (YES or NO)
//
//
// Example:
//
//		int previous_state=STATE_engine_ignore_dynamic_objects_for_collision_detection(YES);
//		some code here ...
//		//restore previous state
//		STATE_engine_ignore_dynamic_objects_for_collision_detection(previous_state);
//
STATE_IMPORT int STATE_WINAPI STATE_engine_ignore_dynamic_objects_for_collision_detection(int YES_or_NO);

// if movement possible returns YES else returns NO
// If movement is not possible it returns the plane equation Ax+By+Cz+D=0
// and the intersection point.
// Note that the normal of the blocking surface is N=( plane[0], plane[1], plane[2])
// If the return value is NO and blocking_object==NULL
// it means that what blocks us is the static part of the world
// see also remarks on STATE_engine_is_movement_possible()
//
//	Remark:
//		Note that this function doesn't take a camera handle as one of its parameters.
//		Note also that the result of the operation is dependent on the current camera that is used.
//		The engine will use the current camera (The camera that was used in the last render)
//		In most of the cases this is the right thing to do. In the case when you want
//		to use a different camera for the calculation then use STATE_camera_set_current()
//		Before calling this function.
//
//
STATE_IMPORT int STATE_WINAPI STATE_engine_is_movement_possible_camera_space(double start_location[3], double end_location[3], DWORD *intersected_polygon, double intersection[3], DWORD *blocking_object);

//Counts how many polygons we will cross going from point1 to point2
//in addition it calculates the average normal of all the polygons between the two points
//
// Parameters:
//
//	point1,point2: A line (actually a segment) starting at point1 and ending at point2
//
//	combined_normal: The combined normal of all the intersected polygons.
//					 if for example we try to move through the corner of a room.
//					 The corner point is created from three intersectiong planes
//					 The combined normal will be the avareage of the three normals.
//
// Return value: The number of polygons that are intersected when going from point1 to point2
//
// Example:
//		double point1[3]={10,0,0};
//		double point2[3]={100,200,300};
//		double combined_normal[3];
//		int num_of_collisions;
//		num_of_collisions=STATE_engine_get_number_of_collisions(point1, point2, combined_normal);
STATE_IMPORT int STATE_WINAPI STATE_engine_get_number_of_collisions(double point1[3], double point2[3], double combined_normal[3]);


// Gets a two dimensional point (x,y) which is a point on the rendered window
// (Usually we get this point from events like OnLButtonDown() )
// If there is a dynamic object at this location (on the rendered image) then this function
// will return the object handle
// See also STATE_engine_2D_point_to_3D() and the STATE_object_ API.
STATE_IMPORT DWORD STATE_WINAPI STATE_engine_get_object_at_point_2D(int x,int y);


// Gets a two dimensional point (x,y) which is a point on the rendered window
// (Usually we get this point from events like OnLButtonDown() )
// If there is a polygon at this location (on the rendered image) then this function
// will return the polygon handle
// See also STATE_engine_2D_point_to_3D()
STATE_IMPORT DWORD STATE_WINAPI STATE_engine_get_polygon_at_point_2D(int x,int y);


//The default rendering window is the window on which the engine render
//when we call STATE_engine_render() with a NULL window
//(example: STATE_engine_render(NULL,camera))
//If the return value is NULL it means that the function failed
//One can use this handle with many of the win32 api function
//that deals with windows
STATE_IMPORT HWND STATE_WINAPI STATE_engine_get_default_rendering_window_hwnd(void);

//The default rendering window is the window on which the engine render
//when we call STATE_engine_render() with a NULL window
//(example: STATE_engine_render(NULL,camera))
STATE_IMPORT void STATE_WINAPI STATE_engine_maximize_default_rendering_window(void);

//The default rendering window is the window on which the engine render
//when we call STATE_engine_render() with a NULL window
//(example: STATE_engine_render(NULL,camera))
STATE_IMPORT void STATE_WINAPI STATE_engine_minimize_default_rendering_window(void);

//The default rendering window is the window on which the engine render
//when we call STATE_engine_render() with a NULL window
//(example: STATE_engine_render(NULL,camera))
STATE_IMPORT void STATE_WINAPI STATE_engine_set_default_rendering_window_title(char *text);

//The default rendering window is the window on which the engine renders
//when STATE_engine_render() is called with a NULL window
//(example: STATE_engine_render(NULL,camera))
//Arguments: 
// left_x, top_y : the top left corner of the window
// width, height: the dimension of the window
STATE_IMPORT void STATE_WINAPI STATE_engine_set_default_rendering_window_size(int left_x, int top_y, int width, int height);

// Obsolete, use STATE_group_create_reduced_copy() instead
// accuracy is used when we try to check whether two polygons
// have the same plane (so we can unite them to one polygon). 
// If this number for example is 0.1 and 
// the X component if the normals are planeA[0]=1 and planeB[0]=0.9
// then the join operation will take place.
// can be done only in editor mode.
// returns the number of polygons that were reduced.
STATE_IMPORT int STATE_WINAPI STATE_engine_reduce_polygons_count(double accuracy);

// Add a polygon to the world. A polygon can be created and manipulated
// but it is not drawn until this function (or STATE_object_add_polygon() ) is called.
//
// This function also calls STATE_polygon_is_valid() to check the validity of the added polygon.
// If STATE_polygon_is_valid() returns NO (not valid) it will try to
// fix the polygon. If the attempt to fix the polygon will fail the
// function will return VR_ERROR.
//
// This function works only in EDITOR_MODE.
// To add a polygon in Viewer Mode (The default mode) use the
// function STATE_object_add_polygon(). 
// In viewer mode it is only possible to add polygons to the dynamic part of the world
// using STATE_object_add_polygon().
//
// Returns OK or VR_ERROR.
STATE_IMPORT int STATE_WINAPI STATE_engine_add_polygon(DWORD polygon_handle);


// Save the 3D world as a file.
//
// Arguments:
//	file_name:
//			The file name should have the extension .wld (for text format) 
//			or .STATE (for binary format) or should be without 
//			an extension at all (.wld will be added automatically)
//			if a relative path is provided it will be relative to
//			the directory of the executable.
//
//  group_to_save: The group to save. Give NULL if you would like to save the whole world.
//				   If a group handle that is not NULL is used then only the given group will be saved.
//
//	save_flags: See below.
//
// returns OK or VR_ERROR
//
// Note: 
//		1)
//		When saving the world the engine automatically calculates the vertices normal. If you set the points normal
//		yourself and you don't want the engine to overwrite it then call
//		STATE_light_calcualte_vertices_normals_automatically() before calling STATE_engine_save()
//
//		2)If the saved world is a terrain, sphere or any round surface then we suggest calling
//		  STATE_light_calcualte_vertices_normals_automatically(SET_VERTEX_NORMAL_EQUAL_TO_AVERAGE_OF_ADJACENT_POLYGONS);
//		  before doing save. This is only relvant if light will be applied on the saved world.
//		
//
// Example: 
//		//set a normal mode calculation that is good for terrain and round surfaces. This is only relvant if light
//		//will be applied on the saved world. If you do use light then we suggest you test  STATE_light_calcualte_vertices_normals_automatically()
//		//with different modes.
//		STATE_light_calcualte_vertices_normals_automatically(SET_VERTEX_NORMAL_EQUAL_TO_AVERAGE_OF_ADJACENT_POLYGONS);
//		//Save the whole world including the bitmaps that will be saved in bmp format
//		STATE_engine_save("World1.3dstate", NULL, SAVE_FLAT_BITMAP_PATH | SAVE_BITMAP_AS_BMP);
//
// possible flags: (note that we can do an OR operation between them)
#define SAVE_DEFAULT			0
#define SAVE_ONLY_WHATS_NEEDED	1 //does NOT save extra resources that are not needed to the saved group. it will save only animations that are needed for the polygons in the models. Cameras backgrounds and tracks will NOT be saved
									// see also SAVE_MODELS_ONLY
#define SAVE_FLAT_BITMAP_PATH	2 //Recomended. STATE World Builder always uses this flag. For every polygon the name of its bitmap wont include the path just the name. the same for animations. this is very convenient if we want to keep all the bitmap in one directory.
								// If we also save bitmaps then the save will be done with flat bitmap path no matter the value of this flag.
// The following options will save the world as a complete project including the bitmap files
// The function will make a directory called Bitmaps bellow the given path, and there it will put all the bitmaps
// The idea here is that the world becomes a "stand alone" it can be send through e-mail, put on a diskette etc ...
// using one of the bitmap flags will cause the save to be done with flat bitmap path no matter the value of this flag.
// using different kind of bitmaps type is allowed. 
#define SAVE_BITMAP_AS_JPG		4  //The bitmaps are saved in a JPG format. This should be done when optimizing for space. The down side is that it would take more time to load the world. and
									// the quality of some bitmaps might suffer. If transparent bitmap have spots or that some parts are missing one can modify by hand
									// The *.flt file for that bitmap.
								    //for more information about the flt files see at the bottom of this file
								    // in the help appendixes

#define SAVE_BITMAP_AS_BMP		8  //The bitmaps are saved in a BMP format. This should be done when we want the ability to modify the bitmaps
									// using bitmaps editors like PhtoPaint, Photoshop, Paintbrush etc ... Remember to delete the jbm file each time you modify the bitmap.
#define SAVE_BITMAP_AS_JBM		16 //The bitmaps are saved in a JBM format. This is the engine internal format. This should be done when we want the loading time of the world to be as short as possible.

//see also the flag SAVE_BITMAP_OPTIMIZE below

#define SAVE_IN_MULTIPLE_FILES	32  // * Not very useful. Instead of putting all the models in one file, it saves each model
									// in a separate file. This way is easier to change the world
									// by hand. In this method it is also very easy to replace components (since each one is
									// separate file.

#define	SAVE_ONLY_MODELS 64 //Save the model and its bitmaps and only the animations that are currently used by
							// the model. This is the same as to call with SAVE_ONLY_WHATS_NEEDED when all the save flags are set to NO
							// ( see STATE_camera_set_save_flag() , STATE_animation_set_save_flag(0 etc ...)

#define SAVE_RELEASE 128 //1-	All the polygons will be put inside module main
						 //		this makes a faster save and a faster load afterwards
						 //		When we load a world that was save using SAVE_RELEASE we
						 //		will lose all group affiliations.
						 //2-	polygons that have their release_save flag set to NO wont be saved
						 //		for example some models have polygons that are used only as a bottom
						 //		(applications like 3DSTATE World Builder need a bottom polygon so when
						 //		we drop an objects on a floor it will know how to put it, in some models
						 //		a bottom polygon is part of the model and in others (like trees) it is just an auxiliary polygon)
						 //		the release_save_flag can be set by STATE_polygon_set_release_save_flag() or
						 //		by specifying NO\YES in the RELEASE_SAVE field in the world text file

#define SAVE_BITMAP_OPTIMIZED 512 //This option overrides the following flags SAVE_BITMAP_AS_JPG, SAVE_BITMAP_AS_BMP, and SAVE_BITMAP_AS_JBM
								  //with this option all the bitmaps will be saved in jpg format
								  //except those that are being used with a transparency, those will be saved
								  //in BMP format. This flag is used when we want to save in jpg format
								  //but we dont want the jpg format to ruin the transparency bitmaps.
								  //The problem is that jpg images are not identical to the original and
								  //this makes a big problem with the transparent color because instead of one background color
								  //for example 255,255,255 we get several like 254,255,255 and 255,254,254 etc ...
								  //the engine will only eliminate one transparent color leaving all the other nuances
								  //If you didnt understand, just try to save a world using the flag SAVE_BITMAP_AS_JPG
								  //and then load the saved world and look at polygons that use a transparent bitmap.
								  //another way to deal with this problem is using the flt file that is saved together with each jpg bitmap
								  //for more information about the flt files see at the bottom of this file
								  // in the help appendixes

#define SAVE_WITHOUT_GROUPS	 1024 //* Not useful. This flag is only valid when saving in the binary format (to save in binary format give a file name that has a "3DSTATE" name extension instead of "wld")
								  //In many applications that use viewer mode, the STATE_group_ API is not used at all
								  //In this case you might want to use this flag in order to decrease the size of the saved file.

#define SAVE_WITH_BSP_TREE_INFORMATION 2048 //The use of this flag is not recommended.
											//This flag is only valid when saving in the binary format (to save in binary format give a file name that has a "STATE" name extension instead of "wld")
											//The bsp information is needed only when the world is loaded in VIEWER_MODE (the drfault mode)
											//If a world will be loaded in viewer mode without the bsp information then the
											//engine will create this information on its own. Creating the bsp tree is a lengthy operation
											//so it is better if it is done at save time rather then in load time. To conclude you should use this flag in
											//the following situation:
											//The world is meant to be used in VIEWER_MODE and this is the release version.
											//Dont use this flag if you still have to do some editing to the world in World Builder.
											//Very Important ! In many cases a world that was saved with this flag will be unusable in editor mode
											//(because in the process of creating the bsp tree many polygons have to be cut into smaller fractions)
											//so because of that you should also save another version without using this flag in case you might want in the future to
											//edit this world.

#define SAVE_WITHOUT_CALCULATING_POINT_NORMALS 4096 // * Very useful (shorten save time). When saving the world the engine calculates the normal of all the points
													//Use this flag if you have modify the points normal and you dont want the engine
													//to overwrite your point normal calculations. Another reason you might want to use this flag is to speed-up
													//the time it takes to save a world. Using this flag one can reduce some significant time
													//since the normal calculations are quite a time consuming process. We don't recommend
													//using this flag for speeding the saving function because if the points normal are not calculated
													//and the world is lit then the lighting result will be inaccurate.
													//See STATE_point_get_normal() for more information. 
							

STATE_IMPORT int STATE_WINAPI STATE_engine_save(char *file_name, DWORD group_to_save, int save_flags);

//Use STATE_engine_save() with a .x extension ...
//STATE_IMPORT int STATE_WINAPI STATE_engine_save_in_directx_format(char *file_name);



// Redraw on the window the last rendered image
// Usually used together with STATE_engine_render_on_dc()
// Note that this function also releases the dc the was obtained
// when calling STATE_engine_render_on_dc() this means that
// all the function that use the returned dc (from dc=STATE_engine_render_on_dc() )
// must be called before calling STATE_engine_copy_image_from_dc_to_screen().
// 
//See STATE_engine_render_on_dc() for more information.
STATE_IMPORT void  STATE_WINAPI STATE_engine_copy_image_from_dc_to_screen(void);

//obsolete. Completely identical (Physically the same) to STATE_engine_copy_image_from_dc_to_screen()
STATE_IMPORT void  STATE_WINAPI STATE_engine_bitblt(void);

//Like STATE_engine_render() only that it renders on a HBITMAP
//A HBITMAP is a WIN32 type (nothing to do with STATE) for more information
//about HBITMAP check your compiler help.
//The size of the created bitmap will match the size of the window that
//was last used for regular rendering.
//Note that there is no need to free the HBITMAP that is created. The engine will automatically
//delete the old HBITMAP when calling STATE_engine_render_on_bitmap() again.
// Return value: on failure returns NULL. On success returns an HBITMAP
// 
//
//Example A: Taking a screenshot
//
//		HBITMAP hbm=STATE_engine_render_on_bitmap(your_window_or_null, your_camera_or_null);
//		STATE_utilities_save_bitmap(hbm, "d:\\screenshots\\shot.bmp");
//
// Example B: Using the rendered image as a regular bitmap that
//			  can be put on a polygon.
//
//		HBITMAP hbm=STATE_engine_render_on_bitmap(NULL, NULL);
//		STATE_utilities_save_bitmap(hbm, "d:\\screenshots\\shot.bmp");
//		DWORD STATE_bmp=STATE_bitmap_load("d:\\screenshots\\shot.bmp", -1);
//		DWORD test_poly=STATE_polygon_get_first_polygon();
//		STATE_polygon_set_bitmap_fill(test_poly, STATE_bmp);
//
// Please note that Example B uses the harddisk to do the conversion between 
// a HBITMAP and a bitmap that can be used with STATE_bitmap API.
// Currently there is no other way to do this. This is not as bad as it sounds
// Since the operating system will use its buffers so when we will do
// STATE_bitmap_load() it doesnt wait for the file to be written to the disk
// it takes the file immediately from the operation system buffers.
//
// See also STATE_utilities_save_bitmap(), STATE_engine_render_on_dc()
STATE_IMPORT HBITMAP  STATE_WINAPI STATE_engine_render_on_bitmap(HWND hwnd, DWORD camera_handle);



//This function is very important. It allows painting and writing over the rendered image
//The function is exactly like STATE_engine_render() only that it doesnt write the
//rendered image into the screen, instead the image is left on the device context ( dc )
//If you dont know what is a Device Context then dont bother about it
//just think of it as a handle that many WIN32 graphic functions get.
//The beauty of this technique that one can use all the many Windows function
// (GDI functions). Please be aware that Windows graphics function are
// some time quite slow.
// After finishing painting on top of the rendered image 
// use STATE_engine_copy_image_from_dc_to_screen() to draw the finished image on the
// window
//
//Here is a sample how to use STATE_engine_render_on_dc()
//
//		HDC dc=STATE_engine_render_on_dc(NULL,camera);	
//
//	    //paints a circle on the device context. 
//		//Note that ELlipse() is a Windows function (WIN32 API)
//		//Look in you compiler help for information about that function.
//		//There are many other function that you can use for writing text, changing
//      //colors, changing the fill pattern etc ...
//		Ellipse(dc, 100,100,200,200);
//		//paint another circle
//		Ellipse(dc, 220,220,320,320);
//		
//		//Get the picture on the screen. Without calling this function you
//      //wont see anything.
//		STATE_engine_copy_image_from_dc_to_screen();
//		
//		//Note that STATE_engine_copy_image_from_dc_to_screen()
//		//releases dc (the device context) so
//		//after calling it one cant use dc any more.
//		//a new valid dc will be obtain when we repeat it in a loop
//		// dc=STATE_engine_render_on_dc();	
//
// For drawing a wire frame lines around polygons note the function STATE_engine_3D_edge_to_2D()
//
STATE_IMPORT HDC STATE_WINAPI STATE_engine_render_on_dc(HWND hwnd, DWORD camera_handle);



//This function converts a given line in the 3D world to its projection on the rendered window.
//This could be very useful when drawing a wire frame to a polygon.
//
// Parameters:
//		p1, p2 : A line (a segment) in the 3D world
//		p1_2D, p2_2D : The equivalent line on the rendered window.
//
// Return value:
//		returns one of the four:
//		EDGE_NOT_SEEN if the edge is outside the rendering window
//					in this case p1_2D and p2_2D are not valid
//		EDGE_FULLY_SEEN if the edge is fully inside the rendering window
//					in this case p1_2D and p2_2D are the desired result

//		EDGE_PARTIALLY_SEEN if some of the edge is inside the window
//					in this case p1_2D and p2_2D are the portion that inside the rendering area

//		VR_ERROR an error occurred 
//					in this case p1_2D and p2_2D are not valid
//
// Example:
//	double p1[3]={10,20,30};
//	double p2[3]={100,200,300};
//	double p1_2d[3], p2_2d[3];
//	inr rc;
//	rc=STATE_engine_3D_edge_to_2D(p1,p2, p1_2d, p2_2d);
//
STATE_IMPORT int STATE_WINAPI STATE_engine_3D_edge_to_2D(double p1[3], double p2[3], int p1_2D[2], int p2_2D[2]);


// returns one of the four
// EDGE_NOT_SEEN if the edge is outside the rendering window
//					in this case p1,p2 == clipped_p1,clipped_p2

// EDGE_FULLY_SEEN if the edge is fully inside the rendering window
//					in this case p1,p2 == clipped_p1,clipped_p2

// EDGE_PARTIALLY_SEEN if some of the edge is inside the window
//					in this case clipped_p1,clipped_p2 is the seen part

// VR_ERROR an error occurred 
//					in this case clipped_p1,clipped_p2 are not valid

STATE_IMPORT int STATE_WINAPI STATE_engine_clip_edge(double p1[3], double p2[3], double clipped_p1[3], double clipped_p2[3]);


// Calling this function will eliminate the render to the given group (and groups that belong to the given group)
// Polygons that are not included in this group wont be rendered
// If a NULL is given then all the polygons will be rendered (the default)
// Arguments: 
//		grp_to_render_handle: The group to render
STATE_IMPORT void STATE_WINAPI STATE_engine_set_group_to_render(DWORD grp_to_render_handle);

// Calling this function will eliminate the render to the given objects and its sons
// Polygons that are not included in this objects wont be rendered
// If a NULL is given then all the polygons will be rendered (the default)
// Arguments: 
//		obj_to_render_handle: The group to render
//
// See also STATE_object_set_father_object()
//
STATE_IMPORT void STATE_WINAPI STATE_engine_set_object_to_render(DWORD obj_to_render_handle);
STATE_IMPORT DWORD STATE_WINAPI STATE_engine_get_rendered_object();

// -- bitmaps stuff

//Returns the number of bitmaps that are currently loaded into the engine.
STATE_IMPORT int STATE_WINAPI STATE_engine_get_number_of_loaded_bitmaps(void);

//Calling this function will free all the bitmaps that are not used at this moment.
//Not used means that there is no polygon or animation that uses them.
// Returns the number of bitmaps that were unloaded
STATE_IMPORT int STATE_WINAPI STATE_engine_unload_unused_bitmaps(void);

//This function is not very useful
//Unload all the bitmaps from the engine.
STATE_IMPORT void STATE_WINAPI STATE_engine_unload_all_bitmaps(void);


//Obsolete. Please use STATE_engine_translate_movement_on_screen_to_world()
STATE_IMPORT int STATE_WINAPI STATE_engine_translate_movement_on_screen_to_movement_in_world(double p3D[3], double result_p3D[3], int delta_x, int delta_y);

// This function is used to solve the following problem:
// An object inside our world is dragged with the mouse delta_x,delta_y pixels.
// Where should we put the object now so that it will reflect the correct move.
// Arguments:
//	p3D -
//		p3D is a point in the world.
//
//	delta_x,delta_y
//		delta_x, delta_y is the desired change in the location on
//		the screen of the projection of p3D.
//
//  result_p3D
//		result_p3D is a location that if p3D will move to this location
//		its projection on the render image will move delta_x pixels right
//		and delta_y pixels down (negative gives an opposite direction)
//
// returns OK or VR_ERROR and the reslut x,y,z point in result_p3D.
STATE_IMPORT int STATE_WINAPI STATE_engine_translate_movement_on_screen_to_world(double p3D[3], double result_p3D[3], int delta_x, int delta_y);


// gets YES or NO. YES will move the objects automatically each
// time we do STATE_engine_render(). The movement is according
// to the instructions in the wld file (CHASE ... ) or as was set in real-time.
// The default is YES.
// See also STATE_camera_advance_all()
// and STATE_object_advance_all()
// Note works only in viewer mode (viewer mode is the default mode)
STATE_IMPORT void STATE_WINAPI STATE_engine_advance_objects_automatically(int yes_no_flag);

// gets YES or NO. YES will move the cameras automatically each
// time we do STATE_engine_render(). The movement is according
// to the instructions in the wld file (CHASE ... ) or as was set in real-time.
// The default is YES.
// See also STATE_camera_advance_all()
// and STATE_object_advance_all()
STATE_IMPORT void STATE_WINAPI STATE_engine_advance_cameras_automatically(int yes_no_flag);



//This function doesn't  work in editor mode. When zbuffer is off
// get YES or NO. zbuffer is the name of the algorithm that is often used to solve
// the hidden surface removal problem. If NO is given the engine will use
// BSP algorithms (binary space partitions) the BSP algorithms are faster
// and more accurate most of the time. There are some cases where BSP algorithms will
// be less accurate. Try testing both methods and see which is better for your world.
// Note that using this function is equivalent to calling STATE_engine_load_world()
// with (or without) the DONT_USE_ZBUFFER flag. Using this function makes it possible to change
// methods at run-time.
//The return value is OK or VR_ERROR (error if we try to do it in editor_mode)
// The default is zbuffer ON.
// example: STATE_engine_use_zbuffer(YES);
STATE_IMPORT int STATE_WINAPI STATE_engine_use_zbuffer(int yes_no_flag);


// returns YES or NO. YES if the zbuffer algorithm is used.
// see STATE_engine_use_zbuffer() for more details.
STATE_IMPORT int  STATE_WINAPI STATE_engine_is_zbuffer(void);



// Returns an array of all the extensions supported by the engine.
// returns also in number_of_supported_formats the number of extensions supported.
// these are the extensions that can be used with STATE_engine_load_world()
// and with STATE_engine_save()
// Example of usage:
//		int num;
//		table=STATE_engine_get_supported_extensions_array(&num);
//		for(int i=0; i<num; i++) MessageBox(table[i]);
//
// The strings that we will get are "wld", "STATE","mft", "raw", "x" and more depending  on the
// version of STATE dll.
STATE_IMPORT char ** STATE_WINAPI STATE_engine_get_supported_formats_extensions_array(int *number_of_supported_formats);

// Usage is exactly like STATE_engine_get_supported_formats_extensions_array()
// The string that we will get is similar  to "STATE's 3D worlds file format" for example.
STATE_IMPORT char ** STATE_WINAPI STATE_engine_get_supported_formats_names_array(int *number_of_supported_formats);

// a value of 0 will make rendering as fast as possible but 
// perspective correction will be at its lowest level (try it out and see if the lake of accuracy influence your world)
// a value of one is the default. values less than one will make rendering faster (but less accurate)
// values bigger than one will make rendering slower but more accurate.
// normally you wouldnt find values bigger than 3 useful.
// the value 1000 has a special meaning it will make perspective correction
// 100% correct. This of course will make rendering very very slow,
// use this value for pause mode or for screen capturing or when ever speed
// is not important at all.
STATE_IMPORT void STATE_WINAPI STATE_engine_set_perspective_correction_accuracy(double value);

// see STATE_engine_set_perspective_correction_accuracy() for more details
STATE_IMPORT double STATE_WINAPI STATE_engine_get_perspective_correction_accuracy(void);


// === Performances Querying  ====== //

// Measure the time between two consecutive calls to render.
// returns the time in milliseconds.
// STATE_engine_render() must be called at least twice before this
// function can be used (if not it will return 0) 
// The function calculate the average time starting with the first render to the last render
// See also STATE_engine_get_computer_speed_factor() which returns the average of the last 50 program cycles
STATE_IMPORT DWORD STATE_WINAPI STATE_engine_get_average_program_cycle_time(void);


// Measure the time between two consecutive calls to render.
// returns the time in milliseconds.
// STATE_engine_render() must be called at least twice before this
// function can be use (if not it will return 0) 
// See also STATE_engine_get_computer_speed_factor() which returns the average of the last 50 program cycles
STATE_IMPORT DWORD STATE_WINAPI STATE_engine_get_last_program_cycle_time(void);

// Returns the execution time (in milliseconds) of the last call to STATE_engine_render().
// Through this number one can calculate the current fps (frames per second)
// t=STATE_engine_get_last_render_execution_time()
// if (t==0) return(0); //STATE_engine_render() wasnt called even once. protect against division by zero
// fps=1000/t;
// see also STATE_engine_get_average_render_execution_time()
STATE_IMPORT DWORD STATE_WINAPI STATE_engine_get_last_render_execution_time(void);

// Returns the average execution time (in milliseconds) of STATE_engine_render().
// Through this number one can calculate the average fps (frames per second)
// t=STATE_engine_get_average_render_execution_time()
// if (t==0) return(0); //STATE_engine_render() wasnt called even once. protect against division by zero
// fps=1000/t;
STATE_IMPORT DWORD STATE_WINAPI STATE_engine_get_average_render_execution_time(void);

// Obsolete. Use the 3D_card API instead (STATE_3D_card....)
// return YES or NO
STATE_IMPORT int STATE_WINAPI STATE_engine_check_3d_hardware_support(void);


// Using this function one can build an application that
// scan a map from the atlas and build a 3D world from it
// the 3D world can be 100% accurate (but never more accurate than
// the map itself ...)
// The function takes a regular windows bitmap (in 256 color format or 24bit per pixel RGB format)
// painted with gray scales and convert it into a 3d model. The 3d model is built according
// to the colors of the bitmap the more white it is the higher it is.
// (the peaks of the mountains will be much brighter than the valleys in the bitmap)
//Input params:
// bitmap_file_name:
//		The name of the bitmap that according to its colors the model is built
//		The bitmap must be in 24 bit rgb format or 8 bit per pixel format (256 colors).
//		The colors of the bitmap should be in gray scale (not a must) since
//		the function calculates height according to their brightness (sqrt (red^2 + green^2 + blue^2)  Never mind if you dont understand the calculation)
//		The bitmap could be in any size.
//		examples: "my_bitmap.bmp" ; "c:\\gs_maps\\France.bmp" ; "..\\rgb24maps\\Jerusalem.map" etc ...
//
// level_of_optimization:
//		This number is responsible to the polygon reduction level.
//		A number in the range [0 - 3000]
//		The bigger the number is the stronger the optimization
//		0 means that there is no optimization so the number of polygon in the output file
//		will be as set by the argument max_number_of_polygons
//		3000 will make such a strong optimization that the model
//		will be simplified to one flat polygon no matter what source bitmap we had !
//		common numbers that you might try are
//		0.1 , 1, 10 ,100 ,200 ,400
//		Use a big numbers if it is important to reduce as many as possible polygons
//		Use small number if the terrain model must be very detailed.
//
//  max_number_of_polygons:
//		A big number makes a very detailed object
//		a small number makes a model with less polygons
//		so it can be rendered fast in real-time
//		Try 2000 to begin with and if you need more details increase this number
//		if on the other hand you need more speed when rendering, use a smaller number
//
//  bitmap_to_wrap_over_handle:
//		This is a handle to the bitmap that will be used to paint the created model
//		for example this bitmap could be a texture of rocks, sand snow , grass etc ...
//		If a NULL is given, then the model will be painted according to the given legend.
//		You get the handle for the bitmap using STATE_bitmap_load()
//		Note that if you want the bitmap to repeat more than once on the model (tile mode)
//		then the dimensions of the bitmap must be in the power of two (for example 32,64,128,256,512,1024,2048 etc ... 
//		all these numbers are powers of two).
//		see also STATE_group_wrap_a_bitmap()
//
//  tile_x, tile_y:
//		The numbers of time that the tile of the bitmap is to repeat itself
//		These numbers are only relevant if bitmap_to_wrap_over_handle
//		is not NULL.
//		examples: 1,1 (no tile) ; 4,4 (bitmap will repeat 16 times) ; 7,3 ; 712,1095 etc ...
//
//	legend, number_of_colors_in_legend:
//		If bitmap_to_wrap_over_handle is NULL, then
//		The created model is painted according to the given legend.
//		legend is an array of RGB colors. If for example we give
//		three colors then the first color will be the color of the bottom
//		third of the model. The second color will be the middle color.
//		the last color will be the color of the top third.
//		If NULL is given (instead of the legend array) then the
//		model is colored according to the default legend.
//		number_of_colors_in_legend is of course the number of colors that
//		the given legend array contains.
//		example: 
//			BYTE my_legend[2][3]={0,255,0,  255,255,255};
//			Here we define two colors. 0,255,0 is green.
//			255,255,255 is white. This would cause the bottom half
//			to be painted with green colors and the top half with white hues
//
//	dimensions:
//		The created model is built so its center is in the origin ( point 0,0,0 )
//		and its width is dimension[0], its length is dimension[1] and its height is dimension[2]
//		Note that if for example dimension[2]==100 (the height of the model)
//		then the model will be built from -50 to +50 (remember that the center is in the origin ...)
//		Chose the number according to the source map.
//		If for example it was a map of Israel then you should give the following dimensions
//		dimension[0]=60 (The width of Israel is 60 Km) (quit small isn't it ?)
//		dimension[1]=300 (The length of Israel is 300 Km)
//		dimension[2]=2 (The difference between the tallest mountain to the lowest place is 2 Km)
//
//	number_of_polygons_in_group:
//		This value is set by the function so it doesn't have to be initialized.
//		When the function returns it contains the number of polygons in the created model
//		This parameter is updated during the time that the function runs and can be
//		accessed by a different process to get an idea on the the function progress
//		This parameter can be used to estimate how much time the function would take
//		If it is not one of your concerns just give NULL.
//		This function can take a lot of time (if we have a huge max_number_of_polygons for example 100000)
//		so you might like to run it in a different process ,while showing
//		a progress bar. Send a pointer to a global variable
//		so that you can read its value outside the process.
//		Though it is not known in advance how much time it takes, this number give some ideas
//		at first this number will be equal to max_number_of_polygons and it will go down by one when ever
//		a polygon is reduced. Till what number it gets to depends on level_of_optimization and
//		the source bitmap itself (If the map is flatter it will be possible to reduce more polygons and vice versa)
//
// Return Value:
//		On failure returns NULL
//		On success returns a handle to the created group
//
//	Example: 
//		double dimensions[3]={20000,20000,6000};
//		DWORD terrain_group=STATE_engine_create_terrain_from_bitmap("map.bmp", 100, 3200, NULL,1,1,NULL, 0,dimensions,NULL);
//		double step[3]={0,0,-3500};
//		STATE_group_move(terrain_group, step, WORLD_SPACE);
//		STATE_camera_set_location(STATE_camera_get_default_camera(),0,0,0);
//		STATE_camera_set_direction(STATE_camera_get_default_camera(),-1,0,0);
//
STATE_IMPORT DWORD STATE_WINAPI STATE_engine_create_terrain_from_bitmap(char *bitmap_file_name, double level_of_optimization, int max_number_of_polygons, DWORD bitmap_to_wrap_over_handle, int tile_x, int tile_y, BYTE legend [][3], int number_of_colors_in_legend,double dimensions[3], int *number_of_polygons_in_group);


// This function is currently in its beta stages.
// Please use STATE_engine_create_terrain_from_bitmap()
// It has several important advantages over STATE_engine_create_terrain_from_bitmap()
//
// 1) It automatically cuts large bitmaps into smaller parts
//	  The bitmap that is used to wrap over the terrain could be very big
//	  though the existing 3D cards are limited in handeling large bitmaps.
//	  Most of the 3D cards today can handle bitmaps not larget than 1024x1024
//
// 2) This function is faster. The function doesnt try to minimize the number of polygons
//	  To reduce the number of polygons call STATE_group_create_reduced_copy() after creating the terrain
//
// 3) The function can also accept a text file with the terrain mesh
//
//
//Parameters:
//
//	elevation_source: 
//		Currently there are two options:
//		1) A bitmap file (bmp)
//			The	brighter pixels present higher eltitude
//
//		2) an XYZ file (extension xyz)
//
//		   xyz file is a text file that contains a mesh of points x,y,z
//		   Example:
// 
//			x1 y1 z01		x2 y1 z02		x3 y1 z03 .......
//			x1 y2 z11		x2 y2 z12       x3 y2 z13 .......
//				.
//				.
//				.
//				.
//			x1 Yn Zn1		x2 Yn zn2       x3 Yn zn3 .......Xn Yn Znn
//
//
//
//	terrain_bmp: 
//		The image that is put on top of the terrain.
//		This image could be very big.
//		In the resulted world this image will be split into
//		pieces according to terrain_bmp_part_width
//
//	world_dimension
//		world_dimension[0] is the length of the world along the world X axis
//				This axis is mapped to the length of the terrain bmp and to the -Y axis of
//				the XYZ file
//
//		world_dimension[1] is the length of the world along the world Y axis
//				This axis is mapped to the width of the terrain bmp and to the X axis of
//				the XYZ file
//
//		world_dimension[2] is the height of the world (max_Z-min_Z) 
//		
//		If any of the given dimensions is negative then the engine will use 
//		calculate the dimension according to the elevation_source ratio
//
//
//	add_polygons_to_create_flat_edges
//		Add polygons to the terrain edges in order to create flat edges of the terrain
//		This way the terrain is nicer to view (so that the terrain doesn't abruptly ends and the 
//		viewer walks on emty space).	It also makes the model a lot easier to integrate with other 
//		models (since the edges are all at the same height)
//		If the speed database is used then it is advised to add the surrounding polygons only
//		after running the getpoints utility.
//		possible values:
//			The number given is the width of the added border.
//			If 0 is given then no polygons are added for creating the flat square edge.
//			Common values are 1, 10, 1000000. Any positive number will be fine.
//		
//		Note that the given width is added to th given dimension. For example if the terrain dimensions given are 1000,1000,300
//		and add_polygons_to_create_flat_edges==700 then the result model will have the following dimesions 1700, 1700, 300
//
//	terrain_bmp_part_width
//		Since the terrain_bmp (the bitmap that is put on top of the terrain)
//		could be very big, there is a need to cut it into smaller pieces
//		Most of the 3D cards today can not handle bitmaps bigger than 1024x1024
//		The GeForce 4 Ti 4600 can not handle bitmaps larger than 4096x4096 
//		Anyhow, for performance sake it is best to divid the terrain bmp into
//		pieces in the size of 256x256 or 1024x1024
//		Common value for terrain_bmp_part_width are 256, 512, 1024
//		The given value must be in the power of two
//
//	max_num_of_polygons
//		Limits the created terrain to this given number. If max_num_of_polygons==0 then
//		the number of polygons will not be limited. This parameter is useful when we want to creat 
//		a terrain quicly (draft terrain ...). Some elevation sources could be highly detailed
//		In these cases creating the terrain and saving it might take a lot of time. Using this parameter
//		we can create a less detailed version first. In many cases we must put a limit to the number of polygons
//		since the terrain is used for realtime and must render quickly on all systems.
//		Note that the resulted terrain might actually have more polygons then the given max_num_of_polygons
//		For example you might set max_num_of_polygons to 50,000 and get a terrain with 51,000 polygons
//		This is normal and should happen frequently. The given limits acrually guarantee that the number of polygons
//		will close to the given limit 
//		
//	color_top_bitmap_flag
//		YES or NO.
//		Sometimes we only have a digital elevation map and no setallite image
//		In this case we use the DEM bmp also as the setallite map. In this case
//		we can ask the engine to color the gray map so it looks more like a physical map
//
//  max_number_of_pixels_in_top_bmp
//		limit the size of the bitmap used. A value of 0 means no limitations.
//
//	temp_dir
//		During the function work it will write some temporary files to the disk.
//		Through this argument one can specify the temp directory to be used.
//		This parameter could be NULL (if one doesnt care where the temp files are written)
//		The function will automatically delete the temporary files once finished.
//
//	original_image_width, original_image_height
//		Through this arguments the function returns
//		the dimensions of the terrain_bmp im age
//
//	height_from_dem
//		The height of the terrain according to the diff between the max_z and the min_z
//		taken from the xyz or grd files. When a bmp is used as the source then -1 is returned
//
//	lowest_point, highest_point
//		only valid when height_from_dem!=-1
//		The X,Y,Z of the lowest and hieghest points
//
//	tfw_file
//		NULL or the tfw file
//
//
// Return Value:
//		On failure returns NULL
//		On success returns a handle to the created group
//
// See also STATE_engine_create_terrain_from_bitmap();
//
// Example: 
//		double dimensions[3]={10000,10000,500};
//		STATE_engine_create_terrain2("elevation_map.bmp", "Setellite_image.bmp",dimensions, 100000, 1024, NO, 10000, NO, NULL, "my_password");
//
STATE_IMPORT DWORD STATE_WINAPI STATE_engine_create_terrain2(char *elevation_source, char *terrain_bmp, double world_dimension[3] , double add_polygons_to_create_flat_edges, int terrain_bmp_part_width, int force_8bit_output, int max_num_of_polygons, int color_top_bitmap_flag, DWORD max_number_of_pixels_in_top_bmp, char *temp_dir, int *original_image_width, int *original_image_height, double *height_from_dem, double lowest_point[3], double highest_point[3], char *tfw_file, char *password);


//If one of the 3DSTATE API functions fails, then one can use this
//function to retrieve a string describing the error
//This is the same string that appears in the STATE.log file
//
// See also:
//		STATE_engine_write_error_messages_to_the_screen(), 
//		STATE_engine_write_to_log_file()
//		STATE_engine_set_error_reporting_mode()
//
STATE_IMPORT char *STATE_WINAPI STATE_engine_get_last_error(void);

// This function is intended to help in writing a program that:
//		1- Keep the same speed all the time even if the frames per second rate changes. This means that when the player in your program \ game
//		   move from one room to another he will continue to have the same speed even if one room
//		   has many more polygons than the other. In other words: it helps to keep the same speed
//		   even if the frames per second rate changes.
//
//		2- Make your program run equally on all computers.
//
// Usage:
// ------
//
//  Example: Lets say we want to move an object by 100 units each second
//  In this case we should write something similar to the following:
//
//		factor=STATE_engine_get_computer_speed_factor();
//		corrected_number= 100*factor;
//		move_my_object(corrected_number);
//
// Important notes:
//
// 1- Since the program cycle duration is the time from the beginning of one render to the next render,
//	  one has to do two renders before getting a non-zero returned value.
//	  This should not be a problem. Here is Why: Consider the usage example above. A returned value of 0 will
//	  cause a call to move_my_object(0) which results that nothing will move before the third render which means
//	  that at the first tenth of a second of your game\application (approximately one tenth) there wont be any movement.
//	  
//
// 2- The return value of this function is constantly changing to reflect
//	  changes in the CPU load. This means you have to call the function each
//	  time before you use it (dont call it once and then use the same value over and over ...). 
//
// 3- This function is extremely fast. Like most of STATE functions it only retrieves an internal value
//	  This means that there is no penalty for calling it frequently.
//  
//	Return Value:
//		The factor returned is actually the average duration of the last 50 rendering cycles (from the beginning of one render to the beginning of the next render)
//		The time is given in tenth of a second (for example if the returned value is 3.2 it means that the average of the last render cycles took 0.32 seconds)
//		The return value will be 0 till there are at least two renders done.
//		
//		
//
//
//  Remarks:
//
//		Here is how to calculate the FPS (Frames Per Second) achived by your program using the STATE_engine_get_computer_speed_factor()
//
//				double average_program_cycle_duration_in_seconds=STATE_engine_get_computer_speed_factor()/10; //The average duration of the last 50 cycles
//				if(average_program_cycle_duration_is_seconds==0) return; //protect against division by zero ...		
//				double FPS=1/average_program_cycle_duration_in_seconds;
//				output_text("The current frames per second rate is %lf", FPS);
//
//
STATE_IMPORT double STATE_WINAPI STATE_engine_get_computer_speed_factor(void);



//When the engine detects an error it makes a beep using the PC loudspeaker
//and writes some words about it to the 3DSTATE.log file (and a less detailed description to error.log)
//Usually you should take care of all the errors and the warnings if
//you didnt, you can use this function to shut down the noise from the speaker.
//Note that the error messages will still be outputted to the 3DSTATE.log and the error.log files
//The default value is ON
//Example: STATE_engine_set_speaker_mode(OFF);
//
// See also: STATE_engine_set_error_reporting_mode()
//
STATE_IMPORT void STATE_WINAPI STATE_engine_set_speaker_mode(int on_off_flag);

// Use this function to stop error reporting for specific code sections
//
// Example:
//		STATE_engine_set_error_reporting_mode(OFF); //disable error reporting
//		your code ...
//		STATE_engine_set_error_reporting_mode(ON); //enable error reporting again
//
// See also: 
//		STATE_engine_set_speaker_mode(), 
//		STATE_engine_write_error_messages_to_the_screen(), 
//		STATE_engine_get_last_error(), 
//		STATE_engine_write_to_log_file()
//
STATE_IMPORT void STATE_WINAPI STATE_engine_set_error_reporting_mode(int on_off_flag);

// *** very usefull.(Note also STATE_object_auto_add_shadows() which uses stencil shadow volume technique)
// This function is everything you'll need to create lighs and shadows
// The function creates and adds new polygons which are the shadow of "entity_creating_shadow"
// on "entity_receiving_shadow" according to the given light source location. 
// Those new added polygons are called patches. 
// entity_receiving_shadow and entity_creating_shadow could be handles to any one of the following
// types POLYGON, OBJECT or GROUP. If a NULL is given the function would treat it as a handle to group
// a(note that a NULL group means the whole world)
// Note that the new added polygons get the same properties
// of the "entity_creating_shadow" (i.e. the same bitmap, the same translucent etc ...)
// Usually you would like to change those properties. For example:
// 
//		double light[3]={0,0,200}
//		STATE_engine_create_shadow(room_group, chair_group, light, 255);
//		//setting the color of the shadows. Note that by changing the color
//		//you can simulate changes in light intensity
//		STATE_group_set_patches_color(room_group, 70,70,70, ALL_PATCHES); 
//
// If we dont call STATE_group_set_patches_color() we wont get a dark shadow on the room floor
// instead we will get the shape of the shadow but with the colors and bitmaps of the chair.
// Note that the last argument in the call to STATE_engine_create_shadow() is called serial_number
// serial_number is a number in the range [10 - 255]
// The number is relevant only if we will call 
// STATE_group_set_patches_color(..., serial_number)
// and the function STATE_group_set_patches_bitmap(..., serial_number)
// This will allow us to have a room with several shadows and to change only one of them.
// 
// Use STATE_polygon_delete_patches() to remove the shadow
// For example: STATE_polygon_delete_patches(room_handle, ALL_PATCHES) will
// delete all the shadows in the room while STATE_polygon_delete_patches(room_handle, 255) will
// delete just the shadows that were created with serial number 255.
//
// Performance note:
//		If you create the shadow in real time and you want it to be fast
//		then call this function with polygon handles. for example
//		STATE_engine_create_shadow(polygon_to_get_shadow, polygon_obscaring_light, light, 255);
//
// Note that function like STATE_polygon_add_patch_easy() are also creating patches though for
// different purposes.
//
// Examples:
// A)
//		double light[3]={100,200,300}
//		STATE_engine_create_shadow(wall_polygon, blocking_polygon, light, 255);
// 
//
// Example B:
// while(user did not press ESC) {
//		STATE_polygon_delete_patches(wall_polygon, ALL_PATCHES);
//		light[0]+=10; //move the light source
//		STATE_engine_create_shadow(wall_polygon, blocking_polygon, light, 255);		
//		STATE_polygon_set_bitmap_fill(shadow_polygon_handle, flare_bitmap);
//		STATE_polygon_set_translucent(shadow_polygon_handle, NORMAL_GLASS);
// }
//
// See also: 
//		STATE_object_auto_add_shadows() this function uses a totaly different method for creating shadows,
//		STATE_engine_set_patches_offset(),
//		STATE_engine_create_dynamic_shadow(),
//		STATE_polygon_delete_patches(), 
//		STATE_polygon_delete_patches(),
//		STATE_group_delete_patches(), STATE_group_set_patches_translucent(), 
//		STATE_group_set_color(), STATE_group_set_patches_bitmap()
//		STATE_polygon_add_patch_easy().
//
STATE_IMPORT void STATE_WINAPI STATE_engine_create_shadow(DWORD entity_receiving_shadow, DWORD entity_creating_shadow, double light_src[3], BYTE serial_number);

// Exactly like STATE_engine_create_shadow() only that it is faster
// The disadvantage is that dynamic shadows can only be deleted with
// STATE_engine_delete_dynamic_shadows() which delete all the existing dynamic shadow.
// (STATE_group_delete_patches() wont work here)
// We suggest that you use this function to create shadows that can be changed from
// one render to another. For example in our world we might have several source
// of lights some of them will be moving and some are fixed. We could also have some moving objects
// in our world (so their shadow is moving too). The shadow that is created
// from the fixed light sources and by the fixed object can be created only once at the beginning of
// the program. To create the fixed shadows use STATE_engine_create_shadow()
// The shadows that are created by the moving light sources or from the moving objects will have to be
// deleted and recreated for each render cycle. Use STATE_engine_create_dynamic_shadow()
// to create them. To delete them use STATE_engine_delete_dynamic_shadows().
// Here is an example:
// 
//		//creating the static shadow. Create the shadow that the trees project on the ground 
//		double sun[3]={10000, 10000, 30000};
//		STATE_engine_create_shadow(ground, trees, sun, 255);
//		while(Escape key is not pressed) {
//			STATE_engine_delete_dynamic_shadows();
//			move_our_ball(ball);
//			STATE_engine_create_dynamic_shadow(ground, ball, light_source);
//			STATE_engine_render(NULL, camera);
//		}
//		
//	See also STATE_engine_set_patches_offset()
STATE_IMPORT void STATE_WINAPI STATE_engine_create_dynamic_shadow(DWORD entity_receiving_shadow, DWORD entity_creating_shadow, double light_src[3], BYTE serial_number);

// Deletes all the existing dynamic shadow. Dynamic shadow is created using
// the function STATE_engine_create_dynamic_shadow()
STATE_IMPORT void STATE_WINAPI STATE_engine_delete_dynamic_shadows(void);


//When rendering takes more than the specified time then the rendering function will exit automatically
//This is very useful when rendering indoor worlds. The engine renders front to back
// so it is usually enough to stop rendering after finishing with the current room.
// Try playing with different values and see the result.
//The default is 1000000 (which is about 17 minutes which is similar to "take all the time in the world to finish every rendering")
//Typical values would be 200,100,50
// Note that this function is only relevant in Viewer Mode (Viewer Mode is the the default mode)
//Example: STATE_engine_set_maximum_rendering_time(100); //Set minimum frames per seconds to 10
STATE_IMPORT void STATE_WINAPI STATE_engine_set_maximum_rendering_time(int max_duration_in_mili_seconds);
STATE_IMPORT int STATE_WINAPI STATE_engine_get_maximum_rendering_time(void);

//Sets a minimum time for each render. If STATE_engine_render() take less time
//than the specified time then the engine will wait (Doing Sleep() ) till it reaches the given
// duration.
// Why should you use this function ? Sometimes when you get to the end of your world
// The engine has only few polygons to render in this case it can get to 1000 frames per seconds and more !!!
// If you are moving a car by X units every render then your car is going to become
// a rocket when you reach the end of the world ...
// The default is 2. This permits a maximum of 500 frames per seconds
// Note also the function STATE_engine_get_computer_speed_factor()
//Example: STATE_engine_set_minimum_rendering_time(0);
STATE_IMPORT void STATE_WINAPI STATE_engine_set_minimum_rendering_time(int min_duration_in_mili_seconds);
STATE_IMPORT int STATE_WINAPI STATE_engine_get_minimum_rendering_time(void);


//Controls the amount of CPU time that the rendering process gets.
//Usually you would like to have the fastest graphics possible
//though sometime when a few application are working in conjunction
//there is need to release some CPU time for the benefit of the others.
// parameters:
//		thread_priority: (For more details please see the win32 function SetThreadPriority() )
//
//			THREAD_PRIORITY_ABOVE_NORMAL 
//				Indicates 1 point above normal priority for the priority class. 
//			THREAD_PRIORITY_BELOW_NORMAL Indicates 
//				1 point below normal priority for the priority class. 
//			THREAD_PRIORITY_HIGHEST 
//				Indicates 2 points above normal priority for the priority class. 
//			THREAD_PRIORITY_IDLE 
//				Indicates a base priority level of 1 for IDLE_PRIORITY_CLASS, NORMAL_PRIORITY_CLASS, or HIGH_PRIORITY_CLASS processes, and a base priority level of 16 for REALTIME_PRIORITY_CLASS processes. 
//			THREAD_PRIORITY_LOWEST 
//				Indicates 2 points below normal priority for the priority class. 
//			THREAD_PRIORITY_NORMAL 
//				Indicates normal priority for the priority class. 
//			THREAD_PRIORITY_TIME_CRITICAL 
//				Indicates a base priority level of 15 for IDLE_PRIORITY_CLASS, NORMAL_PRIORITY_CLASS, or HIGH_PRIORITY_CLASS processes, and a base priority level of 31 for REALTIME_PRIORITY_CLASS processes. 
//
//		Class priorities: ( For more details please see the win32 function SetPriorityClass() )
//
//			HIGH_PRIORITY_CLASS 
//				Specify this class for a process that performs time-critical tasks that must be executed immediately. The threads of the process preempt the threads of normal or idle priority class processes. An example is the Task List, which must respond quickly when called by the user, regardless of the load on the operating system. Use extreme care when using the high-priority class, because a high-priority class application can use nearly all available CPU time. 
//			IDLE_PRIORITY_CLASS 
//				Specify this class for a process whose threads run only when the system is idle. The threads of the process are preempted by the threads of any process running in a higher priority class. An example is a screen saver. The idle-priority class is inherited by child processes. 
//			NORMAL_PRIORITY_CLASS 
//				Specify this class for a process with no special scheduling needs. 
//			REALTIME_PRIORITY_CLASS 
//				Specify this class for a process that has the highest possible priority. The threads of the process preempt the threads of all other processes, including operating system processes performing important tasks. For example, a real-time process that executes for more than a very brief interval can cause disk caches not to flush or cause the mouse to be unresponsive. 
//Examples:
// A)
//		Setting High priority (The default)
//		STATE_engine_set_thread_priority(THREAD_PRIORITY_HIGHEST, HIGH_PRIORITY_CLASS);
//
// B)
//		Setting the fastest rendering possible
//		STATE_engine_set_thread_priority(THREAD_PRIORITY_TIME_CRITICAL, REALTIME_PRIORITY_CLASS);
//
// For more details please see the win32 functions SetPriorityClass() and SetThreadPriority()
STATE_IMPORT void STATE_WINAPI STATE_engine_set_thread_priority(int thread_priority, DWORD class_priority);

//This function must be called before your program exits.
//Not calling this function at the end of your program might 
//cause your program not to exit cleanly (Crashing at the end)
//This function should not be called from inside a destructor
//No function can be called after calling STATE_engine_close()
//
// NOTE ! This function should only be called before exiting your program !!!
//		No function can be called after calling STATE_engine_close() !!!
//
// To close one world and load a different world simply call STATE_engine_load_world()
// This will automatically unload the previous world.
//
STATE_IMPORT void STATE_WINAPI STATE_engine_close(void);


//Returns STATE's logo
STATE_IMPORT HBITMAP STATE_WINAPI STATE_engine_get_logo(void);

#define INFORMATION_NONE 0
#define INFORMATION_FPS 1
#define INFORMATION_ANTIALIASING_MODE 2
#define INFOMATION_ZBUFFER_MODE 4 //The zbuffer and the stencil buffer mode
#define INFORMATION_3DCARD 8
#define INFORMATION_NUMBER_OF_POLYGONS 16
#define INFORMATION_ALL 0xFFFFFFFF

// Shows the frames per second rate and other information at the top left of the rendered window.
// The default it is (INFORMATION_FPS|INFORMATION_ANTIALIASING_MODE | INFORMATION_3DCARD | INFORMATION_NUMBER_OF_POLYGONS)
//
// Example:
//		STATE_engine_show_frames_per_second_rate(INFORMATION_FPS | INFORMATION_ANTIALIASING_MODE |INFOMATION_ZBUFFER_MODE); 
//
STATE_IMPORT void STATE_WINAPI STATE_engine_show_frames_per_second_rate(int information_level);

// *** Very Useful


// What does this function do:
// ---------------------------
// The default is NO, which means that each render will
// clear the result of the previous rendering before painting the new picture.
//
// Why do we need this function:
// ------------------------------
//	There are two cases in which one should consider calling this function
//	with the argument YES
//	1) When our world is an indoor world (see defenition below). In this case clearing the screen is simply not needed.
//	   (by not clearing the previous screen we could improve a bit the speed of each render)
//	2) When we want to create special effects.
//
//
// Terms:
// -----
// An inddor world for this matter is a world that no matter where you are
// inside the world, each pixel on the rendered image belongs to a polygon in your world
// or in other world, you never get to see the background.
//
// Remarkds
// --------
// If there is one object inside the world which encompass the whole world then
// use STATE_object_use_as_skybox(). This would give better results then using this function.
//
// Example:
// --------
// To understand the effect of this function simply call it and see what you get.
// Example: STATE_engine_render_on_top_of_previous_rendering(YES);
//
STATE_IMPORT void STATE_WINAPI STATE_engine_render_on_top_of_previous_rendering(int YES_or_NO);




// Use this function only with the commercial version of STATE 3D Engine.
// This function should be the first STATE function that is called .
// If this function is not called then a popup message will appear.
// Note: It is better to call this function after you have created the main window
// of your application, otherwise when launching your application your main window
// might be behind the engine windows (there is nothing wrong with that, it is simply not as nice as)
// Arguments:
// Enter the argument exactly as they appear on the license that you got from STATE.
// Failing to do so will prompt a popup window with a warning.
// 
// Return value:
// Returns OK if the registration was successful.
// Returns VR_ERROR if one of the arguments was wrong.
// If you receive VR_ERROR then please check that you type everything
// exactly as they appear on the license that you got from STATE.
// 
//
// Example:
// STATE_register_license("XXXXXX", "John Smith", "johnsmith124@ibm.com", "IBM", "OS2 Wrap",1234567890.987654321);
int STATE_register_license(char *decleration_text, char *your_full_name, char *password ,char *company, char *product_name ,double registration_number);

//This function could be very important for performance if you find in STATE.log that
// a significant number of polygons were added to your world while loading it.
// Background:
//		When the world is loaded in Viewer Mode (the default) the engine needs to build a BSP data structure (Never mind if you
//		dont have any idea what BSP is. Keep reading) it then looks to see if such file exist in the same folder of the world.
//		(in binary format this file is sometime inside the world file so there is no sparate file) If such a file does not exists
//		then the engine start building its own file (a very lengthy process. When the world is big it could even take 30 minutes !)
//		During this process many polygons are cut into smaller pieces. For example if a tree is stuck into the ground in a way
//		that the ground intersect one of the polygons of the tree then this polygon or the polygon of the ground will be cut into two pieces,
//		adding to the number of total polygons in the world. To avoid that it is recommended that polygons in your world should not
//		intersect with each other. Nevertheless even if no polygon intersects others the engine might need to split some of the polygons.
//		(If you really want to know why, you can search the Internet for articles about BSP trees == Binary Space Partition Trees)
//
// This function controls the numbers of polygons that the engine will add in the process of forming
// the bsp tree. The bigger this number is the less polygons will be added. The default is 1.
// Possible numbers are anything bigger then 0.
// Examples: 0.01, 0.1, 1, 10, 50, 100, 1e20
//
// Calling this function with a bigger number helps in reducing the number of added polygons but reduces the accuracy of the BSP
// structure. This could cause the world to appear incorrectly. Surfaces that should be in front
// of other surfaces will be behind. We suggest that you keep this number as low as possible as long as it doesnt increase significantly.
// the number of polygons in your world. Note that in the log file (STATE.log) you can find how many polygons were added 
// during the process of creating the BSP data structure.
//
// STATE_engine_set_bsp_precision() should be called before loading the world.
// Each time that the precision variable is changed a new bsp tree will be created.
STATE_IMPORT void STATE_WINAPI STATE_engine_set_bsp_precision(double precision);
STATE_IMPORT double STATE_WINAPI STATE_engine_get_bsp_precision(void);

//This is a very important function.
//The Speed database is used to speed up rendering. 
//One of the most time consuming test for the engine is to determine in real-time which
//of the polygons in the world are seen and which are totally hidden by other polygons.
//The information in the Speed database will reduce the amount of calculations that are needed in real-time
//in order to determine from each location in the world the polygons that are seen and those that are hidden. 						
//The Speed database contains information for every possible area in the world. The information tells the engine which 
//polygons in the world are seen from this area. In order to keep the size of the Speed  database reasonable it is needed to supply
//to the engine a list of points that we would like to speed the rendering in the vicinity of the given points.
//In order to create the Speed database it is first needed STATE_engine_load_world() with the flag OUTPUT_RENDER_LOCATIONS
//Here is a summary of how to speed up the rendering
//		1) Call STATE_engine_load_world() with the OUTPUT_RENDER_LOCATIONS flag and walk trough your world. This will create the visited_points.txt
//		   Try to visit every possible area inside your world. The direction in which a certain location is approached does not matter
//		   All that mater is the visited location themselves. Areas that wont be visited wont benefit from the Speed database
//		2) Call STATE_engine_create_speed_database() Sit back and relax while the engine will create the Speed database.
//		   Use the visited_points.txt you have created in step "1". The speed.pvs file that is created is the Speed database that will
//		   speed rendering in real-time.
//		3) Run your application. Make sure that the speed.pvs file is located in your application working directory (the folder where you have your exe file).
//		   If you encounter areas in the world that do not enjoy any speed up, it is usually because those areas were not
//		   visited in step "1". In order to fix this there is no need to redo the whole process. Here are two ways you can edit the speed.pvs database.
//		   A) Do step "1" again but visit only the locations where the speed optimization didn't take place.
//			  Do step "2" but now call STATE_engine_create_speed_database() with reset_database set to NO.
//			  The new information will be added to the original speed.pvs file.
//		   
//		   OR
//
//		   B) Add locations manually to the visited_points.txt file and then repeat step "2"
//
//
//Please note the following facts:
//1) The bigger your world is the more impact you will gain by using the Speed database. 
//	 On very small worlds you might not even notice the effect.
//2) To increase the speed up by this process build your world in a way that a minimum number of polygon is visible
//	 from every location in the world. The most perfect world for this kind of optimization is 
//	 an indoor world where one cant see from one room to another because the door leads to a corridor.
//   Here is an example of such structure:
//
//	--------
//	| Room |
//	|   A  |	
//	|	   |
//	| ------
//	------ |
//	|Room  |
//	|B     |
//	|------|
//
// Note that from Room A it is not possible to see room B and the other way around
// Many games (like Quake ...) use this technique extensively.

// Parameters:
//
//	visited_points_file_name: A file that holds a list of x,y,z points. This file is created automatically
//							  when calling STATE_engine_load_world() with the OUTPUT_RENDER_LOCATIONS.
//
//	reset_database: should be YES or NO. If NO then the function will use the previously created speed.pvs file and will
//					add to it new information derived from the given visited_points_file_name
//
//
//  make_compact_database: YES or NO. If YES the database will be significantly smaller. In some worlds the compact database
//							will give a much smaller rendering speed up. In other worlds it will give better results then the 
//							detailed database (none compact). One has to check both methods on the final world.
//							The best performance will be achieved when make_compact_database==NO and the points
//							in the visited_points_file_name are selected cafully so that there are not too many but 
//							there are enough to represent all the location that deffer by the polygon visible from these locations.
//							for example, if there are two ajacent points that when the camera is located in each one of them 
//							there are xxx visible polygons. So if from both location there are the same polygons visible
//							so one of these points might be ommited.
STATE_IMPORT int STATE_WINAPI STATE_engine_create_speed_database(char *visited_points_file_name, int reset_database, int make_compact_database);


//Determines the distance that the engine will set for all created patches from the polygon
//the patches are put on. (To learn about patches and what they are used for please see
//STATE_polygon_add_patch_easy() or STATE_engine_create_shadow() )
//
// Q: Why do we need this function ?
//
// A: When putting a patch over a polygon we get two polygons one on top of the other
//	  Some 3D cards can not handle this situation cleanly because of small floating point
//    glitches when zbuffer is done. The artifact is stripes on the patch (coming from the polygon behind)
//    This function can be used to solve this problem so that patches look better.
//
//The default distance is 2. This function was introduced with version 5.4 of the engine.
//previous versions of the engine made patches with 0 distance.
//
// Parameters:
//		offset_size: The distance is explained above.
//					 Use the smallest number that makes the patches look good
//					 example of possible values: 0, 0.4, 1, 10, 40
//					 A too big number would make shadows created with STATE_engine_create_shadow()
//					 to be inaccurate
//
// See also:
// STATE_engine_get_patches_offset(), STATE_polygon_add_patch_easy() 
// STATE_polygon_add_patch(), STATE_polygon_add_shadow_patch(), STATE_engine_create_shadow()
STATE_IMPORT void STATE_WINAPI STATE_engine_set_patches_offset(double offset_size);

//For more details please see STATE_engine_set_patches_offset()
//Returns the distance that is used by the engine when creating patches
// (The distance from the created patch and the polygon that the patch is put on)
STATE_IMPORT double STATE_WINAPI STATE_engine_get_patches_offset(void);

// Why do we need this function
//------------------------------
//This function is used to group together many small bitmaps into several bigger bitmaps.
//This is very useful in improving rendering frames per second and shortening loading
//time of the world. One of the techniques used in today's leading games is to create
//small lightmaps (second bitmaps) for every polygon in the world (This can be done using STATE 3D Webmaker\WorldBuilder)
//This technique can drastically improve the graphics quality of the world though they create a huge number of small bitmaps
//For example in one of Half-Life worlds there are about 4000 lightmaps each one at the average size of 5*9 pixels.
//Loading 4000 bitmaps can take quite sometime. Using STATE_engine_reduce_number_of_bitmaps()
//one can group those small bitmaps into bigger ones. For example 4000 5x9 bitmaps can be grouped to only 3 bitmaps of 256x256 !!!
//
// Please note that bitmap that are transparent or that are used by a STATE 2D animation or that are tiled will not be grouped together.
//
// How to use this function
// -------------------------
//	Load your world in editor mode (Give the EDITOR_MODE flag when calling STATE_engine_load_world() )
//	Call STATE_engine_reduce_number_of_bitmaps()
//	Call STATE_engine_save()
//
//	This will save a new copy of your world with the small bitmaps grouped in big bitmaps.
//	Note that not always we would like to  group all the small bitmaps in big bitmaps
//	For example if we have a bitmap that we use as a shooting mark (to put on walls after shooting at them)
//	then we would like this bitmap to stay separate in its own file so that we could easily
//	call STATE_polygon_set_bitmap_fill()
//	the include_filter and exclude_filter parameters are used in order to give 
//  the programmer the freedom to group together only the bitmaps that he wants to
//
//  Example:
// ---------
//	//The following example will group together all the bitmaps that are equal or smaller than 32x32 bitmap into bitmaps in the size of 256x256
//  STATE_engine_reduce_number_of_bitmaps(32, 256, "..\\bitmaps", NULL, NULL, YES);
//
// Return Value
// -------------
//	Returns the number of bitmaps that were replaced and group together in bigger bitmaps
//
//
// Parameters:
//	replace_below_equal_this_width:
//		Only bitmaps equal or smaller than this size will be replaced
//		Here are some common values to use 32, 20 and so on.
//		a value of 32 means that only bitmaps that are smaller then a 32x32 bitmap will be included
//		a bitmap of 33x2 is not consider smaller since it does not fit in inside the 32x32 square.
//
//	big_bitmaps_width
//		The size of the bitmaps that will be created and will be filled with the small bitmaps
//		Values must be in the power of to (i.e 256, 512, 1024, 2048, 4096 etc.)
//		For example giving 256 means that the engine will use 256x256 bitmaps.
//		This bitmap is always square.
//		By giving a big value (such as 1024 or 2048) you can group all the bitmaps in your world into
//		one big bitmap !
//		So what is the right value to use ? We recommend using 256 since
//		there are some 3D card that can not display bitmaps larger than 256x256
//		One can test different values and check how it effect the rendering speed.
//
//	bitmaps_folder
//		The directory where the bitmaps can be found. Bitmaps that are not in this folder are not going to be grouped 
//		inside big bitmaps.
//
//	include_filter
//		Only bitmaps with name that contains the "include filter" string will be grouped\replaced
//		If this parameter is NULL then all the bitmaps in the given folder will be included
//		Example: If the include_filter is "xx"  then the following bitmaps will be included xx.bmp xx12345.bmp 123xx.bmp but not x123x.bmp
//
//	exclude_filter
//		Bitmaps with name that contains the "Exclude filter" string will NOT be grouped\replaced
//
//
//	create_8bit_bitmaps
//		Whether the group bitmaps are in 8 bit or 24 bit format.
//		Could be YES or NO. YES is recommended for faster loading time.
//		In some cases NO could give better graphics quality.
//
STATE_IMPORT int STATE_WINAPI STATE_engine_reduce_number_of_bitmaps(int replace_below_equal_this_width, int big_bitmaps_width, char *bitmaps_folder, char *include_filter, char *exclude_filter, int create_8bit_bitmaps);


#define STEREOSCOPIC_OFF 0
#define STEREOSCOPIC_CYAN_RED 1		//The left lens\eye is cyan. The right lens\eye is red.
#define STEREOSCOPIC_RED_CYAN 2		//The right lens\eye is cyan. The left lens\eye is red.
#define STEREOSCOPIC_BLUE_RED 3		//The right lens\eye is red. The left lens\eye is blue
#define STEREOSCOPIC_RED_BLUE 4		//The left lens\eye is red. The right lens\eye is blue
#define STEREOSCOPIC_GREEN_RED 5	//The right lens\eye is red. The left lens\eye is green
#define STEREOSCOPIC_RED_GREEN 6	//The left lens\eye is red. The right lens\eye is green


#define STEREOSCOPIC_NUMBER_OF_MODES 7

// If you have a pair of 3D paper glasses (Anaglyphic glasses) 
// then this function can do amazing things for you.
// Once this function is called, the engine will render
// your scene in a way that will match your 3D glasses.
// The result is a real 3D rendering with a real sense of depth.
// If you don't have  3D paper glasses then you should quickly get a pair.  
// 3D paper glasses should not cost more than 1 dollar.
// There are some Web Sites which even give them for free.
// Note that Cyan is the color that one gets 
// when adding green to blue (CYAN RGB= 0,255,255).
// www.rainbowsymphonystore.com is a good place to buy 3D Glasses.
// 3D glasses are very cheap ( $0.4 for a pair)
// Check out http://www.rainbowsymphonystore.com/3dglasses.html
// www.d3.com is another option.
// Any other place is probably just as good.

// We recommend the glasses sold by www.d3.com though
// any other suitable product from other companies can do the work.
//
// If your company is using a type of 3D glasses or any type of 3D virtual reality head set that
// is not supported by the engine then you can write to us and we will
// try to add support for this equipment.
// 
// Parameters:
//		stereoscopic_mode:
//			set the mode of work
//
//		distance_between_eyes:
//			Any number starting from 0.
//			A bigger number will increase the 3D effect
//			0 will result in no 3D effect.
//			The 3D effect is created using a combination of two images
//			taken from adjacent locations. This mimics the way it is done
//			in nature. In nature our brain receives two images, one from each eye.
//			
// Note that getting an anaglyphic image is not without a price.
// In this mode the rendering operation will be about 40 percent slower.
//
//	Examples:
//	1)
//	//Initializing anaglyphic mode. You can call this function anytime
//	//after the world is loaded.
//	//Try different numbers for distance_between_eyes till you get the best effect
//	STATE_engine_set_stereoscopic_mode(STEREOSCOPIC_RED_CYAN, 100);			
//
//  2)
//	//Canceling the anaglyphic mode. 
//  STATE_engine_set_stereoscopic_mode(STEREOSCOPIC_OFF, 100);	
STATE_IMPORT void STATE_WINAPI STATE_engine_set_stereoscopic_mode(int stereoscopic_mode, double distance_between_eyes);

STATE_IMPORT char *STATE_WINAPI STATE_engine_get_stereoscopic_mode_name(void);

//Switch between the different stereoscopic modes.
//See also STATE_engine_get_stereoscopic_mode_name()
//returns the new mode number
STATE_IMPORT int STATE_WINAPI STATE_engine_switch_stereoscopic_mode(void);

//Returns the current stereoscopic mode
//For more information see STATE_engine_set_stereoscopic_mode()
STATE_IMPORT int STATE_WINAPI STATE_engine_get_stereoscopic_mode(void);

// Obsolete. Please use STATE_layer_create() which does exactly the same.
STATE_IMPORT DWORD STATE_WINAPI STATE_engine_add_layer(double left_x, double top_y, double right_x, double bottom_y, DWORD bitmap_handle, BYTE glass_type);

// Who needs this function ?
// -----------------------------
// This function is not very important. 99% of all users will never have to use it.
// Users who import code from other engines might need this function.
// This function is also useful if your code is built to support only triangles.
// Using this function one can limit the engine to use only triangles.

// What does it do ?
// -------------------
// Limits the number of points that are permitted per polygon.
// This function can be called at any time, before or after loading the world.
// This function will split polygons that have more points than what is permitted into smaller polygons.
// The engine is smart enough to isolate the internally created polygon fractions from
// the software API. For example if we limit the number of permitted points per polygon to 3
// in this case polygons with 10 points will be split to 8 triangles.
// When we use functions such as STATE_polygon_set_bitmap_fill() on one of the
// newly created fractions (triangles) the engine will be smart enough to set the given
// bitmap to all the fractions as if it is still one polygon.
//
// Remarks:
// -----------
// 
// IMPORTANT:	Please note that polygons that have more than the permitted number of points will
//				deleted ! This means that one should consider all polygon handles as not legal
//				after calling this function.
//				Example:
//				DWORD floor_handle=STATE_polygon_get_handle_using_name("floor");
//				STATE_engine_set_max_num_of_points_per_polygon(4);
//				// If the polygon "floor" had more than 4 point, it was deleted. In this case we cannot use its handle any more
//				//Repeat the call to be sure that we get a valid handle
//				DWORD floor_handle=STATE_polygon_get_handle_using_name("floor");
//				//We now have a valid fraction of the floor (or the whole floor if it was not deleted)
//				API functions that are executed on the floor fraction will be carried out on all the fractions
//				that make the floor.
//
//
// Example:
//	STATE_engine_set_max_num_of_points_per_polygon(3);
//
// See also STATE_engine_get_max_num_of_points_per_polygon()
STATE_IMPORT void STATE_WINAPI STATE_engine_set_max_num_of_points_per_polygon(int max_number);

// Returns the maximum permited number of points per polygon
// for more details please see STATE_engine_set_max_num_of_points_per_polygon().
STATE_IMPORT int STATE_WINAPI STATE_engine_get_max_num_of_points_per_polygon(void);

//Returns the width of the rendered target. If in full screen mode, it will return the width
//of the screen. At least STATE_engine_render() should be called before
//this function is used.
STATE_IMPORT int STATE_WINAPI STATE_engine_get_rendering_window_width(void);
STATE_IMPORT int STATE_WINAPI STATE_engine_get_rendering_window_height(void);

// *** a very useful function
//
// Terms
// -----
// A skybox object is any regular object that encompass the whole world. In other words: 
// 1) Has a close shape such as a box or a sphere (or any other closed shape)
// 2) Is big enough so that all the other parts of the world reside inside the skybox object.
// For example, a skybox could be a room a sphere with clouds bitmaps etc.
//
// What does this function do and why do we need it ?
// --------------------------------------------------
// Actually if one doesn't use this function, nothing bad happens.
// you can still use your skybox objects and they will work fine.
// The only pupose of this function is to give the engine the information
// that this object encompass the whole world .
// The engine can use this information and get about %10 gain in Frams Per Second Speed
// That's why if you have some object that encompass the whole world it is a good deed
// to tell the engine about it.
// Note that if you use this function, the function STATE_engine_render_on_top_of_previous_rendering()
// would have no effect. If one of your objects is a skybox (encompass the whole world)
// then using this function would get you more increase in Frames Per Second then using STATE_engine_render_on_top_of_previous_rendering()
//
// Parameters
// ----------
// object_handle: The object which is used as a skybox. Give NULL to tell the engine that 
// there is bo skybox in use anymore.
// 
// Example:
//  DWORD my_room=STATE_object_create_from_file("my_room.x");
//	STATE_engine_using_skybox(my_room)
//
// Remakrs:
// --------
//	1) skybox object should not have transparent or translucent polygons.
//	If they have it simply wont look good.
//
//  2) If you ever get to render outside the skybox area , you will see
//	   a mishmash of colors. To avoid this simply don't call STATE_engine_using_skybox()
//	   One can only use this function if you never render outside the bounding box.
//
//
// Behind the scene (for the curious reader)
// -----------------------------------------
// When there is an object that is declered as skybox object then the engine does not
// clear the screen between two consecutive rendering (since there is no need)
// In addition there is also no need for clearing the Zbuffer.
// When there is no skybox object but the world is an inddor world,
// you can still speed up rendering by calling STATE_engine_using_skybox()
// This will prevent the engine from clearing the screen between renders
// but the engine would still need to clear the ZBuffer.
//
// See also STATE_engine_render_on_top_of_previous_rendering()
//
STATE_IMPORT void STATE_WINAPI STATE_engine_using_skybox(DWORD object_handle);


// *** quite important
// Use this function before calling STATE_object_create_from_file()
// in order to set a different bitmap folder than the default.
// If this function is not called the engine will look for the bitmaps
// in the folder specify by STATE_engine_load_world(), in the current directory
// and in the Bitmaps directory if exists. In short, you will need to use this functin
// only if the engine doesn't succeed in finding the object bitmaps.
//
// Note that instead of using this function one can use STATE_object_create_from_file2()
//
STATE_IMPORT void STATE_WINAPI STATE_engine_set_bitmaps_folder(char *bitmap_folder);
STATE_IMPORT char * STATE_WINAPI STATE_engine_get_bitmaps_folder();

// *** Not an important function. 99.99 percent of all users will never use it
// When setting the texture coords the engine will automatically check that that the value of the coordinates
// can be linearly interpolated. This is relevant only for polygons which have more
// than 3 points.
// The default is ON
STATE_IMPORT void STATE_WINAPI STATE_engine_set_coordinates_auto_correct_mode(int ON_or_OFF);

// *** Useful though one can do without ...
// By default for every error message the engine beeps and write the message to the 3dstate.log and error.log files
// By calling this function errors will also appear on the screen (while rendering)
// This function is for debugging purposes. Should be removed in release version
// Note that messages on the screen appear only when rendering. So errors that
// have to do with loading the world will only appear in the 3dstate.log file (and error.log)
//
//
// Example: 
//	  STATE_engine_write_error_messages_to_the_screen(ON);
//
// See also:	
//				STATE_engine_set_error_reporting_mode(), 
//				STATE_engine_get_last_error(),
//				STATE_engine_write_to_log_file()
//
STATE_IMPORT void STATE_WINAPI STATE_engine_write_error_messages_to_the_screen(int ON_or_OFF);

#define FOG_MODE_AUTO 1
#define FOG_MODE_VERTEX 2

#define FOG_START_DISTANCE_DEFAULT -1

// *** very useful
// Controls the fog parameters. Fog can be used
// to create night time effect, fog effect and under water effect.
// These different effects are set by selecting a different color for the fog
//
// Parameters:
//		ON_or_OFF: could be ON or OFF. The default is OFF
//				   Using fog would usually increase the world realism
//				   without costing too much in terms of frames per second
//				   If OFF is given then the rest of the parameters are ignored
//
//		red, green, blue:
//					The fog color. Gray shades such as (128,128,128) or  (200,200,200) etc always look good.
//					Try also colors that are similar to the average world color.
//					For making a night time effect set fog color to black -(0,0,0) 
//					For blue sky colors such as (200,200,230) would give a much natural result
//					then 0,0,255 (which would give an under water effect
//
//		fog_mode:
//				Could be one of the following FOG_MODE_AUTO, FOG_MODE_VERTEX
//				If FOG_MODE_VERTEX is selected then the fog is controled manually using the function
//				STATE_point_set_fog()
//
//		max_visible_distance:
//				Beyond this distance the fog is 100% thick and nothing is seen
//				This parameter is only relevant when FOG_MODE_AUTO is selected
//
//		fog_start_distance:
//			Closer then this distnace, fog has no effect.
//			If FOG_START_DISTANCE_DEFAULT is given then fog_start_distance
//			Would be automatically set to max_visible_distance/20
//
//		fog_type:
//			reserved for future use
//
// See also:
//		STATE_point_set_fog()
//
// Example A:
//		//switch fog on
//		STATE_engine_set_fog(ON, 200, 200, 200, FOG_MODE_AUTO, 1000000, FOG_START_DISTANCE_DEFAULT, 0);
//
// Example B:
//		//switch fog off. This saves rendering time. The rest of the parameters are ignored
//		STATE_engine_set_fog(OFF, 200, 200, 200, FOG_MODE_AUTO, 1000000, FOG_START_DISTANCE_DEFAULT, 0);
//
STATE_IMPORT void STATE_WINAPI STATE_engine_set_fog(int ON_or_OFF, BYTE red, BYTE green, BYTE blue, int fog_mode, double max_visible_distance, double fog_start_distance, int fog_type);

// Returns the current fog settings:
// For more details see STATE_engine_set_fog();
// 
// Example:
//		int ON_or_OFF, fog_mode, fog_type;
//		BYTE red, green, blue;
//		double max_visible_distance, fog_start_distance;
//		STATE_engine_get_fog_parameters(&ON_or_OFF, &red, &green, &blue, &fog_mode, &max_visible_distance, &fog_start_distance, &fog_type);
//
STATE_IMPORT void STATE_WINAPI STATE_engine_get_fog_parameters(int *ON_or_OFF, BYTE *red, BYTE *green, BYTE *blue, int *fog_mode, double *max_visible_distance, double *fog_start_distance, int *fog_type);


//--------------------------------------------------------//
//=========== T H E    C A M E R A   A P I ===============//
//--------------------------------------------------------//

// This API	controls properties of the camera like
// zoom location direction etc...




// returns a handle to a camera.
// Used in conjunction with STATE_camera_get_next_camera()
// to traverse all the existing cameras
// example:
//	for(handle=STATE_camera_get_first_camera() ; handle!=NULL ; handle=STATE_camera_get_next_camera(handle) )
//	{
//		... 
//
//	}
STATE_IMPORT DWORD STATE_WINAPI STATE_camera_get_first_camera(void);
STATE_IMPORT DWORD STATE_WINAPI STATE_camera_get_next_camera(DWORD camera_handle);

//Checks whether the given camera handle is legal and represents a handle to a camera.
//If the handle is not a legal camera handle then a popup window will pop out (only if popup_title!=NULL).
//This function is used mainly for debugging (similar to the C++ function assert() )
//Return value: YES or NO.
//Arguments:
//	camera_handle: The handle of the camera to be examined.
//  popup_title: The title of the popup that will pop-out in case the given handle does not represent
//				a legal camera handle. If popup_title==NULL then there wont be any popup
//				The best is to give the function name so if you get the error you know
//				what function should be fixed.
STATE_IMPORT int STATE_WINAPI STATE_camera_is_camera(DWORD camera_handle, char *function_name);

// insensitive to upper / lower case  (it doesn't make a difference if it is a or A)
STATE_IMPORT DWORD STATE_WINAPI STATE_camera_get_using_name(char *camera_name);

// Returns a pointer to camera name which is stored internally inside the 3D Engine.
// Visual Basic users should use STATE_entity_get_name() instead.
// See also STATE_entity_get_name(), STATE_camera_set_name();
STATE_IMPORT char *STATE_WINAPI STATE_camera_get_name(DWORD camera_handle);


STATE_IMPORT void STATE_WINAPI STATE_camera_set_name(DWORD camera_handle, char *name);

// *** Function level of importance = High.
//
// The human eye can not see clearly objects that are too near the eye.
// Try reading a newspaper when your eye is one centimeter a way from the page ...
// The camera object (see the STATE_camera_ API ) which is the computer eye for rendering, act 
// similarly to the human eye. This function determines the closest distance in which the camera
// is able to see objects. The parts of objects that are closer than this number will not be seen.
// Calling this function with a number too small has its price.
// The bigger this number is the more accurate the rendering of the objects will be
// On the other hand a number too big will hide objects that are near
// the camera. Typical values are 1, 4, 10, 20 100.
// Try to make this number as big as you can. Stop increasing this number
// when near objects are not drawn.
//
// Important :
//		In some cases increasing the number can improve the rendering speed dramatically.
//
// return OK or VR_ERROR
STATE_IMPORT void STATE_WINAPI STATE_camera_set_distance_from_eye(DWORD camera_handle, double distance);

//See STATE_camera_set_distance_from_eye() 
STATE_IMPORT double STATE_WINAPI STATE_camera_get_distance_from_eye(DWORD camera_handle);


// Creates a camera and returns a handle to that camera
// Note that the values for the camera setting (location, direction , field view etc ...)
// are given default values. In probably all cases you
// would like to set those values yourself before you
// You use the camera specially the location and the direction
// given parameters
//----------------- 
//	camera_name: a string that identifies the camera. Usually this
//		string wont be in use since it is more efficient to
//		retrieve a camera using the handle (the return value of this
//		function).
// see also STATE_camera_recreate() STATE_camera_save()
STATE_IMPORT DWORD STATE_WINAPI STATE_camera_create(char *camera_name);

// Returns the last camera that was used with the last STATE_engine_render();
STATE_IMPORT DWORD STATE_WINAPI STATE_camera_get_current(void);

// The current camera is the camera that was used in the last render.
// Many functions such as STATE_engine_3D_point_to_2D(), STATE_engine_2D_point_to_3D, is_movement_possible() and more,  dont take
// a camera_handle as one of their parameters instead they do
// STATE_camera_get_current() ( note that the result of functions such as STATE_engine_2D_point_to_3D() depends on the camera that is used)
// This assumption is usually correct though in some occasions you might want to call STATE_engine_2D_point_to_3D() (or similar functions )for a camera
// other then the one that was used in the last render. For these occasions you will need 
// to call STATE_camera_set_current().
// If you are rendering on more than one window then you are most likely to use STATE_camera_set_current()
// (for example, if you render on 3 different windows with three different camera and then you do STATE_engine_3D_point_to_2D() 
//  How will the engine know with which window\camera you want this calculation to be done).
// STATE_camera_set_current() takes little time to execute so if you are in doubt, there is no harm in calling this function.
//
// If your program doesn't call STATE_engine_render() at all then you will need to call STATE_camera_set_current()
// in order that the engine will know what camera to use for calculations done in functions such as STATE_engine_2D_point_to_3D()
//
// hwnd is the window that we are using the camera with
// ( If you are using MFC you can do CWnd->m_hWnd )
STATE_IMPORT int  STATE_WINAPI STATE_camera_set_current(DWORD camera_handle, HWND hwnd);

//STATE_IMPORT int  STATE_WINAPI STATE_camera_set_current(DWORD camera_handle);


// Returns a handle to the first camera that was created.
STATE_IMPORT DWORD STATE_WINAPI STATE_camera_get_default_camera(void);


//save the current status of the given camera to the given buffer
// later a camera with the same status (location, direction, field_of_view etc ...)
// can be created using the given buffer.
STATE_IMPORT void  STATE_WINAPI STATE_camera_save(DWORD camera_handle, DWORD save_buffer[CAMERA_DESCRIPTOR_SIZE]);

// creates a new camera and returns the handle to that camera
// The new camera status (location, direction, field_of_view etc ...) is
// set according to the data in restore_buffer.
// this buffer should be filled before calling this function
// using STATE_camera_save(). This mechanism could be used to
// duplicate cameras. Another usage could be to send the buffer across
// the internet so that the same camera could be created on
// the far computer. see also STATE_camera_create()
STATE_IMPORT DWORD STATE_WINAPI STATE_camera_recreate(char *camera_name, DWORD restore_buffer[CAMERA_DESCRIPTOR_SIZE]);

// The new camera status (location, direction, field_of_view etc ...) is
// set according to the data in "values_data" buffer.
// this buffer should be filled before calling this function
// using STATE_camera_save().
STATE_IMPORT void STATE_WINAPI STATE_camera_set_values(DWORD camera_handle, DWORD values_data[CAMERA_DESCRIPTOR_SIZE]);

// return the width of the core image created on the camera film
// this value multiplied by the value return from STATE_camera_get_height()
// indicates the quality of the picture that will be created
// you can modify the picture quality by using the following functions
// 1- STATE_engine_set_picture_quality(int quality);
// 2- STATE_engine_increase_picture_quality(void);
// 3- STATE_engine_decrease_picture_quality(void);
STATE_IMPORT int STATE_WINAPI STATE_camera_get_width(DWORD camera_handle);
STATE_IMPORT int STATE_WINAPI STATE_camera_get_height(DWORD camera_handle);

// Set the direction towards which the camera will point .
//
// Example:
//
//		DWORD camera=STATE_camera_get_default_camera();		
//		//Make the camera points towards the positive Y axis (pointing right)
//		STATE_camera_set_direction(camera, 0,1,0);
//
// Remarks:
//		Note that STATE_camera_set_direction(camera, 10,20,30);
//		has exactly the same result as STATE_camera_set_direction(camera, 1,2,3);
//
//		For more information about coordination systems and setting directions see the book (download the book free from www.3dstate.com )
//
// See also: STATE_camera_set_direction1()
//
STATE_IMPORT void STATE_WINAPI STATE_camera_set_direction(DWORD camera_handle, double x, double y, double z);


// Set the direction towards which the camera will point. Exactly the same as the function STATE_camera_set_direction()
// only that here we use just 1 parameter.
//
// Example:
//
//		double xyz[3]={10,20,30}; //Note that setting 10,20,30 is the same as 1,2,3 (the engine normalizes the vector (make it a unit vector) )
//		DWORD camera=STATE_camera_get_default_camera();
//		STATE_camera_set_direction1(camera, xyz);
//
// Remarks:
//
//		For more information about coordination systems and setting directions see the book (download the book free from www.3dstate.com )
// 
// See also STATE_camera_set_direction()
//
STATE_IMPORT void STATE_WINAPI STATE_camera_set_direction1(DWORD camera_handle, double xyz[3]);

// Returns the direction towards which the camera points.
//
//
// Example A:
//		
//		double x,y,z;
//		DWORD camera=STATE_camera_get_default_camera();
//		STATE_camera_get_direction(camera, &x, &y, &z);
//
// Example B:
//
//		//Note that instead of doing it as shown below, it is easier to use the function STATE_camera_get_direction1() instead
//		double xyz[3];
//		DWORD camera=STATE_camera_get_default_camera();
//		STATE_camera_get_direction(camera, &xyz[0], &xyz[1], &xyz[2]);
//
//
//
// See also STATE_camera_get_direction1()
//
//
// Remarks:
//
//		The returned direction is normalized (meaning it is a unit vector)
//		For more information about coordination systems and setting directions see the book (download the book free from www.3dstate.com )
//
STATE_IMPORT void STATE_WINAPI STATE_camera_get_direction(DWORD camera_handle, double *x, double *y, double *z);

// Returns the direction towards which the camera points. Exactly the same as the function STATE_camera_get_direction()
// only that here we use just 1 parameter.
//
// Example:
//
//		double xyz[3];
//		DWORD camera=STATE_camera_get_default_camera();
//		STATE_camera_get_direction1(camera, xyz);
//
//
//
// See also STATE_camera_get_direction()
//
//
// Remarks:
//
//		The returned direction is normalized (meaning it is a unit vector)
//		For more information about coordination systems and setting directions see the book (download the book free from www.3dstate.com )
//
STATE_IMPORT void STATE_WINAPI STATE_camera_get_direction1(DWORD camera_handle, double xyz[3]);



//Gets a point\location in the world and make the camera point at this location
// See also STATE_camera_point_at_2D(), STATE_engine_2D_point_to_3D()
STATE_IMPORT void STATE_WINAPI STATE_camera_point_at(DWORD camera_handle, double x, double y, double z);

// Gets a two dimensional point=> (x,y) is a point in the rendered window
// Usually we get this point from events like OnLButtonDown()
// The function calculates the corresponding three dimensional point
// in the world and sets the camera to point towards this point.
// If there is no corresponding 3D point then it returns VR_ERROR
// (happens when we point to an empty place in the rendered image.
// see also STATE_camera_point_at() and STATE_engine_2D_point_to_3D();
STATE_IMPORT int STATE_WINAPI STATE_camera_point_at_2D(DWORD camera_handle, int x, int y);


// Set the location of the camera
//
// Example A:
//		
//		DWORD camera=STATE_camera_get_default_camera();
//		STATE_camera_set_location(camera, 10, 20, 30);
//
// Example B:
//		
//		double x=10,y=20,z=30;
//		DWORD camera=STATE_camera_get_default_camera();
//		STATE_camera_set_location(camera, x, y, z);
//
// Example C:
//
//		//Note that instead of doing it as shown below, it is easier to use the function STATE_camera_set_location1() instead
//		double xyz[3]={10,20,30};
//		DWORD camera=STATE_camera_get_default_camera();
//		STATE_camera_set_location(camera, xyz[0], xyz[1], xyz[2]);
//
// See also STATE_camera_set_location1()
//
STATE_IMPORT void STATE_WINAPI STATE_camera_set_location(DWORD camera_handle, double x, double y, double z);


// Set the location of the camera. Exactly the same as the function STATE_camera_set_location()
// only that here we use just 1 parameter.
//
// Example:
//
//		double xyz[3]={10,20,30};
//		DWORD camera=STATE_camera_get_default_camera();
//		STATE_camera_set_location1(camera, xyz);
//
// See also STATE_camera_set_location()
//
STATE_IMPORT void STATE_WINAPI STATE_camera_set_location1(DWORD camera_handle, double xyz[3]);


//Returns the location of the camera.
//
// Example A:
//		
//		double x,y,z;
//		DWORD camera=STATE_camera_get_default_camera();
//		STATE_camera_get_location(camera, &x, &y, &z);
//
// Example B:
//
//		//Note that instead of doing it as shown below, it is easier to use the function STATE_camera_get_location1() instead
//		double xyz[3];
//		DWORD camera=STATE_camera_get_default_camera();
//		STATE_camera_get_location(camera, &xyz[0], &xyz[1], &xyz[2]);
//
//
//
// See also STATE_camera_get_location1()
//
STATE_IMPORT void STATE_WINAPI STATE_camera_get_location(DWORD camera_handle, double *x, double *y, double *z);

// Returns the location of the camera. Exactly the same as the function STATE_camera_get_location()
// only that here we use just 1 parameter.
//
// Example:
//		
//		double xyz[3];
//		DWORD camera=STATE_camera_get_default_camera();
//		STATE_camera_get_location(camera, xyz);
//
// See also STATE_camera_get_location()
//
STATE_IMPORT void STATE_WINAPI STATE_camera_get_location1(DWORD camera_handle, double xyz[3]);

//Sets the axis system of the camera explicitly.
// X,Y,Z are three unit vectors that defines the view space. -X is the direction
// of viewing.
// One of the useful usage of this function together with STATE_camera_get_axis_system() and get\set location
// is to save the camera exact position.
//The function works exactly like STATE_object_set_axis_system()
//See STATE_object_set_axis_system() for more information.
// Returns OK or VR_ERROR
STATE_IMPORT int STATE_WINAPI  STATE_camera_set_axis_system(DWORD camera_handle, double X[3], double Y[3], double Z[3]);

//Gets the axis system of the camera..
// X,Y,Z are three unit vectors that defines the view space. -X is the direction
// of viewing.
// One of the useful usage of this function together with STATE_camera_set_axis_system() and get\set location
// is to save the camera exact position.
// See also STATE_camera_set_axis_system()
STATE_IMPORT void STATE_WINAPI STATE_camera_get_axis_system(DWORD camera_handle, double X[3], double Y[3], double Z[3]);



// Returns the new tilt angle
// examples:
// tilt 0 mean looking straight.
// tilt 90 looking up
// tilt -90 means looking down
STATE_IMPORT double STATE_WINAPI STATE_camera_set_tilt(DWORD camera_handle, double angle);

// Returns the new tilt angle
// examples:
// tilt 0 mean looking straight.
// tilt 90 looking up
// tilt -90 means looking down
STATE_IMPORT double STATE_WINAPI STATE_camera_get_tilt(DWORD camera_handle);


// Returns the new tilt angle
// examples:
// 90 means rotate camera 90 degrees up
// -90 means rotate camera 90 degrees down
// The rotation is done in world cords (rather then in object cords)
STATE_IMPORT double STATE_WINAPI STATE_camera_modify_tilt(DWORD camera_handle, double change);

//Changes the location of the camera by the given X,Y,Z offset
// New_location=old_location + (X,Y,Z)
// space_flag could be WORLD_SPACE or CAMERA_SPACE
// space_flag determines if the step is according to the camera's
// coordinate system or the world coordinate  system.
// For example if space_flag==CAMERA_SPACE and step==(-1,0,0)
// This means that the camera will advance forward.
// Try both options and see what happens.
STATE_IMPORT void STATE_WINAPI STATE_camera_move(DWORD camera_handle, int space_flag, double x, double y, double z);


//This is an important function. It helps in moving the camera nicely
//so it doesn't go through polygons.
//The function tries to move the camera to the given wanted location.
//If it is impossible because of collision, it tries to move as close as it can
//In every respected game, when the player moves towards a wall, his movement won't be stopped
//when he hits the wall. Instead the player will move smoothly along the wall.
//Using this function one can achieve this effect automatically. 
//
// Parameters:
//
//	DWORD camera, 
//			The camera that we want to move
//
//	double wanted_location[3], 
//			The location in world space coordinates where we want to move the camera
//
//	double camera_physical_width
//			The physical camera is actually a point though in order to 
//			simulate real life the engine apply real dimensions to the camera
//			The size of the camera is calculated as a square with the dimension 
//			camera_physical_width x camera_physical_width
//			The smaller this number is the more we will be able to get nearer to the walls in the world.
//			

//Return value
// Returns YES or NO. YES is returned if the camera was moved. NO is returned
// if the function could not move the camera at all
// 
STATE_IMPORT int STATE_WINAPI STATE_camera_move_with_collision_detection(DWORD camera, double wanted_location[3], double camera_physical_width);

//Move the camera up or down so its height above the ground
//is according to the given parameters.
//This function is quite useful, usually one calls it before moving the camera.
//Imagine for example that we have stairs to climb. If calling this function
// after moving the camera forward the result will be a very natural stairs climbing.
// Note that this function is very simple inside. Some people might not use this function and
// would rather write their own code.
//
// Return Value.
//		Returns a handle to the polygon bellow.
//		If there is nothing below it will return NULL
// Parameters:
//		camera_handle: the camera to be dropped.
//
//		double height_above_ground : set this argument to the desire height above the ground.
//
//		gravity : The maximum fall that the camera is allowed to make for each call of the function.
//				  For example if the camera falls down from a building. The bigger this number is
//				  the fewer calls (to this function) it will take till the camera reaches the ground.
//				  In other words if the camera height above the ground is 100 and the camera is moving forward
//				  on the top of a building, once it falls from the building the camera height from the floor will suddenly be
//				  quite a big number, lets say 10000. If the gravity is 50 then after calling this function the camera will be set
//				  at a height of 10000-50 above the ground.
//				  If you didn't get all of this it doesn't matter, try different values till you reach the desired effect.
//				  
//
//		point_on_the_ground : The function returns through this parameter a point on the ground that is exactly
//							  bellow the camera. If there is no polygon bellow the camera then this parameter will contain (0,0,0)
//							  and the function return value will be NULL
//
// See also STATE_camera_drop_down();
//
// Example
//	double point_on_the_ground[3];
//  DWORD floor_polygon=STATE_camera_set_height_above_ground(my_camera, 100, 40,point_on_the_ground);
//  if(floor_polygon==NULL)
//			error_message("We are walking on air !!!);
//
//
STATE_IMPORT DWORD STATE_WINAPI STATE_camera_set_height_above_ground(DWORD camera_handle, double height_above_ground, double gravity ,double point_on_the_ground[3]);

// Calculates the camera height above ground.
//
// Return value:
//		Returns the polygon below the camera. Returns NULL if there is nothing below.
//
// Parameters:
//		[IN] camera_handle: 
//				the camera handle
//
//		[IN] camera_size:
//				The function calculates an imagenary square around the camera.
//				The function calculates the height above ground of each corner of this square and
//				the height of the camera itself.
//				The function returned value is the minimum height above ground from the five measurments (4 corners and the camera location itself).
//				If camera_size==0, the function will only calculate and return the height of the camera point.
//				See the remarks section for "why do we need this parameter"
//
//		[OUT] height:
//				Through this parameter the function returns the calculated height above the ground
//			
// Remarks:
//		- What's the use of the camera_size parameter ?
//		  Imagine your player (the camera) is moving on a roof of a building
//		  When he passes the edge of the roof you detect it and start dropping the camera down
//		  This methos is not good because this way your player can get too close to the building walls.
//		  When using this function you will be able to detect when all of the player body is in
//		  the air and only then to start dropping the player down.
//
//		2) In many times the function STATE_engine_ignore_dynamic_objects_for_collision_detection(YES);
//		   will be called before dropping down the camera. This will make the camera fall through dynamic objects
//
// Example:
//		double height;
//		DWORD polygon=STATE_camera_get_height_above_ground(my_camera, 50, &height);
// 
STATE_IMPORT DWORD STATE_WINAPI STATE_camera_get_height_above_ground(DWORD camera_handle, double camera_size, double *height);


//The function is not in use
// Returns the new focus distance
STATE_IMPORT double STATE_WINAPI STATE_camera_set_focus(DWORD camera_handle, double focus_distance);

//The function is not in use
// Returns the new focus distance
STATE_IMPORT double STATE_WINAPI STATE_camera_modify_focus(DWORD camera_handle, double change_in_focus_distance);


// Returns the new field of view angle
// the field of view could be anything between 1 to 179 degrees
// field of view == 1 means a very strong zoom in 
// field of view == 179 means a very strong zoom out
STATE_IMPORT double STATE_WINAPI STATE_camera_set_zoom(DWORD camera_handle, double field_of_view_angle);
STATE_IMPORT double STATE_WINAPI STATE_camera_get_zoom(DWORD camera_handle);

//Changes the current field of view
// New_vield_of_view = old_field_of_view + field_of_view_change
// field_of_view_change could be positive or negative.
// Returns the new field of view angle
STATE_IMPORT double STATE_WINAPI STATE_camera_modify_zoom(DWORD camera_handle, double field_of_view_change);

//totaly identical to STATE_camera_get_zoom()
STATE_IMPORT double STATE_WINAPI STATE_camera_get_field_of_view(DWORD camera_handle);

//Returns the vertical field of view. Note that the vertical field of you cannot be set.
//It is automatically set according to the window size and the horizontal field of view (set by STATE_camera_set_zoom() )
STATE_IMPORT double STATE_WINAPI STATE_camera_get_vertical_field_of_view(DWORD camera_handle);


// Sets the new head angle (also the return value)
// examples:
// 0 means look forward into the screen
// 90 means look left
// -90 or 270 means look right
// 180 means camera looking from the screen out
// if we add or subtract 360*n to the angle it
// will be the same
STATE_IMPORT double STATE_WINAPI STATE_camera_set_head_angle(DWORD camera_handle, double head_angle);

// Returns the new head angle
// Could be use the determine absolute direction (North, Soth , West, East ...)
// examples:
// 0 means look forward into the screen (in the opposite direction of the world X axis)
// 90 means look left (in the opposite direction of the world Y axis)
// -90 or 270 means look right (in the direction of the world Y axis)
// 180 means camera looking from the screen out (in the direction of the world X axis)
// Adding or subtracting 360*n to the angle has no influence on the result
STATE_IMPORT double STATE_WINAPI STATE_camera_get_head_angle(DWORD camera_handle);


// Returns the new head angle
// examples:
// 0 do nothing (can be use as get_current_head_angle() ...)
// 90 means turn 90 degrees counterclockwise
// 90 means turn 90 degrees clockwise.
// 180 means camera turns half a circle
STATE_IMPORT double STATE_WINAPI STATE_camera_modify_head_angle(DWORD camera_handle, double head_angle);


// Returns the new bank angle
// the angle is given in degrees from the horizon
// examples:
// 0 means looking straight.
// 90 means turned 90 degrees clockwise
// -90 means turned 90 degrees clockwise
// giving alpha +360*k (k= ... -5,-4 , .... 0, 1 ,2 ...)
// is the same as giving alpha
STATE_IMPORT double STATE_WINAPI STATE_camera_set_bank(DWORD camera_handle, double angle);

// Returns the new bank angle
// the angle is given in degrees from the horizon
// examples:
// 0 means looking straight.
// 90 means turned 90 degrees clockwise
// -90 means turned 90 degrees clockwise
// The return value is in the range -90 - 270
STATE_IMPORT double STATE_WINAPI STATE_camera_get_bank(DWORD camera_handle);

// Returns the new bank angle
// the angle is given in degrees from the horizon
// examples:
// 0 means looking straight.
// 90 means turned 90 degrees clockwise
// -90 means turned 90 degrees clockwise
STATE_IMPORT double STATE_WINAPI STATE_camera_modify_bank(DWORD camera_handle, double change);


// Rotates the camera in the given angle.
// space_flag can be OBJECT_SPACE or  WORLD_SPACE
// space_flag determines if the step is according to the camera's
// coordinate system or the world coordinate  system.
// For example if space_flag==CAMERA_SPACE and step==(-1,0,0)
// This means that the camera will advance forward.
// Try both options and see what happens.
STATE_IMPORT void STATE_WINAPI STATE_camera_rotate_x(DWORD camera_handle, double degrees, int space_flag);
STATE_IMPORT void STATE_WINAPI STATE_camera_rotate_y(DWORD camera_handle, double degrees, int space_flag);
STATE_IMPORT void STATE_WINAPI STATE_camera_rotate_z(DWORD camera_handle, double degrees, int space_flag);

// The same just with radians.
// space flag can be OBJECT_SPACE or  WORLD_SPACE
STATE_IMPORT void STATE_WINAPI STATE_camera_rotate_x_radians(DWORD camera_handle, double radians, int space_flag);
STATE_IMPORT void STATE_WINAPI STATE_camera_rotate_y_radians(DWORD camera_handle, double radians, int space_flag);
STATE_IMPORT void STATE_WINAPI STATE_camera_rotate_z_radians(DWORD camera_handle, double radians, int space_flag);

//Converts a point in camera space to a point in world space.
// Returns OK or VR_ERROR if failed
//Here are some examples:
// The location of the camera (world space) will always translate to (0,0,0) in camera space
// (-1,0,0) in camera space is one unit in front of the camera this is equal to
// the camera location plus the camera direction (a unit vector) in camera space.
// (1,0,0) in camera space is one unit in the back of the camera
// (0,1,0) in camera space is always one unit to the right of the camera.
// (0,-1,0) in camera space is always one unit to the left of the camera.
// (0,0,1) in camera space is always one unit up above the camera.
//See also STATE_camera_convert_point_to_camera_space()
STATE_IMPORT int STATE_WINAPI STATE_camera_convert_point_to_world_space(DWORD camera_handle, double camera_space_point[3], double result[3]);

//Converts a point in world space to a point in camera space.
// Returns OK or VR_ERROR if failed
//Here are some examples:
// The point (0,0,0) in camera space will always translate to the location of the camera in world space
// (-1,0,0) in camera space is one unit in front of the camera this is equal to
// the camera location plus the camera direction (a unit vector) in camera space.
// (1,0,0) in camera space is one unit in the back of the camera
// (0,1,0) in camera space is always one unit to the right of the camera.
// (0,-1,0) in camera space is always one unit to the left of the camera.
// (0,0,1) in camera space is always one unit up above the camera.
//See also STATE_camera_convert_point_to_world_space()
STATE_IMPORT int STATE_WINAPI STATE_camera_convert_point_to_camera_space(DWORD camera_handle, double world_space_point[3], double result[3]);

// deletes a camera.
// remember not to use any handle after the instance was deleted
// doing:
//	STATE_camera_delete(my_camera);
//	my_camera=NULL;
// is good practice.
// returns OK or VR_ERROR 
// VR_ERROR could be receive if there is no more camera with this handle
// (if we already deleted the camera before)
STATE_IMPORT int STATE_WINAPI STATE_camera_delete(DWORD camera_handle);

STATE_IMPORT void STATE_WINAPI STATE_camera_delete_all(void);


// *************************


//Returns the name of the track that the camera is associated with.
//See also STATE_camera_set_track()
STATE_IMPORT char * STATE_WINAPI  STATE_camera_get_track_name(DWORD camera_handle);

STATE_IMPORT DWORD STATE_WINAPI STATE_camera_get_track_handle(DWORD camera_handle);

// Returns VR_ERROR, OK
// if track_handle ==NULL it doesn't cancel the chase type so
// for canceling chasing use STATE_camera_set_chase_type(NO_CHASE);
// Automatically set the closest point on the track (closest to the object)
// as the next point to go towards (to override this setting use STATE_camera_set_next_point_on_track() )
STATE_IMPORT int STATE_WINAPI STATE_camera_set_track(DWORD camera_handle, DWORD track_handle);


// return OK or VR_ERROR
// the offset is return through argument "offset"
STATE_IMPORT int STATE_WINAPI STATE_camera_get_track_offset(DWORD camera_handle, double offset[3]);

// return OK or VR_ERROR
STATE_IMPORT int STATE_WINAPI STATE_camera_set_track_offset(DWORD camera_handle, double offset[3]);


// Returns an index to a point on the track. This point is the point that the camera is moving towards
// The exact cords of the point can be retrieved through STATE_track_get_point()
// 0 is the first point and number_of_points-1 is the last index (use STATE_track_get_number_of_points() to get this number
STATE_IMPORT int STATE_WINAPI STATE_camera_get_next_point_on_track(DWORD camera_handle);

// returns the point on the track that the camera is moving towards
// 0 is the first point and number_of_points-1 is the last index (use STATE_track_get_number_of_points() to get this number
// if point_index<0 then it set it to 0
// if point_index is bigger than the maximum index then it is set to the maximum allowed. In those cases it returns VR_ERROR 
// to notify the change from the given value.
// If there is no track associated with the camera it returns VR_ERROR without setting the value
STATE_IMPORT int STATE_WINAPI STATE_camera_set_next_point_on_track(DWORD camera_handle, int point_index);

// Set the object to chase after
// the given arguments should NOT be NULL
// Note that if the camera was previously chasing a camera or a group
// , those chasing operation are canceled. (The camera can chase only one thing at a time).
STATE_IMPORT void STATE_WINAPI STATE_camera_set_object_to_chase(DWORD camera_handle, DWORD object_to_chase_handle);

// If the return value is NULL it means no object is being chased
STATE_IMPORT DWORD STATE_WINAPI STATE_camera_get_chased_object(DWORD camera_handle);


// Set the camera to chase after
// the given argument should NOT be NULL
// Note that if the camera was previously chasing an object or a group
// , those chasing operation are canceled. (The camera can chase only one thing at a time).
STATE_IMPORT void STATE_WINAPI STATE_camera_set_camera_to_chase(DWORD camera_handle, DWORD camera_to_chase_handle);

// If the return value is NULL it means no camera is being chased
STATE_IMPORT DWORD STATE_WINAPI STATE_camera_get_chased_camera(DWORD camera_handle);

// Set the group to chase after
// the given argument should NOT be NULL
// Note that if the camera was previously chasing a camera or an object
// , those chasing operation are canceled. (The camera can chase only one thing at a time).
STATE_IMPORT void STATE_WINAPI STATE_camera_set_group_to_chase(DWORD camera_handle, DWORD group_to_chase_handle);

// If the return value is NULL it means no group is being chased
STATE_IMPORT DWORD STATE_WINAPI STATE_camera_get_chased_group(DWORD camera_handle);

// return OK or VR_ERROR
// the offset is return through argument "offset"
STATE_IMPORT int STATE_WINAPI STATE_camera_get_chase_offset(DWORD camera_handle, double offset[3]);

// return OK or VR_ERROR
// The offset should be in camera space
STATE_IMPORT int STATE_WINAPI STATE_camera_set_chase_offset(DWORD camera_handle, double offset[3]);

// The returned number is between 0 and one.
// A number close to 0 means that the tracking is weak
// 1 means that the tracking is very sharp.
STATE_IMPORT double STATE_WINAPI STATE_camera_get_chase_softness(DWORD camera_handle);

// The returned number is between 0 and one
// a number close to 0 means that the tracking is weak
// 1 means that the tracking is very sharp.
// it the given number is bigger than one it is set as one
// if it is smaller than 0 than it set as 0
STATE_IMPORT void STATE_WINAPI STATE_camera_set_chase_softness(DWORD camera_handle, double softness);

// look in the header file for possible values and their meanings
// (look for NO_CHASE, CHASE_LOCATION and others ...)
STATE_IMPORT int STATE_WINAPI STATE_camera_get_chase_type(DWORD camera_handle);

// look in the header file for possible values and their meanings
// (look for NO_CHASE, CHASE_LOCATION and others ...)
STATE_IMPORT void STATE_WINAPI STATE_camera_set_chase_type(DWORD camera_handle, int chase_type);


STATE_IMPORT void STATE_WINAPI STATE_camera_set_chase_distance(DWORD camera_handle, double chase_distance);

STATE_IMPORT double STATE_WINAPI STATE_camera_get_chase_distance(DWORD camera_handle);

// ************************

// Advance the object on the track according to its
// track, track_speed, track_offset, chase_offset, target_point_on_track and chasing softness
// Use STATE_camera_advance() instead (set the right chase type before calling ...)
// XXXX STATE_IMPORT void STATE_WINAPI STATE_camera_advance_on_track(DWORD camera_handle);

// Advance the camera according to its chase_type
STATE_IMPORT void STATE_WINAPI STATE_camera_advance(DWORD camera_handle);



//Relevant only when CHASE_PHYSICS is used.
//Set the speed of the camera.
//	Example
//			//Give the camera speed of 10 in the positive direction of the Y axis
//			STATE_camera_set_speed(camera, 0,10,0);
//
// See also STATE_object_set_speed_units()
//
STATE_IMPORT void STATE_WINAPI STATE_camera_set_speed(DWORD camera_handle, double speed[3]);
STATE_IMPORT void STATE_WINAPI STATE_camera_get_speed(DWORD camera_handle, double speed[3]);

//Returns the size of the speed.
//Using this function we can only retrieve the magnitude of the speed but not its direction.
//For example if the function returns 10, it means that the speed of the camera is 10.
//Q: "speed of ten" to which direction ?
//A: To know the direction of the speed call STATE_camera_get_speed()
//
STATE_IMPORT double STATE_WINAPI STATE_camera_get_absolute_speed(DWORD camera_handle);

//Relevant only when CHASE_PHYSICS or CHASE_TRACK is used.
//Set the size of the speed.
//
// Read the example below to learn how it all works. 
//
// Example:
//		STATE_camera_set_speed(camera, 0,10,0);
//		speed_size=STATE_camera_get_absolute_speed(camera);
//		//speed_size is now 10
//
//		STATE_camera_set_absolute_speed(camera, 100);
//
//		double speed[3];
//		STATE_camera_get_speed(camera, speed);
//		//speed is now { 0, 100, 0}
//
//
// Advanced:
//	In physics speed is a vector it has a size and a direction
//	Q: If the speed vector is 6,8,0 ( STATE_camera_set_speed(camera, 6,8,0) );
//	   what is the size of the speed (what will STATE_camera_get_absolute_speed(camera) return ?)
//
//
//	A:	Let calculate the vector size according to Pythagoras Theorem
//		speed size=  Square root of ( A*A+ B*B+ C*C ) = square root of ( 6*6 + 8*8 + 0*0) = Square root of (100) == 10
//
//	
STATE_IMPORT void STATE_WINAPI STATE_camera_set_absolute_speed(DWORD camera_handle, double speed);

// Set the force that will work on the camera. For example to simulate gravity force
// do like that:
//		double force[3];
//		force[0]=0;
//		force[1]=0;
//		force[2]= -10; //simulate gravity 10 in the direction of the -Z axis.
//		STATE_camera_set_force(my_camera,force);
//
//	Note that this will only relevant when CHASE_PHYSICS is applyed
STATE_IMPORT void STATE_WINAPI STATE_camera_get_force(DWORD camera_handle, double force[3]);
STATE_IMPORT void STATE_WINAPI STATE_camera_set_force(DWORD camera_handle, double force[3]);

//Limits the maximum possible speed of a camera
//If there is no limit then by applying force on the camera
//The speed could reach infinity (unless some friction is given too)
//Note that this will only relevant when CHASE_PHYSICS is applyed
STATE_IMPORT void STATE_WINAPI STATE_camera_set_max_speed(DWORD camera_handle, double value);
STATE_IMPORT double STATE_WINAPI STATE_camera_get_max_speed(DWORD camera_handle);



//Friction is a value smaller or equal to 1
// if friction==0 it means no friction
// if friction==1 it means infinite friction nothing can move
// if friction is smaller than 0, it acts like pressing on the gas
// The exact calculation.
// Each render the absolute_speed is multiply with (1-friction).
//
// Note, a negative number can used for increasing the speed.
// a friction of -1 will double the speed.
// -2 will multiply the speed by three each rendering.
// so choose number carefully.
// typical numbers can be 0.03 or -0.03 etc ...
STATE_IMPORT void STATE_WINAPI STATE_camera_set_friction(DWORD camera_handle, double value);
STATE_IMPORT double STATE_WINAPI STATE_camera_get_friction(DWORD camera_handle);

//This number is used when the camera hits a polygon. 
//A value of one will make the new speed (after collision)
//equal to the old speed (effects only the absolute speed value and not the direction)
//a value of 0.5 will cut speed by half. 
//We can also give values greater than one.
//For example a value of 2 will make the new speed twice as big
//as before the collision.
STATE_IMPORT void STATE_WINAPI STATE_camera_set_elasticity(DWORD camera_handle, double value);
STATE_IMPORT double STATE_WINAPI STATE_camera_get_elasticity(DWORD camera_handle);

// (left_x, top_y) and (right_x ,bottom_y) are points on the window that
// was used to render an image using the given camera.
// The function returns through new_location a location that if we would
// do STATE_camera_set_location() the rectangle area will fill our screen
// returns OK or VR_ERROR
STATE_IMPORT int STATE_WINAPI STATE_camera_get_location_to_fit_rectangle(DWORD camera_handle, int left_x, int top_y, int right_x, int bottom_y, double new_location[3]);


// x,y are points on the window
// usually we get this point from events like OnLButtonDown()
// The function moves the camera towards the point (or far from the point depending 
// on the given value, factor).
// Note that the direction of the camera doesn't change. (Use STATE_camera_point_at() to make the target point at the center)
// returns OK, VR_ERRORR.
// factor can be any number. Here are some examples:
// factor== -2 , the camera will triple its distance from the given point
// factor== -1 , the camera will double its distance from the given point
// factor==0 means, the camera wont move
// factor==0.5 the camera will move half way to the point
// factor==1 the camera will move to the given point x,y
// factor==2 the camera will pass the point (will be in the same distance but on the other side of the line that connects the point and the line).
STATE_IMPORT int STATE_WINAPI STATE_camera_move_toward_point_2D(DWORD camera_handle, int x, int y, double factor);

// advance all the cameras. If STATE_engine_advance_cameras_automatically(YES) is called
// then this function will be called automatically each time we do render
STATE_IMPORT void STATE_WINAPI STATE_camera_advance_all();

STATE_IMPORT void STATE_WINAPI STATE_camera_set_name_to_chase(DWORD camera_handle, char *name_to_chase);

STATE_IMPORT char * STATE_WINAPI STATE_camera_get_name_to_chase(DWORD camera_handle);

// sending YES will write the given camera to file after call to STATE_engine_save() with SAVE_ONLY_WHATS_NEEDED 
// Note that it if STATE_engine_save() is called without SAVE_ONLY_WHATS_NEEDED, all the cameras will be saved
// sending NO will cause  STATE_engine_save() not to save this camera (only if SAVE_ONLY_WHATS_NEEDED is used)
// The default value is NO
// see also STATE_animation_set_save_flag()
STATE_IMPORT void STATE_WINAPI STATE_camera_set_save_flag(DWORD camera_handle, int yes_no_flag); 

// returns YES or NO
// The default value is NO
// see STATE_camera_set_save_flag() for more details
STATE_IMPORT int STATE_WINAPI STATE_camera_get_save_flag(DWORD camera_handle); 

// Some words about parallel and perspective projection:
// Though we have a 3D world eventually when we create an image
// of that world it is a 2D image. In order to get this image the engine
// projects the 3D world onto a projection plane. There are two main methods to do this projection
// They are called parallel projection and perspective projection.
// Perspective projection (the default ) is similar to the way the human
// eye sees things. Far away objects look smaller while closer objects look bigger.
// Parallel projection is the same as orthographic projection.
// The depth dimension is not shown on the result image.
// If you never heard of projection method simply try both method
// and see what happens. For the vast majority of applications
// you will use only perspective projection (the default)
// Parallel projection should interest you only if your application is in fields such as
// Architecture design, 3D modeling etc ...
// Parallel projection has little use in games if at all.
STATE_IMPORT void STATE_WINAPI STATE_camera_set_perspective_projection(DWORD camera_handle); 
STATE_IMPORT void STATE_WINAPI STATE_camera_set_parallel_projection(DWORD camera_handle, int width, int heigh); 
STATE_IMPORT int STATE_WINAPI STATE_camera_is_perspective_projection(DWORD camera_handle); 
STATE_IMPORT int STATE_WINAPI STATE_camera_get_parallel_projection_width(DWORD camera_handle);
STATE_IMPORT int STATE_WINAPI STATE_camera_get_parallel_projection_height(DWORD camera_handle);


//Drops the camera down.
//In other words: Move the camera down till it is height_above_ground above the polygon below the camera
//If there is nothing below the camera then the location of the camera will not be changed.
//In many times the function STATE_engine_ignore_dynamic_objects_for_collision_detection(YES);
// will be called before dropping down the camera. This will make the camera fall through dynamic objects
// 
// Return value: 
//		returns a handle to the polygon below the camera
//
// parameters:
//		camera_handle: the camera to be dropped
//
//		height_above_ground: the height above the polygon below the camera in which the camera should be positioned.
//
//
// Examples: 
//	A)	//drop the camera down
//		STATE_camera_drop_down(my_camera, 100); //put the camera 100 units above the ground 
//
//	B)  //drop the camera through objects
//		STATE_engine_ignore_dynamic_objects_for_collision_detection(YES);
//		STATE_camera_drop_down(my_camera, 100); //put the camera 100 units above the ground 
//		STATE_engine_ignore_dynamic_objects_for_collision_detection(NO); //restore regular collision detection
//
STATE_IMPORT DWORD STATE_WINAPI STATE_camera_drop_down(DWORD camera_handle, double height_above_ground);


// Who needs this function ?
// --------------------------
//	Users that are porting code from other 3D engines might want to use this function.
//	All the rest can skip this function
//
// General background
// ------------------
//	Please note that when moving or rotating the camera all that the engine does is change
//	the camera's matrix which is a quick and fast to do.
//
//
// What is this matrix ?
// ---------------------
//	To learn more about transformation matrixes you should read a computer graphics math book.
//  This is just if you are interested in the subject. One of the great things about the STATE engine
//	is that it makes it completely unnecessary to learn these subjects.
//	Anyhow, since you are still reading it means that you want to know more, so here it is:
//		the first top 9 cells of the matrix are the rotation matrix
//		The bottom row is the translation values.
//		The diagnal is the scale values
//
//		S R R 0
//		R S R 0
//		R R S 0
//		T T T 1
//
//		Above is the matrix. T stands for translation. S for scale. R for rotation.
//		If you don't know what we are talking about then you should read a book in the subject
//
//		By taking a point in camera space and multiplying it by the inverse_matrix we get the point in World Space.
//		for example the point (0,0,0) in camera space is the location of the camera.
//		when multiplying (0,0,0) by the inverse_matrix we get (T T T) which means that
//		(T,T,T) is the camera location
//		The code below shows how to multiply a point (vec) by the matrix (mat).
//		The result is saved in dst.
//
//		void 	mul_vec_mat(vector	dst, vector vec, matrix mat) //does dst=vec*mat
//		{
//		  double	res[4];
//		  double	one_over_res3;
//
//
//			res[0]=vec[0]*mat[0][0] + vec[1]*mat[1][0] + vec[2]*mat[2][0] + mat[3][0];
//			res[1]=vec[0]*mat[0][1] + vec[1]*mat[1][1] + vec[2]*mat[2][1] + mat[3][1];
//			res[2]=vec[0]*mat[0][2] + vec[1]*mat[1][2] + vec[2]*mat[2][2] + mat[3][2];
//			res[3]=vec[0]*mat[0][3] + vec[1]*mat[1][3] + vec[2]*mat[2][3] + mat[3][3];
//
//			if(res[3]<1e-20 && res[3]>-1e-20) { //protect agains floating point errors.
//				ERROR_MSG(SCREEN,"mul_vec_mat: devision by zero\n");
//		
//				dst[0]=res[0];
//				dst[1]=res[1];
//				dst[2]=res[2];
//				return;
//			}
//
//			one_over_res3=1/res[3];
//
//			dst[0]=res[0]*one_over_res3;
//			dst[1]=res[1]*one_over_res3;
//			dst[2]=res[2]*one_over_res3;
//
//		}
//
//
//	Parametar:
//	----------
//
//		camera_handle: the camera handle
//
//		camera_matrix: The camera matrix. Note that multiplying a point in world space by this matrix will result
//					   a point in camera space.
//
//		inverse_matrix: The inverse of camera_matrix. Multiplying a point in camera space by this matrix will
//						result a point in world space.
//
//
// Example:
//
//		double matrix[4][4], int_matrix[4][4];
//		STATE_camera_get_transformation_matrix(my_camera, matrix);
//
STATE_IMPORT void STATE_WINAPI STATE_camera_get_transformation_matrix(DWORD camera_handle, double camera_matrix[4][4], double inverse_matrix[4][4]);

// *** An advanced function. Not very useful.
// The camera projection plane is the plane on which the seen world is projected on.
// The projection plane equation is X+d=0 or in other words all the points in camera space
// that have their X coordinate equals to -d.
//
// Q: What is this function good for ?
// A: Lets say you have an object in the world and you would like to put it
//    so it is seen (by the camera) in the lower right corner of the screen.
//	  Here are several ways that you can achive this.
//	  Method 1:
//			double offset[3]={-10, 11, -12}; // 10 units in front of the camera, 11 units to the right of the camera, 12 units below the camera
//											//the 10,11,12 numbers have no special meaning, check other values till you have the correct position
//			double object_location[3];
//			STATE_camera_convert_point_to_world_space(camera, offset, object_location);
//			STATE_object_set_location1(my_object, object_location);
//
//			This method is very simple though if the size of the rendering window changes
//			it will also change the position of the object relative to the rendering window corners
//
//	  Method 2: Use STATE_camera_get_zoom() to get the camera field of view and calculate it from there.
//
//	  Method 3: Use STATE_camera_get_projection_plane_d() (this function)
//		 
//			STATE_camera_set_current(my_camera, m_hWnd); //m_hWnd is the window on which we render
//			double d=STATE_camera_get_projection_plane_d(my_camera);
//			//get the dimension of the rendering window
//			RECT rect;
//			GetClientRect(&rect);
//			double object_location[3];
//			object_location[0]=d;
//			object_location[1]=rect.right/2;
//			object_location[2]=-rect.bottom/2;
//			//Now we have a point at the bottom left corner of the screen
//			//This point might be closer than the allowed distance from the camera (eye)
//			//so we will now take care of it.
//			double allowed_distance=STATE_camera_get_distance_from_eye(my_camera);
//			//Remeber that the camera is looking towards the negativ X axis
//			object_location[0]= -allowed_distance;
//			object_location[1]= object_location[1]*allowed_distance/-d;
//			object_location[2]= object_location[1]*allowed_distance/-d;
//			//The point that we have now is in camera space and its location
//			//is in the bottom left corner of the viewed rendering window
//			//Note that you will probably want to move the location alittle bit
//			//forward, left and upper so it is not cut by the viewing frustom
//			double object_location_world_space[3];
//			STATE_camera_convert_point_to_world_space(camera, object_location, object_location_world_space);
//			STATE_object_set_location1(my_object, object_location);
//
// Return Value
//		Returns the projection plane equation d parameter (plane equation: X+d=0) which is the
//		the distance of the projection plane from the camera. The return number is negative
//		
STATE_IMPORT double STATE_WINAPI STATE_camera_get_projection_plane_d(DWORD camera_handle);

// *** quite a useful function
// Moves the camera on a sphere around the given location.
// The point on the sphere is determined according to the mouse location.
// This function is very easy to use. Just call it and see how it works.
// A typical use would be to call this function before or after each call to 3DSTATE_engine_render()
//
// Parameters:
//		camera_handle: 
//			the camera handle
//		center:
//			the center of the sphere on which the camera will move
//			The camera will always be directed towards the center
//
//		radius:
//			The radius of the sphere on which the camera moves
//			The camera will always be at "radius" distance from the center.
//
//		activation_trigger:
//			Sometimes you might want to move the camera only when dragging
//			with the mouse left button down (for example) as common in 3D modeling programs
//			In games you might want to move the camera when the mouse move (without any mouse key being pressed)
//			This parameter is used to tune the way the function works.
//			The activation_trigger can be any of Windows Virtual-Key Codes
//			For example VK_LBUTTON (Mouse left button). These constanst are define by Windows
//			See examples below. Using activation_trigger=0 has the advantage
//			that by putting the cursor in the middle of the rendering window
//			we get back to the normal position.
//
//		reverse_direction: YES or NO. If YES is given then the direction of the camera movement is reversed
//
//	Example A:
//		//Moves around the origin when the mouse moves (no trigger)
//		//Call this code before each render
//		double center[3]={0,0,0};		
//		DWORD camera=STATE_camera_get_current();
//		STATE_camera_move_around_point_according_to_mouse_position(camera, center, 10, 0, NO);
//
//	Example B:
//		//Moves around the origin when the left mouse button is being dragged
//		//Call this code before each render
//		double center[3]={0,0,0};		
//		DWORD camera=STATE_camera_get_current();
//		STATE_camera_move_around_point_according_to_mouse_position(camera, center, 10, VK_LBUTTON, NO);
//
//	Example C:
//		//Moves around the origin when the key 'A' on the keyboard is pressed (and the mouse moves)
//		//Call this code before each render
//		double center[3]={0,0,0};		
//		DWORD camera=STATE_camera_get_current();
//		STATE_camera_move_around_point_according_to_mouse_position(camera, center, 10, 'A', NO);
//
//	Example D:
//		//Moves the camera around a flying airplane.
//		//Call this code before each render
//		double center[3];		
//		STATE_object_get_location1(my_plane, center);
//		DWORD camera=STATE_camera_get_current();
//		STATE_camera_move_around_point_according_to_mouse_position(camera, center, 10, 0, NO);
//
STATE_IMPORT void STATE_WINAPI STATE_camera_move_around_point_according_to_mouse_position(DWORD camera_handle, double center[3], double radius, int activation_trigger, int reverse_direction);

//-----------------------------------------------------//
//======== T H E    O B J E C T    A P I ==============//
//-----------------------------------------------------//

// An object is a collection of polygons that can be manipulated together.
// This API is used in viewer_mode (the default mode) to manipulate moving objects (also referred\called dynamic objects)
// In viewer mode every world has a static part, for example the terrain the buildings etc.
// All the elements that can be moved or change shape are called objects or dynamic objects.
// Note that there are no static objects. There is just the static part of the world which is built from polygon
// and objects which are a collection of polygons that can be moved and manipulate together.
// 
// Behind the scenes:
//		When turning (called rotating) or moving the object as well as any other of the object API function,
//		the 3DSTATE engine only changes the internal transformation matrixes
//		This means that even if you turn or move an object with thousands of polygons
//		the operation will be very qucik since the engine doesnt change the point values
//		of each one of the object's polygons
//		The complicated math is well hidden from the API user but at the same time
//		this API is totaly flexible allowing total freedom to the programmer
//
//
// See the GROUP API for manipulating elements in EDITOR mode.
// Note that some functions of the Group API also works in viewer mode
// (mainly those functions which do not move polygons or change their points) but not the other way around.

// Returns YES or NO.
// YES , if the given handle is an object handle. NO if not.
// If it is not an object handle , a message box will appear,
// the title of this message box will be the parameter "function_asking"
// if function_asking==NULL than no popup will be created.
// See also IS_OBJECT()
STATE_IMPORT int STATE_WINAPI STATE_object_is_object(DWORD object_handle, char *function_asking);


// Checks whether a given object can move from one point to another inside the world
// Return value: YES or NO.
// If movement is possible the function will return YES else returns NO
//
// If movement is not possible it returns the plane equation Ax+By+Cz+D=0
// and the intersection point.
// Note that the normal of the blocking surface is N=( plane[0], plane[1], plane[2])
// on return blocking_object is the object that make movement impossible
// if the return value is NO and blocking_object==NULL it means
// that what blocks us is the static part of the world.
// the difference between this function end STATE_engine_is_movement_possible()
// is that here we also give the handle to the dynamic object that wants to move.
// This handle is needed so that the function can eliminate the object itself
// from being  the obstacle that makes the movement impossible.
// see also remarks on STATE_engine_is_movement_possible()
//
//	Arguments:
//		object_handle: The object that we want to move. The engine will disable this object
//						for doing the collision detection as explained in the remarks above.
//		
//		start_point, end_point: The function will check collision between the world and a line going from start_point to end_point
//
//		intersected_polygon:  If there is a collision going from the start point to the end point then the function will return
//							  the blocking polygon using this argument. 
//
//		intersection: an x,y,x point on the intersected_polygon where the line from start_point to end_point intersects
//
//		blocking_object: If the intersection is caused because of another dynamic object that the blocking object will be returned
//						using this argument.
//		
//
//
//	Remark:
//		Note that this function doesn't take a camera handle as one of its parameters.
//		Note also that the result of the operation is dependent on the current camera that is used.
//		The engine will use the current camera (The camera that was used in the last render)
//		In most of the cases this is the right thing to do. In the case when you want
//		to use a different camera for the calculation then use STATE_camera_set_current()
//		Before calling this function.
//
//
// Example:
//		double start_point[3]={1,2,3};
//		double end_point[3]={100,200,300};
//		DWORD intersected_polygon;
//		double intersection[3];
//		DWORD blocking_object[3];
//		STATE_object_is_movement_possible(my_object ,start_location, end_location, &intersected_polygon, intersection, &blocking_object);
//
STATE_IMPORT int STATE_WINAPI STATE_object_is_movement_possible(DWORD object_handle ,double start_location[3], double end_location[3], DWORD *intersected_polygon, double intersection[3], DWORD *blocking_object);


//Searches for an object with this name.
// If no object has this name then it
// returns a NULL
// If there are several objects with the same name then 
// it returns the first one. 
STATE_IMPORT DWORD STATE_WINAPI STATE_object_get_object_using_name(char *object_name);

// Returns the name of the given object
// The function returns a pointer to the name which is stored internally inside the 3D Engine
// Visual Basic users should use STATE_entity_get_name() instead.
// See also STATE_entity_get_name()
STATE_IMPORT char *STATE_WINAPI STATE_object_get_name(DWORD object_handle);

STATE_IMPORT char *STATE_WINAPI STATE_object_get_type_name(DWORD object_handle);

// Returns OK if was successful else VR_ERROR;
// Arguments:
//		new_name: the name to set for this object
//See also STATE_object_get_name(), STATE_object_get_object_using_name()
STATE_IMPORT int STATE_WINAPI STATE_object_set_name(DWORD object_handle, char *new_name);

// Returns OK if was successful else VR_ERROR;
STATE_IMPORT int STATE_WINAPI STATE_object_set_type_name(DWORD object_handle, char *new_name);

//Use can use this function to associate a number with the object.
//This function does nothing so you can take use of it to save a number associated with the object
//This number will also be saved when calling STATE_engine_save()
//See also STATE_object_set_control_type_number(), STATE_object_attach_my_own_data()
STATE_IMPORT void STATE_WINAPI STATE_object_set_object_type_number(DWORD object_handle, int object_type_number);
STATE_IMPORT int  STATE_WINAPI STATE_object_get_object_type_number(DWORD object_handle);

//Use can use this function to associate a number with the object.
//This function does nothing so you can take use of it to save a number associated with the object
//This number will also be saved when calling STATE_engine_save()
//See also STATE_object_set_object_type_number(), STATE_object_attach_my_own_data()
STATE_IMPORT void STATE_WINAPI STATE_object_set_control_type_number(DWORD object_handle, int control_type_number);
STATE_IMPORT int  STATE_WINAPI STATE_object_get_control_type_number(DWORD object_handle);


STATE_IMPORT char *STATE_WINAPI STATE_object_get_track_name(DWORD object_handle);

STATE_IMPORT DWORD STATE_WINAPI STATE_object_get_track_handle(DWORD object_handle);


// returns a handle to an object.
// Used in conjunction with STATE_object_get_next_object()
// to traverse all the existing objects
// example:
//	for(handle=STATE_object_get_first_object() ; handle!=NULL ; handle=STATE_object_get_next_object(handle) )
//	{
//		... 
//
//	}
STATE_IMPORT DWORD STATE_WINAPI STATE_object_get_first_object(void);
STATE_IMPORT DWORD STATE_WINAPI STATE_object_get_next_object(DWORD object_handle);

//Another way of looping through all the objects that are loaded into the engine is by using the FOREACH_OBJECT macro.
// Example:
//		FOREACH_OBJECT(obj) {
//			//do something with the object. For rxsmple lets check its name
//			char *name=STATE_object_get_name(obj);
//		}
#define FOREACH_OBJECT(obj) for(DWORD obj=STATE_object_get_first_object(); obj!=NULL; obj=STATE_object_get_next_object(obj))



// Set the location of the object
//
// Example A:
//		
//		DWORD object=STATE_object_get_first_object();
//		STATE_object_set_location(object, 10, 20, 30);
//
// Example B:
//		
//		double x=10,y=20,z=30;
//		DWORD object=STATE_object_get_first_object();
//		STATE_object_set_location(object, x, y, z);
//
// Example C:
//
//		//Note that instead of doing it as shown below, it is easier to use the function STATE_object_set_location1() instead
//		double xyz[3]={10,20,30};
//		DWORD object=STATE_object_get_first_object();
//		STATE_object_set_location(object, xyz[0], xyz[1], xyz[2]);
//
// See also STATE_object_set_location1()
//
STATE_IMPORT void STATE_WINAPI STATE_object_set_location(DWORD object_handle, double x, double y, double z);

// Set the location of the object. Exactly the same as the function STATE_object_set_location()
// only that here we use just 1 parameter.
//
// Example:
//
//		double xyz[3]={10,20,30};
//		DWORD object=STATE_object_get_first_object();
//		STATE_object_set_location1(object, xyz);
//
// See also STATE_object_set_location()
//
STATE_IMPORT void STATE_WINAPI STATE_object_set_location1(DWORD object_handle, double xyz[3]);

//Returns the location of the object.
//
// Example A:
//		
//		double x,y,z;
//		DWORD object=STATE_object_get_first_object();
//		STATE_object_get_location(object, &x, &y, &z);
//
// Example B:
//
//		//Note that instead of doing it as shown below, it is easier to use the function STATE_object_get_location1() instead
//		double xyz[3];
//		DWORD object=STATE_object_get_first_object();
//		STATE_object_get_location(object, &xyz[0], &xyz[1], &xyz[2]);
//
//
//
// See also STATE_object_get_location1()
//
STATE_IMPORT void STATE_WINAPI STATE_object_get_location(DWORD object_handle, double *x, double *y, double *z);

// Returns the location of the object. Exactly the same as the function STATE_object_get_location()
// only that here we use just 1 parameter.
//
// Example:
//		
//		double xyz[3];
//		DWORD object=STATE_object_get_first_object();
//		STATE_object_get_location(object, xyz);
//
// See also STATE_object_get_location()
//
STATE_IMPORT void STATE_WINAPI STATE_object_get_location1(DWORD object_handle, double xyz[3]);

// Advance the object (move relative)
// 
// Parameters:
//		space_flag: could be WORLD_SPACE or OBJECT_SPACE or CAMERA_SPACE
//					space_flag determines if the step is according to the objects
//					coordinate system or the world coordinate  system etc ...
//					For example if space_flag==OBJECT_SPACE and step==(-1,0,0)
//					This means that the object will advance forward (If it is a car than
//					forward is the direction of the front window).
//					Try both options and see what happens.
//
STATE_IMPORT void STATE_WINAPI STATE_object_move(DWORD object_handle, int space_flag, double x, double y, double z);

// Set the direction towards which the object will point .
//
// Example:
//
//		DWORD object=STATE_object_get_first_object();		
//		//Make the object points towards the positive Y axis (pointing right)
//		STATE_object_set_direction(object, 0,1,0);
//
// Remarks:
//		Note that STATE_object_set_direction(object, 10,20,30);
//		has exactly the same result as STATE_object_set_direction(object, 1,2,3);
//
//		For more information about coordination systems and setting directions see the book (download the book free from www.3dstate.com )
//
// See also STATE_object_set_direction1()
//
STATE_IMPORT void STATE_WINAPI STATE_object_set_direction(DWORD object_handle, double x, double y, double z);

// Set the direction towards which the object will point. Exactly the same as the function STATE_object_set_direction()
// only that here we use just 1 parameter.
//
// Example:
//
//		double xyz[3]={10,20,30}; //Note that setting 10,20,30 is the same as 1,2,3 (the engine normalizes the vector (make it a unit vector) )
//		DWORD object=STATE_object_get_first_object();
//		STATE_object_set_direction1(object, xyz);
//
// Remarks:
//
//		For more information about coordination systems and setting directions see the book (download the book free from www.3dstate.com )
// 
// See also STATE_object_set_direction()
//
STATE_IMPORT void STATE_WINAPI STATE_object_set_direction1(DWORD object_handle, double xyz[3]);

// Returns the direction towards which the object points.
//
//
// Example A:
//		
//		double x,y,z;
//		DWORD object=STATE_object_get_first_object();
//		STATE_object_get_direction(object, &x, &y, &z);
//
// Example B:
//
//		//Note that instead of doing it as shown below, it is easier to use the function STATE_object_get_direction1() instead
//		double xyz[3];
//		DWORD object=STATE_object_get_first_object();
//		STATE_object_get_direction(object, &xyz[0], &xyz[1], &xyz[2]);
//
//
//
// See also STATE_object_get_direction1()
//
//
// Remarks:
//
//		The returned direction is normalized (meaning it is a unit vector)
//		For more information about coordination systems and setting directions see the book (download the book free from www.3dstate.com )
//
STATE_IMPORT void STATE_WINAPI STATE_object_get_direction(DWORD object_handle, double *x, double *y, double *z);

// Returns the direction towards which the object points. Exactly the same as the function STATE_object_get_direction()
// only that here we use just 1 parameter.
//
// Example:
//
//		double xyz[3];
//		DWORD object=STATE_object_get_first_object();
//		STATE_object_get_direction1(object, xyz);
//
//
//
// See also STATE_object_get_direction()
//
//
// Remarks:
//
//		The returned direction is normalized (meaning it is a unit vector)
//		For more information about coordination systems and setting directions see the book (download the book free from www.3dstate.com )
//
STATE_IMPORT void STATE_WINAPI STATE_object_get_direction1(DWORD object_handle, double xyz[3]);


// Rotates the object so that the orientation of the object matches the given directions
// returns OK or VR_ERROR
// One of the useful usage of this function together with STATE_object_get_axis_system() and get\set location
// is to save the objact exact position.
// For more explaniton see inside the book "3D programming for windows using STATE"
// One of the given axises could be 0,0,0 in this case this axis is calculated automatically(derived from the other two axises).
// If the given axises are not orthogonal, the function 
// handle it by building an orthogonal system that is close as possible
// to the given axis.
//
// Arguments:
//
//		double x[3], double y[3], double z[3];
//			The X Axis, the Y axis and the Z axis.
//			The given axises do not have to be a unit vector. and one of the axises
//			could be left for the function to calculate by giving(0,0,0)			
//			
//
// Here are some examples, note that
// all the examples give the same result.
// Example A:
//	x_axis[0]=0; x_axis[1]=1; x_axis[2]=0; 
//	y_axis[0]=-1; y_axis[1]=0; y_axis[2]=0; 
//	z_axis[0]=0; z_axis[1]=0; z_axis[2]=1; 
//	STATE_object_set_axis_system(x_axis, y_axis, z_axis);
//
// Example B:
//	//Example B would give exactly the same result as example A
//	x_axis[0]=0; x_axis[1]=111; x_axis[2]=0; 
//	y_axis[0]=-17; y_axis[1]=0; y_axis[2]=0; 
//	z_axis[0]=0; z_axis[1]=0; z_axis[2]=12; 
//	STATE_object_set_axis_system(x_axis, y_axis, z_axis);
//
// Example C:
//	//Note that  z_axis=(0,0,0) and will be automatically 
//  // calculated by the function
//	//Example C would give exactly the same result as example A and B
//	x_axis[0]=0; x_axis[1]=111; x_axis[2]=0; 
//	y_axis[0]=-17; y_axis[1]=0; y_axis[2]=0; 
//	z_axis[0]=0; z_axis[1]=0; z_axis[2]=0; 
//	STATE_object_set_axis_system(x_axis, y_axis, z_axis);
//
// Example D:
//  // Note that z_axis=(0,0,0) and that the x_axis and the y_axis are not orthogonal to each other.
//  //Note that we will get again the same result
//	x_axis[0]=0; x_axis[1]=1; x_axis[2]=0; 
//	y_axis[0]=-1; y_axis[1]=11; y_axis[2]=0; 
//	z_axis[0]=0; z_axis[1]=0; z_axis[2]=0; 
//	STATE_object_set_axis_system(x_axis, y_axis, z_axis);
//
// Example E:
//  // Note that z_axis=(-1,0,0) which doesn't give a right hand system
//	// in this case the z_axis will be ignored and calculated by doing cross between the x_axis and the y_axis
//  //Note that we will get again the same result
//	x_axis[0]=0; x_axis[1]=1; x_axis[2]=0; 
//	y_axis[0]=-1; y_axis[1]=0; y_axis[2]=0; 
//	z_axis[0]=0; z_axis[1]=0; z_axis[2]=-1; 
//	STATE_object_set_axis_system(x_axis, y_axis, z_axis);
STATE_IMPORT int STATE_WINAPI STATE_object_set_axis_system(DWORD object_handle, double x[3], double y[3], double z[3]);

// One of the useful usage of this function together with STATE_object_set_axis_system() and get\set location
// is to save the objact exact position.
//Returns the axis system of the object. For more details please see STATE_object_set_axis_system()
STATE_IMPORT void STATE_WINAPI STATE_object_get_axis_system(DWORD object_handle, double x[3], double y[3], double z[3]);

//Retrieves the X axis of the object. Note the following:   x_axis * -1 == the object direction
//For more details see STATE_object_set_axis_system
STATE_IMPORT void STATE_WINAPI STATE_object_get_x_axis(DWORD object_handle, double x_axis[3]);

//Retrieves the Y axis of the object. 
//For more details see STATE_object_set_axis_system
STATE_IMPORT void STATE_WINAPI STATE_object_get_y_axis(DWORD object_handle, double y_axis[3]);

//Retrieves the Z axis of the object. 
//For more details see STATE_object_set_axis_system
STATE_IMPORT void STATE_WINAPI STATE_object_get_z_axis(DWORD object_handle, double z_axis[3]);



#ifndef WORLD_SPACE
	#define WORLD_SPACE	0
#endif

#define OBJECT_SPACE (WORLD_SPACE+1)

// Turns an object around its center.
// Use STATE_object_set_rotation_center() to change the object center
// space flag can be OBJECT_SPACE or  WORLD_SPACE
//
// Remarks:
//		Note that there are several functions for rotating an object. Here are some of them
//		and help when to use what.
//
//		1) Rotation around the object center
//			If you need to turn/rotate an object around its center (for example a propeller of an helicopter)
//			then use one STATE_object_rotate_x/y/z()
//
//		2) If you need to rotate an object around a point in the object body
//		   for example to turn a door around its hinge then use
//			STATE_object_rotate_x/y/z() together with STATE_object_set_rotation_center()
//
//		3) If you need to turn an object around a point in the world.
//			for example earth circling around the sun.
//			in this case use STATE_object_rotate_around_point()
//
// See also: 
//		STATE_object_rotate_x(), STATE_object_rotate_y(), STATE_object_rotate_z()
//		STATE_object_rotate_x_radians(), STATE_object_rotate_y_radians(), STATE_object_rotate_z_radians()
//		STATE_object_rotate_around_point()
//		STATE_object_set_rotation_center()
//		STATE_object_rotate_including_sons()
//
STATE_IMPORT void STATE_WINAPI STATE_object_rotate_x(DWORD object_handle, double degrees, int space_flag);
STATE_IMPORT void STATE_WINAPI STATE_object_rotate_y(DWORD object_handle, double degrees, int space_flag);
STATE_IMPORT void STATE_WINAPI STATE_object_rotate_z(DWORD object_handle, double degrees, int space_flag);

// The same just with radians.
// space flag can be OBJECT_SPACE or  WORLD_SPACE
STATE_IMPORT void STATE_WINAPI STATE_object_rotate_x_radians(DWORD object_handle, double radians, int space_flag);
STATE_IMPORT void STATE_WINAPI STATE_object_rotate_y_radians(DWORD object_handle, double radians, int space_flag);
STATE_IMPORT void STATE_WINAPI STATE_object_rotate_z_radians(DWORD object_handle, double radians, int space_flag);

// obsolete. Use STATE_object_get_tilt() instead
// The return value is in [-1  -   1] according to the
// angle between the object direction and the world Z axis
// 1 means we are looking up . -1 means down.
// For example if we have an airplane we can use this function to test
// whether the plane is diving, going up or strait (strait would return 0)
STATE_IMPORT double	 STATE_WINAPI STATE_object_get_cos_pitch(DWORD object_handle);

// The return value is in [-90  -   90] according to the
// angle between the object direction and the world Z axis
// 90 means we are looking up . -90 means down.
// For example if we have an airplane we can use this function to test
// whether the plane is diving, going up or strait (strait would return 0)
STATE_IMPORT double	 STATE_WINAPI STATE_object_get_tilt(DWORD object_handle);
STATE_IMPORT void STATE_WINAPI STATE_object_set_tilt(DWORD object_handle, double tilt_angle);



// The return value is in [-1  -   1] according to the
// angle between the object Y axis and the world Z axis
// If our object is a plane then this function will return the
// cos() of the angle between the plane wing and the world Z axis
// 0 means the object is strait. 
// 1 means one wing is pointing up to the sky and the other wing into the ground.
// -1 means that the other wing is pointing up to the sky and the first wing into the ground.
STATE_IMPORT double STATE_WINAPI STATE_object_get_cos_bank(DWORD object_handle);

//The same as STATE_object_get_cos_bank() just that the return value is in degrees
// 0 means the object is strait. 
// 90 means one wing is pointing up to the sky and the other wing into the ground.
// -90 means that the other wing is pointing up to the sky and the first wing into the ground.
STATE_IMPORT double STATE_WINAPI STATE_object_get_bank(DWORD object_handle);

// Set the bank angle
// 0 means the object is strait. 
// 90 means one wing is pointing up to the sky and the other wing into the ground.
// -90 means that the other wing is pointing up to the sky and the first wing into the ground.
STATE_IMPORT void STATE_WINAPI STATE_object_set_bank(DWORD object_handle, double bank_angle);

// Used to check if the object is heading North
// south east west etc ...
STATE_IMPORT double	 STATE_WINAPI STATE_object_get_cos_head(DWORD object_handle);

//obsolete. Use STATE_object_get_transformation_matrix() instead.
//STATE_IMPORT void STATE_WINAPI STATE_object_get_total_move_mat(DWORD object_handle, double object_matrix[4][4]);

// Who needs this function ?
// --------------------------
//	Users that are porting code from other 3D engines might want to use this function.
//	All the rest can skip this function
//
// General background
// ------------------
//	Please note that when moving/turning an object the STATE engine never modifies the X,Y,Z values of the points of the polygons of a dynamic object.
//	When the object is rotated scaled or moved, the engine only modify the object matrix.
//	Not modifing all the object points saves a lot of time.
//
// What is this matrix ?
// ---------------------
//	To learn more about transformation matrixes you should read a computer graphics math book.
//  This is just if you are interested in the subject. One of the great things about the STATE engine
//	is that it makes it completely unnecessary to learn these subjects.
//	Anyhow, since you are still reading it means that you want to know more, so here it is:
//		the first top 9 cells of the matrix are the rotation matrix
//		The bottom row is the translation values.
//		The diagnal is the scale values
//
//		S R R 0
//		R S R 0
//		R R S 0
//		T T T 1
//
//		Above is the matrix. T stands for translation. S for scale. R for rotation.
//		If you don't know what we are talking about then you should read a book in the subject
//
//		By taking a point in object space and multiplying it by the matrix we get the point in World Space.
//		for example the point (0,0,0) in object space is the center of the object.
//		when multiplying (0,0,0) by the matrix we get (T T T) which means that
//		(T,T,T) is the object location
//		The code below shows how to multiply a point (vec) by the matrix (mat).
//		The result is saved in dst.
//
//		void 	mul_vec_mat(vector	dst, vector vec, matrix mat) //does dst=vec*mat
//		{
//		  double	res[4];
//		  double	one_over_res3;
//
//
//			res[0]=vec[0]*mat[0][0] + vec[1]*mat[1][0] + vec[2]*mat[2][0] + mat[3][0];
//			res[1]=vec[0]*mat[0][1] + vec[1]*mat[1][1] + vec[2]*mat[2][1] + mat[3][1];
//			res[2]=vec[0]*mat[0][2] + vec[1]*mat[1][2] + vec[2]*mat[2][2] + mat[3][2];
//			res[3]=vec[0]*mat[0][3] + vec[1]*mat[1][3] + vec[2]*mat[2][3] + mat[3][3];
//
//			if(res[3]<1e-20 && res[3]>-1e-20) { //protect agains floating point errors.
//				ERROR_MSG(SCREEN,"mul_vec_mat: devision by zero\n");
//		
//				dst[0]=res[0];
//				dst[1]=res[1];
//				dst[2]=res[2];
//				return;
//			}
//
//			one_over_res3=1/res[3];
//
//			dst[0]=res[0]*one_over_res3;
//			dst[1]=res[1]*one_over_res3;
//			dst[2]=res[2]*one_over_res3;
//
//		}
//
// Example:
//
//		double matrix[4][4];
//		STATE_object_get_transformation_matrix(my_object, matrix);
//
STATE_IMPORT void STATE_WINAPI STATE_object_get_transformation_matrix(DWORD object_handle, double object_matrix[4][4]);

//Set the transformation matrix. For more details see STATE_object_get_transformation_matrix()
//Note that it also rest the scaling so if STATE_object_set_scale() was used then it has to be called again.
STATE_IMPORT void STATE_WINAPI STATE_object_set_transformation_matrix(DWORD object_handle, double object_matrix[4][4]);



// returns a handle to the an animation that belongs to the object
// If the object has no animations or it has more then one the function
// returns NULL
STATE_IMPORT DWORD STATE_WINAPI STATE_object_get_animation(DWORD object_handle);


// Returns OK or VR_ERROR
// The function search the given object for a polygon that has an animation with the name old_animation_name
// The function then replaces the animation for this specifc polygon. 
// This polygon is set with an animation that has the name new_animation_name
// If the given object has more than one polygon with the animation old_animation_name
// then the function will return VR_ERROR
// See also STATE_object_replace_animation()
// 
// Arguments:
//		object_handle  
//			The object handle.
//		old_animation_name
//			The name of the animation to be replaced.
//		new_animation_name
//			The name of the animation to to be set instead of old_animation_name.
//
STATE_IMPORT int STATE_WINAPI STATE_object_replace_animation_using_names(DWORD object_handle, char *old_animation_name ,char *new_animation_name);

// Returns OK or VR_ERROR
// Exactly like STATE_object_replace_animation_using_names() only that instead of getting the names of the animation
// it receives the handles (faster)
// For more details see STATE_object_replace_animation_using_names()
// Arguments:
//		object_handle  
//			The object handle.
//		old_animation_handle
//			The handle to the animation to be replaced.
//		new_animation_handle
//			The handle to the animation to to be set instead of old_animation_name.
//
//
STATE_IMPORT int STATE_WINAPI STATE_object_replace_animation(DWORD object_handle, DWORD old_animation_handle ,DWORD new_animation_handle);


//Converts a point in object space to a point in world space.
// Returns OK or VR_ERROR if failed
//Here are some examples:
// The location of the object (world space) will always translate to (0,0,0) in object space
// (-1,0,0) in object space is one unit in front of the object this is equal to
// the object location plus the object direction (a unit vector) in object space.
// (1,0,0) in object space is one unit in the back of the object
// (0,1,0) in object space is always one unit to the right of the object.
// (0,-1,0) in object space is always one unit to the left of the object.
// (0,0,1) in object space is always one unit up above the object.
//See also STATE_object_convert_point_to_object_space()
STATE_IMPORT int STATE_WINAPI STATE_object_convert_point_to_world_space(DWORD object_handle, double object_space_point[3], double result[3]);

//Converts a point in world space to a point in object space.
// Returns OK or VR_ERROR if failed
//Here are some examples:
// The point (0,0,0) in object space will always translate to the location of the object in world space
// (-1,0,0) in object space is one unit in front of the object this is equal to
// the object location plus the object direction (a unit vector) in object space.
// (1,0,0) in object space is one unit in the back of the object
// (0,1,0) in object space is always one unit to the right of the object.
// (0,-1,0) in object space is always one unit to the left of the object.
// (0,0,1) in object space is always one unit up above the object.
//See also STATE_object_convert_point_to_world_space()
STATE_IMPORT int STATE_WINAPI STATE_object_convert_point_to_object_space(DWORD object_handle, double world_space_point[3], double result[3]);


// **************************

// returns VR_ERROR, OK

// if track_handle ==NULL it doesn't cancel the chase type so
// for canceling chasing use STATE_object_set_chase_type(NO_CHASE);
// Automatically set the closest point on the track (closest to the object)
// as the next point to go towards
// (to override this setting use STATE_object_set_next_point_on_track() )
STATE_IMPORT int STATE_WINAPI STATE_object_set_track(DWORD object_handle, DWORD track_handle);

// return OK or VR_ERROR
// the offset is return through argument "offset"
STATE_IMPORT int STATE_WINAPI STATE_object_get_track_offset(DWORD object_handle, double offset[3]);

// return OK or VR_ERROR
STATE_IMPORT int STATE_WINAPI STATE_object_set_track_offset(DWORD object_handle, double offset[3]);

// returns an index to a point on the track. This point is the point that the object is moving towards
// The exact cords of the point can be retrieved through STATE_track_get_point()
// 0 is the first point and number_of_points-1 is the last index (use STATE_track_get_number_of_points() to get this number
STATE_IMPORT int STATE_WINAPI STATE_object_get_next_point_on_track(DWORD object_handle);

// Setss the point on the track that the object is moving towards
// 0 is the first point. number_of_points-1 is the last index (use STATE_track_get_number_of_points() to get this number
// Returns OK or VR_ERROR.
// Important: note that calling the functions STATE_object_set_chase_type() or STATE_object_set_track()
//			  will initialize the target point, so call STATE_object_set_next_point_on_track() after these functions (if are used )
//			  and not before.
STATE_IMPORT int STATE_WINAPI STATE_object_set_next_point_on_track(DWORD object_handle, int point_index);

// Set the object to chase after
// the given argument should NOT be NULL
// Note that if the object was previously chasing a camera or a group
// , those chasing operation are canceled. (The object can chase only one thing at a time).
STATE_IMPORT void STATE_WINAPI STATE_object_set_object_to_chase(DWORD object_handle, DWORD object_to_chase_handle);

// If the return value is NULL it means no object is being chased
STATE_IMPORT DWORD STATE_WINAPI STATE_object_get_chased_object(DWORD object_handle);

// Set the camera to chase after
// the given argument should NOT be NULL
// Note that if the object was previously chasing another object or a group
// , those chasing operation are canceled. (The object can chase only one thing at a time).
STATE_IMPORT void STATE_WINAPI STATE_object_set_camera_to_chase(DWORD object_handle, DWORD camera_to_chase_handle);

// If the return value is NULL it means no camera is being chased
STATE_IMPORT DWORD STATE_WINAPI STATE_object_get_chased_camera(DWORD object_handle);

// Set the group to chase after
// the given argument should NOT be NULL
// Note that if the object was previously chasing a camera or an object
// , those chasing operation are canceled. (The object can chase only one thing at a time).
STATE_IMPORT void STATE_WINAPI STATE_object_set_group_to_chase(DWORD object_handle, DWORD group_to_chase_handle);

// If the return value is NULL it means no group is being chased
STATE_IMPORT DWORD STATE_WINAPI STATE_object_get_chased_group(DWORD object_handle);


// return OK or VR_ERROR
// the offset is return through argument "offset"
STATE_IMPORT int STATE_WINAPI STATE_object_get_chase_offset(DWORD object_handle, double offset[3]);

// return OK or VR_ERROR
STATE_IMPORT int STATE_WINAPI STATE_object_set_chase_offset(DWORD object_handle, double offset[3]);

// The returned number is between 0 and one
// a number close to 0 means that the tracking is weak
// 1 means that the tracking is very sharp.
STATE_IMPORT double STATE_WINAPI STATE_object_get_chase_softness(DWORD object_handle);


// The returned number is between 0 and one
// a number close to 0 means that the tracking is weak
// 1 means that the tracking is very sharp.
// it the given number is bigger than one it is set as one
// if it is smaller than 0 than it set as 0
STATE_IMPORT void STATE_WINAPI STATE_object_set_chase_softness(DWORD object_handle, double softness);

// look in the header file for possible values and their meanings
// (look for NO_CHASE, CHASE_LOCATION and others ...)
STATE_IMPORT int STATE_WINAPI STATE_object_get_chase_type(DWORD object_handle);

// look at the top of the header file for possible values and their meanings
// (look for NO_CHASE, CHASE_LOCATION and others ...)
STATE_IMPORT void STATE_WINAPI STATE_object_set_chase_type(DWORD object_handle, int chase_type);


/*
	Function for automatic falling from a track.
	--------------------------------------------
	Example of usage:
	Say we have a track for a runner on a road, if the track is not 100 percent precise
	meaning that some time the runner will be in the air and some times inside the ground.
	With these functions we can solve that problem.
	We give an offset for the track in the Z axis 
	so the track will be above the road. Now if "falling"
	is ON, the object will fall down till it reaches the
	the track.
	Another example:
	If we want to move the object on a terrain model
	very close to the ground. It will be very difficult to 
	calculate the exact track. Now we can define a very simple
	track like a basic square and "falling" will do the rest.

	NOTE: to use it directly from the wld file, add _FALLxx to the
	chase_type. for example CHASE_TRACK_AIRPLANE would become CHASE_TRACK_AIRPLANE_FALL17
	17 here is the height above ground that we want to fall.
*/

// Calling this function will make the object fall down from
// its track till it reaches "height_above_ground" distance from the ground
// Here is an example for an ocasion that makes this function very useful:
//		If we have a terrain and we want a runner to run on a square track on this terrain then using
//		this function we can use a track with only 4 points (a square) that will be up in the air.
//		The engine will automatically place our runner at the right spot on the terrain.
// fall_through_dynamic_objects could be YES or NO. The idea here is that if for example we have a runner (human being)
// and the track is very high in the air then it might fall on the top of the airplanes so we would like it to fall through the planes on the ground
// if on the other hand there are cars on the ground we would like it to fall on the top of the cars and not to pass through.
// See also STATE_object_get_location_on_track_before_falling()
STATE_IMPORT void STATE_WINAPI STATE_object_set_falling_from_track(DWORD object_handle, double	height_above_ground, int fall_through_dynamic_objects);


//Calling this function would cancel previous calls to STATE_object_set_falling_from_track()
//See also STATE_object_set_falling_from_track()
STATE_IMPORT void STATE_WINAPI STATE_object_unset_falling_from_track(DWORD object_handle);

// Returns the current status in the three params: set_flag, height_above_ground, fall_through_dynamic_objects
// set_flag could be YES or NO 
// fall_through_dynamic_objects could be YES or NO . See STATE_object_set_falling_from_track() for more details
STATE_IMPORT void STATE_WINAPI STATE_object_get_falling_from_track_params(DWORD object_handle, int *set_flag, double	*height_above_ground, int *fall_through_dynamic_objects);


//This function is relevant only when STATE_object_set_falling_from_track() is used.
// It is not a very important function though if you are using STATE_object_set_falling_from_track()
//you might want to take a look at it.
//Lets imagine that we have a world that has some mountains. Our object is a car that 
//is following a square track in the air. We tell the engine by using STATE_object_set_falling_from_track()
//to drop the car from the air to the ground so that the car drive in a square path on the terrain.
//We can use this function to find out that the location of the object before the object was dropped.
//See also STATE_object_set_falling_from_track()
STATE_IMPORT void STATE_WINAPI STATE_object_get_location_on_track_before_falling(DWORD object_handle, double location[3]);


//Relevant only when CHASE_PHYSICS is used.
//Set the speed of the object.
//	Example
//			//Give the object speed of 10 in the positive direction of the Y axis
//			STATE_object_set_speed(object, 0,10,0);
//
// See also STATE_object_set_speed_units()
//
STATE_IMPORT void STATE_WINAPI STATE_object_set_speed(DWORD object_handle, double speed[3]);
STATE_IMPORT void STATE_WINAPI STATE_object_get_speed(DWORD object_handle, double speed[3]);


//Returns the size of the speed.
//Using this function we can only retrieve the magnitude of the speed but not its direction.
//For example if the function returns 10, it means that the speed of the object is 10.
//Q: "speed of ten" to which direction ?
//A: To know the direction of the speed call STATE_object_get_speed()
//
STATE_IMPORT double STATE_WINAPI STATE_object_get_absolute_speed(DWORD object_handle);

//Relevant only when CHASE_PHYSICS or CHASE_TRACK is used.
//Set the size of the speed.
//
// Read the example below to learn how it all works. 
//
// Example:
//		STATE_object_set_speed(object, 0,10,0);
//		speed_size=STATE_object_get_absolute_speed(object);
//		//speed_size is now 10
//
//		STATE_object_set_absolute_speed(object, 100);
//
//		double speed[3];
//		STATE_object_get_speed(object, speed);
//		//speed is now { 0, 100, 0}
//
//
// Advanced:
//	In physics speed is a vector it has a size and a direction
//	Q: If the speed vector is 6,8,0 ( STATE_object_set_speed(object, 6,8,0) );
//	   what is the size of the speed (what will STATE_object_get_absolute_speed(object) return ?)
//
//
//	A:	Let calculate the vector size according to Pythagoras Theorem
//		speed size=  Square root of ( A*A+ B*B+ C*C ) = square root of ( 6*6 + 8*8 + 0*0) = Square root of (100) == 10
//
//	
STATE_IMPORT void STATE_WINAPI STATE_object_set_absolute_speed(DWORD object_handle, double speed);


// Set the force that will work on the object. For example to simulate gravity force
// do like that:
//		double force[3];
//		force[0]=0;
//		force[1]=0;
//		force[2]= -10; //simulate gravity 10 in the direction of the -Z axis.
//		STATE_object_set_force(my_object,force);
//
//	Note that this will only relevant when CHASE_PHYSICS is applyed
STATE_IMPORT void STATE_WINAPI STATE_object_set_force(DWORD object_handle, double force[3]);
STATE_IMPORT void STATE_WINAPI STATE_object_get_force(DWORD object_handle, double force[3]);

//Similar to STATE_object_set\get_absolute_speed() only that it works on the size of the force vector
STATE_IMPORT double STATE_WINAPI STATE_object_get_absolute_force(DWORD object_handle);
STATE_IMPORT void STATE_WINAPI STATE_object_set_absolute_force(DWORD object_handle, double force_size);


//This function doesn't change the size of the speed but only changes the speed direction according to the given value
//Relevant only for CHASE_PHYSICS
//
// Why do we need this function
// ----------------------------
//
// Consider the following example: You use CHASE_PHYSICS for moving a car \ missile \ bullet or whatever.
// now you want to change the direction of the missile. If you call STATE_object_set_direction()
// or STATE_object_rotate_z() it will only change orientation of the car \ missile model.
// It is not going to change the direction in which it is heading. For example if your
// car turns in 90 degrees it will continue going in the same direction as before according to the car speed vector
// (only that after you turn it, it will look as if the car is sliding sideways)
// For these cases we actually want to do several things. To turn the car (shapewize) but
// also to turn the direction in which it is heading. We might also want to change the 
// direction of the force (simulating the car engine). The car engine pushes the car
// forward in the direction of the car so if our object is a car or a missile we would like
// to also adjust the force direction according to the new direction. If the source of force
// is external (gravity force for example) then we would not want to change the direction of the force
// since gravity is not influenced by the car turning ...
//
// Example A:
//			
//			double direction[3];
//			STATE_object_get_direction(obj, &direction[0], &direction[1], &direction[2]);
//			//set the direction of movement to the direction of the object
//			STATE_object_set_speed_direction(obj, direction);
//
// Example B:
//
//			//STATE_object_set_speed_direction() code is equivalent to the following code
//			STATE_object_set_speed_direction(DWORD object_handle, double direction[3])
//			{
//				double save_speed_size=STATE_object_get_absolute_speed(object_handle);
//				STATE_object_set_speed(object_handle, direction);
//				//restore speed size
//				STATE_object_set_absolute_speed(object_handle, save_speed_size);
//			}
//
//		
STATE_IMPORT void STATE_WINAPI STATE_object_set_speed_direction(DWORD object_handle, double direction[3]);


//This function doesn't change the size of the force but only changes the force direction according to the given value
//Relevant only for CHASE_PHYSICS
//Similar to STATE_object_set_speed_direction() only that it works on the force.
//For more details see STATE_object_set_speed_direction()
STATE_IMPORT void STATE_WINAPI STATE_object_set_force_direction(DWORD object_handle, double direction[3]);


//Limits the maximum possible speed of an object.
//If there is no limit then by applying force on the object 
//The speed could reach infinity (unless some friction is given too)
//Note that this will only relevant when CHASE_PHYSICS is applyed
STATE_IMPORT void STATE_WINAPI STATE_object_set_max_speed(DWORD object_handle, double value);
STATE_IMPORT double STATE_WINAPI STATE_object_get_max_speed(DWORD object_handle);


//Friction is a value smaller or equal to 1
// if friction==0 it means no friction
// if friction==1 it means infinite friction nothing can move
// if friction is smaller than 0, it will actually cause accelleration and not friction
// The exact calculation.
// Each render the absolute_speed is multiplied by (1-friction).
//
// Note, a negative number can use for increasing the speed.
// a friction of -1 will double the speed.
// -2 will multiply the speed by three each rendering.
// so choose a number carefully.
// typical numbers can be 0.03 or -0.03 etc ...
STATE_IMPORT void STATE_WINAPI STATE_object_set_friction(DWORD object_handle, double value);
STATE_IMPORT double STATE_WINAPI STATE_object_get_friction(DWORD object_handle);

//This number is used when the object hits a polygon. 
//A value of one will make the new speed (after collision)
//equal to the old speed (effects only the absolute speed value and not the direction)
//a value of 0.5 will cut speed by half. 
//We can also give values greater than one.
//For example a value of 2 will make the new speed twice as big
//as before the collision.
STATE_IMPORT void STATE_WINAPI STATE_object_set_elasticity(DWORD object_handle, double value);
STATE_IMPORT double STATE_WINAPI STATE_object_get_elasticity(DWORD object_handle);

// advance all the objects. If STATE_engine_advance_objects_automatically(YES) is called
// then this function will be called automatically each time we do render
// Note works only in viewer mode (viewer mode is the default mode)
STATE_IMPORT void STATE_WINAPI STATE_object_advance_all();

// Moves it according to the chase type.
// Note works only in viewer mode (viewer mode is the default mode)
STATE_IMPORT void STATE_WINAPI STATE_object_advance(DWORD object_handle);


//This function is probably useless for 99.9999% of our users.
//You will need it only if you choose to write your own version STATE_engine_render()
//The 3D_animation is updated automatically according to need each time that STATE_engine_render()
//is called and the given object is seen. If one doesn't call STATE_engine_render()
//(because one chooses to use his own renderer) then the 3D_animation frame is not updated.
// Only in this case you should call this function so that your external renderer can benefit from
// the 3D_animation API.
STATE_IMPORT void STATE_WINAPI STATE_object_update_3D_animation_frame(DWORD object_handle);

//When the engine moves dynamic objects according to their chase type it also monitor
//the distance that these object have passed. Calling this function will reset this counter.
//Please note that the counter is not updated when calling functions such as
//STATE_object_set_location() , STATE_object_move(). The counter is only updated when
//the object is automatically moved by the engine according to its chase type.
STATE_IMPORT void STATE_WINAPI STATE_object_reset_distance_counter(DWORD object_handle);


//Returns the distance that the given object has passed. For more details please see
//STATE_object_reset_distance_counter()
STATE_IMPORT double STATE_WINAPI STATE_object_get_distance_counter(DWORD object_handle);

// The function returns the object bounding box.
// The function will also take into account all the rotation and 
// scaling of the object.
// The function is very fast so feel free to call it often.
//
// parameters:
//
//		box
//			Used to return the object bounding box	
//			box[0][0] is the minimum X
//			box[0][1] is the minimum Y
//			box[0][2] is the minimum Z
//			box[1][0] is the max X 
//			box[1][1] is the max Y
//			box[1][2] is the max Z
//
// See also: STATE_object_get_dimensions(), STATE_object_set_dimensions()
//
// Example:
//		
//		double box[2][3];
//		STATE_object_get_bounding_box(my_obj, box);
//
//
STATE_IMPORT void STATE_WINAPI STATE_object_get_bounding_box(DWORD object_handle, double box[2][3]);

// Retrieves the width , the length and the height of the given object.
// The "length" is computed as the length of the object along the X axis
// The "width" is computed as the length of the object along the y axis
// The "height" is computed as the length of the object along the Z axis
//
// The difference between this function and STATE_object_get_bounding_box()
// is that that the values returend from this function are not going to change
// when you move the object or turn it around in 90 degrees (or any number of degrees).
// The width , length and height values are properties of the object and are 
// set according to the object data located in the world file or the object file.
// In run time the only function that changes these properties is STATE_object_set_scale()
//
// Q: If I have a car object in my world, what is consider the width and what is the length ?
// A: It depends on how the car model is built or in other words the information of the car that is inside
//	  the world file. If the car is built so that its back is towards the +X axis and its side is towards the Y axis
//	  then the width return by this function will be the distance from one door to the other
//	  and the length will be from the front of the car till the end (as you might expect)
//	  If on the other hand the car is built so that the front window is towards the Y axis
//	  than the width returned is actually the length and vice versa.
//	  Note that you have an option in WorldBuilder to save all objects so that they are
//	  facing the -X axis. This way the width and length get to be what you expect them to be.
//
// Parameters:
//		[IN] object_handle:
//			The handle to the object
//
//		[OUT] length:
//			The length of the object along the X axis according to the initial state of the object
//			as defined in the world file (or the object file). If STATE_object_set_scale()
//			is used then the length returned by this function will be accordingly adjusted (according to scale[1])
//
//		[OUT] width:
//			The length of the object along the Y axis according to the initial state of the object
//			as defined in the world file (or the object file). If STATE_object_set_scale()
//			is used then the width returned by this function will be will be adjusted accordingly (according to scale[1])
//
//		[OUT] height:
//			The length of the object along the Z axis according to the initial state of the object
//			as defined in the world file (or the object file). If STATE_object_set_scale()
//			is used then the height returned by this function will be will be adjusted accordingly (according to scale[2])
//
// Example:
//
//		double scale[3]={1,1,1};
//		STATEobject_set_scale(my_object, scale); //reset the scaling so that the dimensions received will not be factored
//		double length, width, height;
//		STATE_object_get_dimensions(my_object, &length, &width, &height);
//
// See also: STATE_object_set_dimensions(), STATE_object_get_bounding_box()
//
STATE_IMPORT void STATE_WINAPI STATE_object_get_dimensions(DWORD object_handle, double *length, double *width, double *height);

// Set the width, length and height of the given object.
// Note that this function uses STATE_object_set_scale() in order to change the 
// dimensions of the object.
// For more details please see STATE_object_get_dimensions()
//
// See also: STATE_object_set_scale(), STATE_object_get_dimensions(), STATE_object_get_bounding_box()
//
// Example:
//		STATE_object_set_dimensions(my_object, 10,20,30);
//
STATE_IMPORT void STATE_WINAPI STATE_object_set_dimensions(DWORD object_handle, double length, double width, double height);



//Create a copy of a given object
//Returns a handle to the created object
//If an error occurred it will return NULL
// duplicate_polygons_flag can be YES or NO.
// if NO is given then the source and the new copy will
// use the same polygons. That means that if we will change the bitmap or the color
// of a polygon it will effect all the copies too.
// If the object is big then duplicating all its polygons can be time consuming,
// That's why we might prefer to set duplicate_polygons_flag to NO.
// Note also that if the polygons are not really duplicated then many
// functions that receive a polygon will fail.
// For example if we call STATE_polygon_get_location()
// and we send a polygon that belongs to a copy of an object. 
// then since we didnt duplicate the object's polygons there is only
// one polygon for both objects so in this case the function will return
// the location of the polygon from the original object.
// The Same story is with functions like STATE_polygon_add_shadow_patch()
STATE_IMPORT DWORD STATE_WINAPI STATE_object_duplicate(DWORD object_handle, int duplicate_polygons_flag);

// *** Not too useful.
// Returns YES or NO.
// YES if this object was created STATE_object_duplicate( .., YES)
STATE_IMPORT int STATE_WINAPI STATE_object_was_duplicated_with_polygons(DWORD object_handle);


//Note that after doing delete you cant use the handle any more
//so good practice is to do the following:
//	STATE_object_delete(obj_handle);
//	obj_handle=NULL;
STATE_IMPORT void STATE_WINAPI STATE_object_delete(DWORD object_handle);

#define STATE_DELETE 1
#define STATE_DISABLE 2

// This function set a timer and an event that should happen after
// the time has elapsed.
// There only two events that could be used.
// STATE_DELETE and STATE_DISABLE.
// If STATE_DELETE is chosen then the object 
// will be automatically deleted after the time has elapsed
// If STATE_DISABLE is chosen then the object 
// will be automatically disabled after the time has elapsed
// The time is in milliseconds 
// Setting the timer to 0 will cancel any going event for this object.
// Note that after doing delete you cant use the handle any more
// so good practice is to do the following:
//	STATE_object_set_event(obj_handle,1000, STATE_DELETE);
//	obj_handle=NULL;
//  
//  Arguments:
//		object_handle: The object that should be associated with the event.
//
//		time_in_milliseconds: How long till the event will take place
//
//		event: STATE_DELETE or STATE_DISABLE. See remarks.
//		
//
// examples:
// A) 
//	  Setting the timer to two seconds
//	  STATE_object_set_event(obj_handle, 2000, STATE_DISABLE);
// B)
//	  Canceling the timer
//	  STATE_object_set_event(obj_handle, 0, any_number_will_do);
STATE_IMPORT void STATE_WINAPI STATE_object_set_event(DWORD object_handle, int time_in_milliseconds, int event);

//This is a very usefull function.
//The main useage of this function is to delete objects automatically after 
// a 2D animation on a specific polygon has ended (explosion effect animation for example)
//Example: 
	//After shooting down a helicoter we would like to have an explosion
	//to do that we do the following:
	//1) Using WorldBuilder\Webmaker we create an object that contains usually one polygon with explosion\fire animation
	//2) At the beginning of the program we disable this object so it is not seen
	//3) When we need to create an explosion we do the following:
	//	  - We duplicate the explosion effect object (using STATE_object_duplicate() )
	//	  - We move the new copy to the location where we need the effect ( STATE_object_get_location() and STATE_object_set_location() ) make sure that the copy is enabled
	//	  - And here comes our function. We set an event using STATE_object_set_event_on_animation_frame()
	//		to delete the object automatically once the animation is over.

// This function is similar to STATE_object_set_event() only that the event is triggered
// by an animation frame and not by time.
// When the animation on the given polygon will finish showing the given frame.
// The event will take place.
// Note that the given polygon must belong to the given object
// There only two events that could be used.
// STATE_DELETE and STATE_DISABLE.
// Example for usage: Lets say we have an explosion and now we have created
// an object for showing the smoke, the smoke will be an animation on a polygon.
// the animation will show smoke evaporating. using this function we can automatically delete
// the object once the animation is finished
// If animation_frame==-1 it has a special meaning, it means expire after the last frame.
// Note that after doing delete you cant use the handle any more
// so good practice is to do the following:
//	STATE_object_set_event_on_animation_frame(obj_handle, polygon_handle, animation_frame, STATE_DELETE);
//	obj_handle=NULL;
//  Arguments:
//		object_handle: The object that should be associated with the event.
//
//		polygon_handle: A polygon of the object that has an animation
//
//		animation_frame: The animation frame number that should trigger the event.
//
//		event: STATE_DELETE or STATE_DISABLE. See remarks.
STATE_IMPORT void STATE_WINAPI STATE_object_set_event_on_animation_frame(DWORD object_handle, DWORD polygon_handle, int animation_frame, int event);


//Objects that are disabled are not drawn and are not
//taking CPU time (they also dont take part in collision detection).
STATE_IMPORT void STATE_WINAPI STATE_object_disable(DWORD object_handle);

//Enables the given object.
//See STATE_object_disable() and STATE_object_is_enabled()
STATE_IMPORT void STATE_WINAPI STATE_object_enable(DWORD object_handle);

//Returns YES or NO.
//Returns whether this object is enabled or disabled.
//Objects that are disabled are not drawn and are not
//taking CPU time (they also dont take part in collision detection).
//By default all objects are enabled.
//See also STATE_object_disable(), STATE_object_enable()
STATE_IMPORT int STATE_WINAPI STATE_object_is_enabled(DWORD object_handle);

#define UNITS_PER_HUNDREDTH_OF_A_SECOND 1
#define UNITS_PER_RENDER 2

// This function is only relevant for object that have their chase type set to
// CHASE_TRAKE or CHASE_TRACK_AIRPLANE or CHASE_PHYSICS (relevant for CHASE_PHYSICS
// only when collison detection for chase physics is turned off. See STATE_object_cancel_collision_test_for_chase_physics() for more details)
//
// When objects are set to go according to a specific track
// they have speed. We have two ways two work with the speed. The given speed could be render cycle related or
// time related. The default is according to time (UNITS_PER_HUNDREDTH_OF_A_SECOND).
// We recommend not to use the default and to call this function after loading the world with UNITS_PER_RENDER
// (example STATE_object_set_speed_units(UNITS_PER_RENDER) );
// If you want to make your world based on time you should use STATE_engine_get_computer_speed_factor() instead.
//
// The reason why we recommend not using UNITS_PER_HUNDREDTH_OF_A_SECOND is because
// when using UNITS_PER_HUNDREDTH_OF_A_SECOND the engine checks the Windows system clock every render
// in order to calculate the correct advancement that should be applied. The problem is that 
// the Windows system clock might not be updated as frequently as FPS of the application.
// For example if there are 100 frames per second in your game and the system clock is updated
// only ten times a second, the result will be that the object will move only in every tenth of a second ...
// To solve this you should alaways work according to render cycles but in order to make your
// program run the same on every computer you should use STATE_engine_get_computer_speed_factor()
// at the beginning of your program to determine the computer speed.
//
//
// Parameters:
// 
//1-  UNITS_PER_HUNDREDTH_OF_A_SECOND
//	  This is the default mode. The advantage of this mode is that
//	  objects will move at the same speed no matter what is the size of the world
//	  or the strength of the computer. On slower computers the objects will simply
//	  move in bigger steps thus make the movement less smooth.
//	  For example an object that has an absolute speed of 10 will 
//	  move in a speed of 10 units per 1 hundredth of a second which means
//	  1000 virtual units in a second.
//

//2-	
//	UNITS_PER_RENDER
//	Each render the object is advance by the speed, notice that we will
//	get a different speed for slow computers than on fast ones, though
//  the steps that the object is doing each render will always be equal.
//
//
//  The default is UNITS_PER_HUNDREDTH_OF_A_SECOND.
//
//
//  
//
//	Remarks:
//		* The speed_units_system is always set for all the objects.
//
//	Question: What is the length of one virtual unit ?
//	Answer:	  This depends on your world. For example lets say that your world
//			  is just one big house. When you build this house in a 3D editor
//			  you can make the length of the house 1 million units or maybe just 100 units
//			  it is up to you. You decide what is the length of one unit inside your world.
//			  The engine doesn't care whether you have chosen to build your world so that one unit is
//			  equal to one meter, one centimeter or one inch.
//
STATE_IMPORT void STATE_WINAPI STATE_object_set_speed_units(int speed_units_system);
STATE_IMPORT int STATE_WINAPI STATE_object_get_speed_units(void);


//Returns a handle to a polygon that belongs to the object and has
//the given name. If polygons_name==NULL, it will return a random polygon from the object.
//If the return value is NULL, it means that no polygon with the given name belongs
//to the object.
STATE_IMPORT DWORD STATE_WINAPI STATE_object_get_polygon(DWORD object_handle, char *polygons_name);

//Returns YES if the given polygon is part of the given object
// NO otherwise.
STATE_IMPORT int STATE_WINAPI STATE_object_is_polygon_part_of(DWORD object_handle, DWORD polygon_handle);

STATE_IMPORT int STATE_WINAPI STATE_object_get_number_of_polygons(DWORD object_handle);

// Returns a handle to the object first polygon
// Used in conjunction with STATE_object_get_next_polygon()
// to traverse all the object polygons
// example:
//		for(DWORD poly=STATE_object_get_first_polygon(obj); poly!=NULL; poly=STATE_object_get_next_polygon(obj, poly))
//		{
//			//do something with the polygon. For rxsmple lets check its name
//			char *name=STATE_polygon_get_name(poly);
//			...
//		}
//
// Note that this 2 functions wont work if the flag CREATE_BSP_FOR_DYNAMIC_OBJECTS 
// is given when calling STATE_engine_load_world() (you will probably never use this flag ...)
// In this rare case use the function STATE_object_get_all_polygons() instead
STATE_IMPORT DWORD STATE_WINAPI STATE_object_get_first_polygon(DWORD object_handle);
STATE_IMPORT DWORD STATE_WINAPI STATE_object_get_next_polygon(DWORD object_handle, DWORD polygon_handle);


//Another way of looping through all an object polygon is by using the FOREACH_OBJECT_POLYGON macro.
// Example:
//		FOREACH_OBJECT_POLYGON(obj, poly) {
//			//do something with the polygon. For rxsmple lets check its name
//			char *name=STATE_polygon_get_name(poly);
//		}
// Note that this 2 functions wont work if the flag CREATE_BSP_FOR_DYNAMIC_OBJECTS 
// is given when calling STATE_engine_load_world() (you will probably never use this flag ...)
// In this rare case use the function STATE_object_get_all_polygons() instead
#define FOREACH_OBJECT_POLYGON(obj, poly) for(DWORD poly=STATE_object_get_first_polygon(obj); poly!=NULL; poly=STATE_object_get_next_polygon(obj, poly))



// polygons_array[] is an array of DWORD. This array will be filled with handles to
// all the polygons of the object. 
// NOTE: you must allocate memory for the array before calling this function
// Example: 
//		int num_of_polys=STATE_object_get_number_of_polygons(obj_handle);
//		// allocating mem using new or malloc etc ...
//		DWORD *poly_array=(DWORD *)malloc(num_of_polys*sizeof(DWORD));
//		STATE_object_get_all_polygons(obj_handle, poly_array);
STATE_IMPORT void STATE_WINAPI STATE_object_get_all_polygons(DWORD object_handle, DWORD polygons_array[]);

//Only relevent when using CHASE_PRECISE.
// It then detrmines the distance between the given object and the object that we chase.
// Arguments:
//		object_handle: a handle to the object
//		chase_distance: The distance that the given object should keep from the chased entity
STATE_IMPORT void STATE_WINAPI STATE_object_set_chase_distance(DWORD object_handle, double chase_distance);

//Only relevent when using CHASE_PRECISE.
// It returns the chase distance 
//For more details see STATE_object_set_chase_distance()
STATE_IMPORT double STATE_WINAPI STATE_object_get_chase_distance(DWORD object_handle);

// Ignore this object for collision detection 
// purposes. This means that when other object will call STATE_object_is_movement_possible()
// ( or STATE_engine_is_movement_possible() ) they would never get collision with
// this object. This function is useful for object like clouds, smoke, grass etc ...
// The default is that all the objects are collisional
STATE_IMPORT void STATE_WINAPI STATE_object_make_non_collisional(DWORD object_handle);
// This is the default. 
STATE_IMPORT void STATE_WINAPI STATE_object_make_collisional(DWORD object_handle);

// Returns YES or NO
// For more details please see STATE_object_make_non_collisional() , STATE_object_make_collisional()
STATE_IMPORT int STATE_WINAPI STATE_object_is_collisional(DWORD object_handle);


// The following three functions are only relevant when the chase type of
// the object is CHASE_PHYSICS. For example if your object is a billiard ball
// then the engine will automatically calculate its direction when it
// hits the tables edges.  If you cancel collision test then the engine would
// still calculate the speed and the friction etc ... but when the ball would hit
// the the table edge it will continue moving as if it was air.
// So why would you want to cancel the automatic collision detection ?
// You will want to cancel it when you do your own checking.
// Say for example that your object is a missile, that is planed to blow up
// when it is near the target. For this case you have to do your own test ...
// The default is: enabled
// Canceling the collision check also improves performaces so if you don't realy need the collision detection
// then we would recommend to cancel it.
//
// Important:
//		Canceling the collision detection also changes the speed units to UNITS_PER_RENDER(this is a design bug in the engine that wasn't fix in order
//		to maintain backwards compatibility). To make life easier we recommend of the following:
//		A) After loading the world call STATE_object_set_speed_units(UNITS_PER_RENDER)
//		B) In order to have your program run the same on all computers and have the same speed in all parts of your world
//		   do factor=STATE_engine_get_computer_speed_factor() in every render cycle. Factor all your objects spped like that
//		   new_speed=original_speed*factor;
//
//		C) Cancel STATE_object_cancel_collision_test_for_chase_physics() for object that do not need a constant collision detection.
//
//
//		Going according to our recommendation will give you the following benefits:
//		1) Your program will be faster. 
//		2) The object will move smoother. 
//		3) Your program will move at the same speed in all parts of your world
//		and in all computers. 
//		4) Calling STATE_object_cancel_collision_test_for_chase_physics() will have no undesired effect on the speed units.
//
//
//		For more details about this please see STATE_object_set_speed_units()
//
//		
//
STATE_IMPORT void STATE_WINAPI STATE_object_cancel_collision_test_for_chase_physics(DWORD object_handle);

//Enables the collision detection for the given object (relevant only for CHASE_PHYSICS)
//For more details please see STATE_object_cancel_collision_test_for_chase_physics()
STATE_IMPORT void STATE_WINAPI STATE_object_enable_collision_test_for_chase_physics(DWORD object_handle);

//Returns YES if collision detection test is performed (when the chase type is CHASE_PHYSICS)
// otherwise returns NO
STATE_IMPORT int STATE_WINAPI STATE_object_is_collision_test_for_chase_physics(DWORD object_handle);


//This function is only relevant when the chase_type is set to CHASE_PHYSICS.
//Use this function to make the object rotate around its center.
//rotation_space: choose between WORLD_SPACE and OBJECT_SPACE
//rotate_around_x_axis: The number of angles that the object will rotate 
//		around the X axis, each time STATE_engine_render() is called
// see also STATE_object_get_physics_rotation()
STATE_IMPORT void STATE_WINAPI STATE_object_set_physics_rotation(DWORD object_handle, int rotation_space ,double rotate_around_x_axis, double rotate_around_y_axis, double rotate_around_z_axis);


//This function is only relevant when the chase_type is set to CHASE_PHYSICS.
// Return value: The rotation space (WORLD_SPACE or OBJECT_SPACE)
// through rotation_xyz[] the function returns the rotation angles
//		rotation_xyz[0] is the number of angles that the object will rotate 
//		around the X axis, each time STATE_engine_render() is called
// see also STATE_object_set_physics_rotation()
STATE_IMPORT int STATE_WINAPI STATE_object_get_physics_rotation(DWORD object_handle, double rotation_xyz[3]);

//Set the rotation center of the object. The default is the center of the object
//Note that the center point is in object space.
//Use this function to make a door turn arroun it axis (and not around its center) etc.
// Remarks:
//		Note that there are several functions for rotating an object. Here are some of them
//		and help when to use what.
//
//		1) Rotation around the object center
//			If you need to turn/rotate an object around its center (for example a propeller of an helicopter)
//			then use one STATE_object_rotate_x/y/z()
//
//		2) If you need to rotate an object around a point in the object body
//		   for example to turn a door around its hinge then use
//			STATE_object_rotate_x/y/z() together with STATE_object_set_rotation_center()
//
//		3) If you need to turn an object around a point in the world.
//			for example earth circling around the sun.
//			in this case use STATE_object_rotate_around_point()
//
// See also: 
//		STATE_object_rotate_x(), STATE_object_rotate_y(), STATE_object_rotate_z()
//		STATE_object_rotate_x_radians(), STATE_object_rotate_y_radians(), STATE_object_rotate_z_radians()
//		STATE_object_rotate_around_point()
//		STATE_object_set_rotation_center()
//		STATE_object_rotate_including_sons()
//
STATE_IMPORT void STATE_WINAPI STATE_object_set_rotation_center(DWORD object_handle, double center[3]);
STATE_IMPORT void STATE_WINAPI STATE_object_get_rotation_center(DWORD object_handle, double center[3]);

//Determines whether the object will be moved automatically according to his chase type (The movement is done each time render is called)
// yes_no_flag should be YES or NO.
//The default is YES
STATE_IMPORT void STATE_WINAPI STATE_object_advance_automatically(DWORD object_handle, int yes_no_flag);


// The following functions are used for creating a hierarchy among objects and manipulating
// multiple number of objects together. For example let's say
// that we want to have a human model in our application. We should construct the model from
// several objects, the hands, the legs, the head etc ... Each object can be constructed from
// smaller objects, the hands for example could contain objects for the fingers.
// When we move the hand we will want the finger objects to move together with the 
// hand object. We do it by setting the hand object as the father of each finger object. 
// Later we can call STATE_object_move_including_sons(hand_object_handle) This will
// move the hand objects including all the fingers. 
// Terminology: The usage of the word TREE
//		Setting father-sons relations creates a structure similar to a family tree
//
// We can call this function with father_object==NULL
// this will mean that there is no father for this object.
// Returns OK or VR_ERROR.
// If the function fails it means that we tried to make a loop in hierarchy of the objects
// (for example setting object B as the father of object A and then setting object A as the father of B)
STATE_IMPORT int STATE_WINAPI STATE_object_set_father_object(DWORD object_handle, DWORD father_object);
STATE_IMPORT DWORD STATE_WINAPI STATE_object_get_father_object(DWORD object_handle);

// returns YES or NO. 
// A NULL object_handle means the whole world
// Note that if objectA_handle==objectB_handle it will return YES
// If objectB_handle==NULL it will always return YES.
STATE_IMPORT int STATE_WINAPI STATE_object_is_objectA_included_in_objectB(DWORD objectA_handle, DWORD objectB_handle);

//Returns the first son of the given object. This function together with STATE_object_get_next_son()
// create an easy way for going over all the descendants (son, grandchild etc ...) of an object.
// for example: 
//	for(son=STATE_object_get_first_son(car_object); son!=NULL; son=STATE_object_get_next_son(son)) {
//		//your code here ....
//	}
// Note that if you just want to go over all the direct sons (no grandchildren ...) you should
// use STATE_object_get_first_direct_son() and STATE_object_get_next_direct_son()
STATE_IMPORT DWORD STATE_WINAPI STATE_object_get_first_son(DWORD object_handle);
STATE_IMPORT DWORD STATE_WINAPI STATE_object_get_next_son(DWORD object_handle, DWORD son_handle);

//Returns the first immediate son of the given object. This function together with STATE_object_get_next_direct_son()
// create an easy way for going over all the direct son (the close family) of an object.
// for example: 
//	for(son=STATE_object_get_first_direct_son(car_object); son!=NULL; son=STATE_object_get_next_direct_son(son)) {
//		//your code here ....
//	}
// Note that if you want to go over all the descendants (including grandchildren ...) you should
// use STATE_object_get_first_son() and STATE_object_get_next_son()
STATE_IMPORT DWORD STATE_WINAPI STATE_object_get_first_direct_son(DWORD object_handle);
STATE_IMPORT DWORD STATE_WINAPI STATE_object_get_next_direct_son(DWORD object_handle, DWORD son_handle);


// Advance the object (move relative) including all of its descendants
// space_flag could be WORLD_SPACE or OBJECT_SPACE or CAMERA_SPACE
// space_flag determines if the step is according to the objects
// coordinate system or the world coordinate  system etc ...
// for example if space_flag==OBJECT_SPACE and step==(-1,0,0)
// This means that the object will advance forward (If it is a car than
// forward is the direction of the front window).
// Try both option and see what happens.
STATE_IMPORT void STATE_WINAPI STATE_object_move_including_sons(DWORD top_object_handle, int space_flag, double x, double y, double z);

// Calculates the center of the collection of objects defined by the given object and all of its descendants
// The result is saved in the center argument.
// ("tree" means the whole family tree of the object)
//Return value: returns the number of objects in the tree
STATE_IMPORT int STATE_WINAPI STATE_object_get_center_of_tree(DWORD top_object_handle, double center[3]);

#define CENTER_OF_TOP_OBJECT 1
#define CENTER_OF_TREE 2
#define CENTER_AS_GIVEN 3
// Move the object and all its descendants to a new location. center_flag can be one of the two: CENTER_OF_TOP_OBJECT or CENTER_OF_TREE
// If CENTER_OF_TREE is given then after calling this function the center of the tree will be in the given new location.
// If CENTER_OF_TOP_OBJECT then after calling this function the center of the top object will be in the given new location.
STATE_IMPORT void STATE_WINAPI STATE_object_set_location_of_tree(DWORD top_object_handle, double new_location[3], int center_flag);

//The object and all its descendants will be rotated according to the given arguments:
// degrees: The number of degrees to rotate.
// axis: this could be 1 or 2 or 3. Give one for rotating around the X axis. 2 for rotating around the Y axis and 3 for the Z axis
// space_flag: OBJECT_SPACE or  WORLD_SPACE. If we have chosen for example to rotate around the X axis
//								this flag determines whether it is the X axis of the object or the X axis of the world.
// center_flag: Could be one of the three, CENTER_OF_TOP_OBJECT, CENTER_OF_TREE, CENTER_AS_GIVEN.
//				This flag determines the center of rotation.
// center: this argument is only relevant if the center_flag is CENTER_AS_GIVEN. In this case the point given by this
//				argument will define the center of rotation.
STATE_IMPORT void STATE_WINAPI STATE_object_rotate_including_sons(DWORD top_object_handle, double degrees, int axis, int space_flag, int center_flag, double center[3]);

//Remove light from the given object.
//If the given object was lit using the STATE_light API then
//this function could be used to restore the object normal appearance
//See also STATE_group_remove_light(), STATE_polygon_remove_light(), STATE_light_remove_light()
STATE_IMPORT void STATE_WINAPI STATE_object_remove_light(DWORD object_handle);

#define DYN_MAX_NUM_OF_BITMAPS 32

// *** very important
// Sets the given bitmap to all the polygons of the object. The bitmap serves as a skin.
// The behavior of the function depends whether the object uses an Effect or not
// When not using an effect:
//		The object can have only two bitmaps associated with it. the rest are ignored
//		Only bitmap_index==0 and bitmap_index==1 are relevant.
//		Calling this function with bitmap_index==0 would set the primary bitmap of all the polygons
//		belonging to this object.
//
// When an Effect is used
//		DYN_MAX_NUM_OF_BITMAPS bitmaps can associated with the object.
//		Whether they are relevent or not depends on the effect.
//		for most effect only 1 or two bitmaps are used
//
// Parameters
//		object_handle: The object handle
//
//		bitmap_handle: The bitmap handle
//
//		bitmap_index: A zero based index. Any number between 0 and DYN_MAX_NUM_OF_BITMAPS-1
//
// Return value:
//		OK or VR_ERROR 
//
// Example:
//	STATE_object_set_bitmap(my_object, my_bitmap, 0);
//
STATE_IMPORT void STATE_WINAPI STATE_object_set_bitmap(DWORD object_handle, DWORD bitmap_handle, int bitmap_index);


// *** a very inportant function
//Similar to STATE_engine_add_world() the difference is that
//the added world file is added as a dynamic object.
//This function is most useful for adding md2 file (quake files)
//If the world contains a 3d animation (all md2 files contain 3d animation)
//The 3D animation will automatically be added to the engine
//see the 3D_animation API and the 3D_sequence API
//Example:
//		A) loading an md2 object
//		DWORD girl=STATE_object_create_from_file("people\\girl.md2");
//
//		B) Loading a 3dstate file (that can be created using WorldBuilder)
//			DWORD boat=STATE_object_create_from_file("models\\boat.3dstate");
//
// Note:
//	1) If the bitmaps for the object are not located in the default directory
// (inside the bitmaps folder) you can set the bitmap folder by calling
// STATE_engine_set_bitmap_folder();
//
//
//  2) This function currently support the following file formats:
//		wld, 3dstate, morfit, md2, anim3d
//
//  3) If you have a model that you have created using 3D Studio Max or any other 3D editor
//	  you would first have to convert it to one of the formats supported by 3DSTATE.
//
//  4) When using the function STATE_object_get_name() for object that were loaded using
//		STATE_object_create_from_file(), the name returned will be the object file name without the path and 
//		without the file extension.
//	  
//
// See also STATE_object_create_from_file2(), STATE_engine_set_bitmap_folder()
STATE_IMPORT DWORD STATE_WINAPI STATE_object_create_from_file(char *file_name);

//The same as STATE_object_create_from_file() only that here there are two parameters.
//The additional parameter is the bitmap folder.
// If the bitmaps for the object are not located in the default directory
// (inside the bitmaps folder) you can use this function. Another option is to call
// STATE_engine_set_bitmap_folder() before calling STATE_object_create_from_file()
//
//for more details see STATE_object_create_from_file()
STATE_IMPORT DWORD STATE_WINAPI STATE_object_create_from_file2(char *file_name, char *bitmap_folder);

//When loading an md2 objects the engine builds the different animation sequences according to the
//names of the frame.
//With md3 it is different. The frames associated with each sequences are usually given in a different file
//usually animation.cfg
// md3 is also very differnt in that that many models are built from 2 or three separate md3 models
//  for example one md3 model for the legs and one md3 model for the torso and one for the head.
//This function is the same as STATE_object_create_from_file only that one can also give the animation.cfg
//and the engine will build the animation accordingto this file.
//The animation.cfg files are simple text files. You can find many on the Internet (look for Quake3 md3)
// below is an example 

/*
// first frame, num frames, looping frames, frames per second (the 4 number fields below)
0	30	0	25		// BOTH_DEATH1
2	1	0	25		// BOTH_DEAD1
90	40	0	20		// TORSO_GESTURE
151	1	0	15		// TORSO_STAND
152	1	0	15		// TORSO_STAND2
153	8	8	20		// LEGS_WALKCR
161	12	12	20		// LEGS_WALK
*/

//Return value: The function returns a handle to the created object

//Parameters:
//	file_name: the file of the md3 object. Example: "..\\my_models\\dog.md3"
//  
//  animation_cfg_file: the file which gives the names of the different sequences and the fram numbers associated with each frame.
//						see example above
//
//	sequence_token: The animation.cfg files describe the sequences for several models. The token is used to associate
//					the loaded model with the relevant sequences.
//					for example if we load an md3 that is only the legs then we only want
//					to take the sequences (from the animation.cfg) that start with "LEGS_"
//

// Example
//		DWORD obj=STATE_object_md3_create_from_file("woman1.md3", "animation.cfg", "TORSO_");

STATE_IMPORT DWORD STATE_WINAPI STATE_object_md3_create_from_file(char *file_name, char *animation_cfg_file, char *sequence_token);

//Returns the number of tags. A tag is used to connect between md3 objects.
//Use STATE_object_md3_connect_models() to connect models using tags.
STATE_IMPORT int STATE_WINAPI STATE_object_md3_get_number_of_tags(DWORD obj);

STATE_IMPORT char *STATE_WINAPI STATE_object_md3_get_tag_name(DWORD obj, int tag_number);

//returns -1 if the tag was not found
STATE_IMPORT int STATE_WINAPI STATE_object_md3_get_tag_number(DWORD obj, char *tag_name);

//*** advanced function. Not very usefull
//Return the tag interpolated transformation matrix according to the last animation frame that was drawn
//[IN] object_handle, tag_name
//[OUT] the transformation matrix  (tag_matrix)
//
//Return value: OK or VR_ERROR
//
STATE_IMPORT int STATE_WINAPI STATE_object_md3_get_tag_transformation_matrix(DWORD object_handle,char *tag_name,  double tag_matrix[4][4]);

//Use for md3 models that come in several parts. For example: legs, torso and head
//The function connects the son_obj to the base_obj
//The given tag name should be included inside the base_obj model.
// Common tag names are "tag_torso", "tag_head", "tag_weapon"
//To check the tags that are included in a model, use STATE_object_md3_get_tag_name() and STATE_object_md3_get_number_of_tags()
// If you are too lazy to check you can also use STATE_object_md3_connect_models_automatic() which finds the tag automatically
// Example: 
//		STATE_object_md3_connect_models(lower, upper, "Tag_Torso");
//
//
// Imporant !
// ------------
// Note that the order is important. 
// STATE_object_md3_connect_models(lower, upper, "Tag_Torso"); would work fine but
// STATE_object_md3_connect_models(upper,lower  "Tag_Torso"); will fail (we changed the order to upper,lower)
// This is because how the tag information is stored inside the models.
//
STATE_IMPORT int STATE_WINAPI STATE_object_md3_connect_models(DWORD base_obj, DWORD son_obj, char *tag_name);

//The same as STATE_object_md3_connect_models() just that the function automatically looks for a matchnig tag
//so there is no need to give the tag name as a parameter.
//The function looks for a tag that exists both in the base_obj and in the son_obj
//The object will be connected according to the first matching tag that is found.
// Q: How would you know if the objects were connected correctly using the right tag
// A: If they look connected correctly then they are.
//
STATE_IMPORT int STATE_WINAPI STATE_object_md3_connect_models_automatic(DWORD base_obj, DWORD son_obj);

//Discconect models that are connected using STATE_object_md3_connect_models();
STATE_IMPORT int STATE_WINAPI STATE_object_md3_disconnect_models(DWORD base_obj, DWORD son_obj);


#define CREATE_3D_TEXT 512
#define CANCEL_AUTO_FACE_CAMERA 1024 //By default the texts are turned so that they faced the camera
									 //this is done so the text can always be read (even when viewed from behind ...)
									 //Use to cancel this property. 

// *** very useful
// Creates an 3D object in the shape of the given text. See example
//
// Parameters:
//
//		font_handle:
//			The font to use to create the model.
//			To create a font call STATE_font_create(). 
//			for example:
//			DWORD ariel14=STATE_font_create("Ariel", 14, FONT_CREATE_DEFAULT);
//
//		text:
//			The text itself that will be created into a 3D model
//
//		red, green, blue:
//			The color of the text created. This flag has no effect when CREATE_3D_TEXT is used
//			When CREATE_3D_TEXT is used set STATE_object_set_color() to change the color of the object
//
//		flags:
//			any of the following:
//				FONT_WRITE_DEFAULT,
//				FONT_WRITE_CENTERED_X, //When FONT_WRITE_CENTERED_X + FONT_WRITE_CENTERED_Y are given then the location of the object will be the center of the text
//				FONT_WRITE_CENTERED_Y,
//				FONT_WRITE_PIXELIZED, //Creates ugly pixelized letters. Have no effect when CREATE_3D_TEXT is used
//				CREATE_3D_TEXT, //This flag will add depth to the text (and many more polygons). 
//								// If you need a text label then don't use this flag. If you need a real 3D text model
//								// Then use this flag.
//								//When using this flag a totaly different technique is used to create the model
//				CANCEL_AUTO_FACE_CAMERA //By default objects will turn automatically to the camera so that the
//										//text is always readable ( even if viewing the model from behind ...)
//										//use this flag to cancel this property. This flag has no effect when 
//										//CREATE_3D_TEXT is given
//
//
// Remarks:
//		1. The text object that is created is very small so scaling the object is usually necessary. See example below
//
// Example:
//
//		DWORD ariel14=STATE_font_create("Ariel", 14, FONT_CREATE_DEFAULT);
//		DWORD text_object=STATE_object_create_from_text(ariel14, "Johnny", 255, 128, 128, FONT_WRITE_CENTERED_X | FONT_WRITE_CENTERED_Y);
//		STATE_object_set_location(text_object, 5000, 5000, 5000);
//		double scale[3]={1000, 1000, 1000};
//		STATE_object_set_scale(text_object, scale);
//
STATE_IMPORT DWORD STATE_WINAPI STATE_object_create_from_text(DWORD font_handle, char *text, BYTE red, BYTE green, BYTE blue, DWORD flags);

// *** very useful
// Similar to STATE_object_create_from_text() only that it adds the text to a given object
// This function is very useful to add labels to objects since the text will stick to the object
// and follow it wherever it moves(a plane , a man walking etc ...)
//
// Maybe a proper name would be STATE_object_set_text() since the function doesn't add text but rather
// replace the current text with the given one (by default there is no text)
//
// This function adds two parameters that are not included in STATE_object_create_from_text()
// The size parameter is used to control the size of the text without effecting the object size
// As would happen if were to use STATE_object_set_scale() (this will increase the size of both the text and the rest of the object)
// The offset parameter is used to control the location of the text relative to the location of the object (in object space coords)
//
// Parameters:
//		object_handle:
//				A handle to an object to which we want to add the text
//
//		font_handle, text, red, green, blue
//				See explanations in STATE_object_create_from_text()
//
//		flags:
//				As in STATE_object_create_from_text() only that here the flag CREATE_3D_TEXT has no effect
//
//		size:
//				The size parameter is used to control the size of the text without effecting the object size
//				As would happen if were to use STATE_object_set_scale() (this will increase the size of both the text and the rest of the object)
//
//		offset:
//				The offset parameter is used to control the location of the text relative to the location of the object (in object space coords)
//
//
// See also:
//		STATE_object_get_text(), STATE_object_create_from_text()
//
// Example:
//			double text_offset[3]={0, 0, 0};
//			STATE_object_add_text(obj, ariel14, "Against All Odds", 255, 100, 32, FONT_WRITE_CENTERED_X | FONT_WRITE_CENTERED_Y  , 1000, text_offset);
//
STATE_IMPORT void STATE_WINAPI STATE_object_add_text(DWORD object_handle, DWORD font_handle, char *text, BYTE red, BYTE green, BYTE blue, DWORD flags, double size, double offset[3]);

//retrieves the text set by STATE_object_add_text()
STATE_IMPORT char *STATE_WINAPI STATE_object_get_text(DWORD object_handle);

// Set the object text to the given color.
// Used together with STATE_object_add_text()
// See also 
//		STATE_object_add_text(), STATE_object_get_text()
void STATE_object_set_text_color(DWORD object_handle, BYTE red, BYTE green, BYTE blue);
void STATE_object_get_text_color(DWORD object_handle, BYTE *red, BYTE *green, BYTE *blue);

//Returns the object 3D_animation or NULL if it doesnt have any 3D animation
// Example:
// my_obj=STATE_object_create_from_file("woman.md2"); //The md2 files contain a 3D animation. The 3D animation is automatically loaded to the engine
// my_3danim=STATE_object_get_3D_animation();
// walk_sequence=STATE_3D_sequence_get_using_name(my_3danim,"walk");
// standing_sequence=STATE_3D_sequence_get_using_name(my_3danim,"stand");
// undressing_sequence=STATE_3D_sequence_get_using_name(my_3danim,"undress");
// if(key_1_is_pressed) STATE_object_set_3D_sequence(my_obj,walk_sequence); 
// if(key_2_is_pressed) STATE_object_set_3D_sequence(my_obj,undressing_sequence); 
// and so on ...
STATE_IMPORT DWORD STATE_WINAPI STATE_object_get_3D_animation(DWORD object_handle);


//Returns OK or VR_ERROR
//Used to attach a 3D animation to an object.
// animation3d_handle could also be NULL (to detach the current used animation)
//For more details please see STATE_object_get_3D_animation()
STATE_IMPORT int STATE_WINAPI STATE_object_set_3D_animation(DWORD object_handle, DWORD animation3d_handle);


//Replaces the current sequence with a new one.
// The engine will interpolate between the old sequence and the new one
// The duration of the interpolation process is controlled by transition_period
// transition_period is in mili seconds.
// If transition_period==0 then the engine wont do any interpolation between the old and the
// new sequences.
// Returns OK or VR_ERROR
// Note that usually this function is used
// together with STATE_3D_sequence_duplicate()
// This is because if two objects use the same 3D_sequence then
// changing the speed of the sequence (or other properties) will
// effect both objects
// Examples:
// A)	//A simple example
//		STATE_object_set_3D_sequence(my_object, walking_sequence, 100);
//
// B)	//A simple example with full code
//		DWORD obj=STATE_object_create_from_file("robot.md2");
//		DWORD anim3d=STATE_object_get_3D_animation(obj);
//		//Every md2 file might have different sequence names depending on the animated character
//		//to know the names of the sequences you have to go through all of them using STATE_3D_sequence_get_first() and get next mechanism (and of course STATE_3D_sequence_get_name)
//		DWORD walking_sequence=STATE_3D_sequence_get_using_name("walk");
//		STATE_object_set_3D_sequence(my_object, walking_sequence, 100);
//
// C)	//How to do it when a multiple number of objects might use the same 3D sequence
//		//Normally you will do it like in this sample
//		DWORD robot1=STATE_object_create_from_file("robot.md2");
//		DWORD anim3d=STATE_object_get_3D_animation(robot1);
//		//Every md2 file might have different sequence names depending on the animated character
//		//to know the names of the sequences you have to go through all of them using STATE_3D_sequence_get_first() and get next mechanism (and of course STATE_3D_sequence_get_name)
//		DWORD walking_sequence=STATE_3D_sequence_get_using_name("walk");
//		robot1_walk=STATE_3D_sequence_duplicate(walking_sequence, "robot1_walk");
//		STATE_object_set_3D_sequence(robot1, robot1_walk, 100);
//		//Note ! now that this object has its own sequence then changing the sequence speed wont effect
//		//other objects which use the same sequence
STATE_IMPORT int STATE_WINAPI STATE_object_set_3D_sequence(DWORD object_handle, DWORD sequence3d_handle, int transition_period);

STATE_IMPORT DWORD STATE_WINAPI STATE_object_get_3D_sequence(DWORD object_handle);


//new_sequence3d_handle is the handle to the sequence that we want to play when the current sequence
//reaches its last frame.
// arguments:
// transition_period	
//		The engine is going to interpolate between the old and the new animation and thus create a smooth animation
//		The duration of this transition period is given by the argument transition_period
//		If this value is 0 then the switch to the new sequence will happen immediately
//		transition_period is in mili seconds.
STATE_IMPORT void STATE_WINAPI STATE_object_replace_3D_sequence_when_finished(DWORD object_handle, DWORD new_sequence3d_handle, int transition_period);


//Scale the object (in object space).
//Example:
// double scale[3]={1,2,3}; //make it twice as wide and triple tall
// STATE_object_set_scale(my_object, scale); 
STATE_IMPORT void STATE_WINAPI STATE_object_set_scale(DWORD object_handle, double scale[3]);
//Returns the scaling factor. The default is 1,1,1
// After calling this function
// scale[0] will hold the X scaling factor
// scale[1] will hold the Y scaling factor
// scale[2] will hold the Z scaling factor
// Example:
//		double scale[3];
//		STATE_object_get_scale(my_object,scale);
STATE_IMPORT void STATE_WINAPI STATE_object_get_scale(DWORD object_handle, double scale[3]);

//Returns the handle to the group that represent this object.
STATE_IMPORT DWORD STATE_WINAPI STATE_object_get_group_handle(DWORD object_handle);

// *** quite useful
//Calls STATE_point_set_rgb() for all the points of the object polygons
//For more details see STATE_polygon_set_light()
STATE_IMPORT void STATE_WINAPI STATE_object_set_light(DWORD object_handle, BYTE rgb[3]);

// *** Very useful
//
// Creates an empty object and returns its handle.
// The object is automatically added to the engine
//
// Using this function together with STATE_object_add_polygon()
// makes it possible to create objects in real-time. These two functions also makes it possible
// to do most of EDITOR Mode tasks in viewer mode.
//
// Parameters:
//   object_name: Give the created object a nmae
//
// Example:
//		DWORD obj=STATE_object_create("my_object");
//
// See also: STATE_object_add_polygon()
//
STATE_IMPORT DWORD STATE_WINAPI STATE_object_create(char *object_name);


// *** Very useful
//
// Adds a polygon to the object.
//
// General remarks about object polygons
// -------------------------------------
// Please note that the points of the polygon should be in OBJECT_SPACE
// (for example 0,0,0 is the center of the object and not the center of the world)
// Adding polygons that are defined in the object space is very easy because we don't
// have to take care whether the object has moved or turned around. For example
// if we have an airplane that is flying then in object space the coordinates
// of the points that define a polygon on the airplane wing will always stay the same
// no matter how many maneuvers the airplane did.
// One can use the function STATE_object_convert_point_to_object_space()
// If the points of the polygon are only known in world space. 
//
// How do I get the object to begin with ?
// ---------------------------------------
// The object can come from three sources:
// 1. You can create a new empty polygon simply by calling STATE_object_create();
// 2. You can load the object from a file by using STATE_object_create_from_file();
// 3. The object can be included in the world. In this case after you load your world with
//    STATE_engine_load_world() you can use STATE_object_get_first_object(), STATE_object_get_next_object() and
//    STATE_object_get_name()to check what objects you have inside the world. Note that
//	  If you use World Builder or Webmaker, you can use make any model to become an object
//	  simply by right clicking and selecting properties and then defining the model as a dynamic object.
//
// Return value
// -------------
// Returns OK or VR_ERROR
// If the return value is VR_ERROR it is probably because that the polygon is not valid.
// (for example it has only two points. 4 points not on the same plane. not convex,
// bitmap coordination are not valid etc ...see the STATE.log to catch the error)
//
// See also: STATE_object_create(), STATE_object_create_from_file()
STATE_IMPORT int STATE_WINAPI STATE_object_add_polygon(DWORD object_handle, DWORD polygon_handle);


//The same as STATE_polygon_create_lightmap() only that it creates lightmaps bitmaps for all
//polygons of the given object.
//Returns the number of bitmaps that were created and assigned to the object.
//For more details see STATE_polygon_create_lightmap()
// Note this important function: STATE_engine_reduce_number_of_bitmaps()
STATE_IMPORT int STATE_WINAPI STATE_object_create_lightmap(DWORD object_handle, int max_bm_side, int min_lightmap_size, double sample_rate, int force_ray_tracing, int bitmap_number);

//Deletes a polygon from the object. Returns OK or VR_ERROR
STATE_IMPORT int STATE_WINAPI STATE_object_delete_polygon(DWORD object_handle, DWORD polygon_handle);



// Drops the object down till it hits the floor\ground
// If the object is below the ground the function will actually move it up
// and put it on top of the above ground
//
// Return Value.
//		Returns a handle to the polygon bellow.
//
// Parameters:
//		object_handle: the object to be dropped.
//
//		fall_through_dynamic_objects: Could be set to YES or NO. 
//									  Notice the following code sample:
//									  Lets assume that we have a terrain world and we want to move our character forward
//
//									  STATE_object_move(my_object, OBJECT_SPACE, -10, 0, 0); //Move the object 10 units forward. If we are in front
//																							  //of a mountain slope, it could get us into the mountain.
//									  STATE_object_move(my_object, WORLD_SPACE, 0, 0, 10000000); //Move the object up in the air so if we have penetrated the mountain
//																								  //with our previous move then now we are above the mountain
//									  STATE_object_drop_down(my_object, NO); //Drop the player nicely on the ground
//
//									  The code above will work very nicely till there is a helicopter or a bird that
//									  flys above our player. If so, our player will drop down on a the helicopter instead
//									  of on the ground. In this case of this game we better call with fall_through_dynamic_objects==YES
//
//
//		match_ground_angle:	YES or NO. If YES is given then the object will turned so it matches the ground surface
//							for example if we drop a car on a road going uphill then giving YES will insure that 
//							the car will be positioned according to the road incline and that its four wheel will contact the road
//							Note that if we drop a tree on a mountain side that we should give NO since trees grow upwards ...
//							
//
//
//
// Example A
//
//  DWORD floor_polygon=STATE_object_drop_down(my_object);
//  if(floor_polygon==NULL)
//			error_message("The object has nothing below !!!);
//
//
// Example B
//
//		//an alternative to STATE_object_drop_down()
//		double min_height, max_height;
//		DWORD poly=STATE_object_get_bounding_box_height_above_ground(my_object, NO, &min_height, &max_height);
//		if(poly!=NULL)
//			STATE_object_move(my_object, WORLD_SPACE, 0,0, -max_height);
//
//		// Note that STATE_object_drop_down() would calculate the height
//		// according to the center of the object and not according the the max_height
//		// as shown here
//
//	Remarks:
//		See also STATE_object_drop_down_fast(), STATE_object_get_bounding_box_height_above_ground()
//
STATE_IMPORT DWORD STATE_WINAPI STATE_object_drop_down(DWORD object_handle, int fall_through_dynamic_objects, int match_ground_angle);


// *** very useful.
// Drops the object down till it hits the floor\ground.
// Use this function when you want to put an object in a certain location.
// In many cases you will know the X,Y of the location but not the height.
// In these cases simple put the object high up For example: (X,Y, 10000000)
// and just drop it down.
//
// Return Value.
//		Returns a handle to the polygon bellow.
//
// Parameters:
//		object_handle: the object to be dropped.
//
//		object_height: The height of our object.
//						You can calculate the height of the object using the object bounding box
//						Give a value bigger than the object height if you want the object to be above the ground
//						Give a value smaller than the object height if you want the object to be sunken in the ground
//						Calculating the height should be a one time operation that should be done at the beginning of the program.
//						The fact that this function doesnt have to do calculate the height on its own
//						make this function a little bit faster then STATE_object_drop_down(). Except that the functions are the same
//
//		fall_through_dynamic_objects: Could be set to YES or NO. 
//									  Notice the following code sample:
//									  Lets assume that we have a terrain world and we want to move our character forward
//
//									  STATE_object_move(my_object, OBJECT_SPACE, -10, 0, 0); //Move the object 10 units forward. If we are in front
//																							  //of a mountain slope, it could get us into the mountain.
//									  STATE_object_move(my_object, WORLD_SPACE, 0, 0, 10000000); //Move the object up in the air so if we have penetrated the mountain
//																								  //with our previous move then now we are above the mountain
//									  STATE_object_drop_down(my_object, NO); //Drop the player nicely on the ground
//
//									  The code above will work very nicely till there is a helicopter or a bird that
//									  flies above our player. If so, our player will drop down on a the helicopter instead
//									  of on the ground. In this case of this game we better call with fall_through_dynamic_objects==YES
//
//		match_ground_angle:	YES or NO. If YES is given then the object will turned so it matches the ground surface
//							for example if we drop a car on a road going uphill then giving YES will insure that 
//							the car will be positioned according to the road incline and that its four wheel will contact the road
//							Note that if we drop a tree on a mountain side that we should give NO since trees grow upwards ...
//							
//
//						
//Example:
//		//calculate the height:
//		//This is a one time operation that should be done at the beginning of the program.
//		//The fact that this function doesnt have to do
//		double box[2][3];
//		STATE_object_get_bounding_box(my_object, box);
//		double height= box[1][2]-box[0][2];
//		STATE_object_drop_down_fast(my_object, height, NO, YES);
//		
STATE_IMPORT DWORD STATE_WINAPI STATE_object_drop_down_fast(DWORD object_handle, double object_height, int fall_through_dynamic_objects, int match_ground_angle);

#ifndef ANIMATION_NO_INTERPOLATION
#define ANIMATION_NO_INTERPOLATION 0
#define ANIMATION_REALTIME_INTERPOLATION 1
#define ANIMATION_NO_INTERPOLATION_SUPER_FAST 2 //faster than but takes more memory
#endif

// Use this function to set animation interpolation mode.
// This function is only relevant to objects that are using 3D animation such as md2 or anim3d
// (see the STATE_3D_animation_  API). By default the STATE engine interpolates
// between frames of the current used 3D sequence. Interpolation means that if for example
// we have in one frame a monster creature standing and in the very next frame we have the monster 
// is dismantled into bits and pieces (explosion effect), the engine will be able to create all the frames
// in between. This ability of the engine makes it possible to create very smooth 3D animations
// with only using some very few frames. Usually you will never want to turn this feature off.
// The only reasons for turning 3D interpolation off will be:
// - You have too many animated models and it runs too slow -

//
// Parameters:
//
//		object_handle: The handle of the object
//
//		mode:  could be one of these three options 
//					ANIMATION_REALTIME_INTERPOLATION // The default. created smooth animation between frames.
//					ANIMATION_NO_INTERPOLATION		 // Not very useful. Will save some CPU cycles while making the animation look more jerky
//					ANIMATION_NO_INTERPOLATION_SUPER_FAST // CURRENTLY NOT SUPPORTED. When having a lot of 3d animated models you might want to test this mode to gain speed.

//

//Example:
//		//Turn off 3D animation interpolation to gain more speed
//		DWORD my_object=STATE_object_create_from_file("monster.md2");
//		STATE_object_set_3D_animation_interpolation_mode(my_object, ANIMATION_NO_INTERPOLATION_SUPER_FAST);
STATE_IMPORT void STATE_WINAPI STATE_object_set_3D_animation_interpolation_mode(DWORD object_handle, int mode);

// *** quite a usefull function (though one can do without it)
//One can use this function to associate your own data structure to an object.
//Whether one decides to use this function or not depends on one's programming style.
//The obvious alternative for using this function is to have your own structure or class
//for each object and that one of the fields in your class would be the object_handle through which 
//you can access the STATE functions.
//
// Parameters:
//		object_handle:
//				a handle to an object
//
//		my_own_data:
//			A pointer to a class or structure or even a function. This pointer can point
//			to whatever you want it to point. The Engine doesnt do anything with this pointer
//			it simply save it for you so you can call STATE_object_get_my_own_data() to retrieve it.
//			This parameter can also be NULL.
//
// Example:
//
// typedef struct object_data 
// {
//		int health;
//		int power;
//		int number_of_times_that_got_hit;
//		double height;
//		int weapon_type;
//		//And anyother data you might need in your program
// } object_data;
//
// object_data obj_data; //or use new(), malloc() or whatever ...
// STATE_object_attach_my_own_data(my_object, &obj_data);
//
// Example B:
//      //there is no function for detach so just call like this
//		STATE_object_attach_my_own_data(my_object, NULL); 
//
// Note that the attached data is not saved when calling STATE_engine_save()
// so you need to save it yourself (or see the See also section).
//
// See also STATE_object_set_object_type_number(), STATE_object_set_control_type_number
//
STATE_IMPORT void STATE_WINAPI STATE_object_attach_my_own_data(DWORD object_handle, void *my_own_data);
STATE_IMPORT void *STATE_WINAPI STATE_object_get_my_own_data(DWORD object_handle);


// Copies the speed, the location and the force of one object to another.
// This function is useful when we want to change the shape of an object with another object
// The function also makes sure that object with chase type CHASE_PHYSICS will have a smooth transition
// from one to another.
//
// Remarks
//		The function could be used also when the chase type is not CHASE_PHISYCS or when there is no chase type at all
//		though the function real need is when CHASE_PHYSICS is used.
//		When CHASE_PHYSICS is used one should use this function instead of writing one own version (for coping the location and speed)
//		because this function also synchronize the timing for maximum smooth transition.
// 
// Example:
//
//	//lets say we have an airplane that was hit by a missile and now we want to
//  //replace the airplane shape with a shape of an airplane which is a bit burnt and has a hole.
//
//		STATE_object_disable(good_airplane_handle); //Disable the intact airplane 
//		STATE_object_enabled(burnt_airplane_handle); //Enable the damaged airplane. From now on, this object will be used.
//		STATE_object_copy_physics(burnt_airplane_handle, good_airplane_handle); //set the location and other properties of
//																			//damaged jet so that we will have a smooth transition between the two.				
//
STATE_IMPORT void STATE_WINAPI STATE_object_copy_physics(DWORD dest_object, DWORD src_object);


#define SHAPE_OUTSIDE 1
#define	SHAPE_INSIDE  2
#define SHAPE_BOTH_SIDES 3

// *** quite useful
//	This function creates simple 3D primatives such as a room a box a tunnel and a line.
//	Based on two points that are given to the function, the engine builds a shape around the two points.
//  by selecting the number of polygons used and the inside/outside flag one can control the shape created.
//  If you thing of it the only difference between a room and a box is that in a room the polygons are seen
//  from inside while in the box they are seen from the outside (remember, in the 3D world, polygons have
//  only one side.) The different between a box and a line is that a line is longer and made with more polygons
//	so that it looks more rounded.
//
//	Parameters:
//		object_name:
//			The name of the object that will be created. Give any name you want.
//			Like with all tother names in the 3DSTATE engine there is no limit on the length of the string.
//
//		P1,P2:
//			Two points that define the a line/segment. A long this segment the shape will be built
//		
//		radius:
//			Determines how thick the line will be (or the height of a box / room)
//
//		close_edges:
//			could be YES or NO . If close_edges==YES the function will add
//			two more polygons to seal the edges of the shape. 
//
//		sides:
//			This parameter can be one of the three following options: SHAPE_OUTSIDE, SHAPE_INSIDE ,SHAPE_BOTH_SIDES 
//			If you build a room , you should choose SHAPE_INSIDE since you will look at the walls of the room
//			from inside the room. If you build a box or a 3D line then select SHAPE_OUTSIDE. 
//			If SHAPE_BOTH_SIDES is used then the function will use double the number of polygons
//			so that the shape can be viewd from the inside and from the outside
//
//		number_of_bitmaps_along_the_line:
//			Controls the tiling of the bitmap coordination along the line
//			from P1 to P2. Usually after calling this function you would
//			want to call STATE_object_set_bitmap() in order to wrap a bitmap
//			around the created line (use the bitmap as one skin for the whole object). 
//			number_of_bitmaps_along_the_line, number_of_bitmaps_around_the_line
//			control how the bitmap will be wraped around the created object.
//			If number_of_bitmaps_along_the_line==0 or number_of_bitmaps_around_the_line==0
//			then each polygon will have its bitmap coordinate set with 0,0 at one
//			corner and 1,1 at the other (in other words, the polygons will be set so that a full bitmap is mapped on each polygon)
//			For example if building a room you might want to set each wall with a different bitmap so in this case use
//			number_of_bitmaps_along_the_line=0. If you create a 3D line and you want to wrap around a bitmap of shiny metal  then use
//			different values (such as 2,3,4,5 etc ...) depending on how densed you want the texture to appear.
//
//		number_of_bitmaps_around_the_line:
//			Exactly like number_of_bitmaps_along_the_line only that it controls the density of the bitmap around the created
//			line and not along the line (as done by number_of_bitmaps_along_the_line);
//
//		use_one_bitmap:
//			This parameter can be YES or NO. It controls how the function set the bitmap coordination of the polygon created.
//			If use_one_bitmap==YES then the function will build the polygons so that one bitmap can be served as a skin and
//			cover the whole shape. Use STATE_object_set_bitmap() to set all the polygons of the shape with one bitmap/skin .
//			If use_one_bitmap==NO, then each polygon will have its bitmap coordinate set with 0,0 at one
//			corner and 1,1 at the other (in other words, the polygons will be set so that a full bitmap is mapped on each polygon)
//			For example if building a room you might want to set each wall with a different bitmap so in this case use
//			use_one_bitmap=NO. If you create a 3D line and you want to wrap around a bitmap of shiny metal  then use
//			use_one_bitmap=YES.
//
//	Return value:
//		Returns a handle to the created object
//
//	Example A:
//		//creating a room
//		double P1[3]={0,0,0};
//		double P2[3]={10,0,0};
//		STATE_object_create_line("my_room", P1, P2, 5, 4, YES, SHAPE_INSIDE ,0,0);
//
//	Example B:
//		//creating a box
//		double P1[3]={0,0,0};
//		double P2[3]={10,0,0};
//		STATE_object_create_line("my_box", P1, P2, 5, 4, YES, SHAPE_OUTSIDE ,0,0);
//
//	Example C:
//		//creating a 3D line
//		double P1[3]={0,0,0};
//		double P2[3]={10,0,0};
//		STATE_object_create_line("my_3d_line", P1, P2, 1, 20, YES, SHAPE_OUTSIDE ,4,2);
//
//	Example D:
//		Creating a line from a track data
//		for(int i=1; i<num_of_points; i++) {
//			vector P1, P2;
//			STATE_track_get_point(track, i-1, P1);
//			STATE_track_get_point(track, i, P2);
//			DWORD obj=STATE_object_create_line(obj_name, P1, P2, radius, 8, YES, SHAPE_OUTSIDE, 5, 2);
//			if( (obj!=NULL) && (my_bitmap!=NULL) ) //my_bitmap is a a bitmap you previously loaded with STATE_bitmap_load()
//				STATE_object_set_bitmap(obj, my_bitmap);
//		}


//
//
STATE_IMPORT DWORD STATE_WINAPI STATE_object_create_line(char *object_name, double P1[3], double P2[3], double radius, int num_of_polygons, int close_edges, int side, int number_of_bitmaps_along_the_line, int number_of_bitmaps_around_the_line);

#define EFFECT_DISABLE 1
#define EFFECT_DYNAMIC_ENVIRONMENT_MAPPING 2 //When using this effect don't forget to call STATE_object_render_into_environment_map()
#define EFFECT_FAST_ENVIRONMENT_MAPPING 3 //when using this effect don't forget to load a cubic texture and call STATE_object_set_bitmap()
#define EFFECT_CLOUD_COVER 4
#define EFFECT_SKYSPHERE 5
#define EFFECT_BUMPWAVES 6
#define EFFECT_WATER 7

// *** a very important function
// Cause the given object to be rendered according to the given effect script. This function is used to create all 
// the new effects such as water, reflection etc. Using this function one can utilize many of the available free
// Shaders and effects found on the Internet. This function also support Microsoft HLSL (High Level Shader Language)
// Another nice feature of this function is that you can modify the script used and thus create different effects.
// If you are very knowlegable in 3D graphics and math you can also try and write your own shader though for most users
// the best policy would be to use one of the available free shaders / effect files. 
// Even if you understand nothing of the Shader script, you can still try and modify some of the code, who knows
// many of the best inventions were discovered accedently ( Serendipity ).
//
// Parameters:
//		object_handle:
//			The object which should use the given effect script
//
//		effect_file:
//			The effect script file. This file could be one of the following:
//			1) *.vsh	- a vertex shader script in assembly language
//			2) *.fx		- an effect file written using Microsoft High Level Shader Language
//			3) *.vso	- An object code of a compiled shader. Note that object code shaders can not be
//						  edited in a text editor. To edit the shader you will need it's assembly source
//						  and then use a compiler such as NVidia Cg compiler (free for downloading from NVidia website)
//						  to compile the shader
//
//		pixel_shader_file:
//			A pixel shader script or NULL if a pixel shader is not in use
//			
//		effect_type:
//			Should be one of the constant above. Note that this constant should match the effect file used.
//			If you are trying an effect file that has no effect_type constant to match with then try all of
//			the constant till you get the effect working.
//
// Example A:
//		//Render the object using an dynamically generated environment mapping effect
//		//Note that using this specific effect would also required calling later STATE_object_render_into_environment_map(teapot, effect);
//		DWORD effect=STATE_object_create_effect(teapot, "..\\Effects\\dyn_env_map.fx", NULL,EFFECT_DYNAMIC_ENVIRONMENT_MAPPING);
//
// Example B:
//		//Disable effect for the specific object and get back to render the object using its polygon textures
//		STATE_object_create_effect(teapot, NULL, NULL,EFFECT_DISABLE);
//
STATE_IMPORT DWORD STATE_WINAPI STATE_object_create_effect(DWORD object_handle, char *effect_file, char *pixel_shader ,DWORD effect_type);

// Exactly like STATE_object_create_effect() only that the effect is created from a string instead of from a file
STATE_IMPORT DWORD STATE_WINAPI STATE_object_create_effect_from_text(DWORD object_handle, char *effect_text, char *pixel_shader_text,DWORD effect_type);

//Retuns VR_ERROR or OK.
STATE_IMPORT int STATE_WINAPI STATE_object_render_into_environment_map(DWORD object_handle, DWORD effect_handle);

// Move an object around a given point.
//
// Parameters:
//		object_handle [in]:
//			The object that will move around the given point
//
//		angle [in]:
//			90 for example will cause the object to complete quarter of a circle around
//			the given point. -90 the same just in the opposite direction
//
//		axis [in]:
//			can be one of the three: X_AXIS, Y_AXIS or Z_AXIS
//			Specify how the object will circle around the point
//			For example a car circling in a roundabout is circling
//			around the world Z axis.
//
//		rotation_center [in]:
//			To point to circle around.
//		
// Example:
//		double world_center[3]={0,0,0};
//		STATE_object_rotate_around_point(my_plane, 0.2, Z_AXIS, world_center);
//
// Remarks:
//		Note that there are several functions for rotating an object. Here are some of them
//		and help when to use what.
//
//		1) Rotation around the object center
//			If you need to turn/rotate an object around its center (for example a propeller of an helicopter)
//			then use one STATE_object_rotate_x/y/z()
//
//		2) If you need to rotate an object around a point in the object body
//		   for example to turn a door around its hinge then use
//			STATE_object_rotate_x/y/z() together with STATE_object_set_rotation_center()
//
//		3) If you need to turn an object around a point in the world.
//			for example earth circling around the sun.
//			in this case use STATE_object_rotate_around_point()
//
// See also: 
//		STATE_object_rotate_x(), STATE_object_rotate_y(), STATE_object_rotate_z()
//		STATE_object_rotate_x_radians(), STATE_object_rotate_y_radians(), STATE_object_rotate_z_radians()
//		STATE_object_rotate_around_point()
//		STATE_object_set_rotation_center()
//		STATE_object_rotate_including_sons()
//
STATE_IMPORT void STATE_WINAPI  STATE_object_rotate_around_point(DWORD object_handle, double angle, int axis, double rotation_center[3]);

//*** could be sometimes useful
// Recenter the object polygons.
// Q: Why do we need this function ?
// A: Many operations like scaling the object are done around the object center
//		If we change the points of the object's polygons or move the polygons,
//		the object wont be anymore built arround 0,0,0 and so some operation like scaling might give
//		not the result that we expect.
//		by using this function the object is being rebuilt so that the polygons
//		point are defined around the object 0,0,0
//		The object will look the same way after calling this function as before !
//		but if calling STATE_object_get_location() you will get a different location
//		this is because the center is changed as well as the location of the polygons
//		but it is done in a way that the overall result is as if nothing has moved.
STATE_IMPORT void STATE_WINAPI STATE_object_recenter_polygons(DWORD object_handle) ;

//*** a very useful and powerful function
// This function uses Stencil shadow volumes techniques to automatically create shadows for the object.
// This is a similar technique to what is used in Doom3 (technique called Carmack's Reverse). There are many good articles 
// on the web describing different techniques for creating shadows. The good news is that you don't need to read any of them
// Currently only objects in Directx format (extension .x) can be used to create shadows.
//
// Parameters:
//		object_handle:
//			a handle to the object that cast a shadow
//
//		ON_or_OFF:
//			can be ON or OFF. If OFF is given no shadows will be created for the given object
//
//
//		number_of_light_sources:
//			A shadow can be created from one or more light sources.
//			The engine will automatically use the closest and strongest light sources.
//			Using two light sources doubles the rendering time. Using more than 4 light sources is very rarly done
//			Note that if we have 10 objects and each one of them is using 2 light sources then there will be between
//			2 to 20 light sources used for creating the shadows (depending whether the object are using the same light sources ....)
//			The only way to ensure there are no more than four light sources used is to make sure there are only 4 light sources defined in the world
//		
//
// Return value:
//		Returns OK or VR_ERROR. An error is returned if the object was not loaded from a directx file
//		for example STATE_object_create_from_file("monster.x"); currently only x files can be used to cast shadows
//		using this function.
//
// Remarks:
//		Note that even when the object is disable it will continue to produce shadows
//		This is because in many case a simplified disabled object is used to produce the shadow of high
//		detailed object. To disable a shadow call STATE_object_auto_add_shadows(my_object, OFF, 0);
//
//
// Tips:
//		1) Creating shadows is very time consuming. The more polygons in the object the more time it takes.
//		Therefor many times a more simplified object should be used for creating the shadows
//		The simplified object should never be rendered (always disabled STATE_object_disable()) 
//		but should always be in the same position and direction as the full detailed version.
//		
//
//		2) Note that the time it takes to render is linked directly to the number of light sources used.
//		It is advised for performance matters that when using this function for creating shadows, to use only 1 to 4 light sources in the world
//
//		3) Use STATE_light_set_distance_reach() to limit the effect of lights and thus minimize the number of source lights
//			used each render for calculating the shadows
//
//	The 3DSTATE Engine offers different techniques for creating shadows.
//  For static shadows one can use the lightmaps function to create shadows
//	The function STATE_engine_create_shadow() and STATE_engine_create_dynamic_shadow()
//	use a different methos to create shadows. To know what is apropriate for your specific implimentation you
//  should test both methods
//  
// See also: STATE_engine_create_dynamic_shadow(), STATE_engine_create_shadow()
//
// Examples
//
//		//disabling shadow creation for an object
//		A) STATE_object_auto_add_shadows(my_object, OFF, 0);
//
//		//Setting on automated shadow generation for this object
//		//One light source will be used. The engine will select the closest/strongest light
//		//acording to the location of the object
//		B) STATE_object_auto_add_shadows(my_object, ON, 1);
//
STATE_IMPORT int STATE_WINAPI STATE_object_auto_add_shadows(DWORD object_handle, int ON_or_OFF, int number_of_light_sources) ;

//Returs YES or NO according to whether automatic shadows are set for the given object
// See STATE_object_auto_add_shadows()
STATE_IMPORT int STATE_WINAPI STATE_object_is_auto_shadow_on(DWORD object_handle) ;

//*** could be needed sometimes
// This function redefines the object axis system accoring to its current position
// If you are moving a car on a track and it is going backwards you might want to have a look at this function
// Ulternatively to using this function you can open the object model in any 3D editor
// and turn it around till it is positioned they way you want it.
// 
// The 3DSTATE 3D Engine assumes that object that are loaded to the engine are constructed
// so that their back side points toward the positive X axis. Thier right side towards the positive Y axis
// and their top side towards the Z axis. Many times you use a ready made models that
// are not positioned in this way; this is OK because usually the engine doesn't care which is the front side and which is the back side of a car 
// or any other model. For the 3D Engine it is just a bunch of polygons. However some functions
// of the engine need to know what is the orientation of the object (what direction is back, which one is front, top etc ...)
// For example when using the STATE_object_chase_ function family.
// When the object is requested to chase a track (to follow a given path in the world) automatically
// for example STATE_object_set_chase_type(my_car, CHASE_TRACK) then the engine needs to know
// which is the front side and which is the back so it can move the car along the track with the front wheels leading the way ...
//
// This function is also useful whenever one wants to redefine the object axis system for conviniency matters
// or to make objects that come from different model library have the same orientation
//
// To use this function, first simply position / rotate the object till its back side is towards the positive X axis
// (a camera located at (X, 0, 0) and looking in the direction (-1,0,0) will see the back of the object ) 
// and then call the function to reset/redefined the object axis system
//
// See also the Scale.exe utility (part of the our 3D Developer Studio). This utility has an option to rotate an object and save it
// so one can use this utility instead of this function
//
// Example:
//		DWORD my_obj=STATE_object_create_from_file("my_car.3dstate");
//		STATE_object_rotate_z(my_obj, 180, WORLD_SPACE);
//		STATE_object_redefine_axis_system(my_obj);
//
STATE_IMPORT void STATE_WINAPI STATE_object_redefine_axis_system(DWORD object_handle) ;

//Returns the height of the center of the object above the ground.
//The height could also be negative this happens when the object is below the ground and there is nothig
//below the object. The function uses STATE_object_is_movement_possible() to calculate the height
STATE_IMPORT double STATE_WINAPI STATE_object_get_center_height_above_ground(DWORD object_handle, int fall_through_dynamic_objects) ;

// *** Might be usefull sometimes
//
// General description
// --------------------
//		Calculates the object bounding box bottom height above ground.
//		The object bounding box (see STATE_object_get_bounding_box() ) has 8 points
//		(like every box we know from real life ...) This function tests the height of
//		the four bottom points of the bounding box.
//		The function returns two numbers. The maximun height and the minimum height
//		Internally the function simply calls STATE_object_is_movement_possible()
//		four times ...
//
// Why do we need this function ?
// -------------------------------
//		For example lets say we want to put a house on a hillside
//		One way to do it is to take the point where the user clicked
//		(see STATE_engine_2D_point_to_3D_point() ) put the house above this point
//		and then move the house down so it's lpositioned correctly
//		The problem is that if we put the house on a hillside one side of the 
//		house would have a different height from the ground then the other side
//		(real life houses are always horizontal). In this situation no matter how much
//		down we drop the house it we will always have some part of the house
//		which are in the air or other parts which are below the ground
//		There is no perfect solution for this situation except to rebuild the
//		house in real-time according to the hill slop.
//		This function wont solve the problem it will only give you the
//		min and max height so you can decide how to continue.
//
// Return value:
//		Returns the polygon below the object at the point where height_max is obtained. Returns NULL if there is nothing below.
//
// Parameters:
//		[IN] object_handle: 
//				the object handle
//
//		[IN] fall_through_dynamic_objects:
//				YES or NO. If YES is given then the height of the object is the length along the world axis
//				between the bottom of the object and the top of the first object below the object
//				If NO is given here then the height is the length along the world axis
//				between the bottom of the object and the top of the first non-object below the object
//				note the the teminologies "object" and "dynamic object" are the same in 3DSTATE and could be
//				interchanged. They both mean an object that is dealt by the STATE_object_ API ...
//				
//
//		[OUT] height:
//				Through this parameter the function returns the calculated height above the ground
//			
//
// Example:
//		//an alternative to STATE_object_drop_down()
//		double min_height, max_height;
//		DWORD poly=STATE_object_get_bounding_box_height_above_ground(my_object, NO, &min_height, &max_height);
//		if(poly!=NULL)
//			STATE_object_move(my_object, WORLD_SPACE, 0,0, -max_height);
//		// Note that STATE_object_drop_down() would calculate the height
//		// according to the center of the object and not according the the max_height
//		// as shown here
// 
// See also: STATE_object_drop_down()
//
STATE_IMPORT DWORD STATE_WINAPI STATE_object_get_bounding_box_height_above_ground(DWORD object_handle, int fall_through_dynamic_objects, double *min_height, double *max_height);

//The default. The object is not rotated/turned automatically
#define OBJECT_ROTATED_TYPE_DISABLE 0 

//This is ideal for trees, people images, bilboards, posts, pillars 
//Specially useful with objects containing one polygon with a bitmap of a tree, a person etc ...
//The object direction is indipendent of the camera (the viewer) direction.
//it is only influenced by the camera location
#define OBJECT_ROTATED_TYPE_TREE 1 

//This is ideal for object that might also be looked at from above or for texts that need 
//specially useful with objects containing one polygon with a bitmap showing some text or symbols
#define OBJECT_ROTATED_TYPE_SYMBOL 2 

// *** quite useful 
// A rotated object is an object that is automatically positioned so
// that it always face the camera. 
// This technique is specially usefull if your object contains only one polygon and
// this polygon has a bitmap (with transparent background) of a tree, a person, a symbol, text etc ...
// One can of course do without this function and turn/rotate the object manually before each render
// but it is easier to use this function.
// An alternative for using rotated objects is to use rotated polygons
// The function STATE_object_set_rotated() was added only after version 7.39
// There are some advantages in using this function rather than rotated polygons.
//
// Changing the object side which faces the camera:
//		If the side of the object that is automatically facing the camera is not the one you want then 
//		do the following:
//		Option1 :
//			//This code should be called once for the specified object.
//			STATE_object_rotate_x/y/z(my_object, offset_angle, OBJECT_SPACE) 
//			STATE_object_redefine_axis_system() //redefining the object axis system according the former STATE_object_rotate_x/y/z() functions
//
//		Option2:
//			Before each render turn the object.
//			for example turn it by 90 degrees to get a different side
//
//		
//
// Parameters:
//			object_handle:
//				The object handle
//
//			rotated_type:
//				One of the following
//					OBJECT_ROTATED_TYPE_DISABLE, OBJECT_ROTATED_TYPE_TREE, OBJECT_ROTATED_TYPE_SYMBOL
//					See above for the specific meaning of each type
//
// See also 
//		STATE_object_redefine_axis_system(), STATE_polygon_set_rotated()
//
// Example
//		STATE_object_set_rotated(my_object, OBJECT_ROTATED_TYPE_SYMBOL);
//
STATE_IMPORT void STATE_WINAPI STATE_object_set_rotated(DWORD object_handle, int rotated_type);
//
// Returns the rotation type set by STATE_object_set_rotated()
// For more information see STATE_object_set_rotated()
STATE_IMPORT int STATE_WINAPI STATE_object_get_rotated_type(DWORD object_handle);

//This function is only relevant for md2 and anim3d objects.
//sometimes when there are many 3d animation objects updating them all every render could slow down the application
//Using this function you can tell the engine to update the animation every n renders.
//The default settings will update the animation every render.
// Note that when the engine updates the 3D animation it is done based on time so
// for example updating the animation every 4 renders instead of every render will not make the animation slower
// but it will make it less smooth.
//
// Parameters:
//		number_of_renders_to_wait:
//				Number of renders between each 3D animation update.
//
//		counter:
//				Usually this parameter is 0 but you can set it to a different number so that not all objects are updated at the same render
//				This parameter defins when will the next update happen.
//
//	
// Example:
//			//The animation update will occur every 4 renders with the first update delayed 2 renders from now.
//			//This means the first update will happen after 2 renders the second after 6 renders the third after 10 and so on.
//			STATE_object_set_3d_animation_update_rate(obj, 4,2);
//
STATE_IMPORT void STATE_WINAPI STATE_object_set_3d_animation_update_rate(DWORD object_handle, int number_of_renders_to_wait, int counter);

// *** might be usefull for advance users.
//Returns YES if the object is seen by the camera (inside the camera field of view)
//Returns NO if the object is not included in the camera field of view.
//Some times you might want to do your own specific calculations only when the object is seen
//(in order to save CPU time) in these cases you can use this function to check whether the object is seen.
STATE_IMPORT int STATE_WINAPI STATE_object_is_seen(DWORD object_handle, DWORD camera_handle);


//--------------------------------------------------------//
//=========== T H E    P O L Y G O N   A P I =============//
//--------------------------------------------------------//

// This API deals with polygons. Creating, setting the color or the bitmap etc ...

// Returns YES or NO.
// YES , if the given handle is a polygon handle. NO if not.
// If it is not a polygon handle , a message box will appear,
// the title of this message box will be the parameter "function_asking"
// if function_asking==NULL than no popup will be created.
// See also IS_POLYGON()
STATE_IMPORT int STATE_WINAPI STATE_polygon_is_polygon(DWORD polygon_handle, char *function_asking);


// Search for a polygon with the given name and
// returns the handle to the polygon.
STATE_IMPORT DWORD STATE_WINAPI STATE_polygon_get_handle_using_name(char *polygon_name);

// returns OK or VR_ERROR
STATE_IMPORT int STATE_WINAPI STATE_polygon_set_name(DWORD poly_handle, char *polygon_name);

// Returns the name of the polygon.
// The function returns a pointer to the name which is stored internally inside the 3D Engine
// Visual Basic users should use STATE_entity_get_name() instead.
// See also STATE_entity_get_name()
STATE_IMPORT char * STATE_WINAPI STATE_polygon_get_name(DWORD poly_handle);

STATE_IMPORT DWORD STATE_WINAPI STATE_polygon_get_handle_using_id_num(int poly_id_num);

// *** a very important function
// Using the functions STATE_polygon_get_first_polygon() and STATE_polygon_get_next()
// it is easy to go all over the polygons that exist inside the world
//
// Example:
//    	for(DWORD poly=STATE_polygon_get_first_polygon(); poly!=NULL; poly=STATE_polygon_get_next(poly) ) {
//			if(STATE_polygon_is_disabled(poly)==YES)
//				continue; //it is up to you if you want to process disabled polygons. All the functions will work on disabled polygons
//
//			DWORD object=STATE_polygon_check_if_belongs_to_an_object(poly);
//			if(object==NULL) {
//				//The polygons belong to the static part of the world
//				do something ...
//			}
//			else {
//				//The polygon belongs to a dynamic object
//				do something ...
//			}
//		}
//
STATE_IMPORT DWORD STATE_WINAPI STATE_polygon_get_first_polygon(void); 
STATE_IMPORT DWORD STATE_WINAPI STATE_polygon_get_next(DWORD polygon_handle); 

// If the diven polygon belongs to a dynamic object that the function will return
// the handle to that dynamic object. If the polygon belongs to the static part
// of the world, the function will return NULL.
// Remarks:
//		Sometimes a polygon can belong to several objects. This happens when
//		the function STATE_object_duplicate() is used with the flag
//		duplicate_polygons_flag==NO. In this case the created object will share the polygons of
//		the original copy. When a polygon belongs to several objects
//		the function STATE_polygon_check_if_belongs_to_an_object() will return
//		a handle to the original object that contains the polygons (also called the master object)
//
// Example:
//			DWORD object=STATE_polygon_check_if_belongs_to_an_object(poly);
//			if(object==NULL) {
//				//The polygons belong to the static part of the world
//				do something ...
//			}
//			else {
//				//The polygon belongs to a dynamic object
//				do something ...
//			}
//
STATE_IMPORT DWORD STATE_WINAPI STATE_polygon_check_if_belongs_to_an_object(DWORD polygon_handle);

//This function returns the polygon plane equation (Ax+By+Cz+D=0)
//The function returns the equation A,B,C,D. Note the (A,B,C) is the plane normal
//in other "words" N==( plane[0], plane[1], plane[2]).
//If the polygons belongs to a dynamic object, the function will return the plane in object space
//(which means that you will always get the same plane no mater how much the object moves or rotates)
//to get in world space also the planes of the polygons which belong to dynamic objects, call
// STATE_polygon_get_plane_extended.
// Example:
//		double plane[4];
//		STATE_polygon_get_plane(my_poly, plane);
//
// see also STATE_polygon_get_plane_extended()
STATE_IMPORT void STATE_WINAPI STATE_polygon_get_plane(DWORD polygon_handle, double plane[4]); 

// Exactly like STATE_polygon_get_plane() only that it also deals with polygons which belong to dynamic objects
// Parameters:
//		polygon_handle:
//			The polygon that we want to get its plane equation (or its normal)
//
//		object_handle:
//			The object which the polygon belongs to. NULL if this polygon belong to the static part of the world.
//
//		[out] plane:
//			Through this parameter the function returns the polygon plane.
//
// Remarks:
//		If object_handle==NULL then this function will act exactly as STATE_polygon_get_plane()
//		The difference between this function and STATE_polygon_get_plane() is that if the polygon belongs to
//		a dynamic object (object_handle!=NULL) then STATE_polygon_get_plane_extended() will return the polygon
//		plane in world space where as STATE_polygon_get_plane() will return the plane in object space.
//
// See also: 
//		STATE_polygon_get_plane();
//
// Example:
//		DWORD polygon, object;
//		double 3d_point[3]
//		STATE_engine_2D_point_to_3D(x, y, 3d_point, &object, &polygon);
//		double plane[4];
//		if(polygon!=NULL)
//			STATE_polygon_get_plane_extended(polygon, object, plane);
//
STATE_IMPORT void STATE_WINAPI STATE_polygon_get_plane_extended(DWORD polygon_handle, DWORD object_handle,double plane[4]);

// if red==-1 && green == -1 && blue== -1
// the current color will be used
STATE_IMPORT void STATE_WINAPI STATE_polygon_set_color_fill(DWORD polygon_handle, int red, int green, int blue); 


// returns OK or VR_ERROR
STATE_IMPORT int STATE_WINAPI STATE_polygon_set_bitmap_fill(DWORD polygon_handle, DWORD bitmap_handle);


// If the return is NULL then no bitmap is associated with the polygon.
// The return value is the name of the bitmap without the extention.
// The bitmap's name is the name+path relative to the path given in STATE_engine_load_world()
// If a bitmap exists in the world file but the engine couldnt find the bitmap file than it will still return its name.
// This is used when loading a world from the Internet, at first only the world is loaded
// and then we start streaming the bitmaps. We use this function to get the bitmap file name.
//If you want to check if the bitmap is in used then use STATE_polygon_get_bitmap_handle()
// Examples:
//
// A) 
//		char *name;
//		name=STATE_polygon_get_bitmap_name();
//
// B)
// The following example is relevent only when the world is saved without the SAVE_FLAT_BITMAP_PATH (not very common ...)
// In this case the bitmaps are not nessecerally in one directory.
//		the executable path is d:\projects\test
//		the bitmap path == d:\projects\test\bitmaps\animal\dog\boxer.bmp
//		the bitmap path given in STATE_engine_load_world() == "bitmaps"
//		the return string here will be == "animal\dog\boxer"
//
// See also STATE_bitmap_get_name()
STATE_IMPORT char * STATE_WINAPI STATE_polygon_get_bitmap_name(DWORD polygon_handle); 

// if the return is NULL then no bitmap is associated with the polygon
STATE_IMPORT DWORD STATE_WINAPI STATE_polygon_get_bitmap_handle(DWORD polygon_handle); 


STATE_IMPORT void STATE_WINAPI STATE_polygon_get_color(DWORD polygon_handle, int *red, int *green, int *blue); 


// Returns -1 if we transparent is not used for the given polygon
// else return the index to the transparent color, usually it
// will be 0 which means that the most used color is the transparent color
// if the return value is 1 it means that the second most popular
// color is the transparent color etc ...
STATE_IMPORT int STATE_WINAPI STATE_polygon_get_transparent_index(DWORD polygon_handle); 


// returns YES or NO
// a rotated polygon will always turn itself to
// face the direction of the camera.
// this technique is used to make 2D images look like a 3D object
// usually we will use this technique for trees animals etc ...
// Objects that look the same from all direction are excellent candidates
// (for example a tree or a vase) 
STATE_IMPORT int STATE_WINAPI STATE_polygon_is_rotated(DWORD polygon_handle); 

// For more information see STATE_polygon_is_rotated()
// Parameters:
//		yes_no_flag:	should be YES for setting the given polygon as rotated
//						NO to set it as a normal polygon
STATE_IMPORT void STATE_WINAPI STATE_polygon_set_rotated(DWORD polygon_handle, int yes_no_flag); 

// Obsolete. Not used anymore
// The given value should describe the percentage of fog increment per 1000 units
// Give 0 to cancel fog for a particular polygon.
// This might come handy when working with translucent polygons.
// Returns OK, VR_ERROR
STATE_IMPORT int STATE_WINAPI STATE_polygon_set_light_diminution(DWORD polygon_handle, double value); 
STATE_IMPORT double STATE_WINAPI STATE_polygon_get_light_diminution(DWORD polygon_handle); 

// return YES or NO
// light intensity values are used with bitmap filled polygons
// instead of using several bitmaps in different 
// brightness levels we can simply use this function
// we can also specify a different brightness value for each point
// of the polygon (using the POINT API functions)
// In that case the rendering of the polygon will be slower
// if all the points have the same brightness the rendering time
// is not damaged.
// relevant only for bitmap fill
// see also:
// STATE_polygon_get_brightness()
// STATE_polygon_set_brightness()
// STATE_point_get_brightness()
// STATE_point_set_brightness()
STATE_IMPORT int STATE_WINAPI STATE_polygon_is_uniform_brightness(DWORD polygon_handle); 

// If the brightness is not the same for all the 
// points we return the average.
// relevant only for bitmap fill
// see also:
// STATE_polygon_set_brightness()
// STATE_polygon_is_uniform_brightness()
// STATE_point_get_brightness()
// STATE_point_set_brightness()
STATE_IMPORT double STATE_WINAPI STATE_polygon_get_brightness(DWORD polygon_handle); 

// returns OK, VR_ERROR
// relevant only for bitmap fill
// 100 means the same brightness as the original bitmap
// 200 means twice as bright as the original bitmap
// 50 is half bright as the original
// see also:
// STATE_polygon_get_brightness()
// STATE_polygon_is_uniform_brightness()
// STATE_point_get_brightness()
// STATE_point_set_brightness()
STATE_IMPORT int STATE_WINAPI STATE_polygon_set_brightness(DWORD polygon_handle, int value); 


STATE_IMPORT int STATE_WINAPI STATE_polygon_get_num_of_points(DWORD polygon_handle); 

// deletes the polygon
// Note an attempt to use a handle after it was 
// deleted will crash the program.
// Deleting a polygon will automatically deletes all his points and all its patches polygons.
// Note that you can also call this function to delete a patch polygon
// see also STATE_polygon_delete_patches();
STATE_IMPORT void STATE_WINAPI STATE_polygon_delete(DWORD polygon_handle); 

// Note an attempt to use a handle after it was 
// deleted will crash the program.
STATE_IMPORT void STATE_WINAPI STATE_polygon_delete_point(DWORD polygon_handle, DWORD point_handle); 


// Checks if all the points are on the same plane
// if there are less than three points etc ... The function also sets internal info
// returns YES if the polygon is valid or NO (if not valid).
// If the answer is NO it is usually because the polygon is
// not convex or because the points are not on the same
// plane or the bitmap cords are not good or the light intensity cords are not good.
// When doing STATE_engine_add_polygon() this check is done
// automatically so it is impossible to add a bad polygon
STATE_IMPORT int STATE_WINAPI STATE_polygon_is_valid(DWORD polygon_handle);


//Return YES if the polygon is convex.
// returns NO if the polygon is concave
// all polygons added to the world must be convex
STATE_IMPORT int STATE_WINAPI STATE_polygon_is_convex(DWORD polygon_handle);
	
//Return YES or NO
// check that all points are on the same plane
// Will also return NO if there are less than 3 points in the poly, or if
// two of the three first points are identical.
STATE_IMPORT int STATE_WINAPI STATE_polygon_are_all_points_on_one_plane(DWORD polygon_handle);


//Returns YES or NO
STATE_IMPORT int STATE_WINAPI STATE_polygon_are_bitmap_x_cords_ok(DWORD polygon_handle);

//Returns YES or NO
STATE_IMPORT int STATE_WINAPI STATE_polygon_are_bitmap_y_cords_ok(DWORD polygon_handle);

//Returns YES or NO
STATE_IMPORT int STATE_WINAPI STATE_polygon_are_brightness_cords_ok(DWORD polygon_handle);



// returns the number of points that are on the same line
// 0 means no redundant points
STATE_IMPORT int STATE_WINAPI STATE_polygon_count_redundant_points(DWORD polygon_handle);
	

// remove points that are on the same line
// return the number of points that where removed
// a negative number means that an error occurred
STATE_IMPORT int STATE_WINAPI STATE_polygon_remove_redundant_points(DWORD polygon_handle); 



// Returns a handle to the first point of a polygon
// use STATE_point_get_next_point() the get the next points
//
//Example: 
//
//	//Go over all the points of a polygon
//	for(DWORD point=STATE_polygon_get_first_point(polygon_handle); point!=NULL ; point=STATE_point_get_next_point(point) ) 
//	{
//		
//
//	}
//
//
//Another way of looping through all the objects that are loaded into the engine is by using the FOREACH_POINT macro.
// Example:
//		FOREACH_POINT(poly, pnt) {
//			//do something with the point. For rxsmple lets check its coords
//			double XYZ;
//			STATE_point_get_xyz(pnt, XYZ);
//		}
#define FOREACH_POINT(poly,pnt) for(DWORD pnt=STATE_polygon_get_first_point(poly); pnt!=NULL; pnt=STATE_point_get_next_point(pnt))

STATE_IMPORT DWORD STATE_WINAPI STATE_polygon_get_first_point(DWORD polygon_handle); 

//Get the handle to the last added point of the polygon.
//This is very useful for getting the handle of a point after 
//calling the function STATE_polygon_add_point();
//
// Example:
//
//	DWORD poly=STATE_polygon_create()
//	double point[6]={1,2,3, 0,0, 199};
//	STATE_polygon_add_point(poly, point);
//  //Now if we want to set the point light/color we need to get the point handle
//  //To get the handle to the last point of the polygon (which is the last point we added) we use:
//	point_handle=STATE_polygon_get_last_point(poly);
//	BYTE rgb[3]={255,0,0}; //red color
//	STATE_point_set_rgb(point_handle, rgb);
//
DWORD STATE_stdcall STATE_polygon_get_last_point(DWORD polygon_handle);


// Creates a new empty polygon.
// Returns a handle to the new polygons
// or NULL if an error occurred.
// Note ! the polygon won't be seen or checked for errors 
// till you do STATE_engine_add_polygon()
// Use STATE_polygon_add_point() to add points to the created polygon.
STATE_IMPORT DWORD STATE_WINAPI STATE_polygon_create(void); 


// returns a handle to the new point
// or NULL if an error occurred.
// Adds the point as the last point (add to tail)
// See also STATE_polygon_get_last_point()
STATE_IMPORT DWORD STATE_WINAPI STATE_polygon_add_point(DWORD polygon_handle, double xyz_bmX_bmY_brightness[6]); 

// returns a handle to the new point
// or NULL if an error occurred.
// Add the point as the first point (add to head)
STATE_IMPORT DWORD STATE_WINAPI STATE_polygon_add_point_to_head(DWORD polygon_handle, double xyz_bmX_bmY_brightness[6]); 


STATE_IMPORT void STATE_WINAPI STATE_polygon_rotate_bitmap(DWORD polygon_handle); 

// Rotates the polygon bitmap. A rotation of one means shift texture coords one position.
// the rotation is done in absolute units so that calling STATE_polygon_set_bitmap_rotation(poly,0);
// will always reset the bitmap to its original position.
// Parametrs:
//		rotation_count
//			defines how many vertices to shift the bitmap coords. The number given is in absolute values so that 
//			calling with the same value will always give the same results no matter how many rotation have been done before.
//			rotation_count could also be negative
//
// Examples:
//		A)
//		//rotates the bitmap so that the texture coords of the second becomes the texture coords of the first point, and so on.
//		STATE_polygon_set_bitmap_rotation(poly, 1);
//
//		B)
//		//reset the bitmap to its priginal position.
//		STATE_polygon_set_bitmap_rotation(poly, 0);
//
STATE_IMPORT void STATE_WINAPI STATE_polygon_set_bitmap_rotation(DWORD polygon_handle, int rotation_count) ;

STATE_IMPORT void STATE_WINAPI STATE_polygon_mirror_horizontal_bitmap(DWORD polygon_handle); 
STATE_IMPORT void STATE_WINAPI STATE_polygon_mirror_vertical_bitmap(DWORD polygon_handle); 


// the rotation is around the center of the polygon
STATE_IMPORT void STATE_WINAPI STATE_polygon_rotate_x(DWORD polygon_handle, double degrees); 
STATE_IMPORT void STATE_WINAPI STATE_polygon_rotate_y(DWORD polygon_handle, double degrees); 
STATE_IMPORT void STATE_WINAPI STATE_polygon_rotate_z(DWORD polygon_handle, double degrees); 
// for general rotation operations.
STATE_IMPORT void STATE_WINAPI STATE_polygon_rotate(DWORD polygon_handle, double trans_mat[3][3]);

STATE_IMPORT void STATE_WINAPI STATE_polygon_move(DWORD polygon_handle, double step[3]); 

// the location is according to the center of the polygon
STATE_IMPORT void STATE_WINAPI STATE_polygon_set_location(DWORD polygon_handle, double location[3]); 

// The location is according to the center of the polygon
// Note that if this polygon belongs to a dynamic object then you
//	have to add the location of the dynamic object to the result.
//	(This note is not relevant in editor mode)

// The location is according to the center of the polygon
// A note regarding the function STATE_object_duplicate() :
// Note that this function will fail if the polygon belongs to an object
// that was duplicated with the "dont_duplicate_polygons_flag"
// In this case we will get the location of the polygon on the original copy of the object
STATE_IMPORT void STATE_WINAPI STATE_polygon_get_location(DWORD polygon_handle, double location[3]); 

// the scaling is according to the center of the polygon 
STATE_IMPORT void STATE_WINAPI STATE_polygon_scale(DWORD polygon_handle, double scale_x, double scale_y, double scale_z); 



// reverse the direction in which the polygon is visible
// remember that all the polygons have only one side which is visible.
STATE_IMPORT void STATE_WINAPI STATE_polygon_flip_visible_side(DWORD polygon_handle); 


// tries to join the two polygons to one polygon.
// if it is possible it deletes the two polygons create a new
// one and return the handle to the new one
// on failure returns NULL.
STATE_IMPORT DWORD STATE_WINAPI STATE_polygon_join(DWORD polygon1_handle, DWORD polygon2_handle); 

// The bitmap cords of the target poly are set so they match the source_poly bitmap cords
// If the polys are adjacent calling this function would make the move from
// polygon to another smooth on the connection. If the polys are not on the same
// plane then the connection line will be smooth but the bitmap density would be
// different. 
// returns OK unless an error occurred (like a polygon with  2 points was received etc ...)
// returns VR_ERROR in case of error.
STATE_IMPORT int STATE_WINAPI STATE_polygon_match_bitmap_cords(DWORD source_polygon_handle, DWORD target_polygon_handle);
 

// Duplicates a polygon.
// returns a handle to the new created polygon.
// Note that in order to see the new polygon it should be added to the engine
// by calling STATE_engine_add_polygon() or STATE_object_add_polygon()
STATE_IMPORT DWORD STATE_WINAPI STATE_polygon_duplicate(DWORD polygon_handle); 

// The given polygon is rotated so it will be parallel to 
// "reference_polygon_handle" (the second argument);
// return OK or VR_ERROR.
STATE_IMPORT int STATE_WINAPI STATE_polygon_rotate_to_match_polygon(DWORD polygon_handle, DWORD reference_polygon_handle);

// The given polygon is moved so his given point will match
// "point_to_match" (the third argument);
// return OK or VR_ERROR.
STATE_IMPORT void STATE_WINAPI STATE_polygon_move_to_match_point(DWORD polygon_handle, DWORD point_belongs_to_polygon, DWORD point_to_match);


// Gets a point in space. returns the handle the vertex of the given polygon that
// is the closest vertex to the given point.
// One important use of this function is in conjunction with STATE_engine_2D_point_to_3D()
// When calling STATE_engine_2D_point_to_3D() keep the 3d point and call this
// function with the 3d point.
STATE_IMPORT DWORD STATE_WINAPI STATE_polygon_get_closest_point(DWORD polygon_handle, double p3d[3]);

//This function is not very important.
// Gets a point in space. returns the handle to the vertex of the given polygon that
// is the furthest away from the given point.
//See also STATE_polygon_move_to_match_point()
STATE_IMPORT DWORD STATE_WINAPI STATE_polygon_get_furthest_point(DWORD polygon_handle, double p3d[3]);


//---------------------------------//
//-Polygons operations with groups-//
//---------------------------------//

// set the group that this polygon belongs to.
STATE_IMPORT void STATE_WINAPI STATE_polygon_set_group(DWORD polygon_handle, DWORD group_handle);
STATE_IMPORT DWORD STATE_WINAPI STATE_polygon_get_group(DWORD polygon_handle);

// returns YES or NO
STATE_IMPORT int STATE_WINAPI STATE_polygon_is_in_group(DWORD polygon_handle, DWORD group_handle);

// What is orientation and why do we need it.
// When we want to drop an object\group on the ground (another surface) we need to know
// what surface of the object\group should be glued to the ground.
// The computer of course doesnt know on his own that a piano should stand on its legs ...
// With this flag the designer of the model can express his will that the piano should stand on its legs ...
//
// In order to fully define the orientation of a group, one polygon of the group should have ORIENTATION_BOTTOM or ORIENTATION_top
// and a second polygon should have ORIENTATION_FRONT or ORIENTATION_BACK beside those two polygons all the rest
// should have ORIENTATION_UNKNOWN. Nothing bad happens if they dont, it is just that it can cause contradict.
//
// See also STATE_group_set_orientation()
STATE_IMPORT void STATE_WINAPI STATE_polygon_set_orientation(DWORD polygon_handle, int orientation_value);

//see also STATE_group_get_orientation()
STATE_IMPORT int STATE_WINAPI STATE_polygon_get_orientation(DWORD polygon_handle);

//Note that this function works only if the polygon is convex.
//The function doesn't check whether the point is on the polygon plane
// to check if the point is on the plane first use STATE_polygon_are_all_points_on_one_plane()
//
// actually this STATE_polygon_is_point_inside_polygon determines whether a point is inside the infinity cylinder\tube shape that
// the given polygon is its profile.
// So we can also call STATE_polygon_is_point_inside_polygon() with a point that is not 
// on the plane the function 
//Returns YES or NO.
STATE_IMPORT int STATE_WINAPI STATE_polygon_is_point_inside_polygon(DWORD polygon_handle, double pnt[3]);

//The function doesn't check whether the point is on the polygon plane
//Use this function if the polygon could also be concave, otherwise use STATE_polygon_is_point_inside_polygon()
//Returns YES or NO.
STATE_IMPORT int STATE_WINAPI STATE_polygon_is_point_inside_polygon_concave(DWORD polygon_handle, double pnt[3]);

STATE_IMPORT int STATE_WINAPI STATE_polygon_set_animation(DWORD polygon_handle, DWORD animation_handle);

STATE_IMPORT DWORD STATE_WINAPI STATE_polygon_get_animation(DWORD polygon_handle);

// Assuming that there is an animation associated with the given polygon
// then this function makes it possible to jump to a specific frame. 
// If animation_frame==-1 then the last frame will be chosen.
// If the given animation_frame is bigger than the last frame then the last frame will be chosen.
// The first frame is numbered as 0
STATE_IMPORT void STATE_WINAPI STATE_polygon_set_animation_frame(DWORD polygon_handle, int animation_frame);
STATE_IMPORT int STATE_WINAPI STATE_polygon_get_animation_frame(DWORD polygon_handle);

// calculate the center and returns in parameter "center"
// The location is returend through the center variable
// The location is always in WORLD_SPACE even if the polygon belongs to
// a dynamic object.
// A note regarding the function STATE_object_duplicate() :
// Note that this function will fail if the polygon belongs to an object
// that was duplicated with the "dont_duplicate_polygons_flag"
// In this case we will get the location of the polygon on the original copy of the object
STATE_IMPORT void STATE_WINAPI STATE_polygon_get_center(DWORD polygon_handle, double center[3]);

//YES or NO. If NO the polygon wont be saved when doing STATE_engine_save() with the SAVE_RELEASE flag
//One can also use this function for simply marking polygons with boolean value,
//since the function is very fast and has no effect beside when doing save
STATE_IMPORT void STATE_WINAPI STATE_polygon_set_release_save_flag(DWORD polygon_handle, int YES_or_NO);

STATE_IMPORT int  STATE_WINAPI STATE_polygon_get_release_save_flag(DWORD polygon_handle);

// Polygons that are disabled are not drawn and are
// do not take part in collision detection.
// All the API functions can work on disable polygons the same way they work on enabled ones
//
//Note the following cases:
// The polygon is not visible and does not take part in collision detection ==> use STATE_polygon_disable
// The polygon is     visible and does not take part in collision detection ==> use STATE_polygon_make_non_collisional()
// The polygon is not visible takes part in collision detection ==> name the polygon $$TRANSPARENT that will tell the engine not to draw it
//
// Naming the polygon $$TRANSPARENT (not seen but participate in collision detection) is useful in order to restrict the player
// from getting out of certain areas. They act like transparent walls.
STATE_IMPORT void STATE_WINAPI STATE_polygon_disable(DWORD polygon_handle);

//for more details see STATE_polygon_disable()
STATE_IMPORT void STATE_WINAPI STATE_polygon_enable(DWORD polygon_handle);

//returns YES if the polygon is disabled. NO if it is enabled
STATE_IMPORT int STATE_WINAPI STATE_polygon_is_disabled(DWORD polygon_handle);

// Ignore this polygon for collision detection 
// purposes. This means that when STATE_object_is_movement_possible()
// will be called ( or STATE_engine_is_movement_possible() ) they would never get collision with
// this polygon. This function is useful for polygons representing clouds, smoke, curtains ,grass etc ...
// The default is that all the polygons are collisional
// see also STATE_polygon_make_collisional() ,STATE_polygon_is_collisional(), STATE_polygon_disable()
//Note the following cases:
// The polygon is not visible and does not take part in collision detection ==> use STATE_polygon_disable
// The polygon is     visible and does not take part in collision detection ==> use STATE_object_make_non_collisional()
// The polygon is not visible takes part in collision detection ==> name the polygon $$TRANSPARENT that will tell the engine not to draw it
//
// Naming the polygon $$TRANSPARENT (not seen but participate in collision detection) is useful in order to restrict the player
// from getting out of certain areas. They act like transparent walls.
STATE_IMPORT void STATE_WINAPI STATE_polygon_make_non_collisional(DWORD polygon_handle);

//This is the default. see also STATE_polygon_make_non_collisional() and STATE_polygon_is_collisional()
STATE_IMPORT void STATE_WINAPI STATE_polygon_make_collisional(DWORD polygon_handle);

//Return YES or NO. see also STATE_polygon_make_collisional() and STATE_polygon_make_non_collisional()
STATE_IMPORT int STATE_WINAPI STATE_polygon_is_collisional(DWORD polygon_handle);

// ------------------------------------------------------
// | Constants for glass types used for blending		|
// | with the function STATE_polygon_set_translucent() |
// | Note that these constants are only relevant when   |
// | hardware rendering is used.								|
// ------------------------------------------------------
//
// One doesnt really have to read the explanations (though it
// is quite simple)	, you can test the different options and 
// pick the one you like.

//The default. The polygon will be 100% opaque.
#define DISABLE_BLENDING 0
	
// Filter glass can be used to block rgb channels
// like when looking through a cellophane paper
// For example if the color of the polygon is (red==255,0,0)
// Then when looking through this polygon the whole world will
// be painted with red. The blending formula for DARK_FILTER_GLASS
// is NEW_PIXEL=SRC*DEST  (SRC is the pixel of our polygon. DEST
// is the pixel in the background and NEW_PIXEL is final result.
// Example: We have a room with a window. We look outside through 
//		the window. Lets examine the rendering process
//		when rendering a specific pixel of the window.
//		The computer takes a pixel from the window 
//		(for example 0,0,64 == dark blue == 0, 0, 0.25 in [0-1] rgb scale)
//		and another pixel from outside the room that is exactly on the
//		line connecting between the eye (camera) and the pixel of the window
//		for example lets say that this line hits a cloud so our second pixel
//		is gray (rgb== 128,128,128 == 0.5, 0.5, 0.5 in [0-1] rgb scale) now
//		lets calculate the final pixel that we will have on the window
//		according to the function NEW_PIXEL=SRC*DEST == (0,0, 0.25)*(0.5, 0.5, 0.5)
//		== 0*0.5, 0*0.5, 0.25*0.5= (0,0,0.125) == (0,0,32 in [0-255] scale) 
//		== very dark blue.
//		This blending formula is called DARK_... because the result pixel is
//		darker or equal to the src pixel
#define DARK_FILTER_GLASS			0x31

//The blending formula for FILTER_GLASS is NEW_PIXEL=2*SRC*DEST
//This formula is usually better than DARK_FILTER_GLASS since
// the result pixel is less dark.
#define FILTER_GLASS				0x33

//The blending formula for OPAQUE_FILTER_GLASS is 
// NEW_PIXEL=SRC*SRC + SRC*DEST. It is similar to FILTER_GLASS
// It is called OPAQUE_... since it is a little bit more opaque than
// FILTER_GLASS
#define OPAQUE_FILTER_GLASS			0x43

// Create the effect of normal glass. Here is what happens when looking through 
// this glass: The light from bright objects easily penetrates the window
// while the light from darker objects is reflected from the glass (in this case 
// you get to see the bitmap of the window and not what is outside ...)
// Using this type of glass you can control the level of translucency
// Bright pixel in the bitmap will be totally opaque while dark pixel will
// blend with the background. Make a bright bitmap if you want your window
// to be more opaque. Make a dark bitmap if you want it to be more transparent.
// Note this side effect, the view outside the window will always look brighter
// than what it really is. This effect simulate our eye behavior. When we sit
// in a dark room and we open a window we are overwhelmed by the light. Once we get outside,
// we get used to it and it doesnt seem to be as bright as we felt before
// Here is the blending formula for this window:
// NEW_PIXEL= SRC(1-DEST)+DEST. Note that if DEST=(R,B,G) than 1-DEST is (1-R, 1-G,1-B)
// when working in [0-1] scale.
#define NORMAL_GLASS				0x52

// It is the only glass
// that make a real blending without decreasing or increasing the light.
// It is also the only one who gives a total control on the level of
// transparency, from being total opaque to being total transparent.
// Bright pixel in the bitmap will be totally opaque while dark pixel will 
// be transparent. Make a bright bitmap if you want your window
// to be more opaque. Make a dark window if you want it to be more transparent.
// The blending formula for this glass is
// NEW_PIXEL=SRC*SRC+DEST*(1-SRC)
#define CONTROL_GLASS				0x44

// Has the side effect of making the outside look brighter
// It is called transparent because the glass is a little bit
// more transparent than the other types.
// A dark bitmap (for the window) will make a very transparent glass
// a bright bitmap will make a bright blend between the window and the outside.
// Here is the blending formula:
// NEW_PIXEL=DEST+SRC*DEST
#define BRIGHT_TRANSPARENT_GLASS	0x32

//Similar to BRIGHT_TRANSPARENT_GLASS only a little bit less transparent
//Here is the blending formula:
// NEW_PIXEL=DEST+SRC*SRC
#define BRIGHT_TRANSLUCENT_GLASS	0x42

//similar to BRIGHT_TRANSLUCENT_GLASS only that
//the glass is more opaque. This glass can also be used to block color channels
//like with the filter modes (FILTER_GLASS).
//Here is the blending formula:
// NEW_PIXEL=SRC+SRC*DEST
#define BRIGHT_OPAQUE_GLASS			0x23

//This glass is special. It has the
//simple blending formula NEW_PIXEL=SRC+DEST
//Usually this type of blending is used for
//simulating light because of its additive characteristic
//If used as a window glass then we should use a dark bitmap
//otherwise we will have a very bright saturated window.
#define LIGHT_SOURCE_GLASS			0x22

//This glass creates a special effect. When looking
//through it, it inverts the world that we see.
//The blending formula is:
//NEW_PIXEL=SRC*(1-DEST).Note that if DEST=(R,B,G) than 1-DEST is (1-R, 1-G,1-B)
// when working in [0-1] scale.
#define INVERSE_GLASS				0x51

// one of the most useful filters
// Using this filter one can control the transparency of the polygon very accurately 
// using the vertices alpha channel or using textures with alpha component
// See STATE_point_set_alpha()
#define ALPHA_FILTER 0x56

//Makes the polygon transparent as if it was glass
//Above are the different types of glass.
//call this function with DISABLE_BLENDING to
//change to normal rendering (100% opaque)
//The default is DISABLE_BLENDING
//Example: STATE_polygon_set_translucent(my_poly,NORMAL_GLASS);
// Note that it will only work with hardware rendering 
// ( see STATE_3D_card_check_hardware_support())
STATE_IMPORT void STATE_WINAPI STATE_polygon_set_translucent(DWORD polygon_handle, BYTE glass_type);

//see STATE_polygon_set_translucent()
STATE_IMPORT BYTE STATE_WINAPI STATE_polygon_get_translucent(DWORD polygon_handle);

//Patches are polygons that are always in front of their father polygon
//For example if we shoot a wall in a game then we can add a patch polygon to
//make it look like a whole in the wall. Another example will be in a racing car
//game when we want to leave wheel marks on the road. We can easily do that
//by using STATE_engine_is_movement_possible() to get the polygon of the road
//that is under the wheel and then to use this function to paste a wheel mark
//on the road polygon.
// The first parameter to this function is the polygon that we want to paste the
// patch on it.
// The second argument is the patch it self.
// Note that it is your responsibility to set the location and bitmaps of
// the patch polygon.
// See also STATE_polygon_add_patch_easy() which saves you all the calculations.
STATE_IMPORT void STATE_WINAPI STATE_polygon_add_patch(DWORD polygon_handle, DWORD patch_polygon_handle);

// For explanation about patches see STATE_polygon_add_patch()
// This function is similar to STATE_polygon_add_patch()
// only that it is much easier to use. For example if we shoot a wall in a game then we can add a patch polygon to
// make it look like a whole in the wall. With this function you just
// have to specify the point on the wall where you want the patch to be put,
// the size of the patch and the bitmap that it will be using.
// The point and the polygon where your bullet hit the wall can be easily obtained
// by using STATE_engine_is_movement_possible()
//
// Parameters
// ----------
// father_polygon
//			The polygon on which we want to place a patch
//			NOTE: the pach will inherit the father_polygon properties
//				  so if the father polygon is translucent the patch will also be translucent
//				  if the father_polygon is rotated so will be the patch
//				  If you want the patch to have different properties you
//				  have to set them after the patch was created. For example:
//				  double direction[3]={0,0,1};
//				  int rgb[3]={255,0,0} ; //red
//				  DWORD patch=STATE_polygon_add_patch_easy(father_polygon, center, radius, direction, bitmap_handle,rgb);
//				  STATE_polygon_set_rotated(patch,NO); //making sure that the patch wont be rotated
//				  STATE_polygon_set_translucent(patch,DISABLE_BLENDING); //making sure that the patch wont be translucent
//				  A note regarding the function STATE_object_duplicate():
//				  This function will fail if the father_polygon belongs to an object
//				  that was duplicated with the "dont_duplicate_polygons_flag" set
//					
// center
//			A point on the father polygon that will be the center of the patch
// radius
//			Used to specify the size of the patch. Note that the patches are actually
//			square polygons. This parameter gives the distance between the center of the square and one of the
//			square edges.
// direction
//			The direction of the patch. For example, if we have a car racing game
//			and we want the wheels to leave marks then we would like the wheel mark
//			to be in the direction of the car. So in this case we would give the direction
//			of the car as the direction of the patch. If you dont know, or if you dont care
//			about the direction you can give direction={0,0,0}.
// texture  
//			The texture that is used to paint the patch.
//			If NULL then the rgb color will be used
// rgb
//			Used only when texture==NULL. Specifies the color
//			rgb[0] is red. rgb[1] is green and rgb[2] is blue .
//
//
// Return value: returns a handle to the created patch polygon.
//
// See also STATE_engine_set_patches_offset()
STATE_IMPORT DWORD STATE_WINAPI STATE_polygon_add_patch_easy(DWORD father_polygon, double center[3], double radius, double direction[3], DWORD bitmap_handle,int rgb[3]);



#define ALL_PATCHES 1 //delete all the patches of a given polygon
#define REGULAR_PATCHES 2 //delete all the regular patches of a given polygon (patches that were mad using the function STATE_polygon_add_patch() and STATE_polygon_add_patch_easy() )
#define SHADOW_PATCHES 4 //delete all the shadow patches of a given polygon (patches that were created in the process of creating shade

//Delete patches of the given polygon. If the given polygon is a patch in itself
//then the function will return immediately (without deleting)
// Example A:
//		//Deleting all the patches of a given polygon
//		STATE_polygon_delete_patches(my_polygon, ALL_PATCHES);
//
// Example B:
//		//Deleting all the patches that were created using the functions
//		//STATE_polygon_add_patch() and STATE_polygon_add_patch_easy()
//		STATE_polygon_delete_patches(my_polygon, REGULAR_PATCHES);
//  
// Example C:
//		Delete all the patches that were created using the function STATE_engine_create_shadow()
//		STATE_polygon_delete_patches(my_polygon, SHADOW_PATCHES);
//
// Example D:
//		Delete patches that were created using the function STATE_engine_create_shadow() with a serial_number==220
//		STATE_polygon_delete_patches(my_polygon, 220);
//
// See also: STATE_engine_create_shadow(), STATE_polygon_add_patch() ,STATE_polygon_add_patch_easy()
//			 STATE_group_delete_patches()
STATE_IMPORT void STATE_WINAPI STATE_polygon_delete_patches(DWORD polygon_handle, int type_of_patches_to_delete_or_serial_number);


//Returns the patch type or serial number. This function can be used to decide whether a certain polygon
//is a patch or not. If the polygon is not a patch then 0 is returned.
//to learn more about the different types of patches please see
// STATE_polygon_delete_patches(), STATE_engine_create_shadow(), STATE_polygon_add_patch() ,STATE_polygon_add_patch_easy()
// STATE_group_delete_patches()
STATE_IMPORT int STATE_WINAPI STATE_polygon_get_patch_type(DWORD polygon_handle);


//Checks whether the given line segment intersect the given polygon.
//Returns YES if there is an intersection. Returns NO otherwise.
//The line segment is define by two points: double P1[3] and double P2[3]
//If there is an intersection the intersection point is saved within double intersection[3]
// Wont work in VIEWER_MODE with polygons that belong to a dynamic object (See the STATE_object API)
STATE_IMPORT int STATE_WINAPI STATE_polygon_check_intersection_with_line_segment(DWORD polygon_handle, double P1[3], double P2[3], double intersection[3]);


//Checks whether the two given polygons intersect each other.
//Returns YES if there is an intersection. Returns NO otherwise.
//If there is an intersection the intersection point is saved within double intersection[3]
// Wont work in VIEWER_MODE with polygons that belong to a dynamic object (See the STATE_object API)
STATE_IMPORT int STATE_WINAPI STATE_polygon_check_intersection_with_another_polygon(DWORD polygon1, DWORD polygon2, double intersection[3]);

//----------------------------------------------------------------------------
//=== Polygon function that has to do with light. See also the Light API =====
//----------------------------------------------------------------------------

// There are four properties that influence how light effects polygons, these are ambient, diffuse specular and specular_shining.
// Every defined light that is activated (see the light API) is combined internally of
// three types of lights (ambient, diffuse and specular)
// By using these functions one can modify how a surface react with each type of light.
// Different materials react differently For example
// materials that tend to reflect light better (metals, marble etc ...)
// will have higher values of the specular property. 
// The value given should be a positive number
// The default value is 86. Typical values are between 0 to 500
// Feel free to play with those values till you get the desired effect
// Here is some more info about the light properties
// Specular:
// The specular property defines for a surface how close it is to an ideal reflecting surface
// A mirror for example is a perfect reflecting surface. 
// Different materials have different values .For example
// materials that tend to reflect light better (metals, marble etc ...)
// will have higher values of the specular property.
// Diffuse:
// Diffuse light comes from a particular direction but is reflected evenly of the surface
// Dull surfaces like solid matte plastic exhibit what is called diffuse reflection
// Ambient:
// Ambient light is light that doesn't come from any particular direction
// Objects illuminated by ambient light are evenly lit on all surfaces.
STATE_IMPORT void STATE_WINAPI STATE_polygon_set_ambient(DWORD polygon_handle, int value);
STATE_IMPORT int STATE_WINAPI STATE_polygon_get_ambient(DWORD polygon_handle);
STATE_IMPORT void STATE_WINAPI STATE_polygon_set_diffuse(DWORD polygon_handle, int value);
STATE_IMPORT int STATE_WINAPI STATE_polygon_get_diffuse(DWORD polygon_handle);
STATE_IMPORT void STATE_WINAPI STATE_polygon_set_specular(DWORD polygon_handle, int value);
STATE_IMPORT int STATE_WINAPI STATE_polygon_get_specular(DWORD polygon_handle);
// The shining property influences only the specular light
// The default is 1. Typical values are between 0 to 10
// The lower this value is the bigger the light spot created from the specular lighting will be.
STATE_IMPORT void STATE_WINAPI STATE_polygon_set_specular_shining(DWORD polygon_handle, BYTE value);
// The shining property influences only the specular light
// The default is 1. Typical values are between 0 to 10
// The lower this value is the bigger the light spot created from the specular lighting will be.
STATE_IMPORT BYTE STATE_WINAPI STATE_polygon_get_specular_shining(DWORD polygon_handle);

//Remove light from the polygon.
//If the given polygon was lit using the STATE_light API then
//this function could be used to restore the polygon normal appearance
//See also STATE_object_remove_light(), STATE_group_remove_light(), STATE_light_remove_light()
STATE_IMPORT void STATE_WINAPI STATE_polygon_remove_light(DWORD polygon_handle);

//Exactly like STATE_group_wrap_a_bitmap_fixed(0 but just for one polygon.
// See also STATE_group_wrap_a_bitmap()
// returns OK or VR_ERROR
//
// If the polygon belongs to an object then father_object should be the handle of this object [IN]
// otherwise it should be set to NULL.
STATE_IMPORT int STATE_WINAPI STATE_polygon_wrap_a_bitmap_fixed(DWORD polygon_handle, DWORD father_object,DWORD bitmap_handle, int repeat_x, int repeat_y, double locationXY_to_get_bitmap_top_left_corner[2], double locationXY_to_get_bitmap_bottom_right_corner[2]);

//A very useful function.
//Set the light values of each vertex directly
//Actually this function merely calls STATE_point_set_rgb() for all the points of the given polygon
//Here are some uses of this function:
//1) make a polygon brighter or darker or to alter its color.
//2) Set different colors to each vertex (If a texture is used for this polygon
//	 the the color of the vertixes will be mixed with the color of the texture
//	 (a multiply mix so one can never make the texture brighter than the origibal)
//3) The light API actually creates the illusion of light by manipulating the color of the
//	 vertixes. One can use this function (together with STATE_point_set_rgb) to create an alternative light mechanism
//	 though usually it is easier to use the light API to set the vertices color.
//4) Create nice effects such as fading the polygon to the color of the background.
//
//See also STATE_point_set_rgb()
STATE_IMPORT void STATE_WINAPI STATE_polygon_set_light(DWORD polygon_handle, BYTE rgb[3]);


//Creates two new polygons. The handles to the newly created polygons are returned
//through the arguments back_piece and front_piece.
//The polygon is divided into two pieces according to the given split_plane.
//You can use the function STATE_math_points_to_plane() to create the splitting plane
// Note that the two created polygons are not added to the engine
// so they wont be rendered until STATE_engine_add_polygon() is called
//Returns OK or VR_ERROR.
//If the given plane does not intersect the given polygon then the function will return NULL
//for both back_part and front_part. In this case the return value will be OK.
//
//
// Example:
//		
//		DWORD back_part=NULL;
//		DWORD front_part=NULL;
//		int rc=STATE_polygon_split(my_poly, splitting_plane, &back_part, &front_part);
//
STATE_IMPORT int STATE_WINAPI STATE_polygon_split(DWORD polygon_to_split, double split_plane[4], DWORD *front_piece, DWORD *back_piece);


// Sets the second bitmap of the polygon. When rendering, the second
// bitmap will be blended into the first one.
// When the first bitmap is not used (for example after calling
// STATE_polygon_set_color_fill() ) then also the second bitmap wont be used.
// See also STATE_polygon_create_lightmap()
// returns OK or VR_ERROR
STATE_IMPORT int STATE_WINAPI STATE_polygon_set_second_bitmap(DWORD polygon_handle, DWORD texture_handle);
STATE_IMPORT DWORD STATE_WINAPI STATE_polygon_get_second_bitmap(DWORD polygon_handle);


// *** very important
// General information about lightmaps and their use in the 3DSTATE Engine.
//--------------------------------------------------------------------------
// Lightmaps are very important for achieving more
// realistic graphics quality. Lightmaps are regular bitmaps. The reason they are called
// lightmaps is because they contain information regarding shadowed and lit areas. 
// Although they are mainly used for creating light and shadows, they can also be used for other special effects.
// Since version 4.2 polygons can have two bitmaps. Usually the second bitmap is a lightmap.
// When rendering using the 3D card the two bitmaps will be blended into one. 
// The blending is done by multiplying one bitmap with another according to this formula:
// For each pixel new_rgb=rgb1*rgb2/255;
// For example  rgb1=(255, 10, 10) == bright red.
//				rgb2=(128, 128,128) ==grey.
//				new_rgb= (128, 5,5) == half of rgb1 == a darker red.
//
// Note that the new_rgb will never be brighter than rgb1 or rgb2.
//
// Here is an example of common use of lightmaps:
// Assume we have a small brick bitmap tiled on the floor in our virtual world.
// This usually looks unrealistic since in the real world we will have many light
// and shadow effects. Light from a lamp, from the door, from an openedd window etc ...
// In the real world we should also have many shadows. If there are chairs in the room
// they should create some shadow on the floor and so on.
// The idea is that we take an additional bitmap that will include information
// about the lit and shadowed area. This bitmap is not tiled. The combination
// of the lightmap bitmap and our tiled brick bitmap will give a realistic
// image of the floor.
//
// Q: How do we create this second bitmap that has all the information about 
//	  the lit and shadowed area ?
// 
// A: It is very simple we just call STATE_polygon_create_lightmap() (there are function for object and group as well)
//	  and the bitmap is created automatically according the light source that
//    are defined in the world. (To define light sources use the STATE_light_ API
//
//
// Q: Whats the big deal about having a second texture, after all if we have
//	  the lightmap bitmap and the original bitmap then we can merge them together
//	  using any 2D painting program (Photo Shop, Corel Photo Paint etc ...)
//	  Then we can take the result bitmap and use it for our polygon. This way
//	  there is no need for two bitmaps for our polygon.
//
// A: If we have a big wall or a floor then in order to get a good graphics quality
//	  We will have to use a huge bitmap (1024x1024 as an example == 2MB in 16bit color depth). If we use a tiled bitmap and a lightmap
//	  as a second bitmap we could do quite well with two bitmaps of 128x128 that is
//	  2x 128x128 == 2x16K==32K (64K in 16bit color depth). This way we consume just 1/32
//	  of bitmaps memory.
//	  
//
// Q: Are lightmaps supported in software rendering ?
//
// A: The second bitmap is ignored when doing software rendering. Even though
//	  the functions for creating lightmaps are very useful in software rendering.
//	  As mentioned in the previous question, functions like STATE_polygon_create_lightmap()
//	  can be used in off-line to create the lightmap then by using a 2D painting program
//	  the lightmap can be blended with the texture.
//
//
// Q: There are three methods for creating light. 
//	  1) Using the functions: STATE_engine_create_shadow() and STATE_engine_create_dynamic_shadow()
//	  2) Through the Light API (STATE_light_ ....)
//	  3) Using lightmaps.
//	  The qustions is: what difference is there between the three and when should each one be used.
//    Another question is why have three methods, that is confusing. It would be preferred to have
//	  just one set of functions.
//
// A: Creating nice lights and shadows is one of the important steps in creating
//	  realistic 3D environments. In real-time applications not only we want it to look good
//	  we also it to run in high frames per second rate.There is no one lighting method that
//	  can give best results in all the cases. Thats why there are several methods.
//	  Generally speaking we could give the following guidelines:
//	  Use lightmaps when: Use lightmaps for big polygons such as floors and walls.
//			Lightmaps are only good for static light (light that comes from an open window
//			but not one that comes from a moving flashlight).
//			If lightmaps are used only where they have a strong impact (big polygons)
//			then the effect on the FPS is small (about 10%).
//
//	  Use STATE_engine_create_dynamic_shadow() to create dynamic lights (and shadow)
//			for example to make a light spot from a moving flashlight. This function could be very fast
//			if it is used correctly. For example projecting the flashlight glass lid (one polygon)
//			on top of the six polygons of the room will be very fast (Order 1X6=6). On the other hand
//			projecting.
//
//	  Use STATE_light API functions for both dynamic and static lights.
//			The light API will give good results only when a high polygon count object is lit.
//			The reason is that light is only calculated for the vertices (Goraud shading)
//			so using it to light floors or walls will only result a darker or a brighter floor
//			but it wont create nice light spots and shadows. We suggest that lightmaps will be used
//			for big polygons like walls and floors and the light API will be used to 
//			illuminate objects with many small polygons. 
//
//	  In games like Unreal small lightmaps are used for the walls and floors and when the
//	  room is lit (someone is shooting a rocket for example) then they use Goraud
//	  shading (The light API) to increase the overall brightness.
//
//
// Q: What are the light sources that are used to build a lightmap.
//
// A: Each pixel in the light map is calculated according to the total intensity of all the light sources
//	  at that point. The light sources are the same light sources that are used by
//	  STATE light API (STATE_light_create()
//
//
//	STATE_polygon_create_lightmap()
//=================================
// Creates a bitmap that includes lighting information for that polygon.
// The light that reaches the polygon is calculated according to the light
// sources in the world (STATE_light_create() ...) and the geometry of the world
// (different objects can block the light from some source lights.
//
// Arguments:
//		sample_rate: Determines the size of the lightmap according to the size of the polygon in the 3D world.
//					 For example, if we have a floor in the size of 1000x1000 virtual units and the sample rate
//					 argument is set to 100 then every pixel in the light map will match to a square of 100x100 virtual units on the floor.
//					 that means that that the light map bitmap that will be created will be created will be in the size of 10x10.
//					 Though it is a very small bitmap it still contains 100 light samples. Compare it to the four samples one gets
//					 when using Goraud shading (values are calculated only the vertices).
//					 If a value of 0 is given for sample_rate then all the bitmaps will be in the size of max_bm_side X max_bm_side
//
//		max_bm_side: The maximum alowed height or width of the created bitmap.
//					 For example a value of 256 will allow bitmaps like 128x256 but not 512x128.
//					 Many 3D cards (like Voodoo 3 ) dont support bitmaps bigger than 256.
//					 Thats why it is not recommended to give higher values.
//					 If your application will run primarily in software mode then bigger bitmaps can be freely used.
//		min_lightmap_size:
//					 In many cases tiny lightmaps are not useful (for example 1x1 or 2x2 size bitmaps)
//					 We can decide that if the number of pixels in the lightmap is less than min_lightmap_size
//					 then no lightmap will be created for the specific polygon.
//		force_ray_tracing: YES or NO. Gives much more accurate lighting that takes into account obstacles between
//					 the lit surface and the source of light. Usually you will set it to YES.
//					 The disadvantage is that creating the lightmap will be slower when.
//					 Consider giving NO only when you plan to calculate lightmap dynamically.
//		bitmap_number: defines whether the light map will be set
//					 as the first bitmap or the second bitmap (possible values are 1 or 2).
//
//
// Returns a handle to the lightmap bitmap handle.
//
// See also: STATE_group_create_lightmap(), STATE_object_create_lightmap()
//			STATE_polygon_set_second_bitmap(), STATE_bitmap_merge(), STATE_polygon_switch_between_primary_and_secondary_bitmaps()
//			STATE_polygon_merge_bitmaps()
//			STATE_engine_reduce_number_of_bitmaps()
//
//
// Example:
//
// double location[3]={0,0,0};
// STATE_polygon_get_center(floor_poloygon, location);
// location[2]+=100; // put the light 100 units above the floor
// DWORD my_light=STATE_light_create("light1", location);
// STATE_light_set_distance_reach(my_light,150); //This will create a nice light spot on the floor
// DWORD lightmap=STATE_polygon_create_lightmap(floor_polygon, 256, 4,  8, YES, 1); //The meaning of the arguments is: Create a light map that wont be bigger 
																//than 256x256. If there are less than 4 pixels then dont create a bitmap. 
																//Every pixel in the light map should correspond with a 8x8 square area on 
																//the polygon. Check for objects that cast shadow over the polygon. The 
																//light bitmap will be set as the primary bitmap.
STATE_IMPORT DWORD STATE_WINAPI STATE_polygon_create_lightmap(DWORD polygon_handle, int max_bm_side, int min_lightmap_size, double sample_rate, int force_ray_tracing, int bitmap_number);

//Switches between the first bitmap and the second bitmap of the polygon.
//This function is usually used with STATE_polygon_create_lightmap()
STATE_IMPORT void STATE_WINAPI STATE_polygon_switch_between_primary_and_secondary_bitmaps(DWORD polygon_handle);




//Moves the bitmap of the polygon
//If you intend to shift the bitmap of only one polygon you should use STATE_polygon_shift_bitmap_including_fractions() instead
//See note below.
//
// Parameters:
// -----------
//Delta_x: The desired change in the x coordinates of the bitmap. 0 means no change. numbers could also be negative.
//Delta_y: The desired change in the y coordinates of the bitmap. 0 means no change. numbers could also be negative.
//bitmap_number: 1 or 2. 1 for moving the primary bitmap. 2 for moving the secondary bitmap
//
// note that if delta_x or delta_y are assigned with an integer number it is as if it is assigned with 0.
// it is like rotating an object 0 degrees oo 360 degrees or -720 degrees. It will all give the same result.

// Examples:
// A)
//		// Shifting the x coordinates of the primary bitmap half way
//		STATE_polygon_shift_bitmap(poly, 0.5, 0, 1); 
//
// B)
//		// Shifting the y coordinates of the secondary bitmap one tenth in the oposit direction
//		STATE_polygon_shift_bitmap(poly, 0, -0.1, 2); 
//
// IMPORTANT NOTE: 
// If you intend to shift the bitmap of only one polygon you should use STATE_polygon_shift_bitmap_including_fractions()
// For example if you want to shift the bitmap on a polygon that is used as a sky
// If the sky are made of one polygon then use STATE_polygon_shift_bitmap_including_fractions() insted
// If the sky are made of several polygons (Works better to have the sky made from several triangles if you are using fog)
// you should continue reading the note below.
// When the world is loaded some of the polygons 
// are split into smaller fractions (as part of BSP Tree creation)
// so it could be that you will have to shift the bitmap of several polygons, not just the one you intended.
// For example lets assume that you put a very big polygon in the sky
// (to create clouds effect using LIGHT_SOURCE_GLASS, see STATE_polygon_set_translucent() ) now you want to shift the bitmap
// you do DWORD sky=STATE_polygon_get_handle_using_name() and you call STATE_polygon_shift_bitmap()
// Now it is very possible that you will notice that only a small part of your polygon shift its bitmap.
// Here is how to solve it:
// At the beginning of your program collect all the fractions that make your sky
// like that:
//	for(DWORD poly=STATE_polygon_get_first_polygon(); poly!=NULL; poly=STATE_polygon_get_next(poly)) {
//		char *name=STATE_polygon_get_name(poly);
//		if(strcmp(name, "My_Sky")==0) {
//			m_sky_polygons[m_number_of_sky_polygons]=poly;
//			m_number_of_sky_polygons++;
//
//		}
//
//	}
//
// Now to shift the bitmap do:
//	for(int i=0; i<m_number_of_sky_polygons; i++) {
//		STATE_polygon_shift_bitmap(m_sky_polygons[i], wind_speed, 0, 1);
//	}
//
//
STATE_IMPORT void STATE_WINAPI STATE_polygon_shift_bitmap(DWORD polygon_handle, double delta_x, double delta_y, int bitmap_number);


//See STATE_polygon_shift_bitmap() for information.
//Use this function instead of STATE_polygon_shift_bitmap() when you are NOT shifting a collection of polygons that belongs to one shape
STATE_IMPORT void STATE_WINAPI STATE_polygon_shift_bitmap_including_fractions(DWORD polygon_handle, double delta_x, double delta_y, int bitmap_number);


//One can use this function to associate your own data structure to a polygon.
//Whether one decides to use this function or not depends on one's programming style.
//The obvious alternative for using this function is to have your own structure or class
//for each polygon and that one of the fields in your class would be the polygon_handle through which 
//you can access the STATE functions.
//
// Parameters:
//		polygon_handle:
//				a handle to an polygon
//
//		my_own_data:
//			A pointer to a class or structure or even a function. This pointer can points
//			to whatever you want it to point. The Engine doesnt do anything with this pointer
//			it simply save it for you so you can call STATE_polygon_get_my_own_data() to retrieve it.
//
// Example:
//
// typedef struct polygon_data 
// {
//		int any_data1;
//		int number_of_times_that_got_hit;
//		//And anyother data you might need in your program
// } polygon_data;
//
// polygon_data poly_data; //or use new(), malloc() or whatever ...
// STATE_polygon_attach_my_own_data(my_polygon, &poly_data);
STATE_IMPORT void STATE_WINAPI STATE_polygon_attach_my_own_data(DWORD polygon_handle, void *my_own_data);
STATE_IMPORT void *STATE_WINAPI STATE_polygon_get_my_own_data(DWORD polygon_handle);


// This function is probably useless for 99.9999% of our users.
// You will need it only if you choose to write your own version STATE_engine_render()
// The 2d polygon animation is updated automatically according to need each time that STATE_engine_render()
// is called and a polygon with animation is rendered. If one doesn't call STATE_engine_render()
// (because one chooses to use his own renderer) then the 3D_animation frame is not updated.
// Only in this case you should call this function so that your external renderer can benefit from
// the 2d animation mechanism.
STATE_IMPORT void STATE_WINAPI STATE_polygon_update_animation(DWORD polygon_handle, DWORD camera_used_for_rendering);

// Returns the first patch that is attached to the given polygon.
// If the given polygon is a patch in itself then the function will return NULL.
// To check whether a polygon is a patch or not, use STATE_polygon_get_patch_type() 
// See also STATE_polygon_get_patch_father.
//
// Example
//		
//		//here is how to make a function that counts how many patches are attached to a given polygon
//
//		int count_patches(DWORD polygon_handle)
//		{
//			int count=0;
//			check_patch=STATE_polygon_get_patch_type(polygon_handle);
//			if(check_patch!=0) {
//				MessageBox("The given polygon is a patch in itself !
//				return(-1); //instead we could do polygon_handle=STATE_polygon_get_patch_father(polygon_handle); 
//							//and continue normal execution of the function
//			}
//
//			for(DWORD poly=STATE_polygon_get_first_patch(polygon_handle); poly!=NULL; poly=STATE_polygon_get_next_patch(poly) )
//				count++;
//			
//			return(count);
//
//		}
STATE_IMPORT DWORD STATE_WINAPI STATE_polygon_get_first_patch(DWORD polygon_handle);


//Returns the next patch. patch_polygon_handle could also be a regular polygon
//that is not a patch, in this case the function will return the first patch
//which is attached to this polygon or NULL if there aren't any patches attached.
//
//Example:
//		//Here is how to go through all the patches of a given polygon
//		for(DWORD poly=STATE_polygon_get_first_patch(polygon_handle); poly!=NULL; poly=STATE_polygon_get_next_patch(poly) )
//		{
//			//do some processing for each patch
//		}

STATE_IMPORT DWORD STATE_WINAPI STATE_polygon_get_next_patch(DWORD patch_polygon_handle);

//Returns the polygon to which the given patch was added.
//if patch_polygon_handle is not a patch (a regular polygon) then the function returns NULL
STATE_IMPORT DWORD STATE_WINAPI STATE_polygon_get_patch_father(DWORD patch_polygon_handle);

// Function level of importance = Medium.
//
// The function turns OFF or ON the bilinear filtering for the given polygon
// Bilinear filtering is a technique for getting better looking textures. When bilinear 
// filtering is ON, it removes the pixelization effect from close textures.
// By default the bilinear mode for all polygons is set according to the 3D card settings
// Using this function one can overide the default settings for specific polygons.
// Here are some reasons why one might consider turning off bilinear filtering for specific polygons
//  - Give the polygon a different look.
//	- When bilinear filtering is turned off, rendering is slightly faster.
//
// To change the default bilinear setting for all the polygons use:
// STATE_3D_card_set_bilinear_filtering() 
// parameters:
//		bilinear_mode:
//			ON  -		The default. Turns on bilinear filtering (for both the primary and the secondary bitmap).
//			OFF -		Turns off bilinear filtering
//			BILINEAR_FOR_PRIMARY_TEXTURE_ONLY -		Turns on bilinear only for the primary texture (each polygon could have two textures, the primary and the secondary)
//			BILINEAR_FOR_PRIMARY_SECONDARY_ONLY -   Turns on bilinear only for the secondary texture (each polygon could have two textures, the primary and the secondary)
//			BILINEAR_USE_3D_CARD_DEFAULTS - Use the setting as defined by STATE_3D_card_set_bilinear_filtering. This is the default option.
//Example:
// STATE_polygon_set_bilinear_filtering(my_polygon, OFF);
//
// See also STATE_polygon_set_bilinear_filtering()
STATE_IMPORT void STATE_WINAPI STATE_polygon_set_bilinear_filtering(DWORD polygon_handle, int bilinear_mode);



// Function level of importance = Medium.
//
// The function turns OFF or ON the mipmap filtering (also known as trilinear filtering) for the given polygon.
// Try switching it off and see how it effect your world (example: STATE_polygon_use_mipmap(my_polygon, OFF); )
// Read the rest for your general knowledge
//
// What are mipmaps
// -----------------
// Mipmaps are used for getting better looking textures. 
// The 3D engine creates several lower detailed bitmaps in decreasing order from each bitmap that is used
// for example: If you use a 64x64 bitmap then the engine will create 6 additional bitmaps:
// 32x32, 16x16, 8x8, 4x4, 2x2, 1x1. Using mipmaps use 33% more texture memory.
// 
// Why do we need mipmaps
// -----------------------
// Imagine that you have a 3D world and in this 3D world you have a house that uses
// a 256x256 bitmap size image for the house front wall. Now imagine that you are looking
// at your house from a very long distance. so far away that the house appear very small on your screen
// Now when the house is very far a way the front of the house is represented by only a few pixels (for 
// example 2x2=4 pixels). And now comes the big questions, which 4 lucky pixels of the bitmap should be selected ?
// You original bitmap has 256x256=65536 pixels. Your 3D card has to select only 4 pixels !
// The problem is that when the location of the camera (the player) is moved a little, the 4 chosen pixels will alos change
// In one frame a pixel from the door will be selected, in another frame a pixel from the window curtain etc ...
// The result will be 4 pixels that are flickering in random colors. In real life when we look at something from a distance
// we see only the main\big features of the object. We spot only the average color. This is exactly what mipmaps can achieve.
// When mipmaps are on (considering our example) the 3D card will use the 2x2 version of our original bitmaps.
//
// Inside the 3D card
// ------------------
// When calculating the color of a pixel the 3D card is actually able (most of them) to use two mipmap levels
// The pixel is calculated twice one for each bitmap that is used, then the 3D card makes an average (according
// to how fit is each of the mipmap level). If bilinear filtering is used then 3D card calculates for each mipmap level 
// a bilinear filtered pixel. Then when making a weighted average of the two pixels it is said that the result pixel
// is trilinear filtered. In sort: Bilinear filtering + mipmaps == Trilinear filtering

// Reasons for using Mipmaps
// -------------------------
// 1) In most cases the graphics will look better
// 2) The fact that far away polygons can be rendered using a very small bitmap can
//    improve rendering speed.
//
//
// Reasons for not using mipmaps
// -----------------------------
// 1) Take 33 percent more texture memory
// 2) In some rare cases the graphics will look better with no mipmaps
// 3) The extra caclulations might slow down the rendering.

// To set the default for all the polygons, use STATE_3D_card_use_mipmaps()
// Note that STATE_polygon_use_mipmap() will overide any setting made by STATE_3D_card_use_mipmaps() 
//
// parameters:
//		mipmap_mode:
//			ON  -		The default. Use mipmaps (for both the primary texture and the secondary texture)
//			OFF -		Mipmaps are not used
//			MIPMAP_FOR_PRIMARY_TEXTURE_ONLY -		Use mipmaps only for the primary texture (each polygon could have two textures, the primary and the secondary)
//			MIPMAP_FOR_PRIMARY_SECONDARY_ONLY -    Use mipmaps only for the secondary texture (each polygon could have two textures, the primary and the secondary)
//
// Example:
//		STATE_polygon_use_mipmap(my_polygon, OFF);
// 
// See also:
//		STATE_3D_card_use_mipmaps(OFF);
//
STATE_IMPORT void STATE_WINAPI STATE_polygon_use_mipmap(DWORD polygon_handle, int mipmap_mode);

//Deletes or add points to the given polygon to match the deisred number of points.
// Be carful not to use points handle of the given polygon after calling this function, since this function could delete some of the polygons points
STATE_IMPORT void STATE_WINAPI STATE_polygon_set_number_of_points(DWORD polygon_handle, int num_of_points);

// *** useful
// Takes a point on the polygon and returns the color and alpha values of the relevant pixel
// This function is very useful in conjunction with STATE_engine_is_movement_possible(), STATE_engine_2D_point_to_3D()
// For example, to receive the color of the pixel where the user point the mouse on
//
// Parameters:
//		XYZ_on_polygon [in]:
//			a point on the polygon.
//			This point can be obtained by STATE_engine_is_movement_possible(), STATE_engine_2D_point_to_3D()
//			or other way.
//			XYZ_on_polygon[0] the x coord
//			XYZ_on_polygon[1] the y coord
//			XYZ_on_polygon[2] the z coord
// 
//		bitmap_index [in]:
//			0 or 1. 0 for the primary bitmap. 1 for the secondary bitmap
//
//		rgba [out]:
//			The color of the pixel in the given location
//			rgba[0]==red
//			rgba[1]==green
//			rgba[2]==blue
//			rgba[3]==alpha
//
//
//		Example:
//			int x=10, y=20; //for example the mouse cursor location
//			double XYZ[3];
//			DWORD selected_object_handle;
//			DWORD selected_polygon_handle;
//			int return_code=STATE_engine_2D_point_to_3D(x,y,XYZ, &selected_object_handle, &selected_polygon_handle);
//			if(return_code!=OK)
//				return;
//
//			BYTE rgba[4];
//			STATE_polygon_get_bitmap_pixel_rgba(selected_polygon_handle, XYZ, 0, rgba);
//
STATE_IMPORT void STATE_WINAPI STATE_polygon_get_bitmap_pixel_rgba(DWORD polygon_handle, double XYZ_on_polygon[3],int bitmap_index ,BYTE rgba[4]);

// *** useful
// Returns the x,y bitmap coordination of a pixel according to a point X,Y,Z on the polygon
// Similar to STATE_polygon_get_bitmap_pixel_rgba() only here we get the x,y bitmap coordinate of the pixel
// The big advantage here, is that we can take these coordinate and use them with STATE_bitmap_set_pixel()
//
// This function is very useful when we want to change bitmaps according to events hapenning in the world
// for example if we shoot on something and now we want to add a mark there.
// we can use STATE_engine_is_movement_possible() to calculate where our bullet has hit
// Then we can take the hitting point X,Y,Z and get the relevant bitmap coordination
// then by using STATE_bitmap_set_pixel() we can modify the pixel and the pixel next to it
// to reflect a bullet hole. (note that a different technique is using poygon patches. See STATE_polygon_add_patch_easy() )
// 
// Parameters:
//		XYZ_on_polygon [in]:
//			a point on the polygon.
//			This point can be obtained by STATE_engine_is_movement_possible(), STATE_engine_2D_point_to_3D()
//			or other way.
//			XYZ_on_polygon[0] the x coord
//			XYZ_on_polygon[1] the y coord
//			XYZ_on_polygon[2] the z coord
// 
//		bitmap_index [in]:
//			0 or 1. 0 for the primary bitmap. 1 for the secondary bitmap
//
//		x,y [out]
//			The return bitmap coordination
//
//
// Example:
//		double XYZ[3]={100, 0, 0}; //assuming this point is on the polygon
//		int x,y;
//		STATE_polygon_get_bitmap_pixel_xy(my_poly, XYZ, 0, &x, &y);
//
STATE_IMPORT void STATE_WINAPI STATE_polygon_get_bitmap_pixel_xy(DWORD polygon_handle, double XYZ_on_polygon[3],int bitmap_index ,int *x, int *y);

//----------------------------------------------------//
//=========== T H E    P O I N T   A P I =============//
//----------------------------------------------------//

// About this API:
// Every polygon is built from points (vertices). This API deals with those points.


// Returns YES or NO.
// YES , if the given handle is a point handle. NO if not.
// If it is not a point handle , a message box will appear,
// the title of this message box will be the parameter "function_asking"
// if function_asking==NULL than no popup will be created.
// See also IS_POINT()
STATE_IMPORT int STATE_WINAPI STATE_point_is_point(DWORD point_handle, char *function_asking);

// Returns the next point
// use STATE_polygon_get_first_point() to get the first point
//
//Example: 
//
//	//Go over all the points of a polygon
//	for(DWORD point=STATE_polygon_get_first_point(polygon_handle); point!=NULL ; point=STATE_point_get_next_point(point) ) 
//	{
//		
//
//	}
//
//
STATE_IMPORT DWORD STATE_WINAPI STATE_point_get_next_point(DWORD point_handle); 

// Gets the point (x,y,z) coordinate.
//Please note that if the point belongs to a dynamic object then the returned (x,y,z) will be in OBJECT_SPACE.
// To get the point in WORLD_SPACE you should write the following code:
//
// double xyz[3];
// STATE_point_get_xyz(my_point, xyz);
// double xyz_world_space[3]
// STATE_object_convert_point_to_world_space(my_object, xyz, xyz_world_space);
STATE_IMPORT void STATE_WINAPI STATE_point_get_xyz(DWORD point_handle, double xyz[3]); 


// Sets the point (x,y,z) coordinate.
// After changing the points of a polygon dont forget to call STATE_polygon_is_valid()
// This call is needed for recalculating internal data of the polygon (such as the polygon's normal)
//
// Example:
//
//		double xyz[3]={100,200,300};
//		STATE_point_set_xyz(my_point, xyz); 
//		STATE_polygon_is_valid(); //Update the polygon data (call this function after you have finished manipulating the polygon's point
//		
//
// Please note that if the point belongs to a dynamic object then the (x,y,z) should be in OBJECT_SPACE.
// To get a x,y,z coorinate in OBJECT_SPACE you should write the following code:
//
//		double xyz[3];
//		xyz[0]=100; xyz[1]=200; xyz[2]=300;  //setting the coordinate (world space)
//		double xyz_object_space[3]
//		STATE_object_convert_point_to_object_space(my_object, xyz, xyz_object_space);
//		STATE_point_set_xyz(my_point, xyz);
//		STATE_polygon_is_valid(); //Update the polygon data (call this function after you have finished manipulating the polygon's point)
//
STATE_IMPORT void STATE_WINAPI STATE_point_set_xyz(DWORD point_handle, double xyz[3]); 

#define FIND_COORDINATE_AUTOMATICALLY  -1
//Set the bitmap coordinates of the given points.
//Examples:
//	A)	//Attach the polygon's bitmap top left corner to the given point.
//		double xy[2]={0,0}
//		STATE_point_set_bitmap_xy(pnt1,owner_polygon_handle, xy);
//
//	B)	//Attach the polygon's bitmap bottom right corner to the given point.
//		double xy[2]={1,1}
//		STATE_point_set_bitmap_xy(pnt1,owner_polygon_handle, xy);
//
//	C)	//This tells the engine to figure out on his own to witch part of the bitmap
//		//this point should be attached. This is very useful since sometimes the right bitmap coord
//		//could look like 0.5543222323 etc... For example if our polygon has a trapez shape or a pentagon shape
//		//Please note that in order for the engine to figure out on his own it must previously give it
//		//Three other coords. The actual calculation of all the missing coordinates is done when
//		//We call STATE_engine_add_polygon()
//		double xy[2]={FIND_COORDINATE_AUTOMATICALLY, FIND_COORDINATE_AUTOMATICALLY}
//		STATE_point_set_bitmap_xy(pnt1,owner_polygon_handle, xy);
//	D)	// Like "C" only that one coordinate element  we give and one we ask the engine to find.
//		// In this example we ask to find the y bitmap coordinate. In order for the engine to find it on its own
//		// We must give him befroehand three other y values.
//		double xy[2]={0, FIND_COORDINATE_AUTOMATICALLY}
//		STATE_point_set_bitmap_xy(pnt1,owner_polygon_handle, xy);
//
//	E)	How to tile a bitmap over a polygon. Lets say we have
//		a square polygon and we want to tile a bitmap .In this example it will be tiled 5x5 .
//
//		double xy[2]={0, 0}
//		STATE_point_get_bitmap_xy(pnt1, xy); //The top left
//		xy[0]=5;
//		STATE_point_set_bitmap_xy(pnt2,owner_polygon_handle, xy); //The top right
//		xy[1]=5;
//		STATE_point_set_bitmap_xy(pnt3,owner_polygon_handle, xy); //The bottom right. xy==[5,5]
//		xy[0]=FIND_COORDINATE_AUTOMATICALLY;
//		xy[1]=FIND_COORDINATE_AUTOMATICALLY;
//		STATE_point_set_bitmap_xy(pnt4,owner_polygon_handle, xy); //We have already three points so the engine
//											  //can figure out the rest on its own. In this case
//											  //If our polygon is a square it should calculate xy={0,5}
//											  //It is better to let the engine handle the forth point and above
//											  //to avoid mistakes.
STATE_IMPORT void STATE_WINAPI STATE_point_set_bitmap_xy(DWORD point_handle, DWORD owner_polygon_handle ,double xy[2]); 
STATE_IMPORT void STATE_WINAPI STATE_point_get_bitmap_xy(DWORD point_handle, double xy[2]); 

//The same as STATE_point_set_bitmap_xy() just for the second bitmap.
//See also STATE_polygon_set_second_bitmap()
STATE_IMPORT void STATE_WINAPI STATE_point_set_second_bitmap_xy(DWORD point_handle, DWORD owner_polygon_handle ,double xy[2]); 
STATE_IMPORT void STATE_WINAPI STATE_point_get_second_bitmap_xy(DWORD point_handle, double xy[2]); 


// This function will return the rgb values of the point
// Red will be returned in rgb[0] 
// Green will be returned in rgb[1] 
// Blue will be returned in rgb[2] 
// Note that these values are only relevant with hardware rendering.
// The default is (255,255,255).
// By changing the rgb value we can create light and atmospheric effects.
//
// How the rgb values are combined with the texture color and the polygon color
// to form the final color of each pixel:
// ----------------------------------------------------------------------------
// The final color of a pixel in a polygon is
// an outcome of the rgb values of the vertexes and the polygon texture or color
// The rgb of the vertexes is interpolated for every pixel of the polygon
// The interpolated value is multiplied by the texture color of every pixel.
// (If there is no texture then it is multiplied by the color of the polygon)
// Before multiplying the rgb is converted to a 0 till 1 scale.
// Example: Lets say we have a red polygon with its color set to (240,10,10)
// Lets assume that this is a triangle and that the rgb value of the vertexes is 
// (255,0,0 ) , (0,255,0) , (0,0,255). If we take a point in the center of the polygon
// then the interpolated value from the three vertexes will be something like (83,83,83)
// when 83 is converted from 0-255 scale to 0-1 scale we get that the interpolated value
// is (0.33, 0.33, 0.33). Now let multiply this value with the color of the polygon
// we get: (0.33, 0.33, 0.33) * (240, 10, 10) == (80, 3, 3)
// The result pixel will be (80,3,3)
//
// Special cases.
// --------------
// 1) If the RGB value of all the vertexes is (255,255,255)
// Then it wont have any effect since in scale of [0-1]
// it will become (1,1,1) and then the multiplication wont change
// the value of the pixel
// 2) If the polygon has no bitmap and its color is white (255,255,255)
// In this case the rgb values of the vertexes determine alone the color
// of the final pixel. This is also known as Gouraud shading
//
STATE_IMPORT void STATE_WINAPI STATE_point_get_rgb(DWORD point_handle, BYTE rgb[3]);

// This function will set the rgb values of the point
// Red should be given in rgb[0] 
// Green should be given in rgb[1] 
// Blue should be given in rgb[2] 
// Note that these values are only relevant with hardware rendering.
// By changing the rgb value we can create light and atmospheric effects.
// The light API is based on this function. Usually one would
// use the light API function though sometimes it nice to have the direct control
// for creating special effects.
//
// How the rgb values are combined with the texture color and the polygon color
// to form the final color of each pixel:
// ----------------------------------------------------------------------------
// The final color of a pixel in a polygon is
// an outcome of the rgb values of the vertexes and the polygon texture or color
// The rgb of the vertexes is interpolated for every pixel of the polygon
// The interpolated value is multiplied by the texture color of every pixel.
// (If there is no texture then it is multiplied by the color of the polygon)
// Before multiplying the rgb is converted to a 0 till 1 scale.
// Example: Lets say we have a red polygon with its color set to (240,10,10)
// Lets assume that this is a triangle and that the rgb value of the vertexes is 
// (255,0,0 ) , (0,255,0) , (0,0,255). If we take a point in the center of the polygon
// then the interpolated value from the three vertexes will be something like (83,83,83)
// when 83 is converted from 0-255 scale to 0-1 scale we get that the interpolated value
// is (0.33, 0.33, 0.33). Now let multiply this value with the color of the polygon
// we get: (0.33, 0.33, 0.33) * (240, 10, 10) == (80, 3, 3)
// The result pixel will be (80,3,3)
//
// Special cases.
// --------------
// 1) If the RGB value of all the vertexes is (255,255,255)
// Then it wont have any effect since in scale of [0-1]
// it will become (1,1,1) and then the multiplication wont change
// the value of the pixel
// 2) If the polygon has a bitmap and its color is white (255,255,255)
// In this case the rgb values of the vertexes determine alone the color
// of the final pixel. This is also known as Gouraud shading
//
// See also STATE_polygon_set_light()
STATE_IMPORT void STATE_WINAPI STATE_point_set_rgb(DWORD point_handle, DWORD father_polygon, BYTE rgb[3]);




// relevant only for bitmap fill
// 100 means the same brightness as the original bitmap
// 200 means twice as bright as the original bitmap
// 50 is half bright as the original
// see also:
// STATE_polygon_get_brightness()
// STATE_polygon_set_brightness()
// STATE_polygon_is_uniform_brightness()
// STATE_point_get_brightness()
STATE_IMPORT double STATE_WINAPI STATE_point_get_brightness(DWORD point_handle); 

// note that if not all the points have the same brightness value
// the rendering of this polygon will be slower.
// relevant only for bitmap fill
// 100 means the same brightness as the original bitmap
// 200 means twice as bright as the original bitmap
// 50 is half bright as the original
// see also:
// STATE_polygon_get_brightness()
// STATE_polygon_set_brightness()
// STATE_polygon_is_uniform_brightness()
// STATE_point_get_brightness()
STATE_IMPORT void STATE_WINAPI STATE_point_set_brightness(DWORD point_handle, double value); 

//The point's normal is relevant only when using light (through the STATE_light_ API)
//The following are three cases where you might want to modify the points normal
// 1) If it is needed to sharpen the contact lines of polygons (see example below)
//	  in order to get better illumination of objects such as rooms boxes etc. 
// 2) If one is writing his own lighting module
// 3) If one is writing a file format converter (from or to STATE).
//
//The normal of a point is set internally to the average of the normals of the adjacent surfaces.
//For example, if we have model of a room (a cube) the normal of a point on the floor
//will point to the center of the cube this is because the coordinates of a point on the floor
//exists in three polygons. The floor polygon and two adjacent walls.
//The average of the normals of these three polygons is a normal that points to the center of the cube.
//Please note that inside the engine a cube is built from 6 polygons each contains
//4 points, so a cube is built from 24 points (and NOT 8).
//Because of the fact that the normal of each point is the average of the adjacent
//it results that points that have the same coordinates will have the same adjacent polygons
//and therefore the same point normal. In other words point that have the same X,Y,Z coordinates
//will also have the same normal.
//
//Why do we need points to have a normal ? What is it good for.
//-------------------------------------------------------------
//The point normal helps a lot in making polygonal shapes to look more rounded
//while lighting the model polygons. Since points with the same X,Y,Z coordinates but on
//different polygons have the same point normal , they will also get the same color
//while the object is lighted and this will hide the connecting line between
//any two polygons.
//
//
//When saving a world on the disk (using STATE_engine_save() or WorldBuilder()
//The engine automatically calculates the polygons normal. This means that if you
//use STATE_point_set_normal() to modify the normals you will then loose
//your changes when the world is saved. To keep your changes
//call STATE_engine_save() with the flag: SAVE_WITHOUT_CALCULATING_POINT_NORMALS
//
// Parameters:
//
//	point_handle: a handle to a point.
//
//	normal:	a three values in the range of [-1  -  1] that corresponds with the
//			X,Y,Z values to be set (normal[0] is the X value)
//			Note that the normal is normalized internally so values such as Normal= {10,0, -20}
//			can also be used to set a normal
//
//  Note that internally the engine uses only 24 bits for the normal X,Y,Z values
//  (for example, three double values would occupied 8*8*3=192 bits)
//  This means that if one calls STATE_point_set_normal() with a normal_x == 0.123456789
//	then internally it will be rounded to something like 0.123
//
// Return Value: OK or VR_ERROR.
//
// Example:
//
// A)	double normal[3]={1,0,0};
//		STATE_point_set_normal(point_handle, normal);
//
// B)
//		The following example will set all the points normal to the same value
//		of their respective polygon. This will make the edges of the polygon much sharper
//		This effect is good for lighting a room were there should be a visible difference
//		between the light that the floor and the wall receive. 
//
//		STATE_light_calcualte_vertices_normals_automatically(DISABLE_AUTOMATIC_NORMAL_CALCULATION); //so that the values that we set wont be overwritten when doing STATE_engine_save().
//    	for(DWORD poly=STATE_polygon_get_first_polygon(); poly!=NULL; poly=STATE_polygon_get_next(poly) ) {
//	  		double normal[4]; //The normal is the three first values
//	  		STATE_polygon_get_plane(poly, normal);
//	  		for(DWORD point=STATE_polygon_get_first_point(poly); point!=NULL ; point=STATE_point_get_next_point(point) ) {
//	  			STATE_point_set_normal(point, normal);
//	  		}
//	  
//	  	}
//
// NOTE: When a group is lit at the first time (when STATE_light_activate() is called)
//		 it will calculate the group normals and will override any setting that were done before.
//		 To avoid this call STATE_light_calcualte_light_normals_automatically(NO);
//
// See also STATE_point_get_normal()
STATE_IMPORT int STATE_WINAPI STATE_point_set_normal(DWORD point_handle, double normal[3]); 

// For more information see STATE_point_set_normal()
//
// Return Value: OK or VR_ERROR.
//
// Example:
//
//		double normal[3];
//		STATE_point_get_normal(point_handle, normal);
STATE_IMPORT int STATE_WINAPI STATE_point_get_normal(DWORD point_handle, double normal[3]); 

// Set the vertex fog value
// Using this function one create interesting effects like setting
// the fog for only specified areas. For example setting fog
// only till a specific height or confining the fog in some volume.
// This value is relevant only when FOG_MODE_VERTEX is used.
// To set FOG_MODE_VERTEX call STATE_engine_set_fog() once after loading the world
// For example: STATE_engine_set_fog(ON, 200, 200, 200, FOG_MODE_VERTEX, 1000000, FOG_START_DISTANCE_DEFAULT, 0);
//
// Note that in the FOG_MODE_VERTEX the points fog value are not updated automatically.
// It is up to the program to modify them. For example lets assume that you set a black fog color
// to simulate night and now you set all the points in the world with values relative to the distance
// of the point from the camera (as if the camera is also a light source and whatever is closer
// to the camera is brighter). This will give a very nice effect but when the camera moves this process 
// has to be repeated otherwise the dark\lit area wont be updated ! 
// This is nor very effective since updating all the points is time consuming
// Therefor vertex fog is usually used for static light effects (see below) or to confine fog
// to specific areas
//
// Vertex fog also can be used as light ! In many cases it even gives better results than using 3DSTATE_light API
// To understand when to use what and what can actually be achieved it is important to understand the following:
//    Each point has three RGB bytes that controls the red green and blue channel. When we have a bitmap/texture on a polygon
//    (this would usually be the case) the final color of a pixel on the screen is the bitmap pixel multiply by the 
//	  the color as contributed from the polygons vertices/points (unless STATE_bitmap_set_operation_mode() is called). Before multiplyong the color
//	  with the bitmap pixel the color is scaled to the range [0 - 1]. For example the color gray (128,128,128) will
//	  be scaled to (0.5, 0.5, 0.5). The important thing to remember here is that when manupulating the point rgb
//	  the final result rendered on the screen will never be brighter than the bitmap it self. Using the point RGB
//	  one can only darken the scene ! The STATE_light API actually manipulate the points RGB to create the illusion
//	  of light. And here we get to the important advantage of vertex fog. When using the vertex fog the result rendered
//	  pixel is between the fog color and the bitmap pixel. So we can actually get colors that are brighter than the bitmap !
//
//	  
// Parameters:
//		fog_intensity:
//			A number between 255 and 0
//			Note that 0 is actually the maximum fog
//			and 255 would give the original color of the bitmap
//			The default is 255
// 
//
// Example:
//		STATE_point_set_fog(point_handle, 128); //The result color in this case will be exactly in the middle between
//											    //the original bitmap color and the fog color
//
STATE_IMPORT void STATE_WINAPI STATE_point_set_fog(DWORD point_handle, BYTE fog_intensity); 
STATE_IMPORT BYTE STATE_WINAPI STATE_point_get_fog(DWORD point_handle); 

// Used to control the transparency of a given polygon.
// Relevant only when STATE_polygon_set_translucent() is called with ALPHA_FILTER or any other filter that uses alpha
STATE_IMPORT void STATE_WINAPI STATE_point_set_alpha(DWORD point_handle, BYTE alpha_value); 
STATE_IMPORT BYTE STATE_WINAPI STATE_point_get_alpha(DWORD point_handle); 


//-----------------------------------------------------//
//======== T H E    L A Y E R   A P I =================//
//-----------------------------------------------------//

// About layers and the STATE_layer API
// -------------------------------------
// Layers are polygons that appear on top of the rendered world.
// Think of layers as the 2D mechanism of the engine.
// One can use layers for displaying information such as the
// player health, a cross hair, simulate looking through binocular etc.
// One can use all the polygon and point functions in order to modify the created layer
// A few of the functions that you might probably want to use are:
//		STATE_polygon_disable(), STATE_polygon_enable(), STATE_polygon_set_bitmap_fill()
//		STATE_polygon_set_light(), STATE_polygon_set_translucent(). STATE_polygon_move() 
//		STATE_polygon_shift_bitmap(), STATE_polygon_add_point() STATE_point_set_xyz()
//		STATE_polygon_rotate_x/y/z() or any other polygon API function.
//
// Layers can also be used as 2D sprites (use STATE_polygon_move() or STATE_polygon_set_location() to move the layer)
//
// Layer have points that you can modify using the STATE_point_api()
// No matter what is the size of the window which you render on the
// top left corner of the rendered screen is mapped to 0,0 and the bottom right corner is 100,100
// You can also change the Z value of the layer points see STATE_layer_set_depth()




// *** very useful
// A very useful and easy to use function.
// The function returns a handle to a polygon which is the created layer.
// After calling this function the created layer is immediately visible.
// (no need to call STATE_engine_add_polygon() )

// Using STATE_layer_create() one can easily create a square polygon
// with a bitmap on it. The first four parameters define the layer size and position.
// No matter what is the size of the window which you render on the
// top left corner is always 0,0 and the bottom right corner is 100,100
// For example if the four numbers 0,0,50,100 are given as the first 4 parameters
// then the layer will occupied the half left of the rendered screen
// If there are two or more polygon layers that overlap then the one which was 
// first created will be on top. Use STATE_layer_set_depth()to control the order and the depth in which the
// layer are rendered.
//
// Return value
//		The function returns a handle to a polygon which is used for the layer.
//		
//
//	Parameters:
//		left_x, top_y:
//				The top left corner of the created layer.
//				Any number in range [0-100]. (0,0) is the top left corner
//				while (100,100) is the bottom right corner
//
//		right_x, bottom_y:
//				The bottom right corner of the created layer.
//				Any number in range [0-100]. (0,0) is the top left corner
//				while (100,100) is the bottom right corner
//
//
//		Note that after the a layer is created it is possible
//		to change/move the points using the Polygon and Point APIs
//		one can also call STATE_layer_set_shape() to change the shape of
//		the layer
//
//
//		bitmap_handle: A handle to a bitmap that will be used.
//				Give NULL if your layer will use color fill.
//				Note that after the layer is created, one can use STATE_polygon_set_bitmap_fill()
//				to change the bitmap.
//
//		glass_type:
//				Using this parameter one can determine if the layer will be
//				opaque or translucent. Use any of the values possible with
//				the STATE_polygon_set_translucent() function.
//				To precisely control the level of translucency 
//				call STATE_polygon_set_light(layer_handle, translucent_level_rgb);
//		
//
// Remarks:
// --------
//
//	A very convenient way to work with layer polygons is
//	to create them all at the beginning of the program and then to disable those
//	which should not be visible right away.
//	Then during the program run, enable layer polygons according to the need.
//	There is no time penalty for having many layer polygons which are disabled
//	Only those which are drawn consume time. Layers are very fast to draw since
//	they exploit the 3D card hardware. The time it takes to draw a layer
//	depends on its size. If the layer covers the whole rendering window\fullscreen
//	then the slowdown in FPS might be measurable.
//
// Examples:
//
//	Example 1:
//		//Creating a layer on the whole window
//		DWORD my_bitmap=STATE_bitmap_load("binocular", -1);
//		if(my_bitmap==NULL) return(VR_ERROR);
//		DWORD layer_polygon=STATE_layer_create(0,0, 100, 100, my_bitmap, FILTER_GLASS);
//		//After the next render the layer will be shown on the screen
//
//	Example 2:
//		//A more advanced example which show how to make a message
//		//appearing on the screen and then fading away and disappearing.
//		//This message could be something like "1000 points bonus"
//		//"Extra life" etc. This example is quite important since once
//		//you will understand it you will be able to create many more nice effects
//		//that have nothing to do with layers. 
//		//The following example is comprised of 3 functions that together
//		//create the mechanism.
//
//		//Call this function at the beginning of the program
//		DWORD create_message_layer(void)
//		{
//			DWORD my_bitmap=STATE_bitmap_load("ExtraLife", -1);
//			if(my_bitmap==NULL) return(NULL);
//
//			//Note that we have chosen LIGHT_SOURCE_GLASS below since
//			//this type of blending will do 'add' between the layer_message polygon and the
//			//rendered world below (new_pixel=src+dest). We need this adding for
//			//doing the fade away effect. We will make the message_layer_polygon
//			//darker and darker till it wont be seen (new_pixel= 0 + dest == dest)
//
//			DWORD message_layer_polygon=STATE_layer_create(20,15, 80,30, my_bitmap, LIGHT_SOURCE_GLASS);
//			if(message_layer_polygon==NULL) return(NULL);
//			//disable the polygon since we dont want it to be seen now
//			STATE_polygon_disable(message_layer_polygon);
//			m_is_message_visible=NO; //remember the status of the message in a global\class member variable. 
//			return(message_layer_polygon);
//		}
// 
//		//Call this function when the message should be seen
//		void make_message_visible(DWORD message_polygon)
//		{
//			STATE_polygon_enable(message_polygon);
//			BYTE rgb[3]={255,255,255);
//			//Show the message in full visibility
//			STATE_polygon_set_light(message_polygon, rgb); //Set the rgb of all the points to 255,255,255
//			m_is_message_visible=YES;
//		}
//		
//
//		//Call this function before each render
//		void make_message_fade_away(DWORD message_polygon)
//		{
//			if(m_is_message_visible==NO) return; //If the message is not visible then there is nothing to do
//			//All the points have the same rgb so lets check the first one
//			DWORD point=STATE_polygon_get_first_point(message_polygon);
//			BYTE rgb[3];
//			STATE_point_get_rgb(point, rgb);
//			if(rgb[0]==0) {
//				STATE_polygon_disable(message_layer_polygon);
//				m_is_message_visible=NO;
//				return;
//			}
//			//Reducing the rgb will make the message darker and because we use LIGHT_SOURCE_GLASS 
//			//The result will be fading away
//			rgb[0]--;
//			rgb[1]--;
//			rgb[2]--;
//			STATE_polygon_set_light(message_layer_polygon, rgb); //Set the rgb of all the points of the polygon
//
//		}
//
//
STATE_IMPORT DWORD STATE_WINAPI STATE_layer_create(double left_x, double top_y, double right_x, double bottom_y, DWORD bitmap_handle, BYTE glass_type);

// Use this function to change the order in which layers are drawn or to
// make them appear behind the render world, in the middle etc. In other words,
// change the default which renders them at the top of the world.
// Eventhough layers appear to be 2D shapes they are actually 3D polygons 
// that have 3D points. By default the Z coordinate value of all the created layer points
// is set to 2. By changing the Z coordinate one can make a layer appear in front of other layers or more deeper 
// inside the world. A bigger value for the depth whould cause the layer to appear more deeprt inside the world
//
// Parameters:
//
//		layer_polygon_handle:
//			This is the handle that is returned from STATE_layer_create()
//
//
//		depth:
//			A number equal or greater than 1.
//
//	Example:
//		
//		// Example A. Make a layer be drawn in front of other layers
//		// by given a value that is less then 2 which is the default.
//		STATE_layer_set_depth(my_layer, 1.5);
//
//		// Example B. Make a layer be drawn behind or in the middle of the world
//		// by given a value that is less then 2 which is the default.
//		// note that if the world totally hides the layer then the layer wont be visible
//		STATE_layer_set_depth(my_layer, 20000);
//		
STATE_IMPORT void STATE_WINAPI STATE_layer_set_depth(DWORD layer_handle, double depth);

//This function provides an easy way for changing the shape of a layer.
//This function is good for creating any shape that is build from 3 or 4 points for eample:
//	any 2D line
//	any kind of triangle
//	any kind of rectangle or any other 4 point shapes.
// For more complex modify the points using STATE_point_set_xyz() and STATE_polygon_add_point()
//
// Parameters:
//
//		layer_handle:
//			This is the handle that is returned from STATE_layer_create()
//			Even if you are not interested in the rectangle layer shape offered by
//			STATE_layer_create(), you should call for creating the layer
//			and then call this function to set the desired shape.
//
//		start, end
//			The function actually creates a line from start to end though by controling the width
//			of the two ends of the line one can create any 4 or 3 points shape.
//
//		base_width, end_width:
//			The width of the created line in each one of its edges
//
//
// Remarks:
//		Note that if the base_width of the end_width is 0, the function would create a triangle
//		and will delete one of the layer points. This means that handles to this layer points
//		should be set to NULL after calling this function.
//
// Example A:
//		//creating a diagnal line. from one side of the screen to the other
//		//First we will create the layer. Note that the initial shape doesn't matter (the 0,0,100,100 below)
//		//since we will overide it later when we call STATE_layer_set_shape()
//		DWORD my_layer=STATE_layer_create(0,0,100,100, NULL, DISABLE_BLENDING);
//		double start[3]={0,0,1};
//		double end[3]={100,100,1};
//		STATE_layer_set_shape(my_layer, start, end, 0.5, 0.5);
//
// Example B:
//		//Creating a triangle (we will assume that my_layer was already created)
//		double start[3]={0,50,1};
//		double end[3]={100,50,1};
//		STATE_layer_set_shape(my_layer, start, end, 50, 0);
//
// Example C:
//		//Creating a non horizontal square (we will assume that my_layer was already created)
//		double start[3]={25,25,1};
//		double end[3]={75,75,1};
//		STATE_layer_set_shape(my_layer, start, end, 50, 50);
//
STATE_IMPORT void STATE_WINAPI STATE_layer_set_shape(DWORD layer_handle, double start[3], double end[3], double base_width, double end_width);

// *** useful and a bit advanced
// Used to create effects of soft shadows.
// This function is used together with STATE_object_auto_add_shadows()
// By default the shadows a painted with a black translucent color (using the ALPHA_FILTER)
// By calling STATE_layer_use_for_painting_shadows()
// You tell the engine to use the setting of this layer to paint the shadows.
// When using a layer as a color fill for shadows it is only used for the shadows and not anything else
// if there are no shadows then it layer is not drawn at all.
// * Note that is works only with shadows that were created using 
//	 STATE_object_auto_add_shadows();
// * Usually you would want this layer to cover the entire screen because shadows that are not included
//	 in the layer area will not be painted.
//
// * There could only be one layer that is used for painting the shadows
//	 The last layer that is specified in the call to STATE_layer_use_for_painting_shadows()
//	 is the layer that is used for painting.
//
// * Note that calling STATE_layer_use_for_painting_shadows(my_layer, NO)
//   will convert this layer to a regular layer which is rendered every time
//   STATE_engine_render() is called. And since usually these layers 
//	 cover the entire screen then ... So if it is not what you meant simply
//	 delete this layer or call STATE_polygon_disable(my_layer) to disable it
//
// Example:
//		DWORD my_bitmap=STATE_bitmap_load("shadow_patern", -1);
//		if(my_bitmap==NULL) return(VR_ERROR);
//		DWORD layer_polygon=STATE_layer_create(0,0, 100, 100, my_bitmap, ALPHA_FILTER);  //cover the whole rendered screen
//		STATE_layer_use_for_painting_shadows(layer_polygon, YES);
//		FOREACH_POINT(layer_polygon, pnt) 
//			STATE_point_set_alpha(pnt, 128); //make the shadow layer half transparent
//
STATE_IMPORT void STATE_WINAPI STATE_layer_use_for_painting_shadows(DWORD layer_handle, int YES_or_NO);

//Returns the layer that is located at the given x,y point
//The x,y point is in client area coordinate.
//This function is usually used with mouse click events to find the layer that the mouse clicked on
STATE_IMPORT DWORD STATE_WINAPI STATE_layer_get_at_point(int x, int y);

//----------------------------------------------------//
//============ T H E    L I N E   A P I ==============//
//----------------------------------------------------//

//Draw 2D lines
//See also STATE_object_create_line()

//*** Useful
// Draw a 2D line on top of the rendered image.
//
// Parameters:
//		x1,y1  x2,y2:
//			The two edges of the line.
//			0,0 is mapped to the top left corner of the screen
//			100,100 is mapped to the bottom right corner of the screen.


// Return Value:
//		The return handle will work with all the polygon and layer APIs so one
//		can use STATE_polygon_set_color_fill() and STATE_polygon_set_translucent() to control the color and the translucency of the line.
//
// See also:	
//		Note that all the STATE_polygon_ and STATE_layer_ function could be used with a line_handle
//		STATE_polygon_disable(), STATE_polygon_set_color_fill(), STATE_polygon_set_translucent()
//		STATE_polygon_delete(), STATE_polygon_move(), STATE_layer_set_shape(), STATE_polygon_set_bitmap_fill()
//		STATE_object_create_line() for creating a 3D line
//
// Example:
//		A)
//		//draw a diagonal line from the top left corner of the screen to the bottom right corner
//		STATE_line_draw(0,0, 100,100, 2, 255,0,0); //draw a red line
//
//		B)
//		//drawing a semi transparent line
//		//Note that line handles, layer handles and polygon handles can use each other functions !
//		DWORD handle=STATE_line_draw(0,0, 100,100, 2, 255,0,0); //draw a red line
//		if(handle==NULL)
//			return(VR_ERROR); //print error message or whatever ...
//		STATE_polygon_set_translucent(handle, NORMAL_GLASS); //line handles can use all the polygon and layer API
//
STATE_IMPORT DWORD STATE_WINAPI STATE_line_draw(double x1, double y1, double x2, double y2, double width, BYTE red, BYTE green, BYTE blue);

//The same as STATE_line_draw just that works with 3D coords input
// Important !!! Will only work after at least one STATE_engine_render() was called
//note that the function does not support ZBuffer so the lines will always be visible
STATE_IMPORT DWORD STATE_WINAPI STATE_line_draw3D(double P1[3], double P2[3], double width, BYTE red, BYTE green, BYTE blue);

//Redefines the edges of a given line. This is much faster then deleting a line and creating a new one
STATE_IMPORT void STATE_WINAPI STATE_line_set_edges(DWORD line_handle, double x1, double y1, double x2, double y2, double width);

//The same as STATE_line_set_edges but simply works with 3D coords input
// Important !!! Will only work after at least one STATE_engine_render() was called
//note that the function does not support ZBuffer so the lines will always be visible
STATE_IMPORT int STATE_WINAPI STATE_line_set_edges3D(DWORD line_handle, double P1[3], double P2[3], double width);


//----------------------------------------------------//
//============ T H E    M E S H   A P I ==============//
//----------------------------------------------------//

// Returns YES or NO.
// YES , if the given handle is a mesh handle. NO if not.
// If it is not a mesh handle , a message box will appear,
// the title of this message box will be the parameter "function_asking"
// if function_asking==NULL than no popup will be created.
// See also IS_MESH()
STATE_IMPORT int STATE_WINAPI STATE_mesh_is_mesh(DWORD mesh_handle, char *function_asking);
STATE_IMPORT char *STATE_WINAPI STATE_mesh_get_name(DWORD mesh_handle);

STATE_IMPORT DWORD STATE_WINAPI STATE_mesh_create_from_file(char *file_name);

//------------------------------------------------------//
//=========== T H E    E F F E C T    A P I ============//
//------------------------------------------------------//

// This API is used to tune the effects and shaders used for rendering.
// 3DSTATE supports Vertex Shaders, Pixel Shaders, fx script file 
// Microsoft HLSL (High Level Shader Language)
// To make an object rendered using a specific effect use the function
// STATE_object_create_effect(); This is the only way to create an effect.
// Once the effect was created the engine will render the given object
// using the specified effect.
// This API (functions starting with 3DSTATE_effect_...) is used to tune the effect used.
// The behavior of the functions of this API is different for every effect used
// Some effects don't support any tuning others support only some of the 
// available functions and only a few support all the functions of this API
// To learn which of the functions of this API can be used to tune a specific Effect
// You should read the specific effect description.
//
// Example:
// If you use EFFECT_SKYSPHERE to render a sky sphere
// You can use STATE_effect_set_color() to set the color of the sky.
// When using the EFFECT_FAST_ENVIRONMENT_MAPPING effect 
// Calling STATE_effect_set_color() will have no effect. 
// In short the functions of this API will give different
// results depending on the Effect used.

// Set the color parameter of the Effect. How this affects the rendering depends
// on the Effect used. If an Effect does not support this function then calling this
// function will not harm (will do nothing)
STATE_IMPORT void STATE_WINAPI STATE_effect_set_color(DWORD effect_handle, BYTE red, BYTE green, BYTE blue, BYTE alpha);

// In some Effects the resulted rendering is also dependent on the location and position
// of another object. Use set the "other object" parameter
// Example:
// The EFFECT_CLOUD_COVER is used to render terrain with cloud shadows.
// use this function to synchronize between the cloud shadows and clouds themselves.
//
STATE_IMPORT void STATE_WINAPI STATE_effect_set_object_to_synchronize_with(DWORD effect_handle, DWORD object_to_synchronize_with);

//Like with all other functions of this API this function will work with some Effects and with
//other it will have no influence.
STATE_IMPORT void STATE_WINAPI STATE_effect_set_light_direction(DWORD effect_handle, float light_dir[3]);

//See remarks above.
STATE_IMPORT void STATE_WINAPI STATE_effect_set_speed(DWORD effect_handle, float speed);

// Returns YES or NO.
// YES , if the given handle is an effect handle. NO if not.
// If it is not a effect handle , a message box will appear,
// the title of this message box will be the parameter "function_asking"
// if function_asking==NULL than no popup will be created.
// See also IS_EFFECT()
STATE_IMPORT int STATE_WINAPI STATE_effect_is_effect(DWORD effect_handle, char *function_asking);

// *** Important function for advanced users.
// If you try to develop your own shaders you will probably need to debug them.
// 3DSTATE Shader engine supports debugging assembly level and high-level language vertex and pixel shaders.
// 3DSTATE supports is integrated into Microsoft Visual Studio .NET 
// Developing your own Shaders requiers a lot of knowledge in 3D and math.
// For most users the best policy would be to use one of the available free shaders / effect files. 
// found on the Internet.Even if you understand nothing of the Shader script, you can still try and modify some of the code, who knows
// many of the best inventions were discovered accedently ( Serendipity ).
//
// How to debug your shaders from inside Microsoft Visual Studio .NET environment.
// -------------------------------------------------------------------------------
//
// 1) You need to have the full DirectX SDK 9.0. Note that installing just the C\C++ subset
//	  will not install the DirectX Extensions for Visual Studio .NET
//	  When asked chose installing Debug Version of Direct3D
//
// 2) Go to the Control Panel and select DirectX. Clcik on the D3D tab and enable all
//	  the debugging oprions you see there. Note that if you want to debug a Pixel Shader
//    then check Enumerate Reference Rasterizer (and uncheck the other two rasterizers)
//
// 3) Make sure you have the call STATE_effect_set_debug_mode()
//	  note that this function must be called before the first call to STATE_engine_render()

// 4) To start your program go to the Debug menu->Direct3D->Start D3D
//	  If you dont have the Direct3D entery in you debug menu it means
//	  that DirectX Extensions for Visual Studio .NET is not installed
//    To install it reinstall DirectX SDK 9.0 (the full version)
//
// 5) You can put breakpoints in your shader code the same way you put in 
//	  your C++ code. The easiest way is to press F2 at the code line where you want to insert
//    a breakpoint. Note that if you put a breakpoint inside a Pixel Shader 
//	  you must use the Reference Rasterizer (see item "2" above)
//
//
// When running in debug mode, your program will be much slower. To get is back to full speed
// do the following:
// 1) In the DirectX control panel, unselect all the debug options.
//    Make sure you enumarate the HAL rasterizer.
//
// 2) Cancel the call to STATE_effect_set_debug_mode()
//
//
STATE_IMPORT void STATE_WINAPI STATE_effect_set_debug_mode();

STATE_IMPORT void STATE_WINAPI STATE_effect_delete(DWORD effect_handle);

//-----------------------------------------------------//
//======== T H E    G R O U P   A P I =================//
//-----------------------------------------------------//

// This API is used in editor mode to manipulate groups of polygons
// See also the OBJECT API for manipulating things in viewer_mode mode.
// With this API you can group polygons together. 
// Example,
// say we have a room with three tables inside. Each table has four legs
// and each leg has 6 polygons. We will probably like to build it like that:
// Group "leg" contains the six polygons
// group "table" which will include the four "leg" groups and the table top polygons
// group "room" which will include the three tables and the walls polygons
//

// A general note for most of the functions in this API:
// After a new group is created, polygons can be associated with this group
// by calling STATE_polygon_set_group()
// IMPORTANT: if the polygons that are associated with the group are not part of the
//			  world (after creating a new polygon, we can add it to the world by calling STATE_engine_add_polygon() )
//			  most of the function in the group API wont work for those polygons.


// Returns YES or NO.
// YES , if the given handle is a group handle. NO if not.
// If it is not a group handle , a message box will appear,
// the title of this message box will be the parameter "function_asking"
// if function_asking==NULL than no popup will be created.
// See also IS_GROUP()
// NOTE, unlike all the other IS_xxx function this function consider a NULL
// as a legal handle, since a NULL group means the whole world.
STATE_IMPORT int STATE_WINAPI STATE_group_is_group(DWORD group_handle, char *function_asking);


// returns a handle of the newly created group
// After a new group is created, polygons can be associated with this group
// by calling STATE_polygon_set_group()
// IMPORTANT: if the polygons that are associated with the group are not part of the
//			  world (after creating a new polygon, we can add it to the world by calling STATE_engine_add_polygon() )
//			  most of the function in the group API wont work for those polygons.
STATE_IMPORT DWORD STATE_WINAPI STATE_group_create(char *name);

// These two functions are used to go over all the existing groups.
// Note that STATE_group_get_first_group() could return any group (and not just the top group)
// Example:
//		//go over all the groups
//		for(DWORD grp=STATE_group_get_first_group(); grp!=NULL; grp=STATE_group_get_next(grp) ) {
//			do something with the group
//		}
//
// See also STATE_group_get_first_included_group(), STATE_group_get_next_included_group()
STATE_IMPORT DWORD STATE_WINAPI STATE_group_get_first_group(void);
STATE_IMPORT DWORD STATE_WINAPI STATE_group_get_next(DWORD group_handle);

STATE_IMPORT DWORD STATE_WINAPI STATE_group_get_using_name(char *name);


STATE_IMPORT DWORD STATE_WINAPI STATE_group_get_first_polygon(DWORD group_handle);
STATE_IMPORT DWORD STATE_WINAPI STATE_group_get_next_polygon(DWORD group_handle, DWORD polygon_handle);


STATE_IMPORT DWORD STATE_WINAPI STATE_group_get_father_group(DWORD group_handle);

// We can call this function with father_group==NULL
// This will mean that this group is at the top of the hierarchy and it doesnt belong to any other group.
// If the function fails it is most likly that we tried to make a loop in hierarchy of the groups
// (for example, groupA father is groupB. groupB father is groupC, and groupC father is groupA ...)
// Returns OK or VR_ERROR.
//
// Example:
//	STATE_group_set_father_group(leg_of_a_chair_group, chair_group);
STATE_IMPORT int STATE_WINAPI STATE_group_set_father_group(DWORD group_handle, DWORD father_group);

// Returns whether groupA is included in groupB.
// A NULL group_handle means the whole world
// Return value: Returns YES or NO. 
//
// Note the following examples: 
// 1)	If groupA_handle==groupB_handle returns YES
// 2)	If groupB_handle==NULL returns YES
//
//	Parameters:
//	groupA_handle, groupB_handle: checks wheter groupA belongs to groupB
//
//	See also STATE_group_is_groupA_included_in_enabled_parts_of_groupB();
//	Version note:
//	In version 5.0 and before STATE_group_is_groupA_included_in_groupB() was identical to
//	STATE_group_is_groupA_included_in_enabled_parts_of_groupB(). In other words, in old version 
//	there was only one function which behaved exactly as STATE_group_is_groupA_included_in_enabled_parts_of_groupB()
//
STATE_IMPORT int STATE_WINAPI STATE_group_is_groupA_included_in_groupB(DWORD groupA_handle, DWORD groupB_handle);

// Checks whether one group belongs to enabled parts of another group.
// (disabled parts are subgroups that are disabled. See STATE_group_disable() )
// A NULL group_handle means the whole world
// Return value: Returns YES or NO. 
// Note the following examples: 
// 1)	If groupA_handle==groupB_handle returns YES
// 2)	If groupB_handle==NULL it will return YES only if groupA does not belong to a disabled group.
// 3)	If groupA belongs to a disabled group that belongs to groupB then returns NO 
//
//	Parameters:
//	groupA_handle, groupB_handle: checks wheter groupA belongs to groupB
//
//	Example:
//		//check if a polygon belongs to a disabled group
//		DWORD my_group=STATE_polygon_get_group(my_polygon)
//		if(STATE_group_is_groupA_included_in_enabled_parts_of_groupB( my_group, NULL)==NO) {
//			//my_group belongs to a disabled group !!!
//		}
//
//	See also STATE_group_is_groupA_included_in_groupB()
//
//	Version note:
//	In version 5.0 and before STATE_group_is_groupA_included_in_groupB() was identical to
//	STATE_group_is_groupA_included_in_enabled_parts_of_groupB(). In other words, in old version 
//	there was only one function which behaved exactly as STATE_group_is_groupA_included_in_enabled_parts_of_groupB()
//
STATE_IMPORT int STATE_WINAPI STATE_group_is_groupA_included_in_enabled_parts_of_groupB(DWORD groupA_handle, DWORD groupB_handle);



// Returns the name of the group.
// The function returns a pointer to the name which is stored internally inside the 3D Engine
// Visual Basic users should use STATE_entity_get_name() instead.
// See also STATE_entity_get_name()
STATE_IMPORT char *STATE_WINAPI STATE_group_get_name(DWORD group_handle);

// A NULL group_handle will get the rotation reference point of the whole world
// This is the point that all the rotations will use as the center of rotation
STATE_IMPORT void STATE_WINAPI STATE_group_get_rotate_reference_point(DWORD group_handle, double center[3]);

// A NULL group_handle will get the rotation reference point of the whole world
// This is the point that all the rotations will use as the center of rotation
// see also STATE_object_set_rotation_center()
STATE_IMPORT void STATE_WINAPI STATE_group_set_rotate_reference_point(DWORD group_handle, double center[3]);

// A NULL group_handle will get the center of mass of all the world
// The operations set_location and move are done according to the center of mass
// This function calls STATE_group_get_bounding_box() and then
// calculates the center of the box.
STATE_IMPORT void STATE_WINAPI STATE_group_get_center_of_mass(DWORD group_handle, double center[3]);

// *** A very useful function
// returns the bounding box of the world or a specific group. 
// Works also in VIEWER_MODE
// A NULL group handle will return the bounding box of the entire world
//
//	the box returned has the following values
//		box[0][0] is the minimum X
//		box[0][1] is the minimum Y
//		box[0][2] is the minimum Z
//		box[1][0] is the max X 
//		box[1][1] is the max Y
//		box[1][2] is the max Z
//
// This function is not very speedy 
// Unlike most of the get function which just return a value
// this function makes all the calculation going over all the vertexes
// in the group (so it is not a very fast function)
//
// Example:
//		//getting the bounding box of the entire world
//		double box[2][3];
//		STATE_group_get_bounding_box(NULL, box);
//
STATE_IMPORT void STATE_WINAPI STATE_group_get_bounding_box(DWORD group_handle, double box[2][3]);

STATE_IMPORT void STATE_WINAPI STATE_group_set_name(DWORD group_handle, char *name);

// A NULL group_handle means the whole world
//
// Reamarks:
//		Polygons	
STATE_IMPORT int STATE_WINAPI STATE_group_get_number_of_polygons(DWORD group_handle);

// Rotates a group around its rotation center. 
// To set the rotation center call: STATE_group_set_rotate_reference_point()
// Groups that include a rotated polygon (See STATE_polygon_is_rotated ) cannot
// be rotated using this function (You can call this function but it will have no effect)
//
// Parameters:
//	group_handle:
//		A NULL group_handle means the whole world
//
//	degrees: In how many degrees to rotate the group. 
//
//	space_flag: CAMERA_SPACE, OBJECT_SPACE, WORLD_SPACE
//	axis_flag:  0 for the X axis. 1 for the Y axis. 2 for the Z axis
//
// see also STATE_group_is_rotation_enabled()
// STATE_group_set_rotate_reference_point()
// STATE_group_get_center_of_mass()
STATE_IMPORT void STATE_WINAPI STATE_group_rotate(DWORD group_handle, double degrees, int space_flag, int axis_flag);


// A NULL group_handle means the whole world
STATE_IMPORT void STATE_WINAPI STATE_group_rotate_using_matrix(DWORD group_handle, double rotate_matrix[3][3]);

STATE_IMPORT void STATE_WINAPI STATE_group_transform_using_matrix4x4(DWORD group_handle, double trans_matrix[4][4]);


// A NULL group_handle means the whole world
// moves the center of mass of the group to the given location
// Note that there is no STATE_group_get_location(), instead use
// STATE_group_get_center_of_mass().
STATE_IMPORT void STATE_WINAPI STATE_group_set_location(DWORD group_handle, double location[3]);

// ZZZ
// I am about to cancel this function since it is the same as
// STATE_group_get_center_of_mass().
// mean while I leave it just so I can link to the executable
STATE_IMPORT void STATE_WINAPI STATE_group_get_location(DWORD group_handle, double *X, double *Y, double *Z);


// A NULL group_handle means the whole world
// moves all the vertex belonging to the group. The result is that the
// center of mass also moves with the given step 
// space flag could be: CAMERA_SPACE, OBJECT_SPACE, WORLD_SPACE
// Note that this function does not update the rotation center of the group
// This means that if you use STATE_group_rotate() after moving the group 
// the rotation center will stay as before. See also STATE_group_set_rotate_reference_point()
// STATE_group_get_rotate_reference_point()
STATE_IMPORT void STATE_WINAPI STATE_group_move(DWORD group_handle, double step[3], int space_flag);

// A NULL group_handle means the whole world
// space flag could be: CAMERA_SPACE, OBJECT_SPACE, WORLD_SPACE
STATE_IMPORT void STATE_WINAPI STATE_group_scale(DWORD group_handle, int space_flag,double scale_x, double scale_y, double scale_z);

//Get the width, height and depth of the group. Please note that
//The dimensions will be returned inside the parameter "dimension"
// dimension[0] will become the depth
// dimension[1] will be the width
//dimension[2] will be the height
//the dimensions are according to the bottom and the front of the group.
//To declarea polygon as being the bottom of the group see those function
// STATE_polygon_set_orientation()
//Returns OK or VR_ERROR;
//Example:
//		double size[3];
//		STATE_group_get_dimensions(my_group, size);
STATE_IMPORT int STATE_WINAPI STATE_group_get_dimensions(DWORD group_handle, double dimensions[3]);


// deletes the group but not the polygon included
// group handle cannot be NULL
// Make sure you dont use the handle after it was ungrouped.
// Example:
//		//The example shows how to go over all the polygons that belong to several group
//		//Note that if we would go over all the polygons of the first group 
//		//and then over all the polygons of the second group we might go over some polygon
//		//twice since it could be for example that groupA belongs to groupB
//		//and therefore by going over all the polygons of groupB will also go
//		//over all the polygons of groupA. The code below show how to do it right so we 
//		//go over each polygon only once. We do it by grouping several objects together and then ungroup them back
//
//		DWORD top_group=STATE_group_creare("Top group");
//		STATE_group_set_father_group(grpA, top_group);
//		STATE_group_set_father_group(grpB, top_group);
//		STATE_group_set_father_group(grpC, top_group);
//		for( DWORD poly=STATE_group_get_first_polygon(top_group); poly!=NULL; poly=STATE_group_get_next_polygon(poly) ) {
//
//			// Your code here ....
//		}
//
//		STATE_group_ungroup(top_group); // We can now delete the top group if we want to.
//		my_group=NULL; //For safty since we cant use it anymore.
STATE_IMPORT void STATE_WINAPI STATE_group_ungroup(DWORD group_handle);

// deletes all the polygons that belong to this group
// and all the groups that belong to this group
// return the number of polygons that were deleted
// A NULL group_handle will delete the whole world.
STATE_IMPORT int STATE_WINAPI STATE_group_delete_members(DWORD group_handle);

// Returns YES or NO.
// The static/dynamic properties are relevant when we load the world
// in viewer mode. Only objects that are declared dynamic (not static) in
// editor mode will be able to move (using the object API)
// see also STATE_group_set_static(), STATE_group_set_dynamic()
STATE_IMPORT int STATE_WINAPI STATE_group_is_static(DWORD group_handle);

// The static/dynamic properties are relevant when we will load the world
// in viewer mode. Only objects that are declared dynamic (not static) in
// editor mode will be able to move (using the object API)
// see also STATE_group_is_static(), STATE_group_set_dynamic()
STATE_IMPORT void STATE_WINAPI STATE_group_set_static(DWORD group_handle);

// The static/dynamic properties are relevant when we will load the world
// in viewer mode. Only objects that are declared dynamic (not static) in
// editor mode will be able to move (using the object API)
// see also STATE_group_set_static(), STATE_group_is_static()
STATE_IMPORT void STATE_WINAPI STATE_group_set_dynamic(DWORD group_handle);

//Usually when we build a world (using STATE World Builder for example)
//we have many models which we want them to be disabled when the world
//is loaded (in viewer mode) . For example, we could have few models of a bird that
//are interchanged to form a 3D animation. If the movement is composed of
// 3 models, then two of them will always be disabled, and one will be enabled.
// Using this function you can decide who is going to be loaded as disabled and
// who as an enabled object.
//Note that this function is only relevant to groups that were set as dynamic
//( using STATE_group_set_dynamic() ). 
// The default is NO
// See also STATE_object_disable()
STATE_IMPORT void STATE_WINAPI STATE_group_load_as_disabled(DWORD group_handle, int YES_or_NO);

//Returns YES or NO.
//See also STATE_group_load_as_disabled(), STATE_object_disable();
STATE_IMPORT int STATE_WINAPI STATE_group_get_load_as_disabled_status(DWORD group_handle);

// returns a handle to the new group
STATE_IMPORT DWORD STATE_WINAPI STATE_group_duplicate_tree(DWORD group_handle);

// The function get three arguments.
// group_handle: the group to rotate
// polygon_in_the_group: a polygon that belongs to the group
// reference_polygon: any polygon.
// inverse_flag: YES or NO. If YES the Normal of the reference polygon is inversed (as if we took the back side of the reference polygon)
// The function rotates the group so that "polygon_in_the_group" will
// be parallel to "reference_polygon"
// returns OK, VR_ERROR.
// A NULL group handle means the whole world
// Note this very nice trick:
//		If the reference polygon also belongs to the group we can call
//		this function several times consecutively.
//		example say we have a box and we call this function as follows:
//		STATE_group_rotate_to_match_polygon(grp, left_poly, front_poly);
//		after the first call the box will rotate in 90 degrees so the left
//		polygon is now were the front poly was. BUT the front poly is part of
//		the group ,so it was rotated too. consecutive calls to this function
//		will cause a 90 degrees rotation each time. like a dog trying to catch its tail.
STATE_IMPORT int STATE_WINAPI STATE_group_rotate_to_match_polygon(DWORD group_handle, DWORD polygon_in_the_group, DWORD reference_polygon, int inverse_flag);

// Similar to STATE_group_rotate_to_match_polygon()
// with the difference that here we have an auxiliary line (going through points p1 and p2)
// To make things clearer let's say we have a square polygon_in_the_group
// and a square reference_polygon, there are four final position for the group
// so that polygon_in_the_group points to the same direction as reference_polygon.
// (each one of the four edges of one group polygon can be aligned with an edge of the
// reference polygon). STATE_group_rotate_to_match_polygon() rotates so it will make
// the minimal rotation. This is usually what we want. So usually we will use
// STATE_group_rotate_to_match_polygon(). We will use this function when we want
// to turn around a specific line. All the rest is exactly like in STATE_group_rotate_to_match_polygon()
// Returns OK or VR_ERROR.
STATE_IMPORT int STATE_WINAPI STATE_group_rotate_around_line_to_match_polygon(DWORD group_handle, DWORD polygon_in_the_group, DWORD reference_polygon, int inverse_flag, double p1[3], double p2[3]);


// The same as STATE_group_rotate_to_match_polygon() only that uses a direction instead of a reference polygon.
STATE_IMPORT int STATE_WINAPI STATE_group_rotate_to_match_direction(DWORD group_handle, DWORD polygon_in_the_group, double direction[3]);


// The given group is moved so his given point will match
// "point_to_match" (the third argument);
// return OK or VR_ERROR.
STATE_IMPORT void STATE_WINAPI STATE_group_move_to_match_point(DWORD group_handle, DWORD point_belongs_to_group, DWORD point_to_match);

STATE_IMPORT int STATE_WINAPI STATE_group_is_bitmap_used(DWORD group_handle, DWORD bitmap_handle);

// finds how many intersections between polygons of groupA (that dont belong to grpB)
// and polygons of group B (that are not included in A)
// groupA or groupB could be NULL (means the whole world) or any other group
STATE_IMPORT int STATE_WINAPI STATE_group_count_intersections(DWORD groupA, DWORD groupB);

// Returns a handle to a polygon.
// The returned polygon is the first polygon that his orientation fits (according to the orientation field of the polygon)
// See also STATE_polygon_set_orientation() and STATE_polygon_get_orientation()
STATE_IMPORT DWORD STATE_WINAPI STATE_group_get_bottom_polygon(DWORD group_handle);
STATE_IMPORT DWORD STATE_WINAPI STATE_group_get_top_polygon(DWORD group_handle);
STATE_IMPORT DWORD STATE_WINAPI STATE_group_get_front_polygon(DWORD group_handle);
STATE_IMPORT DWORD STATE_WINAPI STATE_group_get_back_polygon(DWORD group_handle);


// What is orientation and why do we need it.
// When we want to drop an object\group on the ground (another surface) we need to know
// what surface of the object\group should be glued to the ground.
// The computer of course doesnt know on his own that a piano should stand on its legs ...
// With this flag the designer of the model can express his will that the piano should stand on its legs ...
//
// In order to fully define the orientation of a group, one polygon of the group should have ORIENTATION_BOTTOM or ORIENTATION_top
// and a second polygon should have ORIENTATION_FRONT or ORIENTATION_BACK beside those two polygons all the rest
// should have ORIENTATION_UNKNOWN. Nothing bad happens if they dont, it is just that it can cause contradiction.
//
// Going back to STATE_group_set_orientation(),
// the orientation_value should be one of the five values ORIENTATION_UNKNOWN ,ORIENTATION_TOP ORIENTATION_BOTTOM, ORIENTATION_FRONT, ORIENTATION_BACK
// if given ORIENTATION_UNKNOWN, then all the polygons of the group will be set with ORIENTATION_UNKNOWN.
//								In this case polygon_handle could be NULL
// if given ORIENTATION_BOTTOM, then the given polygon will be set with ORIENTATION_BOTTOM, and all the other polygons of the group that had ORIENTATION_BOTTOM or ORIENTATION_TOP will be set with ORIENTATION_UNKNOWN
// if given ORIENTATION_TOP, then the given polygon will be set with ORIENTATION_TOP, and all the other polygons of the group that had ORIENTATION_BOTTOM or ORIENTATION_TOP will be set with ORIENTATION_UNKNOWN
// if given ORIENTATION_FRONT, then the given polygon will be set with ORIENTATION_FRONT, and all the other polygons of the group that had ORIENTATION_FRONT or ORIENTATION_BACK will be set with ORIENTATION_UNKNOWN
// if given ORIENTATION_BACK, then the given polygon will be set with ORIENTATION_BACK, and all the other polygons of the group that had ORIENTATION_FRONT or ORIENTATION_BACK will be set with ORIENTATION_UNKNOWN
// See also STATE_polygon_set_orientation()
STATE_IMPORT void STATE_WINAPI STATE_group_set_orientation(DWORD group_handle, DWORD polygon_handle, int orientation_value);


//Gets the group axis system according to the orientation of its polygons.
//If there are more than one polygon with the same orientation
// it will choose first, those polygons that belong directly to the group.
// In order to define the group axis system we need to have
// at least two polygons with the orientation flag given.
// one polygon should have the orientation FRONT or BACK and
// the other should be BOTTOM or TOP
// If there is only one polygon with orientation, the function will complete
// the rest according to the computer's understanding ....
// If there is no polygon with a given orientation it will again try to build
// the best system it can ...
// Return value
//--------------
// OK or VR_ERROR. In any case the axis_system given is legal.
// in case of error the axis system will be the same as the world axis system (1,0,0; 0,1,0; 0,0,1)
STATE_IMPORT int STATE_WINAPI STATE_group_calculate_axis_system(DWORD group_handle, double x_axis[3], double y_axis[3], double z_axis[3]);


/*
// Set the object to chase after
// the given argument should NOT be NULL
// Note that if the group was previously chasing a camera or a group
// , those chasing operation are canceled. (The group can chase only one thing at a time).
STATE_IMPORT void STATE_WINAPI STATE_group_set_object_to_chase(DWORD group_handle, DWORD object_to_chase_handle);

// If the return value is NULL it means no object is being chased
STATE_IMPORT DWORD STATE_WINAPI STATE_group_get_chased_object(DWORD group_handle);

// Set the camera to chase after
// the given argument should NOT be NULL
// Note that if the group was previously chasing another object or a group
// , those chasing operation are canceled. (The object can chase only one thing at a time).
STATE_IMPORT void STATE_WINAPI STATE_group_set_camera_to_chase(DWORD group_handle, DWORD camera_to_chase_handle);

// If the return value is NULL it means no camera is being chased
STATE_IMPORT DWORD STATE_WINAPI STATE_group_get_chased_camera(DWORD group_handle);

// Set the group to chase after
// the given argument should NOT be NULL
// Note that if the group was previously chasing a camera or an object
// , those chasing operation are canceled. (The object can chase only one thing at a time).
STATE_IMPORT void STATE_WINAPI STATE_group_set_group_to_chase(DWORD group_handle, DWORD group_to_chase_handle);

// If the return value is NULL it means no group is being chased
STATE_IMPORT DWORD STATE_WINAPI STATE_group_get_chased_group(DWORD group_handle);
*/


// name could be a name of a camera , object or a group.
STATE_IMPORT void STATE_WINAPI STATE_group_set_name_to_chase(DWORD group_handle, char *name);

// If the return value is NULL it means nothing is being chased
STATE_IMPORT char * STATE_WINAPI STATE_group_get_chased_name(DWORD group_handle);


// return OK or VR_ERROR
// the offset is returned through argument "offset"
STATE_IMPORT int STATE_WINAPI STATE_group_get_chase_offset(DWORD group_handle, double offset[3]);

// return OK or VR_ERROR
STATE_IMPORT int STATE_WINAPI STATE_group_set_chase_offset(DWORD group_handle, double offset[3]);

// The returned number is between 0 and one
// a number close to 0 means that the tracking is weak
// 1 means that the tracking is very sharp.
STATE_IMPORT double STATE_WINAPI STATE_group_get_chase_softness(DWORD group_handle);


// The returned number is between 0 and one
// a number close to 0 means that the tracking is weak
// 1 means that the tracking is very sharp.
// it the given number is bigger than one it is set as one
// if it is smaller than 0 than it set as 0
STATE_IMPORT void STATE_WINAPI STATE_group_set_chase_softness(DWORD group_handle, double softness);

// look in the header file for possible values and their meanings
// (look for NO_CHASE, CHASE_LOCATION and others ...)
STATE_IMPORT char *STATE_WINAPI STATE_group_get_chase_type_name(DWORD group_handle);

// Like STATE_group_get_chase_type_name() only that returns the equivalent constant
STATE_IMPORT int STATE_WINAPI STATE_group_get_chase_type(DWORD group_handle);

// look in the header file for possible values and their meanings
// (look for NO_CHASE, CHASE_LOCATION and others ...)
STATE_IMPORT void STATE_WINAPI STATE_group_set_chase_type(DWORD group_handle, int chase_type);


STATE_IMPORT void STATE_WINAPI STATE_group_set_track_name(DWORD group_handle, char *track_name);

STATE_IMPORT char * STATE_WINAPI STATE_group_get_track_name(DWORD group_handle);

// the offset is return through argument "offset"
STATE_IMPORT void STATE_WINAPI STATE_group_get_track_offset(DWORD group_handle, double offset[3]);

STATE_IMPORT void STATE_WINAPI STATE_group_set_track_offset(DWORD group_handle, double offset[3]);



// Calling this function will make the group fall down from
// it's track till it reaches "height_above_ground" distance from the ground
// fall_through_dynamic_objects could be YES or NO. the idea here is that if for example we have a runner (human being)
// and the track is very high in the air then it might fall on the top of the airplanes so we would like it to fall through the planes on the ground
// if on the other hand there are cars on the ground we would like it to fall on the top of the cars and not to pass through.
STATE_IMPORT void STATE_WINAPI STATE_group_set_falling_from_track(DWORD group_handle, double	height_above_ground, int fall_through_dynamic_objects);

STATE_IMPORT void STATE_WINAPI STATE_group_unset_falling_from_track(DWORD group_handle);

// Returns the current status in the three params: set_flag, height_above_ground, fall_through_dynamic_objects
// set_flag could be YES or NO 
// fall_through_dynamic_objects could be YES or NO . See STATE_group_set_falling_from_track() for more details
STATE_IMPORT void STATE_WINAPI STATE_group_get_falling_from_track_params(DWORD group_handle, int *set_flag, double	*height_above_ground, int *fall_through_dynamic_objects);


STATE_IMPORT void STATE_WINAPI STATE_group_get_speed(DWORD group_handle, double speed[3]);

STATE_IMPORT void STATE_WINAPI STATE_group_set_speed(DWORD group_handle, double speed[3]);

STATE_IMPORT double STATE_WINAPI STATE_group_get_absolute_speed(DWORD group_handle);

STATE_IMPORT void STATE_WINAPI STATE_group_set_absolute_speed(DWORD group_handle, double speed);

// rotates the group so that the orientation of the group matches the given directions
// The given directions dont have to be normalized.
// one of the direction can be 0,0,0 in this case it is calculated automatically.
// If the given axis are not orthogonal, the function 
// handle it by building an orthogonal system that is close as possible
// to the given axis.
// see also STATE_group_calculate_axis_system()
// Here are some examples, note that
// all the examples give the same result.
// Example A:
//	x_axis[0]=0; x_axis[1]=1; x_axis[2]=0; 
//	y_axis[0]=-1; y_axis[1]=0; y_axis[2]=0; 
//	z_axis[0]=0; z_axis[1]=0; z_axis[2]=1; 
//	STATE_group_rotate_to_match_axis_system(x_axis, y_axis, z_axis);
//
// Example B:
//	//Example B would give exactly the same result as example A
//	x_axis[0]=0; x_axis[1]=111; x_axis[2]=0; 
//	y_axis[0]=-17; y_axis[1]=0; y_axis[2]=0; 
//	z_axis[0]=0; z_axis[1]=0; z_axis[2]=12; 
//	STATE_group_rotate_to_match_axis_system(x_axis, y_axis, z_axis);
//
// Example C:
//	//Note that  z_axis=(0,0,0) and will be automatically 
//  // calculated by the function
//	//Example C would give exactly the same result as example A and B
//	x_axis[0]=0; x_axis[1]=111; x_axis[2]=0; 
//	y_axis[0]=-17; y_axis[1]=0; y_axis[2]=0; 
//	z_axis[0]=0; z_axis[1]=0; z_axis[2]=0; 
//	STATE_group_rotate_to_match_axis_system(x_axis, y_axis, z_axis);
//
// Example D:
//  // Note that z_axis=(0,0,0) and that the x_axis and the y_axis are not orthogonal to each other.
//  //Note that we will get again the same result
//	x_axis[0]=0; x_axis[1]=1; x_axis[2]=0; 
//	y_axis[0]=-1; y_axis[1]=11; y_axis[2]=0; 
//	z_axis[0]=0; z_axis[1]=0; z_axis[2]=0; 
//	STATE_group_rotate_to_match_axis_system(x_axis, y_axis, z_axis);
//
// Example E:
//  // Note that z_axis=(-1,0,0) which doesn't give a right hand system
//	// in this case the z_axis will be ignored and calculated by doing cross between the x_axis and the y_axis
//  //Note that we will get again the same result
//	x_axis[0]=0; x_axis[1]=1; x_axis[2]=0; 
//	y_axis[0]=-1; y_axis[1]=0; y_axis[2]=0; 
//	z_axis[0]=0; z_axis[1]=0; z_axis[2]=-1; 
//	STATE_group_rotate_to_match_axis_system(x_axis, y_axis, z_axis);
//
// Returns OK or VR_ERROR.
STATE_IMPORT int STATE_WINAPI STATE_group_rotate_to_match_axis_system(DWORD group_handle, double x_axis[3], double y_axis[3], double z_axis[3]);

// Rotates the group around the given line.
// Returns OK or VR_ERROR.
STATE_IMPORT int STATE_WINAPI STATE_group_rotate_around_line(DWORD group_handle, double p1[3], double p2[3], double degrees);

// Stretch a bitmap on all the polygons that belong to a group.
// works best if the group is flat like floor or terrain.
// If group_handle==NULL it will effect the whole world
// If bitmap_handle==NULL it wont change the bitmap.
// Note that if you want the bitmap to repeat more than once on the model (tile mode)
// then the dimensions of the bitmap must be in the power of two (for example 32,64,128,256,512,1024,2048 etc ... 
// all these numbers are powers of two).
//
// set_second_bitmap:
//		If 1 is given then the second bitmap will be set. If 0 then the first bitmap will be set
//
// returns OK or VR_ERROR
//
STATE_IMPORT int STATE_WINAPI STATE_group_wrap_a_bitmap(DWORD group_handle, DWORD bitmap_handle, int repeat_x, int repeat_y, int set_second_bitmap);


// Obsolete, use STATE_group_create_reduced_copy() instead
// This function reduces the number of polygons by
// eliminating redundant polygons and by joining several polygons
// into one polygon. 
// accuracy is used when we try to check whether two polygons
// have the same plane (so we can unite them to one polygon). 
// If this number for example is 0.1 and 
// the X component if the normals are planeA[0]=1 and planeB[0]=0.9
// then the join operation will take place.
// can be done only in editor mode.
// returns the number of polygons that were reduced.
// Returns VR_ERROR on failure on success returns the number of polygons that were reduced
STATE_IMPORT int STATE_WINAPI STATE_group_reduce_polygons_count(DWORD group_handle ,double accuracy);


//This is a very powerfull function for doing polygon reduction.
//optimization_level: a bigger number will give a stronger polygon reduction.
//					  one should play with this value to achieve the optimum result.
//					  tipical values are 0.1, 1, 10 , 100.
//					  a value of 1 for example will snap the model into a grid that
//					  its smallest unit is 1.
//
//returns a handle to the new group of polygons.
STATE_IMPORT DWORD STATE_WINAPI STATE_group_create_reduced_copy(DWORD group_to_copy, double optimization_level) ;

// similar to STATE_group_duplicate_tree() just that the new copy will be made of triangles only.
// Returns a handle to the new copy
STATE_IMPORT DWORD STATE_WINAPI STATE_group_create_copy_made_of_triangles(DWORD group_to_copy) ;



// Returns YES or NO.
// NO means that you cant use STATE_group_rotate() to rotate the group
// The function will return NO, only if the group has at least one rotated polygon inside
STATE_IMPORT int STATE_WINAPI STATE_group_is_rotation_enabled(DWORD group_handle);

//Returns OK or VR_ERROR
STATE_IMPORT int STATE_WINAPI STATE_group_convert_point_to_world_space(DWORD group_handle, double group_space_point[3], double result[3]);
STATE_IMPORT int STATE_WINAPI STATE_group_convert_point_to_group_space(DWORD group_handle, double world_space_point[3], double result[3]);

STATE_IMPORT void STATE_WINAPI STATE_group_set_chase_distance(DWORD group_handle, double chase_distance);
STATE_IMPORT double STATE_WINAPI STATE_group_get_chase_distance(DWORD group_handle);

//Exactly like STATE_polygon_delete_patches() only that it does it for the whole polygons of the group.
STATE_IMPORT void STATE_WINAPI STATE_group_delete_patches(DWORD group_handle, int type_of_patches_to_delete_or_serial_number);



//Set the color of all the patches in the group
// values for red green and blue should be in the range [0-255]
// patches_type should be one of the following: ALL_PATCHES, SHADOW_PATCHES, REGULAR_PATCHES
// or the serial number that was given when the patches were created
// Example A:
//		//Setting the color to all the patches of the given group
//		STATE_group_set_patches_color(my_room, ALL_PATCHES);
//
// Example B:
//		//Setting the color to all the patches that were created using the functions
//		//STATE_polygon_add_patch() and STATE_polygon_add_patch_easy()
//		STATE_group_set_patches_color(my_room, REGULAR_PATCHES);
//  
// Example C:
//		Setting the color to all the patches that were created using the function STATE_engine_create_shadow()
//		STATE_group_set_patches_color(my_room, SHADOW_PATCHES);
//
// Example D:
//		Delete patches that were created using the function STATE_engine_create_shadow() with a serial_number==220
//		STATE_group_set_patches_color(my_room, 39);
//
// See also: STATE_engine_create_shadow(), STATE_polygon_add_patch() ,STATE_polygon_add_patch_easy()
//			 STATE_polygon_delete_patches(), STATE_group_delete_patches()
STATE_IMPORT void STATE_WINAPI STATE_group_set_patches_color(DWORD group_handle, int red, int green, int blue, int patches_type);
STATE_IMPORT void STATE_WINAPI STATE_group_set_patches_bitmap(DWORD group_handle, DWORD bitmap_handle, int patches_type);
STATE_IMPORT void STATE_WINAPI STATE_group_set_patches_translucent(DWORD group_handle, BYTE translucent_type, int patches_type);





//The functions below are the equivalent of STATE_group_get\set_physics_...
//They will be useful for you only if you build some kind of World Editor (Such as STATE World Builder)
//The functions wont show any change till we save the world and reload it in Viewer mode
//and when the chase_type is set to CHASE_PHYSICS.
//
// Return value: The rotation space (WORLD_SPACE or OBJECT_SPACE)
// through rotation_xyz[] the function returns the rotation angles
//		rotation_xyz[0] is the number of angles that the object will rotate 
//		around the X axis, each time STATE_engine_render() is called
// see also STATE_object_set_physics_rotation()
//Note that this is only relevant when applied to a dynamic group and that CHASE_PHYSICS is applied (only in viewer mode)
STATE_IMPORT int STATE_WINAPI STATE_group_get_physics_rotation(DWORD group_handle, double rotation_xyz[3]);
STATE_IMPORT void STATE_WINAPI STATE_group_set_physics_rotation(DWORD group_handle, int rotation_space ,double rotate_around_x_axis, double rotate_around_y_axis, double rotate_around_z_axis);


// Set the force that will work on the object. For example to simulate gravity force
// do like that:
//		double force[3];
//		force[0]=0;
//		force[1]=0;
//		force[2]= -10; //simulate gravity 10 in the direction of the -Z axis.
//		STATE_group_set_force(my_object,force);
//
//Note that this is only relevant when applied to a dynamic group and that CHASE_PHYSICS is applied (only in viewer mode)
STATE_IMPORT void STATE_WINAPI STATE_group_set_physics_force(DWORD group_handle, double force[3]);
//The force is returned through force argument.
STATE_IMPORT void STATE_WINAPI STATE_group_get_physics_force(DWORD group_handle, double force[3]);


//Friction is a value smaller or equal to 1
// if friction==0 it means no friction
// if friction==1 it means infinite friction nothing can move
// if friction is smaller than 0, it act like pressing on the gas
// The exact calculation.
// Each render the absolute_speed is multiply with (1-friction).
//
// Note, a negative number can use for increasing the speed.
// a friction of -1 will double the speed.
// -2 will multiply the speed by three each rendering.
// so choose number carefully.
// typical numbers can be 0.03 or -0.03 etc ...
//Note that this is only relevant when applied to a dynamic group and that CHASE_PHYSICS is applied (only in viewer mode)
STATE_IMPORT double STATE_WINAPI STATE_group_get_physics_friction(DWORD group_handle);
STATE_IMPORT void STATE_WINAPI STATE_group_set_physics_friction(DWORD group_handle, double friction);

//This number is used when the object hits a polygon. 
//A value of one will make the new speed (after collision)
//equal to the old speed (effects only the absolute speed value and not the direction)
//a value of 0.5 will cut speed by half. 
//We can also give values greater than one.
//For example a value of 2 will make the new speed twice as big
//as before the collision.
//Note that this is only relevant when applied to a dynamic group and that CHASE_PHYSICS is applied (only in viewer mode)
STATE_IMPORT double STATE_WINAPI STATE_group_get_physics_elasticity(DWORD group_handle);
STATE_IMPORT void STATE_WINAPI STATE_group_set_physics_elasticity(DWORD group_handle, double elasticity);


//Limits the maximum possible speed of an object.
//If there is no limit then by applying force on the object 
//The speed could reach infinity (unless some friction is given too)
//Note that this is only relevant when applied to a dynamic group and that CHASE_PHYSICS is applied (only in viewer mode)
STATE_IMPORT double STATE_WINAPI STATE_group_get_physics_maxspeed(DWORD group_handle);
STATE_IMPORT void STATE_WINAPI STATE_group_set_physics_maxspeed(DWORD group_handle, double max_speed);


//This function is not so important ...
//This function associates a number with the group
//later (When the world is saved and load in viewer mode
//we can get this number using STATE_object_get_control_type_number()
//Note that the function does nothing it is just a set and get mechanism. 
//The number that you give has no meaning to the Engine. 
//The meaning is as given by the application that is using the engine.
//parameters:
// num: any integer that is equal or greater than 64
// (the first 63 numbers are reserved for future use) 
STATE_IMPORT void STATE_WINAPI STATE_group_set_control_number(DWORD group_handle, int num);
STATE_IMPORT int STATE_WINAPI STATE_group_get_control_number(DWORD group_handle);

// Similar to STATE_group_wrap_a_bitmap()
// The difference is that STATE_group_wrap_a_bitmap() will
// stretch the bitmap so that the top right corner of the bitmap
// will fit to a corner of the group. This function
// will fit the corner of the bitmap the given location in the world
// locationXY_to_get_bitmap_top_left_corner[0], locationXY_to_get_bitmap_top_left_corner[1]
// This is the (X,Y) coordinate in the world that the top left corner of the bitmap will
// be associated with. 
// This way the given group might only get a portion of the bitmap.
// In the special case where locationXY_to_get_bitmap_top_left_corner, and locationXY_to_get_bitmap_bottom_right_corner
// are the bounding box of the group then this function will do the same as
// STATE_group_wrap_a_bitmap()
// Returns OK or VR_ERROR
STATE_IMPORT int STATE_WINAPI STATE_group_wrap_a_bitmap_fixed(DWORD group_handle, DWORD bitmap_handle, int repeat_x, int repeat_y, double locationXY_to_get_bitmap_top_left_corner[2], double locationXY_to_get_bitmap_bottom_right_corner[2]);

// Obselete .(works only in software mode)
// Set the brightness of each polygon of the group according to the intensity of the light source.
// values for ambient and are in the range [0-400]
// values for light_intensity light_intensity are in the range [0-400]
// see STATE_polygon_set_brightness() for explanation of what those values mean
// ambient is the minimum light intensity that is, if the light source has zero intensity or points
// to the opposite direction , the polygon will have the ambient intensity. This is the light that reflects
// from the surroundings.
// Examples:
// A)	direction[0]=1; direction[1]=0; direction[2]=0;
//		STATE_group_light_group(grp, direction,20, 400) //A very strong source of light in a dark environment
//
// B)	direction[0]=1; direction[1]=1; direction[2]=0; //The direction is diagonally
//		STATE_group_light_group(grp, direction, 100, 100) //An average source of light in an average lit environment
//
// Note that actually one can also try negative values for the ambient and the light_intensity.
// note that the formula is final_intensity= ambient + angle_between_polygon_and_light_direction* light_intensity
// where angle_between_polygon_and_light_direction is normalized to the range [0-1]
// a negative light_intensity means that our light source gives darkness instead of light ! (a great new invention !)
STATE_IMPORT void STATE_WINAPI STATE_group_light_group(DWORD group_handle, double direction[3], double ambient, double light_intensity);


//Remove light from the group.
//If the given group was lit using the STATE_light API then
//this function could be used to restore the group normal appearance
//If the given group handle is NULL then light will be removed from the whole world.
//See also STATE_object_remove_light(), STATE_polygon_remove_light(), STATE_light_remove_light()
STATE_IMPORT void STATE_WINAPI STATE_group_remove_light(DWORD group_handle);

// Set the color of all the polygons that belong to the given group.
// rgb[0] ==red
// rgb[1]==green;
// rgb[2]==blue
//Example:
//	BYTE rgb[3]={255,0,0}; //red color
//  STATE_group_set_color(NULL, rgb); //Set the whole world to red
STATE_IMPORT void STATE_WINAPI STATE_group_set_color(DWORD group_handle, BYTE rgb[3]);

//Calls STATE_polygon_set_light() for all the polygons in the group
//For more details see STATE_polygon_set_light()
STATE_IMPORT void STATE_WINAPI STATE_group_set_light(DWORD group_handle, BYTE rgb[3]);

//Set the ambient light value to all the polygons contained in the given group
//For more details see STATE_polygon_set_ambient()
STATE_IMPORT void STATE_WINAPI STATE_group_set_ambient(DWORD group_handle, int value);

//Set the diffuse light value to all the polygons contained in the given group
//For more details see STATE_polygon_set_diffuse()
STATE_IMPORT void STATE_WINAPI STATE_group_set_diffuse(DWORD group_handle, int value);

//Set the specular light value to all the polygons contained in the given group
//For more details see STATE_polygon_set_specular()
STATE_IMPORT void STATE_WINAPI STATE_group_set_specular(DWORD group_handle, int value);


//The same as STATE_polygon_create_lightmap() only that it creates lightmaps bitmaps for all
//polygons of the given group.
//Returns the number of bitmaps that were created and assigned to the group.
//For more details see STATE_polygon_create_lightmap()
// Note this important function: STATE_engine_reduce_number_of_bitmaps()
STATE_IMPORT int STATE_WINAPI STATE_group_create_lightmap(DWORD group_handle, int max_bm_side, int min_lightmap_size, double sample_rate, int force_ray_tracing, int bitmap_number);


//Disable or enabled a group.
//A disabled group is not included in rendering and in collision detection.
//This is a very important function for making large fast worlds using LOD (More details bellow)
//This function can be used to disable parts of the world that are not seen
//(This technique is done automatically when using the speed database)
// Another important usage is doing LOD (level of details)
// Each model in the world (building, mountain , a chair, etc ...) should be built
// several times. Each version of the model should use less polygons (for example the higher
// version should be built from 100 polygons, the second version from 50 polygons, the third from 20 polygons and
// the lowest level from just one rotated polygon that has a bitmap of the tree\chair\building or whatever model we
// are using).
// Before calling STATE_engine_render() check the distance between the camera (the view point)
// and each of the models that have LOD versions. Disable all the versions that should not be used
// and enable just the one that fits to the distance from the viewer
// The function is very fast (just set a flag and returns)
// Parameters:
//	group_handle: the group to be disabled \ enabled
//	YES_or_NO: YES to disable. NO to enable
//
// Example:
//	//disabling a group 
//	STATE_group_disable(a_group_handle, YES);
STATE_IMPORT void STATE_WINAPI STATE_group_disable(DWORD group_handle, int YES_or_NO);

//Returns YES or NO. NO is return if the group is enabled.
//YES is returned if the group is disabled.
// See also STATE_group_disable();
STATE_IMPORT int STATE_WINAPI STATE_group_is_disable(DWORD group_handle);


//Set the given bitmap to all the polygon belonging to the given group.
// Parameters:
//	group_handle: A handle to a group. NULL means the whole world.
//	bitmap_handle: A handle to a bitmap to be set on all the polygons of the given group.
STATE_IMPORT void STATE_WINAPI STATE_group_set_bitmap(DWORD group_handle, DWORD bitmap_handle);


// Used to go over all the groups that belong to a certain group.
//
// Example:
//		for(DWORD house_part=STATE_group_get_first_included_group(house_group); house_part!=NULL; house_part=STATE_group_get_next_included_group(house_group, house_part) ) {
//			// Deal with the various house parts such as roof, windows, door etc ...
//
//		}
//
// Note:
//		Don't mix this function with STATE_group_get_first_group() and 
//		STATE_group_get_next() which are used to go over ALL the existing groups.
//
//		To go over all the groups the belong to a certain object use STATE_object_get_group_handle() as the top group handle
//
STATE_IMPORT DWORD STATE_WINAPI STATE_group_get_first_included_group(DWORD top_group_handle);

// Used to go over all the groups that belong to a certain group.
//
// Example:
//		for(DWORD house_part=STATE_group_get_first_included_group(house_group); house_part!=NULL; house_part=STATE_group_get_next_included_group(house_group, house_part) ) {
//			// Deal with the various house parts such as roof, windows, door etc ...
//
//		}
//
// Note:
//		Don't mix this function with STATE_group_get_first_group() and 
//		STATE_group_get_next() which are used to go over ALL the existing groups
//
//		To go over all the groups the belong to a certain object use STATE_object_get_group_handle() as the top group handle
//
STATE_IMPORT DWORD STATE_WINAPI STATE_group_get_next_included_group(DWORD top_group_handle, DWORD last_included_found);


// *** not needed - not useful
// The points' normals are calculated automatically so there is no need to call this function.
// The function was used for internal 3DSTATE debugging.
STATE_IMPORT void STATE_WINAPI STATE_group_calculate_points_normals(DWORD group_handle);


//-----------------------------------------------------------//
//========= T H E   3 D _ A N I M A T I O N   A P I =========//
//-----------------------------------------------------------//


//This API deals with 3D model animation (mesh morphing)
//A 3D animation is a set of 3D models that are slightly different from one another
//Those 3D models are called 3D frames. For example a 3D animation of a man walking
//would contain several frames for completing a whole walking cycle.
//Usually the 3D character has several movements that it can perform.
//(For example, walk, jump, hit, fall down etc ...) each movement is combined
// of several frames and is called a 3D sequence.
// To summarize it: Each 3D animation is built from many 3D frames. The 3D frames are divided
// into separate 3D sequences. If it all sounds complicated then the following example will 
//clear things up. 

// my_obj=STATE_object_create_from_file("woman.md2"); //The md2 files contain a 3D animation. The 3D animation is automatically loaded to the engine
// my_3danim=STATE_object_get_3D_animation(my_obj);
// walk_sequence=STATE_3D_animation_get_sequence_using_name(my_3danim,"walk");
// standing_sequence=STATE_3D_animation_get_sequence_using_name(my_3danim,"stand");
// undressing_sequence=STATE_3D_animation_get_sequence_using_name(my_3danim,"undress");
// if(key_1_is_pressed) STATE_object_set_3D_sequence(my_obj,walk_sequence); 
// if(key_2_is_pressed) STATE_object_set_3D_sequence(my_obj,undressing_sequence); 
// and so on ...
//
//Please also note the 3D_sequence API which is used in conjunction with this API



// This function, together with STATE_3D_animation_get_next()
// are used for going over all the defined 3D animations
// Example
// for(DWORD anim3d=STATE_3D_animation_get_first(); anim3d!=NULL; anim3d=STATE_3D_animation_get_next(anim3d) {
//		//Write here the code to do for each anim3d
// }
// Note that the function STATE_3D_animation_create() adds a new anim3d
// to top of the queue. So STATE_3D_animation_get_first()
// will always return the last anim3d that was created.
// The same rule is for all STATE_xxxx_get_first() functions.
STATE_IMPORT DWORD STATE_WINAPI STATE_3D_animation_get_first(void);

STATE_IMPORT DWORD STATE_WINAPI STATE_3D_animation_get_next(DWORD anim3d_handle);


//Deletes all the existing 3d animations
STATE_IMPORT void STATE_WINAPI STATE_3D_animation_delete_all(void);

//Deletes the given anim3d.
// Note not to use the handle after it the anim3d was deleted
// Example:
// STATE_3D_animation_delete(my_anim3d);
// my_anim3d=NULL;
STATE_IMPORT void STATE_WINAPI STATE_3D_animation_delete(DWORD anim3d_handle);


//Returns a handle to the anim3d according to the given name
STATE_IMPORT DWORD STATE_WINAPI STATE_3D_animation_get_using_name(char *anim3d_name);

// Returns the name of the animation
// The function returns a pointer to the name which is stored internally inside the 3D Engine
// Visual Basic users should use STATE_entity_get_name() instead.
// See also STATE_entity_get_name()
STATE_IMPORT char *STATE_WINAPI STATE_3D_animation_get_name(DWORD anim3d_handle);

//Returns YES or NO. NO is returned if the given handle
//is not a handle of a 3D_animAtion. YES is returned otherwise
STATE_IMPORT int STATE_WINAPI STATE_3D_animation_is_3D_animation(DWORD anim3d_handle, char *function_asking);

// ===============================================================================
// === More 3D animation functions (beginners can skip this group of functions)===
// ===============================================================================
//
// Use the function below for the following purposes:
// 1) To change your 3d animation in real time
//		For example your game character has been beaten severly
//		you can now modify in real-time the points of the animation
//		to create a swollen face \ damaged body etc ...
//		To modify the 3D animation polygons use STATE_3D_animation_set_triangle_xyz()
//
// 2) To create various effects. For example you can build the first frame as a duplication of a house
//		the second frame can be a car. When the engine will interpolates between the frames it will
//		create 3D morphing animating gradually from a house to a car.
//		You could get the same effect by building an md2 animation
//		(using one of the md2 editors available on the Internet) with two frames.
//		and loading the aniamtion into the engine (for example STATE_object_create_from_file("woman.md2") )
//
//	3) To write converters from other 3d animation formats to STATE.
//	   To do so simply, create an empty 3d animation using STATE_3D_animation_create_for_editing()
//	   Set the points of all the polygons in each frame to their right value
//	   according to the file format you want to import.
//	   Use STATE_3D_animation_set_triangle_xyz() and STATE_3D_animation_set_triangle_bmp_uv()
//	   to set the polygons. When finished call STATE_3D_animation_save()
//		Example:
//			//converting a 3D frames based animation from your favorite format to STATE anim3d format
//			void main()
//			{
//				DWORD anim3d=STATE_3D_animation_create_for_editing("my_anim", 600, 20);
//
//				int num_of_frames=STATE_3D_animation_get_number_of_frames(anim3d);
//				int num_of_triangles=STATE_3D_animation_get_number_of_triangles(anim3d);
//
//				for(int frame=0; frame<	num_of_frames; frame++) {	
//					for(int tri=0; tri<num_of_triangles; tri++) {
//						for(point=0; point<3; point++) {
//							float xyz[3];
//							get_the_point_from_your_favorite_format(frame, tri,point, xyz); //You should write your own implementation to this function
//							float bmp_uv[2];
//							get_the_point_texture_coords_from_your_favorite_format(frame, tri,point, xyz); //You should write your own implementation to this function
//							STATE_3D_animation_set_triangle_xyz(anim3d, tri, point, frame, xyz);
//							STATE_3D_animation_set_triangle_bmp_uv(anim3d, tri, point, frame, bmp_uv);
//						}
//					}
//				}
//
//				STATE_3D_animation_save(anim3d,"my_anim.anim3d");
//			}



		

//Creates a new empty 3D animation that can be edited.
// Use the functions below to edit the 3D animation.
// Returns a handle to the newly created 3D Animation.
// After finishing editing call STATE_3D_animation_finished_editing()
// Only after calling STATE_3D_animation_finished_editing() one can use the 3D animation
// in functions such as STATE_object_set_3D_animation()
STATE_IMPORT DWORD STATE_WINAPI STATE_3D_animation_create_for_editing(char *name, int number_of_triangles, int number_of_frames);

//Rebuilds the animation sequances and make it ready for use.
//See also STATE_3D_animation_create_for_editing(), STATE_3D_animation_set_frame_name(),
//STATE_3D_animation_set_triangle_xyz()
STATE_IMPORT int STATE_WINAPI STATE_3D_animation_finished_editing(DWORD anim3d_handle);

//Note that when the different sequences are built (when STATE_3D_animation_finished_editing() is called)
//they are built according to the frames names. for example if we have 6 frames with the names
// walk1, walk2, walk3, run1, run2, run3
//then the engine will build 2 squences with the name Walk and Run.
//By setting the frames name you control later the names and number of the sequences.
STATE_IMPORT char *STATE_WINAPI STATE_3D_animation_get_frame_name(DWORD anim3d_handle, int frame_number);
STATE_IMPORT void STATE_WINAPI STATE_3D_animation_set_frame_name(DWORD anim3d_handle, int frame_number, char *name);


STATE_IMPORT int STATE_WINAPI STATE_3D_animation_get_number_of_triangles(DWORD anim3d_handle);
STATE_IMPORT int STATE_WINAPI STATE_3D_animation_get_number_of_frames(DWORD anim3d_handle);

//Returns a point of a given triangle in a given frame.
//The result is returned through xyzuv
// xyzuv[0] is the X value. xyzuv[1] is the Y value
//xyzuv[3] is the bitmap u coordinate of the requested point.
//xyzuv[4] is the bitmap v coordinate of the requested point.
//
// triangle_number
//		The index of the triangle in the specified frame. 0 is the first triangle.
//
// frame_number
//		The index of the animation frame . 0 is the first frame
//
// triangle_point
//		can be 0,1,2. Determins the triangle point to retrieve information from.
//
// Returns OK or VR_ERROR.
STATE_IMPORT int STATE_WINAPI STATE_3D_animation_get_triangle_point(DWORD anim3d_handle, int triangle_number, int triangle_point, int frame_number, float xyzuv[5]);

//The same as STATE_3D_animation_get_triangle_point() only that it doesnt retrieve the bitmap coordinations
STATE_IMPORT int STATE_WINAPI STATE_3D_animation_get_triangle_xyz(DWORD anim3d_handle, int triangle_number, int triangle_point, int frame_number, float xyz[3]);

//Set a specific point of the animation
//See STATE_3D_animation_get_triangle_point() for more information
//
// Note 1
//--------
//Please remember to give a name to a frame before you change its point
//set the frame name using STATE_3D_animation_set_frame_name()
//The name should start with letters and end with digits. For example
//walk02. After calling STATE_3D_animation_finished_editing() all consequtive frames that have the same letters
//will be grouped into one sequence. For example walk01, walk02, walk03 will be grouped to sequence walk.
//
// Note 2
//--------
//Please note that modifying points of the animation doesnt work very well with the md2 file format
//This is because the md2 file format represent each point with three BYTES (Thats right, only one byte for X, one for Y and one for Z !!!)
//In order to modify an md2 animation in realtime it is best to convert it first to the anim3d format.
//To convert from md2 to the anim3d format simply do a short program that looks like this
//
//  void main()
//	{
//		DWORD obj=STATE_object_create_from_file("woman.md2");
//		DWORD anim3d=STATE_object_get_3D_animation(obj);
//		STATE_3D_animation_save(anim3d,"woman.anim3d");
//	}
//
// Note that the anim3d format is a lot better for manipulating frames in realtime
// though the file created is a lot bigger. This means that it will use more memory
// and thus is less recommended to use if you have many models and you are concern
// with the speed of your application.
//
STATE_IMPORT int STATE_WINAPI STATE_3D_animation_set_triangle_xyz(DWORD anim3d_handle, int triangle_number, int triangle_point, int frame_number, float xyz[3]);

//Set the bitmap coord of a specific animation
//See STATE_3D_animation_get_triangle_point() for more information
STATE_IMPORT int STATE_WINAPI STATE_3D_animation_set_triangle_bmp_uv(DWORD anim3d_handle, int triangle_number, int triangle_point, int frame_number, float bmp_uv[2]);


//Save the animation in STATE 3D animation file format.
//The saved file can be later loaded using STATE_object_create_from_file();
// One of the uses for this function could be if you are writing a converter
// from your own 3D animation file format to STATE.
STATE_IMPORT int STATE_WINAPI STATE_3D_animation_save(DWORD anim3d_handle, char *anim3d_file_name);


//Contrls the speed of the 3D animation
//for more details see STATE_3D_sequence_set_speed()
//The same as STATE_3D_sequence_set_speed() just that the speed is set for all the sequences of the animation
//it is the same as:
//
//	FOREACH_SEQUENCE(seq) {
//		STATE_3D_sequence_set_speed(seq, speed);
//	}
STATE_IMPORT void STATE_WINAPI STATE_3D_animation_set_speed(DWORD anim3d_handle, double speed);


//-----------------------------------------------------------//
//========= T H E    3 D _ S E Q U E N C E    A P I =========//
//-----------------------------------------------------------//

//This API deals with 3D animation sequence (mesh morphing)
// Each 3D animation is built from many 3D frames. The 3D frames are divided
// into separate 3D sequences. For example there could be a "walking sequence"
// which is made of several frames that animate a full walking cycle.
//See also the STATE_3D_animation API,
// STATE_object_set_3D_sequence(), STATE_object_get_3D_sequence()
// STATE_object_replace_3D_sequence_when_finished(), STATE_object_create_from_file()



// This function, together with STATE_3D_sequence_get_next()
// are used for going over all the defined 3D sequences
// Example
// for(DWORD seq3d=STATE_3D_sequence_get_first(); seq3d!=NULL; seq3d=STATE_3D_sequence_get_next(seq3d) {
//		//Write here the code to do for each seq3d
// }
// Note that the function STATE_3D_sequence_create() adds a new seq3d
// to top of the queue. So STATE_3D_sequence_get_first()
// will always return the last seq3d that was created.
// The same rule is for all STATE_xxxx_get_first() functions.
STATE_IMPORT DWORD STATE_WINAPI STATE_3D_sequence_get_first(DWORD animation3D_handle);
STATE_IMPORT DWORD STATE_WINAPI STATE_3D_sequence_get_next(DWORD sequence3d_handle);

#define FOREACH_SEQUENCE(seq, anim) for(DWORD seq=STATE_3D_sequence_get_first(anim); seq!=NULL; seq=STATE_3D_sequence_get_next(seq))



//Returns a handle to the seq3d according to the given name
STATE_IMPORT DWORD STATE_WINAPI STATE_3D_sequence_get_using_name(DWORD animation3D_handle, char *sequence3d_name);

// Returns the 3D sequence name.
// The function returns a pointer to the name which is stored internally inside the 3D Engine
// Visual Basic users should use STATE_entity_get_name() instead.
// See also STATE_entity_get_name()
STATE_IMPORT char *STATE_WINAPI STATE_3D_sequence_get_name(DWORD sequence3d_handle);

//Deletes all the existing 3d sequences
STATE_IMPORT void STATE_WINAPI STATE_3D_sequence_delete_all(DWORD animation3D_handle);

//Deletes the given seq3d.
// Note not to use the handle after it the seq3d was deleted
// Example:
// STATE_3D_sequence_delete(my_seq3d);
// my_seq3d=NULL;
STATE_IMPORT void STATE_WINAPI STATE_3D_sequence_delete(DWORD animation3D_handle,DWORD sequence3d_handle);



//Returns YES or NO. NO is returned if the given handle
//is not a handle of a 3D_sequence. YES is returned otherwise
STATE_IMPORT int STATE_WINAPI STATE_3D_sequence_is_3D_sequence(DWORD sequence3d_handle, char *function_asking);


//The default speed is 1. A speed of 2 means twice slower. A speed of 0.5 means twice as fast.
//The default duration between to consecutive frames is 1 tenth of a second.
// See also STATE_3D_sequence_set_frame_duration()
// Example:
// Lets assume that the frame duration wasn't changed from it default (you can change it using STATE_3D_sequence_set_frame_duration() )
//That means 1 tenth of a second between two frames.
//So if the speed is equal to two then the duration between consecutive frames will be 2 tenth of a second
STATE_IMPORT void STATE_WINAPI STATE_3D_sequence_set_speed(DWORD sequence3d_handle, double speed);
STATE_IMPORT double STATE_WINAPI STATE_3D_sequence_get_speed(DWORD sequence3d_handle);


//Returns the number of frames that belongs to the given sequence.
STATE_IMPORT int STATE_WINAPI STATE_3D_sequence_get_number_of_frames(DWORD sequence3d_handle);

//The time is in mili of seconds
//0 is the first frame of the sequence and (number_of_sequence_frames-1) is the last
//returns -1 if an error occurred.
STATE_IMPORT int STATE_WINAPI STATE_3D_sequence_get_frame_duration(DWORD sequence3d_handle, int frame_number);
//returns OK or VR_ERROR
STATE_IMPORT int STATE_WINAPI STATE_3D_sequence_set_frame_duration(DWORD sequence3d_handle, int frame_number, int duration);


//Returns YES or NO
//A cyclic sequence means that when the last frame is reached is starts all over again
// A non cyclic sequence will stop animating when the sequencs reaches the last frame.
STATE_IMPORT int STATE_WINAPI STATE_3D_sequence_is_cyclic(DWORD sequence3d_handle);
STATE_IMPORT void STATE_WINAPI STATE_3D_sequence_set_cyclic(DWORD sequence3d_handle, int YES_or_NO);


//Move the sequence to a specific position.
//current_frame should be in the range [0 - number_of_frames_in_sequence]
//Note that current_frame is double. Values like 3.76 are allowed
//In this case frames 3 and frame 4 will be interpolated to represent the given frame number
STATE_IMPORT void STATE_WINAPI STATE_3D_sequence_set_current_frame(DWORD sequence3d_handle, double current_frame);


//Note that current_frame is double. Values like 3.76 mean
//that frames 3 and frame 4 are interpolated in the correct proportions
//
STATE_IMPORT double STATE_WINAPI STATE_3D_sequence_get_current_frame(DWORD sequence3d_handle);

//Will stop the animation (model morphing)
STATE_IMPORT void STATE_WINAPI STATE_3D_sequence_pause(DWORD sequence3d_handle);

//This function will continue playing the animation after it was paused.
STATE_IMPORT void STATE_WINAPI STATE_3D_sequence_play(DWORD sequence3d_handle);

//Return YES or NO
STATE_IMPORT int STATE_WINAPI STATE_3D_sequence_is_paused(DWORD sequence3d_handle);

//Change the direction of the 3D sequence. For example
//If our character walks then one we will walk backwards we will want to change
//the order of the sequence.
STATE_IMPORT void STATE_WINAPI STATE_3D_sequence_backwards_play(DWORD sequence3d_handle, int YES_or_NO);
STATE_IMPORT int STATE_WINAPI STATE_3D_sequence_is_backwards(DWORD sequence3d_handle);

//Returns the number of times that we have passed the last frame in the sequence
//If the sequence is not cyclic then this function will return 0 or 1.
//This function can be used to check if the sequence was finished.
STATE_IMPORT int STATE_WINAPI STATE_3D_sequence_get_number_of_laps(DWORD sequence3d_handle);

//This function is very useful and is often used
// together with STATE_object_set_3D_sequence()
//This function creates a copy of the given sequence and add it
//to the 3D_sequence list of the given 3D_animation. 
//Usually the given animation3D_handle is the 3D animation that
// sequence_to_duplicate belongs too.
// Using this function we can create for each object
// its own copy of the 3D_sequence thus when we will change the sequence
// properties it will effect only this object.
// name is the name to be given to the duplicated copy.
//Return value: The function returns a handle to the new sequence that was created.
//See also STATE_object_set_3D_sequence() (very recommended)
// Example:
// STATE_3D_sequence_duplicate(robot_animation, walk_sequence,"robot1_walk");
STATE_IMPORT DWORD STATE_WINAPI STATE_3D_sequence_duplicate(DWORD animation3D_handle, DWORD sequence_to_duplicate, char *name);


//------------------------------------------------//
//========= T H E    L I G H T 2   A P I =========//
//------------------------------------------------//

//This API replaces the STATE_light_ API.
//The older API (STATE_light_) could still be used (for lightmap generation and others) and we continue its support.

// Light effects are usually better the more polygons we have. This is because the lighting is based on the vetex color and normal
// Note that in order to improve the graphic quality dramatically one need to create lightmaps
// (after defining the light sources) for more details please see STATE_polygon_create_lightmap()
// see also STATE_engine_create_shadow() which uses projection method techniques for creating light

// some common background about lighting
// ------------------------------------
// Each source of light is combined of three different channels, they are
// called Ambient, Diffuse and specular. 
// Ambient light:
//		Ambient light is light that doesn't come from any particular direction
//		Objects illuminated by ambient light are evenly lit on all surfaces.
//		For example if we create a room with a few light sources then even
//		if all the light sources are turned off we will still have some light in the room.		
//		The ambient channel used to simulate this non-directional environment light
// Diffuse light:
//		Diffuse light comes from a particular direction but is reflected evenly of the surface		
//		A good example of a diffuse light source is fluorescent lighting.
//		The angle in which the diffuse light component hit the surface influence the strength of the light reflected
//		( a right angle with the surface will give maximum reflection)
//
// Specular lighting:
//		Typical for metal objects and mirrors.
//		Specular Lighting identifies the bright specular highlights that occur when light 
//		hits an object surface and reflects back toward the camera. 
//		It is more intense than diffuse light and falls off more rapidly across the object surface. It takes longer to calculate specular lighting than diffuse lighting, however the benefit of using it is that it adds significant detail to a surface. 
//		
//
//
//
// Every source light is actually a combination of the three type of lights
// Feel free to play with these values till you get the desired results.
// The rule is that if it looks good then it is correct..	

// Important !
// -----------
// The resulted lighting is a combination of the material characteristic and the type of light
// For controlling the material light properties see STATE_polygon_set_ambient(), STATE_group_set_ambient(), STATE_object_set_ambient()


STATE_IMPORT DWORD STATE_WINAPI STATE_light2_create_default_light(char *light_name);

STATE_IMPORT DWORD STATE_WINAPI STATE_light2_create_directional_light(char *light_name, double direction[3]);

STATE_IMPORT DWORD STATE_WINAPI STATE_light2_create_point_light(char *light_name, double location[3], double range, double attenuation);

STATE_IMPORT DWORD STATE_WINAPI STATE_light2_create_spot_light(char *light_name, double location[3], double direction[3], double range, double attenuation, double theta, double phi, double falloff);

//There are only about 8 lights that can be active simultanuesly. The exact number is dependant on the 3D hardware
//There could be endless number of light sources (1000 different lights for example) but only about 8 active simultanuesly.
//This function returns the number of lights that can be enabled simultaneously
//When the number of enabled lights exceeds the max number possible the exceeding light sources wont effect the rendering. 
// To enable or disable lights see :STATE_WINAPI STATE_light2_disable(), STATE_light2_enable(), STATE_light2_is_disabled()
STATE_IMPORT int STATE_WINAPI STATE_light2_get_number_of_lights_supported_by_hardware();

//Returns a handle to the light according to the given name
STATE_IMPORT DWORD STATE_WINAPI STATE_light2_get_using_name(char *light_name);



STATE_IMPORT void STATE_WINAPI STATE_light2_disable_specular_lighting(int YES_or_NO);

STATE_IMPORT void STATE_WINAPI STATE_light2_disable_all_lighting(int YES_or_NO);



// This function, together with STATE_light2_get_next_light()
// are used for going over all the defined lights
// Example
// for(DWORD light=STATE_light2_get_first_light(); light!=NULL; light=STATE_light2_get_next_light(light) {
//		//Write here the code to do for each light
// }
// Note that the function STATE_light2_create() adds a new light
// to top of the queue. So STATE_light2_get_first_light()
// will always return the last light that was created.
// The same rule is for all STATE_xxxx_get_first_xxxx() functions.
STATE_IMPORT DWORD STATE_WINAPI STATE_light2_get_first_light(void);

STATE_IMPORT DWORD STATE_WINAPI STATE_light2_get_next_light(DWORD light_handle);

#define FOREACH_light(light) for(DWORD light=STATE_light2_get_first_light(); light!=NULL; light=STATE_point_get_next_point(light))

//Sets the location of the light.
// Example
// double location[3]={10,20,30} //setting the X,Y,Z coordinates
// STATE_light2_set_location(my_light,location);
STATE_IMPORT void STATE_WINAPI STATE_light2_set_location(DWORD light_handle, double location[3]);

STATE_IMPORT void STATE_WINAPI STATE_light2_get_location(DWORD light_handle, double location[3]);


// Sets the direction of the light.
// This function is relevant only for spot light
// The direction is set by giving a point in the world towards which the light should be pointed
// See also STATE_light2_set_direction()
//
// The reason is it is not relevant for directional lighting is because directonal lighting doesnt have location (so it cant point to a specific point)
// The reason is it is not relevant for point lighting is because point lighting doesnt have a direction (so it cant point to a specific point)
//
STATE_IMPORT void STATE_WINAPI STATE_light2_point_at(DWORD light_handle, double point[3]);


//Sets the direction of the light. The direction of the light is relevant only if the 
// light type is set to be directional.
// Note that a direction of {1,1,1} is exactly the same as {2,2,2}
STATE_IMPORT void STATE_WINAPI STATE_light2_set_direction(DWORD light_handle, double direction[3]);

STATE_IMPORT void STATE_WINAPI STATE_light2_get_direction(DWORD light_handle, double direction[3]);

STATE_IMPORT void STATE_WINAPI STATE_light2_set_type_of_light(DWORD light_handle, int type_of_light);

STATE_IMPORT int STATE_WINAPI STATE_light2_get_type_of_light(DWORD light_handle);

STATE_IMPORT void STATE_WINAPI STATE_light2_set_range(DWORD light_handle, double distance_reach);

STATE_IMPORT double STATE_WINAPI STATE_light2_get_range(DWORD light_handle);


STATE_IMPORT void STATE_WINAPI STATE_light2_set_attenuation(DWORD light_handle, double attenuation);

STATE_IMPORT double STATE_WINAPI STATE_light2_get_attenuation(DWORD light_handle);

STATE_IMPORT void STATE_WINAPI STATE_light2_set_complex_attenuation(DWORD light_handle, double attenuation0, double attenuation1, double attenuation2);

STATE_IMPORT void STATE_WINAPI STATE_light2_get_complex_attenuation(DWORD light_handle, double attenuation[3]);

//The six function below are relevant only to light of type SPOT_LIGHT (theta, phi, falloff)
STATE_IMPORT void STATE_WINAPI STATE_light2_set_inner_cone_angle(DWORD light_handle, double theta);
STATE_IMPORT double STATE_WINAPI STATE_light2_get_inner_cone_angle(DWORD light_handle);
STATE_IMPORT void STATE_WINAPI STATE_light2_set_outer_cone_angle(DWORD light_handle, double phi);
STATE_IMPORT double STATE_WINAPI STATE_light2_get_outer_cone_angle(DWORD light_handle);
STATE_IMPORT void STATE_WINAPI STATE_light2_set_inner_to_outer_cone_falloff(DWORD light_handle, double faloff);
STATE_IMPORT double STATE_WINAPI STATE_light2_get_inner_to_outer_cone_falloff(DWORD light_handle);


//Note that the values of the rgb could exceed 255 and could even be negative.
//A negative value will actually suck light (make darker)
STATE_IMPORT void STATE_WINAPI STATE_light2_set_ambient_color(DWORD light_handle, double rgb[3]);
STATE_IMPORT void STATE_WINAPI STATE_light2_set_diffuse_color(DWORD light_handle, double rgb[3]);
STATE_IMPORT void STATE_WINAPI STATE_light2_set_specular_color(DWORD light_handle, double rgb[3]);
STATE_IMPORT void STATE_WINAPI STATE_light2_get_ambient_color(DWORD light_handle, double rgb[3]);
STATE_IMPORT void STATE_WINAPI STATE_light2_get_diffuse_color(DWORD light_handle, double rgb[3]);
STATE_IMPORT void STATE_WINAPI STATE_light2_get_specular_color(DWORD light_handle, double rgb[3]);

//Returns YES or NO. NO is returned if the given handle
//is not a handle of a light. YES is returned otherwise
STATE_IMPORT int STATE_WINAPI STATE_light2_is_light(DWORD light_handle, char *function_asking);

//Deletes all the existing lights
STATE_IMPORT void STATE_WINAPI STATE_light2_delete_all(void);

//Deletes the given light.
// Note not to use the handle after it the light was deleted
// Example:
// STATE_light2_delete(my_light);
// my_light=NULL;
STATE_IMPORT void STATE_WINAPI STATE_light2_delete(DWORD light_handle);

//Returns the light source name
// The function returns a pointer to the name which is stored internally inside the 3D Engine
// See also STATE_entity_get_name()
STATE_IMPORT char *STATE_WINAPI STATE_light2_get_name(DWORD light_handle);

STATE_IMPORT void STATE_WINAPI STATE_light2_set_name(DWORD light_handle, char *name);

//Disables or enables a light source.
//There are only about 8 lights that can be active simultanuesly. The exact number is dependant on the 3D hardware
//There could be endless number of light sources (1000 different lights for example) but only about 8 active simultanuesly.
//This function returns the number of lights that can be enabled simultaneously
//When the number of enabled lights exceeds the max number possible the exceeding light sources wont effect the rendering. 
// To enable or disable lights see :STATE_WINAPI STATE_light2_disable(), STATE_light2_enable(), STATE_light2_is_disabled()
//
// Example:
//	STATE_light2_disable(my_light);
STATE_IMPORT void STATE_WINAPI STATE_light2_disable(DWORD light_handle);
STATE_IMPORT void STATE_WINAPI STATE_light2_enable(DWORD light_handle);
STATE_IMPORT int STATE_WINAPI STATE_light2_is_disabled(DWORD light_handle);

// By default the light engine ignores the normal values in the loaded file and
// calculate them automatically. The default normal calculatin mode is:
// SET_VERTEX_NORMAL_USING_MIXED_METHOD
// use this function to set one of the three modes. 
//
// SET_VERTEX_NORMAL_EQUAL_TO_AVERAGE_OF_ADJACENT_POLYGONS:
//		Use this mode to make 3d object look more round and less polygonal
//		This mode will set the normal of a vertex to be equal to the average of the normal of all
//		the adjacent polygons. For example the normal of points in a cube shape room
//		will all point to the center of the room.
//		Note that when using this mode, vertices that have the same x,y,z will
//		also have the same normal which menas that when lighting an object the edges where the polygons
//		connect each other wont be visible (smoother , rounder effect)
//
// SET_VERTEX_NORMAL_EQUAL_TO_POLYGON_NORMAL:
//		The points of a given polygon will have the same normal as the polygon.
//		Note that points with identical x,y,z coordinate might have a different normal
//		if the polygons to which they belong are not on the same plane.
//
// SET_VERTEX_NORMAL_USING_MIXED_METHOD:
//		The default. combines between the two other methods (SET_VERTEX_NORMAL_EQUAL_TO_AVERAGE_OF_ADJACENT_POLYGONS & SET_VERTEX_NORMAL_EQUAL_TO_POLYGON_NORMAL)
//		If the angle between two adjacent polygons is less then given threshold degrees than the average technique is used
//		if the angle between the polygons is more or equal to the given threshold (the default is 30) than the normals used for the points are the polygon normals
//		Actually any number between 0 and 180 can be used. The default is 30 but one can use other numbers as well
//		When 0 is given then it acts exactly like SET_VERTEX_NORMAL_EQUAL_TO_POLYGON_NORMAL
//		when 180 is given then it acts exactly as SET_VERTEX_NORMAL_EQUAL_TO_AVERAGE_OF_ADJACENT_POLYGONS
//		This methos was devised by Chandan Pawasker (chandan@3dstate.com)
//
// DISABLE_AUTOMATIC_NORMAL_CALCULATION:
//		Use the values as given in the world file.
//		Note that the file might not contain normal values
// 
// Example A:
//		STATE_light_calcualte_vertices_normals_automatically(SET_VERTEX_NORMAL_EQUAL_TO_POLYGON_NORMAL);
//
// Example B:
//		//the SET_VERTEX_NORMAL_USING_MIXED_METHOD will be used but with a smaller threshold the the default (30)
//		//This will will increase the SET_VERTEX_NORMAL_EQUAL_TO_POLYGON_NORMAL methos effect
//		STATE_light_calcualte_vertices_normals_automatically(20);
//
STATE_IMPORT void STATE_WINAPI STATE_light2_calcualte_vertices_normals_automatically(int mode);

//--------------------------------------------------------//
//=========== T H E    A N I M A T I O N   A P I =========//
//--------------------------------------------------------//

// animation is a set of bitmaps that replaces itself according
// to a given time sequence.
// A polygon can have a regular bitmap or it can have an animation.
// The idea is that one can just give a list of bitmaps with time sequence
// and the engine deals with all the rest.
// Animations are very useful when used on a rotated polygons.
// You can then define animation  sequences from 8 sides
// and the graphic engine sees that the right bitmap is
// shown each time.
// Each animation can include up to 8 different bitmap sequences
// This is used for representing a rotated polygon from 8 different sides.
// actually there are only 5 bitmaps sequences (the rest are figured automatically
// by the engine by mirroring some bitmaps)
// Here are the 5 constants that represent the 5 different list
#define  BITMAP_LIST_FRONT 1  //a list of bitmaps representing the view from the front
#define  BITMAP_LIST_FRONT_SIDED 2  //a list of bitmaps representing the view from the front-side (45 degrees) angle
#define  BITMAP_LIST_SIDE 3  //a list of bitmaps representing the view from the side
#define  BITMAP_LIST_BACK_SIDED 4  //a list of bitmaps representing the view from the back-side (135 degrees) angle
#define  BITMAP_LIST_BACK 5  //a list of bitmaps representing the view from the back
//
// To understand animation better, just look at one of the sample worlds
// that contain an animation. Try playing with the list of bitmaps and the other params.


// Returns YES or NO.
// YES , if the given handle is an animation handle. NO if not.
// If it is not an animation handle , a message box will appear,
// the title of this message box will be the parameter "function_asking"
// if function_asking==NULL than no popup will be created.
// See also IS_ANIMATION()
STATE_IMPORT int STATE_WINAPI STATE_animation_is_animation(DWORD animation_handle, char *function_asking);


// The given name should be in capital letters
// If the return value is 0 it means 
// that there is no animation with the given name.
STATE_IMPORT DWORD STATE_WINAPI STATE_animation_get_handle(char *animation_name);


STATE_IMPORT DWORD STATE_WINAPI STATE_animation_get_first_animation(void); 
STATE_IMPORT DWORD STATE_WINAPI STATE_animation_get_next(DWORD animation_handle); 

//Creates an empty animation 
//name: give any name you want. 
//		NOTE that if the name is already in use by another animation
//		the function would automatically add a number to the end of the name
// After creating the animation use STATE_animation_add_bitmap() to add bitmaps
STATE_IMPORT DWORD STATE_WINAPI STATE_animation_create(char *name); 


// bitmap_list: is one of the five constants: BITMAP_LIST_FRONT, BITMAP_LIST_FRONT_SIDED, BITMAP_LIST_SIDE, BITMAP_LIST_BACK_SIDED, BITMAP_LIST_BACK
// bitmap_handle: is the frame to add
// bitmap_position_index: 0 means put this frame as the first frame
//						  1 means that we want to insert the bitmap after the first bitmap
//								(our bitmap will become the second bitmap)
//						  n means that we want to insert the bitmap AFTER the n bitmap
//						  -1 has a special meaning it will put the frame as the last one
//						  If the given index is too big than the frame will be set as the last frame.
//
// The return value: returns the location of the added bitmap in the given list.
// usually this value is equal to the given "bitmap_position_index" argument (unless
// bitmap_position_index==-1 or is too big)
STATE_IMPORT int STATE_WINAPI STATE_animation_add_bitmap(DWORD animation_handle, int bitmap_list ,DWORD bitmap_handle, int bitmap_position_index); 

// removes a bitmap from the given list
// bitmap_list: is one of the five constants: BITMAP_LIST_FRONT, BITMAP_LIST_FRONT_SIDED, BITMAP_LIST_SIDE, BITMAP_LIST_BACK_SIDED, BITMAP_LIST_BACK
// bitmap_position_index: 0 means remove the first frame
//						  1 means remove the second frame etc ...
//						  -1 has a special meaning it will remove the frame at the end of the list
//						  If the given index is too big than the last frame will be removed.
//
STATE_IMPORT int STATE_WINAPI STATE_animation_remove_bitmap(DWORD animation_handle, int bitmap_list ,int bitmap_position_index); 

// Returns a handle to the bitmap in the specified list and index
// bitmap_list: is one of the five constants: BITMAP_LIST_FRONT, BITMAP_LIST_FRONT_SIDED, BITMAP_LIST_SIDE, BITMAP_LIST_BACK_SIDED, BITMAP_LIST_BACK
// bitmap_position_index: 0 will return the first bitmap
//						  1 the second bitmap etc ...
STATE_IMPORT DWORD STATE_WINAPI STATE_animation_get_bitmap(DWORD animation_handle, int bitmap_list ,int bitmap_position_index); 

// time_for_each_frame: should be equal to STATE_animation_get_number_of_frames()
// before calling this function you should initialize the array ( DWORD time_for_each_frame[])
// the time is in hundreds of seconds.
// Returns OK or VR_ERROR
// Example:
//		time_for_each_frame[0]= 100; //The first frame will be shown for one second
//		time_for_each_frame[1]= 200; //The second frame will be shown for one second
//		STATE_animation_set_times(my_animation, time_for_each_frame, 2);
// 
STATE_IMPORT int STATE_WINAPI STATE_animation_set_times(DWORD animation_handle, DWORD time_for_each_frame[], int size_of_array); 

// time_for_each_frame: should be equal to STATE_animation_get_number_of_frames()
// the time is in hundreds of seconds.
// Returns OK or VR_ERROR
// Example:
//		DWORD time_for_each_frame[10]
//		STATE_animation_set_times(my_animation, time_for_each_frame, 10);
// 
STATE_IMPORT int STATE_WINAPI STATE_animation_get_times(DWORD animation_handle, DWORD time_for_each_frame[], int size_of_array); 

//Get the duration of the given frame. (in hundreds of seconds)
// frame_index: 0 means the first frame
//			    1 means the second frame etc ...
// to get the last frame frame_index should be equal to the numbers of frames minus one.
//For Example:
//	time=STATE_animation_get_frame_time(my_anim,0); // This would return the time
//												// in hundreds of second that
//												// the first bitmap will be shown
//												// before it replaced by the second bitmap \ frame.
STATE_IMPORT DWORD STATE_WINAPI STATE_animation_get_frame_time(DWORD animation_handle, int frame_index); 

// Set the duration of the given frame. (in hundreds of seconds)
// returns OK or VR_ERROR
// For Example:
//	STATE_animation_set_frame_time(my_anim,0,100); // This would set the duration 
//												// in hundreds of second (100== 1 second) of
//												// the first bitmap will be shown
//												// before it replaced by the second bitmap \ frame.
STATE_IMPORT int STATE_WINAPI STATE_animation_set_frame_time(DWORD animation_handle, int frame_index, DWORD time); 

// Returns the animation name.
// The function returns a pointer to the name which is stored internally inside the 3D Engine
// Visual Basic users should use STATE_entity_get_name() instead.
// See also STATE_entity_get_name()
STATE_IMPORT char * STATE_WINAPI STATE_animation_get_name(DWORD animation_handle); 

//NOTE that if the name is already in use by another animation
//the function would automatically add a number to the end of the name
STATE_IMPORT void STATE_WINAPI STATE_animation_set_name(DWORD animation_handle, char *name); 

// examples:
// factor==10 would make the animation ten time slower
// factor==0.1 would make the animation ten time faster
//		 only if all the numbers in the array are bigger then 10 
// Say we have this times series 1, 2, 20
// now if we call with factor 0.1 we get the series 1, 1, 2
// Note !! If we call again with factor=10
// we get back the series we started with 1,1,2 => 1,2,20
// The mechanism is smart enough to deal with that.
STATE_IMPORT void STATE_WINAPI STATE_animation_factor_speed(DWORD animation_handle, double factor);


// speed==1 means restore normal speed
// speed==2 means twice slower then normal speed
// speed==0.5 means twice faster then normal speed
//		animation frames are never skipped. See
//		remarks on STATE_animation_factor_speed() to
//		understand the implication.
STATE_IMPORT void STATE_WINAPI STATE_animation_set_speed(DWORD animation_handle, double speed);


STATE_IMPORT void STATE_WINAPI STATE_animation_delete_all(void);

//Note that you cant use the handle after it was deleted
// A good practice will be like that:
//		STATE_animation_delete(my_animation);
//		my_animation=NULL;
STATE_IMPORT void STATE_WINAPI STATE_animation_delete(DWORD animation_handle);


// sending YES will write the given animation to file after call to STATE_engine_save() with SAVE_ONLY_WHATS_NEEDED 
// Note that if STATE_engine_save() is called without SAVE_ONLY_WHATS_NEEDED, all the animations will be saved
// Note ! If the animation is used directly by one of the polygons
// in the saved group it will be saved no matter how the save_flag is set !
// The need for this function is because though the engine can figure out itself
// what animations are needed to the saved group it doesn't know what animation will
// be needed in run time. Using this flag one can determine the animation resources
// to use in run-time.
STATE_IMPORT void STATE_WINAPI STATE_animation_set_save_flag(DWORD animation_handle, int yes_no_flag); 

// returns YES or NO
// see STATE_animation_set_save_flag() for more details
STATE_IMPORT int STATE_WINAPI STATE_animation_get_save_flag(DWORD animation_handle); 

//Returns the number of frames in the animation.
// On Error returns 0
// Example: Say we have an animation of a runner. The animation is combined
// from 5 bitmaps series, the runner from the front,45 degrees, side, 135 degrees, and from the back
// (look how it is defined in the wld file, FRONT, FRONT_SIDED, SIDE, BACK).
// Every series must contain the same number of bitmaps.
// calling this function will return the number of bitmap in each series.
STATE_IMPORT int STATE_WINAPI STATE_animation_get_number_of_frames(DWORD animation_handle); 

//Returns the number of bitmaps in the animation.
// On Error returns 0
// Example: Say we have an animation of a runner. The animation is combined
// from 5 bitmaps series, the runner from the front,45 degrees, side, 135 degrees, and from the back
// (look how it is defined in the wld file, FRONT, FRONT_SIDED, SIDE, BACK).
// Every series must contain the same number of bitmaps.
// calling this function will return the number of bitmap in each series multiply by the number
// of series. 
STATE_IMPORT int STATE_WINAPI STATE_animation_get_number_of_bitmaps(DWORD animation_handle); 

// This function returns an array with the handle to all the 
// bitmaps.
// bitmaps_array[] is an array of DWORD. This array will be filled with handles to
// all the bitmaps of the animation. 
// NOTE: you must allocate memory for the array before calling this function
// Example: 
//		int num_of_bitmaps=STATE_animation_get_number_of_bitmaps(animation_handle);
//		// allocating mem using new or malloc etc ...
//		DWORD *bitmaps_array=(DWORD *)malloc(num_of_bitmaps*sizeof(DWORD));
//		STATE_animation_get_all_bitmaps(animation_handle, bitmaps_array);
//
// Note that if a certain bitmaps is used twice by the animation, than its handle will
// appear twice in the bitmaps_array.
// handles 
STATE_IMPORT void STATE_WINAPI STATE_animation_get_all_bitmaps(DWORD animation_handle, DWORD bitmaps_array[]); 

// Returns YES if the last world that was loaded, using STATE_engine_load_world()
// or STATE_engine_add_world().
// Returns NO otherwise 
STATE_IMPORT int STATE_WINAPI STATE_animation_is_part_of_the_last_world(DWORD animation_handle); 

// Returns the number of polygons that are using the given bitmap
STATE_IMPORT int STATE_WINAPI STATE_animation_get_number_of_polygons_with_animation(DWORD animation_handle);


//Create a new animation that has the same bitmaps and timing as the given animation.
// Returns the new animation. If an error occurred, it will return NULL.
//
// Here are some ideas for using this function.
// You will need this function if you want to use the same animation but with different timing.
// For example, let say that your animation is a tree blowing in the wind. You probably
// have lots of trees and each of them use the same animation. Now if you dont want
// all the trees to move as one then you can do one of the following:
// 1) Use STATE_polygon_set_animation_frame() to give each tree a different frame
// Or
// 2) Use this function to duplicate an animation and then to give different timing.
STATE_IMPORT DWORD STATE_WINAPI STATE_animation_duplicate(DWORD animation_handle, char *new_name);

// This function can be used to replace the bitmap of a certain frame in the animation.
// bitmap_list: is one of the five constants: BITMAP_LIST_FRONT, BITMAP_LIST_FRONT_SIDED, BITMAP_LIST_SIDE, BITMAP_LIST_BACK_SIDED, BITMAP_LIST_BACK
// frame_index: 0 means the first frame. -1 has a special meaning, it means the last frame
// bitmap_handle: A handle of the bitmap that will be set for the given frame.
// See also STATE_animation_get_bitmap()
// Return OK or VR_ERROR
STATE_IMPORT int STATE_WINAPI STATE_animation_set_bitmap(DWORD animation_handle, int bitmap_list ,int frame_index, DWORD bitmap_handle);


//The return value is The name of the bitmap without the extension 
// the bitmap_name is the name+path relative to the path given in STATE_engine_load_world()
//If a bitmap exists in the world file
//but the engine couldnt find the bitmap file than it will still return its name.
//This is used when loading a world from the Internet, at first only the world is loaded
//and then we start streaming the bitmaps. We use this function to get the bitmap file name.
//If you want to check if the bitmap is in used then use STATE_animation_get_handle()
STATE_IMPORT char *STATE_WINAPI STATE_animation_get_frame_bitmap_name(DWORD animation_handle, int bitmap_list ,int frame_index);


//This function is very similar to STATE_bitmap_get_transparent_index()
//For info check STATE_bitmap_get_transparent_index()
//This function is needed only in the rare cases that some of the
//animation bitmaps could not be loaded when the world was loaded.
//For example when loading a world from the Internet, at first only the world is loaded
//and then we start streaming the bitmaps. We use this function to get the bitmap transparent index
//If you want to check if the bitmap is in used then use STATE_animation_get_frame_bitmap_handle()
STATE_IMPORT int STATE_WINAPI STATE_animation_get_frame_transparent_index(DWORD animation_handle, int bitmap_list ,int frame_index);


//--------------------------------------------------------//
//=========== T H E   B I T M A P   A P I ================//
//--------------------------------------------------------//

// This API deals with bitmaps/textures and the way they are used by the Engine
//
// Some words about how it works inside the engine
// ----------------------------------------------
//		From version 7 the bitmaps are loaded into memory and into the 3D graphics card
//		when the first STATE_engine_render() is called.
//		When calling STATE_bitmap_load() before the first render the operation is registered but
//		the actual loading only happens in the first render.
//		This means that all the functionss in this API that involve direct access
//		to the bitmap memory can only work after the first render()




// Returns a handle to a previously  loaded bitmap (using the file name)
// If the return value is NULL it means 
// that no bitmap with the given name and the given transparent index was loaded to the engine.
//Think of the transparent_index as part of the name 
//there could be several bitmaps with the same name but each can have a different
//transparent index. Note that the engine keeps a separate copy for each transparent index
//Example:
// STATE_bitmap_get_handle("bitmaps\\people\\man12.bmp",-1);
STATE_IMPORT DWORD STATE_WINAPI STATE_bitmap_get_handle(char *file_name, int transparent_index);


STATE_IMPORT DWORD STATE_WINAPI STATE_bitmap_get_first_bitmap(void); 
STATE_IMPORT DWORD STATE_WINAPI STATE_bitmap_get_next(DWORD bitmap_handle); 

// Supported formats:  ".bmp", ".dds", ".dib", ".jpg", ".png", ".tga"
// File name extension (if given) is ignored. The engine will search
// for recognized extensions (jbm, bmp or jpg) and will load which ever
// exists. If the engine finds more then one recognized extension 
// (for example grass.bmp and grass.jpg) it will load the file
// which was most recently modified 
// If the specified bitmap was already loaded then it just retrieves
// its handle (in that case it is exactly like STATE_bitmap_get_handle() )
// transparent_index should be -1 for no transparent and 0 
// for removing the most frequent color in the bitmap (usually
// the most used color is the background)
// transparent_index==1 means remove the second most used color etc ...
// Note the difference between this command and STATE_utilities_load_bitmap_from_file()
// This command loads a bitmap into the engine, while STATE_utilities_load_bitmap_from_file()
// is just an extension to the Win32 API making it easy to load a bitmap from
// the disk and to get its HBITMAP handle (a Win32 type)
//
// Important: 
//		Bitmaps that are not it the power of two dimensions will be automatically resampled (this is because 3D cards can only handle dimensions that are of the power of two !).
//		This is why it is best to have all your bitmaps in the power of two dimensions.
//		Numbers of the power of two are: 1,2,4,8,16,32,64,128,256,512,1024
//		A bitmaps in the size of 128x256 is OK
//		A bitmap in the size of 100x200 is not OK and there for will be automatically resampled by the engine (consuming more time )
//		Though the engine does a good job in resampling the bitmaps, Corel PhotoPaint or Adobe PhotoShop are probably better ...
//		Meaning that it is better if you resample you bitmaps your self and let the engine handle only bitmaps that are in the power of two
//
// Examples:
//	A-
//		note that we dont have to give the bitmap extension 
//		if we do it is simply ignored
//		STATE_bitmap_load("c:\\bitmaps\\my_bitmap",-1); 
//	B-
//		Load a bitmap and set the most used color as transparent
//		STATE_bitmap_load("c:\\bitmaps\\my_bitmap",0); 
//	C-
//		Load a bitmap and set the second most used color as transparent
//		STATE_bitmap_load("c:\\bitmaps\\my_bitmap",1); 
//
//	NOTE this case !
//	D-
//		This example has some good news and some bad news.
//		The good news is that if c:\\bitmaps\\grass27
//		is exactly the same as c:\\NEW_bitmaps\\grass27
//		Then we shouldnt worry about wasting memory by loading two identical bitmaps
//		When we try to load the second bitmap the engine
//		will see that it already has a bitmap with that name and
//		it will return the handle to the first loaded bitmap without doing anything.
//		The bad news is that if the bitmaps are different, the engine
//		wont overwrite the old one. It will just return the handle to the already
//		loaded bitmap without doing anything else.
//		handle1=STATE_bitmap_load("c:\\bitmaps\\grass27",-1); 
//		handle2=STATE_bitmap_load("c:\\NEW_bitmaps\\grass27",-1); 
//		if(handle1!=handle2) error_message(); //Never happens
//
//		A solution for this in case we want to overwrite the old bitmap
//		is to first delete the old bitmaps and then to load the new one.
//
//	E-
//		If we have two bitmaps in the same directory one is called
//		my_bitmap.jpg and the other is called my_bitmap.bmp
//		handle=STATE_bitmap_load(my_bitmap.jpg,-1);
//		Note that the engine will load my_bitmap.bmp and not
//		my_bitmap.jpg. Thats because loading bitmaps in jpg format is much slower
//		And also because the jpg version of a bitmap is usually not as good as the original
//		(depending how much compression we used)
//
STATE_IMPORT DWORD STATE_WINAPI STATE_bitmap_load(char *file_name, int transparent_index);

STATE_IMPORT DWORD STATE_WINAPI STATE_bitmap_load_cubic_texture(char *file_name);


// returns -1 if we transparent is not used for the given bitmap
// else return the index to the transparent color, usually it
// will be 0 which means that the most used color is the transparent color
// if the return value is 1 it means that the second most popular
// color is the transparent color etc ...
STATE_IMPORT int STATE_WINAPI STATE_bitmap_get_transparent_index(DWORD bitmap_handle) ;


// Deletes the bitmap
// The handle of the bitmap should never be used after the bitmap was unloaded (deleted)
//
// Example:
//		STATE_bitmap_unload(my_bitmap);
//		my_bitmap=NULL; //god practice in cas we accidentally try to use my_bitmap afterwards ...
STATE_IMPORT void STATE_WINAPI STATE_bitmap_unload(DWORD bitmap_handle);

//Returns the name according to the bitmap file name (without the extension)
// The function returns a pointer to the name which is stored internally inside the 3D Engine
// Visual Basic users should use STATE_entity_get_name() instead.
// See also STATE_entity_get_name()
STATE_IMPORT char *STATE_WINAPI STATE_bitmap_get_name(DWORD bitmap_handle);

STATE_IMPORT int STATE_WINAPI STATE_bitmap_get_number_of_polygons_using_bitmap(DWORD bitmap_handle);

//returns the width in pixels of the given bitmap
// The function can only be called after at least one render was done.
STATE_IMPORT int STATE_WINAPI STATE_bitmap_get_width(DWORD bitmap_handle);

//returns the height in pixels of the given bitmap
// The function can only be called after at least one render was done.
STATE_IMPORT int STATE_WINAPI STATE_bitmap_get_height(DWORD bitmap_handle);

// Returns YES or NO.
// YES , if the given handle is a bitmap handle. NO if not.
// If it is not a bitmap handle , a message box will appear,
// the title of this message box will be the parameter "function_asking"
// if function_asking==NULL than no popup will be created.
// See also IS_BITMAP()
STATE_IMPORT int STATE_WINAPI STATE_bitmap_is_bitmap(DWORD bitmap_handle, char *function_asking);

// *** not so useful.
// Returns the number of mipmap levels that were created for the given bitmap
// The only use of this function is if you intend to change pixels in the bitmap mipmap levels
// For most usage this function is not needed since it is easier to access only mipmap 0
// and to cancel mipmaps for the given bitmap. See below for more info.
//
// This function should only be called after there was at least one STATE_engine_render() call
// Return value:
//		The number of mipmaps or -1 if an error occured.
//		
//
// Some general info about mipmaps:
// ---------------------------------
//	When bitmapss are loaded into the engine, the engine automatically creates
//  different version of the bitmap. For example if the bitmap loaded is 256x256
//	then the engine will create additional bitmaps in the size of 128x128, 64x64, 32x32, ... 1x1
//	This is a known technique that improves graphic quality significantly.
//	When rendering a polygon that uses this bitmap the most suitable mipmap levels are selected
//	and blended together.
//	This is done automaticall and internally so you shouldn't concern about it at all
//	except when using STATE_bitmap_set_pixel(). Using this function you can
//	access each mipmap level.
//
STATE_IMPORT int STATE_WINAPI STATE_bitmap_get_number_of_mipmap_level(DWORD bitmap_handle);

// *** not so useful.
// Returns the width of the selected mipmap level
// The only use of this function is if you intend to change pixels in the bitmap mipmap levels
// For most usage this function is not needed since it is easier to access only mipmap 0
// and to cancel mipmaps for the given bitmap. See below for more info.
// 
//	parameters:
//		mipmap_level [in]:
//			a zero based index.
//
// Return value:
//		The number of mipmaps or -1 if an error occured.
//
// Remarks:
// 1. This function should only be called after there was at least one STATE_engine_render() call
// 
// 2. When mipmap_level==0 this function will be the same as STATE_bitmap_get_width()
//
// 3. For more information about mipmaps please see STATE_bitmap_get_number_of_mipmap_level() and STATE_bitmap_set_pixel()
//
// See also STATE_bitmap_get_width(), STATE_bitmap_get_mipmap_height(), STATE_bitmap_get_number_of_mipmap_level()
//
STATE_IMPORT int STATE_WINAPI STATE_bitmap_get_mipmap_width(DWORD bitmap_handle,  int mipmap_level);
//
// The same as STATE_bitmap_get_mipmap_width() just for the height.
STATE_IMPORT int STATE_WINAPI STATE_bitmap_get_mipmap_height(DWORD bitmap_handle, int mipmap_level);


// The bitmap format should BITMAP_FORMAT_A8R8G8B8 or any other BITMAP_FORMAT (search this file for where BITMAP_FORMAT_A8R8G8B8 is defined
// and use any of the formats defined there)
// Name is the name you want to assign to the new bitmap.
// The new bitmap will be saved under this name.
STATE_IMPORT DWORD STATE_WINAPI STATE_bitmap_create(char *name, int width, int height, DWORD format, int use_mipmaps);

// Get the color of a specific pixel
// The function can only be called after at least one render was done.
// The function support the following bitmap formats
// D3DFMT_A8R8G8B8, D3DFMT_X8R8G8B8, D3DFMT_A4R4G4B4, D3DFMT_A1R5G5B5
// Note that the engine default bitmap format is BITMAP_FORMAT_DXT1 (which is a compressed format)
// This means that if you plan to use the STATE_bitmap_get_pixel() or STATE_bitmap_set_pixel()
// You need to call STATE_bitmap_set_default_format(BITMAP_FORMAT_A8R8G8B8); (for example)
// at the beginning of the program (or at least before the first render ).
//
// Parameters:
//		
//		bitmap_handle [IN]:
//			The bitmap handle.
//
//		x,y [IN]:
//			the specific pixel. 0,0 it the top left corner of the bitmap
//			(width-1, height-1) is mapped is the bottom right corner.
//			To get the width and height of the bitmap use STATE_bitmap_get_height()
//			and STATE_bitmap_get_width()
//
//		RGBA [OUT]:
//			returns the color and alpha value of the pixel.
//			RGBA[0] is red
//			RGBA[1] is green
//			RGBA[2] is blue
//			RGBA[3] is the alpha value
//		
//			Note that even if the bitmap is is in 16 bit format the pixel format will be conveted to the RGBA format
//
// Return Value:
//		Return OK or VR_ERROR. VR_ERROR is returned if the function is called before the first render or when x,y are out of range.
//
//
// See also STATE_bitmap_get_height(), STATE_bitmap_get_width()
// STATE_bitmap_set_pixel()
STATE_IMPORT int STATE_WINAPI STATE_bitmap_get_pixel(DWORD bitmap_handle, int x, int y, BYTE RGBA[4]);

// *** useful for doing some special effects and advanced stuff
// Set a pixel in the given bitmap
//
// Parameters:
//		bitmap_handle [IN]:
//			The bitmap handle.
//
//		x,y [IN]:
//			the specific pixel. 0,0 it the top left corner of the bitmap
//			(width-1, height-1) is mapped is the bottom right corner.
//			To get the width and height of the bitmap use STATE_bitmap_get_height()
//			and STATE_bitmap_get_width()
//
//		mipmap_level [IN]:
//			Zero based index. If there are 8 mipmaps level then 0-7 are legal mipmap levels
//			When bitmapss are loaded into the engine, the engine automatically creates
//			different version of the bitmap. For example if the bitmap loaded is 256x256
//			then the engine will create additional bitmaps in the size of 128x128, 64x64, 32x32, ... 1x1
//			This is a known technique that improves graphic quality significantly.
//			When rendering a polygon that uses this bitmap the most suitable mipmap levels are selected
//			and blended together.
//			This is done automaticall and internally so you shouldn't concern about it at all
//			except when using STATE_bitmap_set_pixel(). Using this function you can
//			access each mipmap level.
//			Except for doing some special effects this is usually not
//			very useful so you might want to set just mipmao 0 (the heighest detailed mipmap)
//			and cancel mipmapping for all the polygons using this bitmap using STATE_polygon_use_mipmap(polygon, NO)
//			Note that if you don't cancel mipmapping for the relevant polygon and you just modify
//			one mipmap level, the changes will only be noticeable when the modified mipmap
//			is used. For example if you write pixel to mipmap 0 then only when the viewer (the camera)
//			is close to the polygon with the bitmap, the high level mipmap level (mipmap 0) will be used
//			and only then the changes to the bitmap will be evident.
//
//
//		RGBA [IN]:
//			returns the color and alpha value of the pixel.
//			RGBA[0] is red
//			RGBA[1] is green
//			RGBA[2] is blue
//			RGBA[3] is the alpha value
//		
//			Note that the engine will automatically convert the give RGBA to the proper format used by the bitmap 
//
//
// Important notes:
// 1.	The function can only be called after at least one render was done.
//
// 2.	The function support the following bitmap formats
//		D3DFMT_A8R8G8B8, D3DFMT_X8R8G8B8, D3DFMT_A4R4G4B4, D3DFMT_A1R5G5B5
//		Note that the engine default bitmap format is BITMAP_FORMAT_DXT1 (which is a compressed format)
//		This means that if you plan to use the STATE_bitmap_get_pixel() or STATE_bitmap_set_pixel()
//		You need to call STATE_bitmap_set_default_format(BITMAP_FORMAT_A8R8G8B8); (or any other supported formats)
//		at the beginning of the program (or at least before the first render ).
// 3. Note that the changes to the bitmap will be lost when the screen resolution is changed.
//
// 4. Unless you know what you are doing, we recommend that you write only to mipmap 0 and cancel
//	  mipmapping to all polygons using the modified bitmap (by calling STATE_polygon_use_mipmap(polygon, NO) )
//
//
//
// Return Value:
//		Return OK or VR_ERROR. VR_ERROR is returned if the function is called before the first render or when x,y are out of range.
//
// See also STATE_bitmap_get_number_of_mipmaps()
//
STATE_IMPORT int STATE_WINAPI STATE_bitmap_set_pixel(DWORD bitmap_handle, int mipmap_level, int x, int y, BYTE RGBA[4]);


// A lot faster than STATE_bitmap_set_pixel() this is almost as fast as direct memory access.
// Before starting to call STATE_bitmap_set_pixel_fast() STATE_bitmap_start_direct_memory_access()
// must be called. STATE_bitmap_end_direct_memory_access() must be call after we finished writing the pixels
// Delphi programmers should use this fun
//
// Note that there is no mipmap argument. The mipmap level is selected when STATE_bitmap_start_direct_memory_access() is called
//
// See also STATE_bitmap_set_pixel();
//
// Example:
//			BYTE *mem_ptr=NULL; ////we actually don't use this argument
//			int  number_of_bytes_per_pixel; //we actually don't use this argument
//			int  number_of_pixels_in_one_line;
//			int  number_of_lines; ////we actually don't use this argument
//			STATE_bitmap_start_direct_memory_access(my_bitmap, 0,&mem_ptr, &number_of_bytes_per_pixel, &number_of_pixels_in_one_line, &number_of_lines);
//			if(mem_ptr==NULL) {
//				error_msg();
//				return(VR_ERROR);
//			}
//			BYTE RGBA[4]={100,200,50,100};
//			for(int i=0; i<100) {
//				STATE_bitmap_set_pixel_fast(my_bitmap_handle, i, i/2, RGBA);
//
//			}
//			STATE_bitmap_end_direct_memory_access(my_bitmap_handle, 0);
//			
STATE_IMPORT int STATE_WINAPI STATE_bitmap_set_pixel_fast(DWORD bitmap_handle, int x, int y, BYTE RGBA[4]);
STATE_IMPORT int STATE_WINAPI STATE_bitmap_get_pixel_fast(DWORD bitmap_handle, int x, int y, BYTE RGBA[4]);

// Write a stream of bytes to the beginning of the bitmap. This function is useful when we want to 
// replace the entire content of a bitmap. This is a lot faster than setting each pixel separately
// Delphi programmers should use STATE_bitmap_write_memory_block()
// instead of STATE_bitmap_start_direct_memory_access(), C/C++ programs can use both according to their preferences
//
// Parameters:
//		bitmap_handle [IN]:
//			The bitmap we want to access its memory
//
//		mipmap_level [IN]:
//			See details in STATE_bitmap_set_pixel()
//
//		pixels_data:
//			The array of bytes that would be copied to the bitmap
//			It is the user responsibility to make sure that the bytes/pixels are in the same format as the
//			bitmap pixels. On that issue see also STATE_bitmap_set_default_format()
//
//		number_of_bytes:
//			The number of bytes in the pixels_data array. For example if it is a 256x256 bitmap in RGBA format (32 bit per pixel)
//			then to replace all the pixels we should have number_of_bytes=65536*4
//			
//
STATE_IMPORT int STATE_WINAPI STATE_bitmap_write_memory_block(DWORD bitmap_handle, int mipmap_level, BYTE *pixels_data, int number_of_bytes);

// Set the given alpha value to all the pixels in the bitmap
// Note !
// 1. The function can only be called after at least one render was done.
// 2. Inorder for the function to have any effect, the bitmap format must be one that
//	  includes an alpha component. For example: BITMAP_FORMAT_A8R8G8B8 or BITMAP_FORMAT_A1R5G5B5  
//	  or  BITMAP_FORMAT_A4R4G4B4. To make sure the loaded bitmaps are in 
//	  the desire format call STATE_bitmap_set_default_format() before loading the bitmap
// 	  
STATE_IMPORT void STATE_WINAPI STATE_bitmap_set_alpha_value(DWORD bitmap_handle, BYTE alpha);

// *** advanced function
// Used to access the texture memory directly
// for mass changes this approach is much faster then using STATE_bitmap_set_pixel()
// The function can only be called after at least one render was done.
// Note that after you finish reading /writing to the bitmap memory you need to call
//
// Parameters:
//		bitmap_handle [IN]:
//			The bitmap we want to access its memory
//
//		mipmap_level [IN]:
//			See details in STATE_bitmap_set_pixel()
//
//		memory_ptr [OUT]:
//			a pointer to the first BYTE of the bitmap memory
//
//		number_of_bytes_per_pixel [OUT]:
//			returns the size in bytes of a pixel for example
//			returns 4 for 32 bit formats such as BITMAP_FORMAT_A8R8G8B8
//			NOTE that when using non standard formats the function will return 0
//			this doesn't mean that one cant access the bitmap memory, it just
//			means that you will have to figure the size yourself.
//			You do it according the format you set when you called
//			STATE_bitmap_set_default_format(). For example if called STATE_bitmap_set_default_format(BITMAP_FORMAT_A8R8G8B8);
//			at the beginning of your program, you know that the texture format is 32 bit (4 bytes per pixel)
//
//		number_of_pixels_in_one_line [OUT]
//			This is the same value as returned by STATE_bitmap_get_width()
//
//		number_of_lines [OUT]
//			This is the same value as returned by STATE_bitmap_get_height()
//
//
// Return value:
//		OK or VR_ERROR. VR_ERROR is returned when directly accessing the bitmap memory memory failed.
//
// Remarks:
//		Delphi programers should use STATE_bitmap_write_memory_block() instead.
// 
// Example:
//		//In this example we will access all the pixels of the texture and make them one shade brighter
//
//		//Bitmaps memory can only be accessed after after we had at least one call to STATE_engine_render() during the program run
//		STATE_engine_render(NULL, NULL);
//
//		STATE_bitmap_set_default_format(BITMAP_FORMAT_A8R8G8B8);
//
//		DWORD my_bitmap=STATE_bitmap_load("picture.jpg", -1);
//		STATE_polygon_set_bitmap_fill(my_polygn, my_bitmap); 
//		STATE_polygon_use_mipmap(my_polygn, NO); //make sure that only mipmap level 0 is used
//      make_bitmap_brighter(my_bitmap);
//
//		//Here is the implementation of the make_bitmap_brighter() function
//
//		int make_bitmap_brighter(DWORD my_bitmap)
//		{
//			BYTE *mem_ptr=NULL;
//			int  number_of_bytes_per_pixel;
//			int  number_of_pixels_in_one_line;
//			int  number_of_lines;
//			STATE_bitmap_start_direct_memory_access(my_bitmap, 0,&mem_ptr, &number_of_bytes_per_pixel, &number_of_pixels_in_one_line, &number_of_lines);
//			if(mem_ptr==NULL) {
//				error_msg();
//				return(VR_ERROR);
//			}
//
//			if(number_of_bytes_per_pixel!=4)
//				return(VR_ERROR); //In this simple example we assume we loaded the texture in BITMAP_FORMAT_A8R8G8B8 see STATE_bitmap_set_default_format() for more details
//
//			BYTE *line_start=mem_ptr;
//			for(int line=0; line<number_of_lines; line++) {
//				for(int x=0; x<number_of_pixels_in_one_line*4; x++) {
//					if(line_start[x]<255)
//						line_start[x]++; //make brighter
//				}
//				line_start+=number_of_pixels_in_one_line*4;
//			}
//
//			STATE_bitmap_end_direct_memory_access(my_bitmap, 0);
//		}
//		//In the next render my_polygon will be brighter. repeating thi
//
// See also STATE_bitmap_end_direct_memory_access(), STATE_bitmap_write_memory_block();
STATE_IMPORT int STATE_WINAPI STATE_bitmap_start_direct_memory_access(DWORD bitmap_handle, int mipmap_level, BYTE **memory_ptr, int *number_of_bytes_per_pixel, int *number_of_pixels_in_one_line, int *number_of_lines);

// Ends direct bitmap memory access.
// Each time we call STATE_bitmap_start_direct_memory_access()
// we have to call STATE_bitmap_end_direct_memory_access()
// after finishing the direct access
//
// Parameters:
//		bitmap_handle [IN]:
//			The bitmap handle
//		mipmap_level [IN]:
//			Use the same mipmap level used when STATE_bitmap_start_direct_memory_access() was called
//
//
// Return Value:
//		Returns OK or VR_ERROR. VR_ERROR is returned when calling this function before the first render or when trying to end direct memory
//		access without first starting it using STATE_bitmap_start_direct_memory_access()
//
STATE_IMPORT int STATE_WINAPI STATE_bitmap_end_direct_memory_access(DWORD bitmap_handle, int mipmap_level);


//The constans below are for advanced use with the function STATE_bitmap_set_default_format()
#ifndef MAKEFOURCC
    #define MAKEFOURCC(ch0, ch1, ch2, ch3)                              \
                ((DWORD)(BYTE)(ch0) | ((DWORD)(BYTE)(ch1) << 8) |       \
                ((DWORD)(BYTE)(ch2) << 16) | ((DWORD)(BYTE)(ch3) << 24 ))
#endif /* defined(MAKEFOURCC) */
#define BITMAP_FORMAT_UNKNOWN                0
#define BITMAP_FORMAT_R8G8B8                20
#define BITMAP_FORMAT_A8R8G8B8              21
#define BITMAP_FORMAT_X8R8G8B8              22
#define BITMAP_FORMAT_R5G6B5                23
#define BITMAP_FORMAT_X1R5G5B5              24
#define BITMAP_FORMAT_A1R5G5B5              25
#define BITMAP_FORMAT_A4R4G4B4              26
#define BITMAP_FORMAT_R3G3B2                27
#define BITMAP_FORMAT_A8                    28
#define BITMAP_FORMAT_A8R3G3B2              29
#define BITMAP_FORMAT_X4R4G4B4              30
#define BITMAP_FORMAT_A2B10G10R10           31
#define BITMAP_FORMAT_A8B8G8R8              32
#define BITMAP_FORMAT_X8B8G8R8              33
#define BITMAP_FORMAT_G16R16                34
#define BITMAP_FORMAT_A2R10G10B10           35
#define BITMAP_FORMAT_A16B16G16R16          36
#define BITMAP_FORMAT_A8P8                  40
#define BITMAP_FORMAT_P8                    41
#define BITMAP_FORMAT_L8                    50
#define BITMAP_FORMAT_A8L8                  51
#define BITMAP_FORMAT_A4L4                  52
#define BITMAP_FORMAT_V8U8                  60
#define BITMAP_FORMAT_L6V5U5                61
#define BITMAP_FORMAT_X8L8V8U8              62
#define BITMAP_FORMAT_Q8W8V8U8              63
#define BITMAP_FORMAT_V16U16                64
#define BITMAP_FORMAT_A2W10V10U10           67
#define BITMAP_FORMAT_UYVY                  MAKEFOURCC('U' 'Y' 'V' 'Y')
#define BITMAP_FORMAT_R8G8_B8G8             MAKEFOURCC('R' 'G' 'B' 'G')
#define BITMAP_FORMAT_YUY2                  MAKEFOURCC('Y' 'U' 'Y' '2')
#define BITMAP_FORMAT_G8R8_G8B8             MAKEFOURCC('G' 'R' 'G' 'B')
#define BITMAP_FORMAT_DXT1                  MAKEFOURCC('D' 'X' 'T' '1')
#define BITMAP_FORMAT_DXT2                  MAKEFOURCC('D' 'X' 'T' '2')
#define BITMAP_FORMAT_DXT3                  MAKEFOURCC('D' 'X' 'T' '3')
#define BITMAP_FORMAT_DXT4                  MAKEFOURCC('D' 'X' 'T' '4')
#define BITMAP_FORMAT_DXT5                  MAKEFOURCC('D' 'X' 'T' '5')
#define BITMAP_FORMAT_D16_LOCKABLE          70
#define BITMAP_FORMAT_D32                   71
#define BITMAP_FORMAT_D15S1                 73
#define BITMAP_FORMAT_D24S8                 75
#define BITMAP_FORMAT_D24X8                 77
#define BITMAP_FORMAT_D24X4S4               79
#define BITMAP_FORMAT_D16                   80
#define BITMAP_FORMAT_D32F_LOCKABLE         82
#define BITMAP_FORMAT_D24FS8                83


#define BITMAP_FORMAT_L16                   81

#define BITMAP_FORMAT_VERTEXDATA           100
#define BITMAP_FORMAT_INDEX16              101
#define BITMAP_FORMAT_INDEX32              102

#define BITMAP_FORMAT_Q16W16V16U16         110

#define BITMAP_FORMAT_MULTI2_ARGB8         MAKEFOURCC('M','E','T','1')

// Floating point surface formats

// s10e5 formats (16-bits per channel)
#define BITMAP_FORMAT_R16F                  111
#define BITMAP_FORMAT_G16R16F               112
#define BITMAP_FORMAT_A16B16G16R16F         113

// IEEE s23e8 formats (32-bits per channel)
#define BITMAP_FORMAT_R32F                  114
#define BITMAP_FORMAT_G32R32F               115
#define BITMAP_FORMAT_A32B32G32R32F         116
#define BITMAP_FORMAT_CxV8U8                117


// *** advanced function
// Set the format in which the loaded bitmaps will be converted to
// Bitmaps that were loaded with transparent_index different then -1 are not effected
// by this function.
// All bitmaps loaded after calling this function will be stored in the given format.
// To make all the bitmaps of a world use the given format, call this function before loading
// the world.
// Note that if STATE_bitmap_set_default_format() is called twice or more before the first render
// then only the last call will have effect. 
// 
// The default format is BITMAP_FORMAT_DXT1 (which is a compressed format)
//
// Q: What do I need this function for ?
//
// A: Here is an example. If you want for example to control the transparency area of 
//	  a given polygon accuratelly using the alpha channel 
//	  then call this function with BITMAP_FORMAT_A8R8G8B8
//	  This will make all the bitmaps that are loaded after this call be in 32 bit format where
//	  the high order byte is used for alpha. Call STATE_polygon_translucency() with the polygon that
//	  uses the bitmap and select a translucensy mode that uses the texture alpha channel
//
// Example:
//		STATE_bitmap_set_default_format(BITMAP_FORMAT_A8R8G8B8); //setting to 32 bit format with alpha channel
STATE_IMPORT void STATE_WINAPI STATE_bitmap_set_default_format(DWORD bitmap_default_format);


// Modulate
#define BITMAP_OPERATION_MODE_MODULATE              4      // multiply args together. The default mode
#define BITMAP_OPERATION_MODE_MODULATE2X            5      // multiply and  shift 1 bit
#define BITMAP_OPERATION_MODE_MODULATE4X            6      // multiply and  shift 2 bits

// Add
#define BITMAP_OPERATION_MODE_ADD                    7   // add arguments together
#define BITMAP_OPERATION_MODE_ADDSIGNED              8   // add with -0.5 bias
#define BITMAP_OPERATION_MODE_ADDSIGNED2X            9   // as above but left  1 bit
#define BITMAP_OPERATION_MODE_SUBTRACT              10   // Arg1 - Arg2, with no saturation
#define BITMAP_OPERATION_MODE_ADDSMOOTH             11   // add 2 args, subtract product
                                    // Arg1 + Arg2 - Arg1*Arg2
                                    // = Arg1 + (1-Arg1)*Arg2

// Linear alpha blend: Arg1*(Alpha) + Arg2*(1-Alpha)
#define BITMAP_OPERATION_MODE_BLENDDIFFUSEALPHA     12 // iterated alpha
#define BITMAP_OPERATION_MODE_BLENDTEXTUREALPHA     13 // texture alpha
#define BITMAP_OPERATION_MODE_BLENDFACTORALPHA      14 // alpha from D3DRS_TEXTUREFACTOR

// Linear alpha blend with pre-multiplied arg1 input: Arg1 + Arg2*(1-Alpha)
#define BITMAP_OPERATION_MODE_BLENDTEXTUREALPHAPM   15 // texture alpha
#define BITMAP_OPERATION_MODE_BLENDCURRENTALPHA     16 // by alpha of current color

// Specular mapping
#define BITMAP_OPERATION_MODE_PREMODULATE             17     // modulate with next texture before use
#define BITMAP_OPERATION_MODE_MODULATEALPHA_ADDCOLOR  18     // Arg1.RGB + Arg1.A*Arg2.RGB
                                        // COLOROP only
#define BITMAP_OPERATION_MODE_MODULATECOLOR_ADDALPHA  19     // Arg1.RGB*Arg2.RGB + Arg1.A
                                        // COLOROP only
#define BITMAP_OPERATION_MODE_MODULATEINVALPHA_ADDCOLOR  20  // (1-Arg1.A)*Arg2.RGB + Arg1.RGB
                                        // COLOROP only
#define BITMAP_OPERATION_MODE_MODULATEINVCOLOR_ADDALPHA  21  // (1-Arg1.RGB)*Arg2.RGB + Arg1.A
                                        // COLOROP only

// Bump mapping
#define BITMAP_OPERATION_MODE_BUMPENVMAP            22 // per pixel env map perturbation
#define BITMAP_OPERATION_MODE_BUMPENVMAPLUMINANCE   23 // with luminance channel

// This can do either diffuse or specular bump mapping with correct input.
// Performs the function (Arg1.R*Arg2.R + Arg1.G*Arg2.G + Arg1.B*Arg2.B)
// where each component has been scaled and offset to make it signed.
// The result is replicated into all four (including alpha) channels.
// This is a valid COLOROP only.
#define BITMAP_OPERATION_MODE_DOTPRODUCT3           24

// Triadic ops
#define BITMAP_OPERATION_MODE_MULTIPLYADD           25 // Arg0 + Arg1*Arg2
#define BITMAP_OPERATION_MODE_LERP                  26 // (Arg0)*Arg1 + (1-Arg0)*Arg2


// *** useful for advanced effects
// A polygon can have a color/light (see STATE_polygon_set_light() or STATE_point_set_rgb()) 
// and up to two bitmaps.
// By default when the 3D card calculates the color of a pixel it multiplies all these "possessions" with each other.
// STATE_bitmap_set_operation_mode() makes it possible to define a different operation for example: addition.
// For example if a specific polygon has two bitmaps (See STATE_polygon_set_second_bitmap() )
// one of the bitmaps is completly green and the other is completely red.
// By default the result would be black ! this is becuase the 2 bitmaps are multiplied by each other
// The pixels in the red bitmap have the color (1,0,0) and the pixels in the green bitmap have the color (0,1,0)
// The result pixel is (1*0, 0*1, 0*0) = (0,0,0) = black !
//
// Parameters: 
//		operation_mode:
//			Any of the above constants (all those that start with BITMAP_OPERATION_MODE_ )
//
// Example:
//		BYTE rgb[3]={0,0,0};
//		STATE_group_set_light(NULL, rgb); //see remarks below
//		STATE_bitmap_set_operation_mode(my_bitmap, BITMAP_OPERATION_MODE_ADD);
//
//
// Important remarks:
//			1. Not all the 3D cards support all the different operations
//
//			2. By default the rgb value of all the points is White = 255,255,255 (or (1,1,1) in 0-1 scale )
//			   This is perfect for the BITMAP_OPERATION_MODE_MODULATE which is the default operation
//			   but it is bad for operations like BITMAP_OPERATION_MODE_ADD
//			   Because if the color already white and we add the texture color it will still be white
//			   So when using BITMAP_OPERATION_MODE_ADD rember to call
//						BYTE rgb[3]={0,0,0};
//						STATE_group_set_light(NULL, rgb);
//			   This will set the RGB values of all the points in the world to 0,0,0
//
STATE_IMPORT void STATE_WINAPI STATE_bitmap_set_operation_mode(DWORD bitmap_handle, int operation_mode);

//Set the default bitmap operation mode. Call this function before loading the world to make sure that all
//the bitmaps have this default.
//For more details see STATE_bitmap_set_operation_mode()
// Example: 
//		STATE_bitmap_set_operation_mode_default(BITMAP_OPERATION_MODE_ADD);
//
STATE_IMPORT void STATE_WINAPI STATE_bitmap_set_operation_mode_default(int operation_mode);

//-------------------------------------------------------//
//======== T H E    S O U N D   A P I ===================//
//-------------------------------------------------------//


/*
	STATE sound engine is built using parts of FMod library. If This API is used for commercial 
	purposes then you must get a propre license from FMod. A ccording to FMod web-site
	as long as you dont genrate any revenues from their model you dont have to purchase a license.
	For updated information regarding the terms of use of the FMod library please contact www.fmod.org 
	In the future we will replace the FMod library with our own library.

	In order to be able to use the sound API you must do the following:
	1) Add the fmod lib to your project. (For example users of Visual C should add the fmodvc.lib) 
	2) Make sure to put the fmod.dll in the same directory as your executable.
*/



#define SOUND_NO_LOOP 0
#define SOUND_LOOP_NORMAL 1


#define SOUND_PAUSED 1
#define SOUND_RESUME_PLAYING 0

//const double SOUND_DISTANCE_DEFAULT=1;
#define SOUND_DISTANCE_DEFAULT 1.0

//Load a sound file into the engine.
//Parameters:
//	file_name: 
//		file to load
//
//	entity:
//		A handle to a polygon or a group or an object
//		The entity parameter makes it very easy to create 3D sound that its volume is sensative
//		to the distance form the entity and the balance is automatically computed for each speaker according to the
//		location of the entity.
//		If entity handle is NULL then the sound is not attached
//		to any entity (polygon,object or group) and is treated as a background sound,
//		otherwise it is a 3D-sound positioned at the entity's location , volume and
//		balance are constantly changed according to the listener location.
//		Important: Make sure that if the sound is attached to an entity and the entity is deleted then
//				   this could cause a fatal error. Make sure to stop a sound or dettach it before deleteing
//				   the object it is associated with.
//
//  distance_reach
//		The third argument is the maximum distance reach of the sound, which can be SOUND_DISTANCE_DEFAULT,
//		or a distance reach that the user decides on. This argument will effect only if the entity handle
//		given is not NULL, meaning that the sound has a defined position.	
//		Try giving different numbers and see the effect. for example: 300, 1000, 10000 etc ...
//
// Examples:
//
// A)
// This will set the mp3 song as a background sound
//		DWORD madona=STATE_sound_load("madona\\like_a_virgin.mp3", NULL,SOUND_DISTANCE_DEFAULT);
//		STATE_sound_play(madona, SOUND_LOOP_NORMAL);

//B)
// When we will get closer or further away from the bird the engine will
// automatically update the volume and the balance according to the camera relative
// position towards the bird object.
//
//		DWORD bird_sound=STATE_sound_load("voices\\bird_sound.mp3", bird_object_handle,SOUND_DISTANCE_DEFAULT);
//		STATE_sound_play(bird_sound, SOUND_LOOP_NORMAL);
///With Wave files it is exactly the same, only with a wav sound file.
// 
//
// C)
// This will set the Wave song as a background sound
// Note that it is much better to use mp3 as a background sound since the files
// are a lot smaller.
//
//		DWORD madona=STATE_sound_load("madona\\like_a_virgin.wav", NULL, SOUND_DISTANCE_DEFAULT);
//		STATE_sound_play(madona, SOUND_LOOP_NORMAL);
STATE_IMPORT DWORD STATE_WINAPI STATE_sound_load(char *file_name, DWORD entity_handle,double distance_reach);
 
// Will start playing the sound.
// Note that to get a handle to the sound you must first call
// STATE_sound_load()
// Note that you can call this function again with the same sound file while the sound of the first
// call is still playing (Canonically ). In this case you will hear both sounds played together.
// The engine will treat the second call as if it is a new sound.
//loop is SOUND_LOOP_NORMAL or SOUND_NO_LOOP
// SOUND_LOOP_NORMAL will make the sound to be played from the beginning each time it ends.
// Example:
//		DWORD madona STATE_sound_load("madona\\like_a_virgin.mp3", NULL,SOUND_DISTANCE_DEFAULT);
//		STATE_sound_play(madona, SOUND_LOOP_NORMAL);
//
// Returns OK or VR_ERROR
STATE_IMPORT int STATE_WINAPI STATE_sound_play(DWORD sound_handle,const int loop);


//This function will attach in run-time an entity handle to the sound, thus changing the location
//of the sound and accordingly the volume and balance.
//If the sound is playing already, the change will take effect only after stopping and replaying the sound.
//entity_handle can be a handle to one of the three: polygon handle, object handle or group handle.
//if entity handle is NULL then the engine actually performs Detach meaning that the sound is treated
//as a background sound that is not attached to any entity.
//Important: Make sure that if the sound is attached to an entity and the entity is deleted then
//		     this could cause a fatal error. Make sure to stop a sound or dettach it before deleteing
//			 the object it is associated with.

//Example:
//		DWORD sound_handle=STATE_sound_load("my_song",chair);
//when playing this sound, it will be heard as coming from the chair.
//		STATE_sound_attach(sound_handle,table);
//now, when playing the sound, it will be heard as coming from the table.
//      STATE_sound_attach(sound_handle,NULL);
//now, the sound is a background sound. 	
STATE_IMPORT int STATE_WINAPI STATE_sound_attach(DWORD sound_handle, DWORD entity_handle);

//This function will start playing all the sounds that have been already loaded.
//The volume and balance of each sound will be calculated automatically by the engine.
//!!! this function plays all sounds without checking if a sound is already being played. 
STATE_IMPORT void STATE_WINAPI STATE_sound_play_all_sounds();

//This will stop playing the given sound
//Returns OK or VR_ERROR
STATE_IMPORT int STATE_WINAPI STATE_sound_stop(DWORD sound_handle);

//This will stop playing all the sounds.
STATE_IMPORT void STATE_WINAPI STATE_sound_stop_all_sounds();

//Using this function togwether with STATE_sound_get_next()
//makes it possible to go all over the sounds that were loaded into the engine.
// Example:
//	for(DWORD sound=STATE_sound_get_first() ; sound!=NULL; sound=STATE_sound_get_next(sound) ) {
//		//do something for each sound
//	}
STATE_IMPORT DWORD STATE_WINAPI STATE_sound_get_first();

//use this function to change the distance reach of a sound that is attached to some entity.
//the engine measures the distance between the entity that is attached to the sound and the
//camera and updates the volume and balance accordingly.You can increase the volume of a sound 
//by increasing the distance reach of the sound, or decreasing the objects' sound respectively.
STATE_IMPORT int STATE_WINAPI STATE_sound_set_distance_reach(DWORD sound_handle,double distance_reach);


//Using this function togwether with STATE_sound_get_next()
//makes it possible to go all over the sounds that were loaded into the engine.
// Example:
//	for(DWORD sound=STATE_sound_get_first() ; sound!=NULL; sound=STATE_sound_get_next(sound) ) {
//		//do something for each sound
//	}
STATE_IMPORT DWORD STATE_WINAPI STATE_sound_get_next(DWORD sound_handle);


//This function will set the volume of the given sound.
//volume should be in the range of [0-1000] . 1000 is the maximum valume possible.
//100 is the default volume.
//See also STATE_sound_set_volume()
//volume should be in the range of [0-1000] . 1000 is the maximum valume possible.
//The difference between this function and STATE_sound_set_volume_realtime()
//Is that this function works even if the sound is not being played.
//If you intend to use a specific sound in a constant volume then
//it is better to use STATE_sound_set_volume() then STATE_sound_set_volume_realtime().
//Note that if the sound is attached to an entity (object, polygon or group)
//then actual volume will be effected by the distance of the viewer\listener from the
//sound source. Note also that you can use
//STATE_sound_set_distance_reach() to effect the result calculated volume.
//This is important because if the listener is located outside the distance reach
//zone then no sound will be heard.
STATE_IMPORT void STATE_WINAPI STATE_sound_set_volume(DWORD sound_handle,int volume);

//This function will set the volume of the given sound.
//See also STATE_sound_set_volume()
//volume should be in the range of [0-1000] . 1000 is the maximum valume possible.
//The difference between this function and STATE_sound_set_volume()
//Is that this function works only while the sound is being played.
//If you intend to use a specific sound in a constant volume then
//it is better to use STATE_sound_set_volume().
//Note that if the sound is attached to an entity (object, polygon or group)
//then actual volume will be effected by the distance of the viewer\listener from the
//sound source. Note also that you can use
//STATE_sound_set_distance_reach() to effect the result calculated volume.
//This is important because if the listener is located outside the distance reach
//zone then no sound will be heard.
STATE_IMPORT void STATE_WINAPI STATE_sound_set_volume_realtime(DWORD sound_handle,int volume);


//You can call this function only if a sound is being played.
//The return number will be in the range [0-1000].
//500 is the default. 
//1000 is the volume defined by the sound file itself.
// The function will return -1 if the sound is not being played or some error has happened.
// See also STATE_sound_set_volume(), STATE_sound_set_distance_reach()
STATE_IMPORT int STATE_WINAPI STATE_sound_get_volume(DWORD sound_handle );

//Will set the frequency of the sound. The range is [0-100000]
//Default is 44100.
//You can use this function for special effect. For example simulating
//a racing car engine sound. Try adding 100 to the frequency each time that the speed
//of the car increases.
//See also STATE_sound_get_frequency()
STATE_IMPORT void STATE_WINAPI STATE_sound_set_frequency(DWORD sound_handle, int frequency);

//You can call this function only if the sound is playing.
// See STATE_sound_set_frequency()
STATE_IMPORT int STATE_WINAPI STATE_sound_get_frequency(DWORD sound_handle);

//Returns YES or NO.
STATE_IMPORT int STATE_WINAPI STATE_sound_is_sound_playing(DWORD sound_handle);

//Set the name of the sound. The default is the name of the sound file that was
//loaded without the path and the extension.. 
//See also STATE_sound_get_handle_using_name(), STATE_sound_get_sound_name()
STATE_IMPORT int STATE_WINAPI STATE_sound_set_sound_name(DWORD sound_handle, char *name);


// Returns the name of the sound
// The function returns a pointer to the name which is stored internally inside the 3D Engine
// Visual Basic users should use STATE_entity_get_name() instead.
// See also STATE_entity_get_name(), STATE_sound_set_sound_name()
STATE_IMPORT char * STATE_WINAPI STATE_sound_get_sound_name(DWORD sound_handle);

//Returns a handle to the sound according to the given name.
STATE_IMPORT DWORD STATE_WINAPI STATE_sound_get_handle_using_name(char *name);

//Returns the sound that is attached to the given entity. entity_handle
//can be a handle to one of the three: polygon handle, object handle or group handle.
//If no sound is attached to the given entity it will return NULL.
// Example:
// sound_handle=STATE_sound_get_handle_using_entity(my_object);
STATE_IMPORT DWORD STATE_WINAPI STATE_sound_get_handle_using_entity(DWORD entity_handle);

//pause can be one of the two: SOUND_PAUSED or SOUND_RESUME_PLAYING
//To pause a specific sound call this function with the SOUND_PAUSED constant.
//To start playing it again call with the SOUND_RESUME_PLAYING constant
//Example:
//		STATE_sound_set_pause_mode(my_sound, SOUND_PAUSED); //this will pause the sound
//		//your code here ....
//		STATE_sound_set_pause_mode(my_sound, SOUND_RESUME_PLAYING); //this will resume playing.
STATE_IMPORT void STATE_WINAPI STATE_sound_set_pause_mode(DWORD sound_handle,BOOL pause);

//Will start playing the CD ROM. The first track is 1
//the last track is of course varied from one CD to another.
//To get the number of tracks use STATE_sound_CD_get_num_of_tracks()
STATE_IMPORT void STATE_WINAPI STATE_sound_CD_play(int track);

//This will stop playing the CD ROM.
STATE_IMPORT void STATE_WINAPI STATE_sound_CD_stop();

//Returns the current played track.
STATE_IMPORT int STATE_WINAPI STATE_sound_CD_get_track();

//Returns the number of tracks on the CD.
STATE_IMPORT int STATE_WINAPI STATE_sound_CD_get_num_of_tracks();

//pause can be one of the two: PAUSED or RESUME_PLAYING
//To pause a specific sound call this function with the PAUSED constant.
//To start playing it again call with the RESUME_PLAYING constant
//Example:
//		STATE_sound_CD_set_pause_mode(PAUSED); //this will pause the sound
//		//your code here ....
//		STATE_sound_CD_set_pause_mode(RESUME_PLAYING); //this will resume playing.

STATE_IMPORT void STATE_WINAPI STATE_sound_CD_set_pause_mode(BOOL pause);

//This will eject the CD.
STATE_IMPORT void STATE_WINAPI STATE_sound_CD_eject();




//--------------------------------------------------------//
//======= T H E    M U L T I P L A Y E R    A P I ========//
//--------------------------------------------------------//

/*

This API allows multiple users to run the same application using 
the Internet or their local network (LAN) as a mean of communication.
The API is focused on making it as simple as possible to write a multiplayer applications.

In the next version we will also 
provide servers that game and other application developers
can use as a lobby for potential users.

Before reading the various functions of this API
one should take a look at this "hello multiplayer" example.
Please also note the samples in the SDK.

Example:
	
void main()
{
	STATE_engine_load_world(....)
	
	//Usually one should ask the user wheter he wants to host the game \ application
	//or to join an existing session.
	 int am_i_the_host=ask_user_if_he_wants_to_host_or_join();
	
	
	if(am_i_the_host==YES) {
		if(STATE_multiplayer_connect("game name", "Player name", NULL, TRUE)!=OK)
			exit(-1);
	}
	else {
		if(STATE_multiplayer_connect("game name","Player name", "domain_name_or_IP_of_hosting_computer", FALSE)!=OK)
			exit(-1);
	}


	while (escape key not pressed)
	{
		STATE_engine_render(NULL,NULL);
		
		//Sending a message  
		
		//Initialize a new message  
		STATE_multiplayer_message_create();
		
		double location[3];
		DWORD camera=STATE_camera_get_current();
		STATE_camera_get_location(camera,&location[0],&location[1], &location[2]);
		
		//Write the location to the internal message buffer.  
		STATE_multiplayer_message_write_buffer(location,sizeof(double)*3);
		STATE_multiplayer_message_send();
		
		//receiving a message		  
		char sender_name[256];
		double *opponent_location;
		int result=STATE_multiplayer_message_get(sender_name);
		if(result==STATE_MULTIPLAYER_USER_MESSAGE) {
			opponent_location=STATE_multiplayer_message_read_buffer();
			//process his location here... 
			double x=opponent_location[0];
			double y=opponent_location[1];
			double z=opponent_location[2];
			...

		}
	}
	
	STATE_engine_close();
  
}


*/




//This function initialize a multiplayer session.
//This function must be called before any multiplayer communication can take place.
//
// Aruments:
//----------
// host:
//		If TRUE then this computer will become the host of the game.
//		Usually the application should ask the user whether he wants to join an existing session or
//		to start a new session on his computer.
//
// ip_address:
//		Should be null if host==TRUE.
//		If host==FALSE then this should be the IP or the domain name of the game \ application we want to join.
//
// player_name:
//  The name of the player. 
//
// Game_name:
//  The name of the game. If there are several sessions running on the given host then 
//	only the session that matches this name will be chosen.
// 
//
// Please also note that STATE_multiplayer_connect() can only connect to sessions that are based on the 3DSTATE Engine. 
//
// Return value:
// OK or VR_ERROR.
STATE_IMPORT int STATE_WINAPI STATE_multiplayer_connect(char *game_name,char *player_name, char *ip_address,BOOL host);


// Call this function to terminate the session.
// If your computer is hosting the session (See STATE_multiplayer_connect() )
// then calling this function will end the session completely. 
// If you have joined a session running on a remote computer then calling this function
// will disconnect your computer from the session. All the other users \ computers
// can continue their session. When closing the engine, this function is automatically called,
// so you don't need to call it unless you need to manually disconnect before closing the engine.

STATE_IMPORT void	STATE_WINAPI STATE_multiplayer_close();

//Calling this function tells the engine that you want now to start 
//creating a new message and the previous message can be deleted.
//The engine has an internal buffer that is used to store the message.
//Calling STATE_multiplayer_message_create() simply reset this buffer.
// Example:
//	STATE_multiplayer_message_create();
STATE_IMPORT void STATE_WINAPI STATE_multiplayer_message_create();

//The engine has an internal buffer that is used to store the message.
//Calling STATE_multiplayer_message_create() simply reset this buffer.
// Calling this function will add the given buffer to the internal message buffer.
// Returns: OK or VR_ERROR.
//
// Examples
//  A)
//		STATE_multiplayer_message_create();
//		double location[3]={100,200,300};
//		STATE_multiplayer_message_write_buffer(location, sizeof(double)*3);
//		STATE_multiplayer_message_send(); // three double numbers will be sent 100,200,300
//
//  B)
//		double location[3]={100,200,300};
//		STATE_multiplayer_message_write_buffer(location, sizeof(double)*3);
//		location[0]=400;
//		//Note that the second buffer is concatenated to the first one.
//		STATE_multiplayer_message_write_buffer(location, sizeof(double)*3);
//		STATE_multiplayer_message_send();// six double numbers will be sent 100,200,300, 400, 200, 300
STATE_IMPORT int STATE_WINAPI STATE_multiplayer_message_write_buffer(void *buffer, DWORD buffer_size_in_bytes);

//Sends a message to all the users who are connected to the current session.
//The engine has an internal buffer that is used to store the message.
//Calling STATE_multiplayer_message_create() simply reset this buffer.
//To compose a message use STATE_multiplayer_message_write_buffer()
STATE_IMPORT int STATE_WINAPI STATE_multiplayer_message_send();


#define STATE_MULTIPLAYER_SYSTEM_MESSAGE 6
#define STATE_MULTIPLAYER_USER_MESSAGE 7

//When the other players\users in this session send messages these messages are accumulated in
//an internal queue inside the engine. This function removes the oldest message waiting
// from the queue and copy it to an internal buffer used for reading messages.
// In order to read the message content, use STATE_multiplayer_message_read_buffer()
//Returns one of this four options:
//1) STATE_MULTIPLAYER_USER_MESSAGE
//		This means that this is a message from another user \ computer.
//
//2) STATE_MULTIPLAYER_SYSTEM_MESSAGE
//		This means that this is a system message.
//		A system message could be for example a notification that  
//		a new user has joined the session. See STATE_multiplayer_message_system_read()
//		to learn how to process system messages.
//
//3) THERE_ARE_NO_MESSAGES
//	 This means that the message queue is empty and that there are no messages waiting
//	 for you to read.
//
//4) VR_ERROR
//	 An error occurred.
//
// Example:
//
//		char sender_name[256];
//		double *opponent_location;
//		int result=STATE_multiplayer_message_get(sender_name);
//		if(result==STATE_MULTIPLAYER_USER_MESSAGE) {
//			opponent_location=STATE_multiplayer_message_read_buffer();
//			//process his location here...
//		}
//
STATE_IMPORT int STATE_WINAPI STATE_multiplayer_message_get(char *name_of_sender);

//Returns a pointer into the internal buffer that stores the message.
//This function can not be used to process system messages. To read system messages
// use STATE_multiplayer_message_system_read() instead.
//
// Example:
//		char sender_name[256];
//		double *opponent_location;
//		int result=STATE_multiplayer_message_get(sender_name);
//		if(result==STATE_MULTIPLAYER_USER_MESSAGE) {
//			opponent_location=STATE_multiplayer_message_read_buffer();
//			//process his location here...
//		}
STATE_IMPORT void *STATE_WINAPI STATE_multiplayer_message_read_buffer();


// A new player has been created in the session
#define STATE_CREATEPLAYER   0x0003  

//A player has been deleted from the session
#define STATE_DESTROYPLAYER  0x0005  

// The player has lost its connection with all the
 //other players in the session.
#define STATE_SESSIONLOST           0x0031

#define THERE_ARE_NO_MESSAGES	NO

//When receiving a system message (i.e. STATE_multiplayer_message_get() returns
//STATE_MULTIPLAYER_SYSTEM_MESSAGE ) use this function to
//understand the type of the system message received.
//This function returns a constant that can be one of the following:
//
// STATE_CREATEPLAYER 
//			A new player has joined the session
//
// STATE_DESTROYPLAYER 
//			A player has left the session
//
//STATE_SESSIONLOST 
//			Our computer has lost its connection with all the
//			other players in the session.
//
STATE_IMPORT unsigned int STATE_WINAPI STATE_multiplayer_message_system_read();
  

//Returns whether this computer serve as the host. 
//The answer is according the the host argument given in STATE_multiplayer_connect().
STATE_IMPORT int STATE_WINAPI STATE_multiplayer_is_host();
  

//Returns the IP address of the computer which is running the application.
//Please note that this function should be called after the user computer is
//connected to the Internet.
//When using a dial-up connection (telephone modem) then the IP number will change
//on each connection.
// Q: Why do I need this function ?
// A: In case that the user computer is hosting the game then all the other player
//    should have his IP number so they can connect to the host.
STATE_IMPORT char * STATE_WINAPI STATE_multiplayer_get_computer_ip();
/*
Writing and reading messages to/from the messages buffer is easily done using the functions:
STATE_multiplayer_message_read_buffer()
STATE_multiplayer_message_write_buffer()
However, in some languages such as Visual Basic,
one cannot pass a buffer as an argument to a function.
Therefor, the functions below were written mainly for the Visual Basic programmers.
(Although if a C programmer feels more comfortable with them he can use them as well)
As you can see, for each type of entry we use a different function, if it is textual entry we use :
STATE3dWorld1.Multiplayer_message_write_string("hello guys");

If we want to send our location, which is a floating point variable we use
STATE3dWorld1.Multiplayer_message_write_double (my_location.x);

If its some integer, we use 
STATE3dWorld1.Multiplayer_message_write_long (123456789);

And if we just need to send a byte (short) variable, we use
STATE3dWorld1.Multiplayer_message_write_byte('A');
*************************************************************************
IMPORTANT!!!
The entries are appended to the message buffer in the same order in which 
they were inserted, so when a user receives that message he should extract
the entries from the message in the same order in which they were written.
***************************************************************************
For example :
Writing ...
STATE_multiplayer_write_string("John Smith");
STATE_multiplayer_write_double(location[0]);
STATE_multiplayer_write_double(location[1]);
STATE_multiplayer_write_double(location[2]);

Reading...
char playerName[100];
double location[3]={0,0,0};
STATE_multiplayer_read_string(playerName);
STATE_multiplayer_read_double(&location[0]);
STATE_multiplayer_read_double(&location[1]);
STATE_multiplayer_read_double(&location[2]);

*/
STATE_IMPORT int STATE_WINAPI STATE_multiplayer_message_write_double(double value);

//See function STATE_multiplayer_message_write_double()
STATE_IMPORT int STATE_WINAPI STATE_multiplayer_message_read_double(double *value);

//See function STATE_multiplayer_message_write_double()
STATE_IMPORT int STATE_WINAPI STATE_multiplayer_message_read_long(long *value);

//See function STATE_multiplayer_message_write_double()
STATE_IMPORT int STATE_WINAPI STATE_multiplayer_message_write_long(long value);

//See function STATE_multiplayer_message_write_double()
STATE_IMPORT int STATE_WINAPI STATE_multiplayer_message_read_string(char *string);	

//See function STATE_multiplayer_message_write_double()
STATE_IMPORT int STATE_WINAPI STATE_multiplayer_message_write_string(char *string);

//See function STATE_multiplayer_message_write_double()
STATE_IMPORT int STATE_WINAPI STATE_multiplayer_message_write_byte(BYTE value);

//See function STATE_multiplayer_message_write_double()
STATE_IMPORT int STATE_WINAPI STATE_multiplayer_message_read_byte(BYTE *value);



  

//-------------------------------------------------------//
//======== T H E    T R A C K   A P I ===================//
//-------------------------------------------------------//

// Track is merely a sequence of points.
// cameras and objects can move on tracks in various  ways

// Returns YES or NO.
// YES , if the given handle is a track handle. NO if not.
// If it is not a track handle , a message box will appear,
// the title of this message box will be the parameter "function_asking"
// if function_asking==NULL than no popup will be created.
// See also IS_TRACK()
STATE_IMPORT int STATE_WINAPI STATE_track_is_track(DWORD track_handle, char *function_asking);


STATE_IMPORT DWORD STATE_WINAPI STATE_track_get_using_name(char *name);

// returns YES if cyclic else returns NO
STATE_IMPORT int STATE_WINAPI STATE_track_is_cyclic(DWORD track_handle);

// Parameters:
//	[ IN ] YES_or_NO
//			YES if this track should be cyclic, NO if not.
//
//	Example:
//		STATE_track_set_cyclic(my_track, YES);
//
STATE_IMPORT void STATE_WINAPI STATE_track_set_cyclic(DWORD track_handle, int YES_or_NO);

// Returns the name of the track.
// The function returns a pointer to the name which is stored internally inside the 3D Engine
// Visual Basic users should use STATE_entity_get_name() instead.
// See also STATE_entity_get_name()
STATE_IMPORT char * STATE_WINAPI STATE_track_get_name(DWORD track_handle);

STATE_IMPORT void STATE_WINAPI STATE_track_set_name(DWORD track_handle, char *name);

STATE_IMPORT int STATE_WINAPI STATE_track_get_number_of_points(DWORD track_handle);

STATE_IMPORT DWORD STATE_WINAPI STATE_track_get_first_track(void);

STATE_IMPORT DWORD STATE_WINAPI STATE_track_get_next(DWORD track_handle);

//Another way of looping through all the tracks that are loaded into the engine is by using the FOREACH_TRACK macro.
// Example:
//		FOREACH_TRACK(trk) {
//			//do something with the track. For rxsmple lets check its name
//			char *name=STATE_track_get_name(trk);
//		}
#define FOREACH_TRACK(trk) for(DWORD trk=STATE_track_get_first_track(); trk!=NULL; trk=STATE_track_get_next(trk))


//returns the point in pt[]
// returns VR_ERROR if the index is too big or negative
// else returns OK
STATE_IMPORT int STATE_WINAPI STATE_track_get_point(DWORD track_handle, int point_index, double pt[3]);

//Use this function to change the value of a specific point on the track.
//Parameters:
//	track_handle: The track handle
//
//	point_index: A value between 0 to (STATE_track_get_number_of_points() -1 )
//
//  pt: The X,Y,Z values to set for the given point index.
//
// Return value:
//	VR_ERROR if the failed (our of range index)
//	OK otherwise.
//
// See also: STATE_track_insert_point(), STATE_track_add_point_to_end()
//
STATE_IMPORT int STATE_WINAPI STATE_track_set_point(DWORD track_handle, int point_index, double pt[3]);

// Adds a point as the last point of the track.
// If the new added point is equal to the currently last point. The function will
// return VR_ERROR and the point wont be added.
// If this is a cyclic track then the added point must not also be 
// equal to the first point (index 0 )
//
//	Parameters:
//		track_handle: The track handle
//
//		pt: The X,Y,Z values to set for the given point index.
//
// Return value:
//		Returns OK or VR_ERROR. VR_ERROR is returned if one of the neighboring points
//		is identicle (if it is a cyclic track then there will be two identicle points)
//
//
//	Example:
//
//		double p[3]={100,200,300};
//		DWORD my_track=STATE_track_create("My Track1", YES, NULL, 0);
//		STATE_track_add_point_to_end(my_track, p);
//
//	See also: 
//		STATE_track_insert_point(), STATE_track_set_point()
//
STATE_IMPORT int STATE_WINAPI STATE_track_add_point_to_end(DWORD track_handle, double p[3]);

// Inserts a point to the given index.
// The points at the given index will be shift one place above and the given point will be inserted
// The total number of points of the track will be increased by one
// Index 0 is the first point.
// Note that consecutive identicale points are not allowed so if the inserted point
// is equal to one of its future neighbors the function will return VR_ERROR
// and the point wont be inserted.
// Not that it is allowed to have identicale points on a track just not consecutively (one after the other)
//
//	Parameters:
//		track_handle: The track handle
//
//		point_index: A value between 0 to STATE_track_get_number_of_points()
//
//		pt: The X,Y,Z values to set for the given point index.
//
// Return value:
//		Returns OK or VR_ERROR. VR_ERROR is returned if one of the neighboring points
//		is identicle
//
//	Example A: 
//		//Adding a point as the first point
//		double p[3]={1,2,3};
//		STATE_track_insert_point(my_track, p, 0);
//
//	Example B:
//		//adding a point as the last point
//		//note that this is equal to STATE_track_add_point_to_end()
//			double p[3]={1,2,3};
//			int number_of_points=STATE_track_get_number_of_points(my_track);			
//			STATE_track_insert_point(my_track, p, number_of_points);
//
//	See also: 
//		STATE_track_add_point_to_end(), STATE_track_set_point()
//
STATE_IMPORT int STATE_WINAPI STATE_track_insert_point(DWORD track_handle, double p[3], int index_of_new_point);


// Removes a point from the track.
// The number of points in the track will be decreased by one.
// The points with indexes greater than the deleted point will
// move one index down.
STATE_IMPORT void STATE_WINAPI STATE_track_delete_point(DWORD track_handle, int index_of_point_to_delete);



// point element can be 0 or 1 or 2
// 0 means get the x element. 1 ==> y. 2 ==>z
// note that if the point_index or the element number
// are out of range the function returns 0. there is no way 
// to know if the 0 is because of error or no.
STATE_IMPORT double STATE_WINAPI STATE_track_get_point_element(DWORD track_handle, int point_index,  int point_element);

// Returns the array of points
// You can then access the array yourself for getting and setting points
// Be careful not access the array with an out-of-range  index
// This could cause your program to crash
// indexes should be in the range 0,1,2 ... ,(number_of_points-1)*3+2
// Access example:
// points_array=STATE_track_get_points(track_handle);
// x=points_array[point_index*3+0];
// y=points_array[point_index*3+1];
// z=points_array[point_index*3+2];
//
// The last index possible is 
// array[(number_of_points-1)*3 +2] we have  (number_of_points-1)  because we start with 0
// to get the number of points do:
// number_of_points=STATE_track_get_number_of_points(track_handle);
// The function will always return the same address (array) for a given track.
//
// Example:
//		//This example shows how to create a copy of a track
//		double *points=STATE_track_get_points(src_track);
//		int number_of_points=STATE_track_get_number_of_points(src_track)
//		STATE_track_create("my_track", YES, points, number_of_points);
//
STATE_IMPORT double *  STATE_WINAPI STATE_track_get_points(DWORD track_handle);

// P is a point anywhere in space
// returns the index to the point on the track that
// is the closest.
// NOTE that if an object or a camera has a track and the track_offset is
// not (0,0,0) then you must do the following to get a correct result
//
//	P[0]=P[0]-track_offset[0];
//	P[1]=P[1]-track_offset[1];
//	P[2]=P[2]-track_offset[2];
//	index=STATE_track_find_closest_point_on_track(track, P);
//	STATE_track_get_point(track, index, pt);
//	pt[0]=pt[0]+track_offset[0];
//	pt[1]=pt[1]+track_offset[1];
//	pt[2]=pt[2]+track_offset[2];
//
STATE_IMPORT int STATE_WINAPI STATE_track_find_closest_point_on_track(DWORD track_handle, double P[3]);

//  The line connecting each 2 consecutive points on the track is called a segment
//  This function finds the segment which is the closest to the given point P
//	This is totally different from finding the closest point.
//	A certain segment of the track could be the closest segment to a given point P
//	despite the fact that its two edges point are the most further away points
//	from the give point P. (think of it ...)
//
//	Parameters:
//		[in] track_handle: 
//				a handle to a track
//
//		[in] P:
//				A point in space.
//
//	Return Value.
//		The function returns the index of the beginning of the closest segment
//		if index 0 is returned then the closest segment is point-0 to point-1
//		if index_of_closest==trk->number_of_points-1 
//		then the closest segment is from the last point to the first point.
//
// Example:
//		double P[3]={1,2,3};
//		int index=STATE_track_find_closest_segment_on_track(my_track, P);
//
STATE_IMPORT int STATE_WINAPI STATE_track_find_closest_segment_on_track(DWORD track_handle, double P[3]);


// Creates a new track.
//
// Parameters:
//		points_buffer:
//				contains the x,y,z of the points (example: x1,y1,z1, x2,y2,z2, x3,y3,z3 ....)
//				note that if number_of_points==3 , we need array with 9 doubles (double points[9] )
//				The function will copy the points from the given buffer into an internal storage
//				so that the given points_buffer can be released after calling STATE_track_create() function
//				You can also give NULL. In this case the engine will create a track with zero points.
//				You can then call STATE_track_add_point_to_end() or STATE_track_insert_point() to add points
//				
//		track_name:
//				The name of the track. Note that if track_name is not unique (there is already another track with the same name),
//				the name will be modified so it will be unique. If you use the track name
//				then it is a good practice to check the name after the track was created.
//				or to make sure in advance that the name is unique.
//
// Example A:
//		DWORD my_track=STATE_track_create("My Track1", YES, NULL, 0); //create an empty track
//		double p[3]={100,200,300};
//		STATE_track_add_point_to_end(my_track, p); //add a point to the track

//
// Example B:
//
//		double points[9]={ 0,0,0,   100,0,0, 0,0,100};
//		DWORD trk=STATE_track_create("trk1", YES, points, 3);

//		int num=STATE_track_get_number_of_points(trk); //num should get the value 3
//		double pt[3];
//		for(int i=0; i<num; i++) {
//			STATE_track_get_point(trk, i, pt);
//
//			//Do something with the point ...
//		}
//

STATE_IMPORT DWORD STATE_WINAPI STATE_track_create(char *track_name, int is_cyclic, double *points_buffer, int number_of_points);

// It is the responsibility of the user that the points buffer is big enough for the number of points.
// The given buffer will be copied into an internal buffer inside the engine
// points_buffer contains the x,y,z of the points (example: x1,y1,z1, x2,y2,z2, x3,y3,z3 ....)
// note for example that if the number_of_points==3 then we will need an array with 9 doubles (double points[9] )
//
STATE_IMPORT void STATE_WINAPI STATE_track_set_points_buffer(DWORD track_handle, double *points, int number_of_points);

// If the given number_of_points is smaller than the number of points of the track
// then the track points wich their index is above the given number will be discarded.
// This will have the effect of shortening the track.
//
// If the given number is bigger then the current number of points of the track then
// the engine will increase the number of points of the track.
// note though that the new added points are all set to 0,0,0
// so you must set their values later using STATE_track_set_point()
//
// To conclude, it seems that this function is not very useful.
//
STATE_IMPORT void STATE_WINAPI STATE_track_set_number_of_points(DWORD track_handle, int number_of_points);

// deletes a track
// be carful not to use this handle after it was deleted
// a good practice is to do like that:
//	STATE_track_delete(my_track);
//  my_track=NULL;
//
STATE_IMPORT void STATE_WINAPI STATE_track_delete(DWORD track_handle);

STATE_IMPORT void STATE_WINAPI STATE_track_delete_all(void);

// *** Not a very useful function.
// sending YES will write the given track to file after call to STATE_engine_save() with SAVE_ONLY_WHATS_NEEDED 
// Note that it if STATE_engine_save() is called without SAVE_ONLY_WHATS_NEEDED, all the tracks will be saved
// sending NO will cause  STATE_engine_save() not to save this track (only if SAVE_ONLY_WHATS_NEEDED is used)
// The default value is NO
// see also STATE_track_get_save_flag()
STATE_IMPORT void STATE_WINAPI STATE_track_set_save_flag(DWORD track_handle, int yes_no_flag); 

// *** Not a very useful function.
// returns YES or NO
// The default value is NO
// see STATE_track_set_save_flag() for more details
STATE_IMPORT int STATE_WINAPI STATE_track_get_save_flag(DWORD track_handle); 

// *** Not a very useful function.
// Save the tracks into the given file name.
// Note that STATE_engine_save_world() will save the wholw world including the tracks
// STATE_track_save_file() is useful for the cases where you only want to save the tracks
// and you don't want to call STATE_track_save_file() which could take several minutes
// The tracks saved with STATE_track_save_file() could later be loaded using
// STATE_track_load_from_file()
//
//	Parameters:
//		file_name: 
//			file name defining where you want the tracks to be saved.
//			examples "C:\\my_tracks\\tracks_file.wld", "my_folder\\tracks.txt"
//			
//		save_all_flag:
//			YES or NO. if YES then all the tracks will be saved.
//			If NO then only the tracks that their save_flag is set 
//			will be saved. See also: STATE_track_set_save_flag()
//
//	See also: STATE_track_load_from_file();
STATE_IMPORT int STATE_WINAPI STATE_track_save_to_file(char *file_name, int save_all_flag);

// *** Not a very useful function.
// load tracks from the given file. The file opened must be a wld file or a file that
// was made using the STATE_track_save_to_file() function.
// Note that when doing STATE_engine_load() the engine will automatically load all the tracks
// so this function is only needed if you have decided to save the tracks separately using the function
// STATE_track_save_to_file()
//
//	Parameters:
//		file_name: the name of the track file to be loaded.
//		examples "C:\\my_tracks\\tracks_file.wld", "my_folder\\tracks.txt"
//		
//	overwrite_flag:
//		Can be YES or NO.
//		Used for deciding what to do when there is a track in the loaded file
//		that has the same name as a track which is already loaded into the engine.
//		If overwrite_flag==YES then the new loaded track will replace the previous one
//		If overwrite_flag==NO then the track in the loaded file will be skipped
//
// Example:
//		STATE_track_load_from_file("Tracks\\tracks.txt", YES);
//
STATE_IMPORT int STATE_WINAPI STATE_track_load_from_file(char *file_name, int overwrite_flag);

// Returns the length of the track. If the track is cyclic that the result will also include
// the length of the last segment connectin the last point and the first point.
//
// Example:
//		double length=STATE_get_track_length(my_track);
//
STATE_IMPORT double STATE_WINAPI STATE_track_get_length(DWORD track_handle);

//Calculates the location on the track that is in the given distance from the start of the track (the first point)
//
// Parameters:
//		[in] track_handle:
//			a handle to a track
//
//		[in] distance:
//			The distance from the first point of the track
//
//		[out] location: 
//			The X,Y,Z coordinate of a point that is in on the track and in the given distance from 
//			the first point of the track
//
// Remarks:
//		If the requested distance is longer than the track length and the track is non-cyclic then
//		the function will return the last point on the track and -1 will be returned 
//		indicating that the end of the track was reached. When the track is cyclic then there
//		is no problem in specifing a distance that is longer then one lap of the track
//
// Return value:
//		Returns the index of the next point on the track (in other words if we will continue moving on the track
//		from the calculated location the return value is the index of the next track point we will meet)
//		A return value of -1 indicates that the distance has reached or passed the track length (apply only to non-cyclic tracks)
//
STATE_IMPORT int STATE_WINAPI STATE_track_get_location_at_distance_from_start(DWORD track_handle, double distance, double location[3]);

//-------------------------------------------------------//
//======== T H E    B A C K G R O U N D   A P I =========//
//-------------------------------------------------------//

// Background is the background on which the world is rendered.
// The idea is that you give a bitmap and the engine automatically 
// deals with it. For example when you turn around the engine shift the background
// image so it is not noticeable that it isnt part of the world.
// The engine also sees to it that you can turn 360 degrees without
// seeing the background edges ...
 

// Returns YES or NO.
// YES , if the given handle is a background handle. NO if not.
// If it is not a background handle , a message box will appear,
// the title of this message box will be the parameter "function_asking"
// if function_asking==NULL than no popup will be created.
// See also IS_BACKGROUND()
STATE_IMPORT int STATE_WINAPI STATE_background_is_background(DWORD background_handle, char *function_asking);


// The given name should be in capital letters
// If the return value is 0 it means 
// that there is no background with the given name.
STATE_IMPORT DWORD STATE_WINAPI STATE_background_get_handle(char *background_name);


STATE_IMPORT DWORD STATE_WINAPI STATE_background_get_first_background(void); 
STATE_IMPORT DWORD STATE_WINAPI STATE_background_get_next(DWORD background_handle); 

//creates a new background
//Returns a handle to the new background
//arguments:
// name: any string will do , example "my_sky"
// bitmap_handle: see STATE_bitmap_load() for getting the handle
// Note that you have to call STATE_background_use() so that the background will be shown
STATE_IMPORT DWORD STATE_WINAPI STATE_background_create(char *name, DWORD bitmap_handle);

STATE_IMPORT void STATE_WINAPI STATE_background_set_name(DWORD background_handle, char *name);

// Returns the name of the background.
// The function returns a pointer to the name which is stored internally inside the 3D Engine
// Visual Basic users should use STATE_entity_get_name() instead.
// See also STATE_entity_get_name()
STATE_IMPORT char *STATE_WINAPI STATE_background_get_name(DWORD background_handle);

//The purpose of the distance value is to make the background act more realistic
//by moving up and down according to the camera movement
//The distance value:
//The bigger this number is, the more apparent are the 
//changes in the height  of the background, 
//when the height of the camera changes.  The  background moves 
//up and down according to the height of the camera, to  make it 
//look real (example: If you have a car going up or down a hill, the sky's
//height must change(the sky is the background).) 
// example: STATE_background_set_distance(bg_handle, 30000);
STATE_IMPORT void STATE_WINAPI STATE_background_set_distance(DWORD background_handle, double value); 

//The purpose of the distance value is to make the background act more realistic
//by moving up and down according to the camera movement
//The distance value:
//The bigger this number is, the more apparent are the 
//changes in the height  of the background, 
//when the height of the camera changes.  The  background moves 
//up and down according to the height of the camera, to  make it 
//look real (example: If you have a car going up or down a hill, the sky's
//height must change(the sky is the background).) 
STATE_IMPORT double STATE_WINAPI STATE_background_get_distance(DWORD background_handle); 

//The bigger this number is, the background image will start from a higher
//place. The default is 0
STATE_IMPORT void STATE_WINAPI STATE_background_set_bottom(DWORD background_handle, double value); 

STATE_IMPORT double STATE_WINAPI STATE_background_get_bottom(DWORD background_handle); 




//return OK on success else VR_ERROR
// a value of 100 means normal brightness at the bottom of the background
// a value of 50 means a color that is the average between the background
// and the atmosphere color.
// a value of 200 means twice as bright than the original hue.
// feel free to test with different values.
STATE_IMPORT int STATE_WINAPI STATE_background_set_bottom_intensity(DWORD background_handle, double value); 

STATE_IMPORT double STATE_WINAPI STATE_background_get_bottom_intensity(DWORD background_handle); 

//return OK on success else VR_ERROR
// the changes in the intensity for each scan line
// going from the bottom of the background up.
// 0 means no changes.
STATE_IMPORT int STATE_WINAPI STATE_background_set_intensity_step(DWORD background_handle, double value); 

STATE_IMPORT double STATE_WINAPI STATE_background_get_intensity_step(DWORD background_handle); 

STATE_IMPORT void STATE_WINAPI STATE_background_delete(DWORD background_handle);
STATE_IMPORT void STATE_WINAPI STATE_background_delete_all(void); 

// Tell the engine to use the given background.
// A NULL handle means that we dont want to use any background
STATE_IMPORT void STATE_WINAPI STATE_background_use(DWORD background_handle); 

//Returns the background handle that is currently in use
STATE_IMPORT DWORD STATE_WINAPI STATE_background_get_current_background(void); 

// sending YES will write the given background to file after call to STATE_engine_save() with SAVE_ONLY_WHATS_NEEDED 
// Note that it if STATE_engine_save() is called without SAVE_ONLY_WHATS_NEEDED, all the backgrounds will be saved
// sending NO will cause  STATE_engine_save() not to save this background
// The default value is NO 
// Note that if it is used it will be saved no matter what the flag is
// see also STATE_animation_set_save_flag()
STATE_IMPORT void STATE_WINAPI STATE_background_set_save_flag(DWORD background_handle, int yes_no_flag); 

// returns YES or NO
// The default value is NO 
// see STATE_background_set_save_flag() for more details
STATE_IMPORT int STATE_WINAPI STATE_background_get_save_flag(DWORD background_handle); 

//Replace the bitmap of the background with the given bitmap.
//See also STATE_bitmap_load()
STATE_IMPORT void STATE_WINAPI STATE_background_set_bitmap(DWORD background_handle, DWORD bitmap_handle); 

// Returns a handle to the bitmap that is in use by the given background.
STATE_IMPORT DWORD STATE_WINAPI STATE_background_get_bitmap(DWORD background_handle); 

// Returns the name of the bitmap. Note the difference between calling STATE_background_get_bitmap()
//and then getting the bitmap name using STATE_bitmap_get_name()
// The difference is that here we will get the name even if the bitmap was not loaded yet.
// For example when our world is on the Internet then we first download the world and only then 
// the bitmaps are downloaded. If we call this function we would still get the name even though the bitmap wasn't
// loaded to the engine yet. If you didnt understand this explanation then relax, it is not so important.
STATE_IMPORT char * STATE_WINAPI STATE_background_get_bitmap_name(DWORD background_handle); 


//-------------------------------------------------------//
//======== T H E       F O N T        A P I =============//
//-------------------------------------------------------//

#define FONT_CREATE_DEFAULT	0
#define FONT_CREATE_BOLD       1
#define FONT_CREATE_ITALIC     2
// *** very useful
// Creates a font that can later be used with the STATE_text_ API
//
// Parameters:
//		font_name: the name of the font. For example: Arial
//
//		font_size: the height of the font. For example 14
//
//		flags: could be any of the following: FONT_CREATE_DEFAULT, FONT_CREATE_BOLD, FONT_ITALIC.
//	  		   To use several flags together use | or + . For example  FONT_CREATE_BOLD | FONT_CREATE_ITALIC
//
// Return Value:
//		returns a handle to the font that can later be used with the STATE_text_ API
//		When the function fails it returns NULL
//
STATE_IMPORT DWORD STATE_WINAPI STATE_font_create(char *font_name, int font_size, DWORD flags); 

#define FONT_WRITE_DEFAULT			0
#define FONT_WRITE_CENTERED_X		1
#define FONT_WRITE_CENTERED_Y		2
#define FONT_WRITE_PIXELIZED		256

// *** Very useful
// Write text on the screen. 
//
//	Using this function one can easily do the following
//		1. write text to the screen that would show for a specific time limit
//		2. Switch off old text that was written to the screen
//		3. Replace old messages with new ones
//		4. Create effects such as moving the text
//
//
//	Parameters:
//		font_handle:
//			a handle to a 3DSTATE font. Use to obtain a handle
//			for example: font_handle=STATE_font_create("Ariel", 14, FONT_CREATE_DEFAULT);
//
//		text:
//			The text itself for writing on the screen
//
//		x,y:
//			The location on the screen where the text will be placed. 
//			0,0 denotes the top left corner
//			100,100 denotes the bottom right corner
//			This means that the size of the rendering window is not important
//
//		alpha:
//			use this to control the level of transparency of the text
//			255 means opaque. 0 means totally transparent
//
//		red, green, blue:
//			The color of the text
//
//		flags:
//			any of the following: 
//				FONT_WRITE_DEFAULT, 
//				FONT_WRITE_CENTERED_X, 
//				FONT_WRITE_CENTERED_Y, 
//				FONT_WRITE_PIXELIZED
//			By default the x,y given is the top left corner of the text.
//			Use FONT_WRITE_CENTERED_X or FONT_WRITE_CENTERED_Y to change it
//			Use FONT_WRITE_PIXELIZED for making ugly looking text (pixelized)
//			Use | ot + to combine several flags together. for example FONT_WRITE_CENTERED_X | FONT_WRITE_CENTERED_Y
//
//		number_of_renderes:
//			When calling STATE_font_write_text() the text doesn't appear immediately but only after STATE_engine_render() is called
//			Using this parameter one can tell the engine to resubmit the text for rendering also in the next x renders
//			There are three common situations where we write text:
//			A- Text that keeps changing and is written before (or after) each render
//			   for example (in a game) the plane altitude, time left etc ...
//			   In this case call with number_of_renderes==1
//
//			B- Writing the text once with the atention that it will
//			   be displayed for a long period of time or uknown period of time.
//			   Example "Gear: Manual", "Auto pilot: Off", "Copyright my company 2003"
//			   in this case use number_of_renderes==INT_MAX (see Example A)
//			   If you later want to change the text simply
//			   call STATE_font_write_text() again giving the same x,y just with different text
//			   if you later want to switch off this text simply call
//			   again with the same text and x,y just with number_of_renderes==0
//
//			C- Writing text for a period of time
//			   for example "1000 points bonus", "Extra life" etc ...
//			   in this case use number_of_renderes==1000 or any other number ...
//			   note that on different computers there is different FPS rate (frames per second rate)
//			   To make the text appear the same period time on all computers use
//			   STATE_engine_get_computer_speed_factor()
//				
//
//	Return Value:
//		The function returns a handle to the text created (or NULL if failed)
//		currently there is nothing to do with this handle. In the future this handle will
//		be used by the STATE_text_ API.
//
//	General remarks:
//		1.	This function is very fast since it is using the 3D graphic accelerator
//			to write the text. This is ~1000 faster then using the Windows GDI ...
//
//		2.	3DSTATE 2D text writing mechanism has some important advantage over
//			traditional 2D text mechanism.
//			For example there is no need to add spaces when writing text with varying length
//			One can switch off old text messages etc ...
//		
//		3.  To make the text moving on the screen simply call the function before (or after) each render
//		    with modified x,y. Set the number_of_renderes to 1. See example 3
//
//		4.  To make a text message appear for specific duration measured in seconds (rather in number of renders)
//			use STATE_engine_get_computer_speed_factor() to help you calculate the number of rendered needed
//
//	See also:
//		STATE_object_create_from_text(), //creats an 3D object from the given letters
//		STATE_engine_get_computer_speed_factor(),
//		STATE_font_write_text_scaled()
//
// Example A:
//		DWORD ariel14=STATE_font_create("Ariel", 14, FONT_CREATE_DEFAULT);
//		STATE_font_write_text(ariel14, "Test123", 10, 10, 255, 255, 255, 120, FONT_WRITE_DEFAULT, INT_MAX);
//
// Example B:
//		//Switching off (removing ) the text message from example A by 
//		//using the same text and x,y but with number_of_renderes==0
//		STATE_font_write_text(ariel14, "Test123", 10, 10, 255, 255, 255, 120, FONT_WRITE_DEFAULT, 0);
//
// Example C:
//		//animating text. Simply call the function before (or after) each render
//		//with modified x,y. Set the number_of_renderes to 1
//		STATE_font_write_text(ariel14, "Test123", x, 10, 255, 255, 255, 120, FONT_WRITE_DEFAULT, 1);
//		x++;
//
STATE_IMPORT DWORD STATE_WINAPI STATE_font_write_text(DWORD font_handle, char *text ,double x, double y, BYTE alpha, BYTE red, BYTE green, BYTE blue, DWORD flags, int number_of_renderes); 

// *** Very useful
// Same as STATE_font_write_text
// only that here we can scale the text 
// To make a text animation (the text getting bigger or smaller) simply continue
// to call the function before (or after) each render each time with a different scale value
//
// Remarks:
//		1. Note that the non-scaled text usually looks a bit better.
//
//		2. Note that the size of the scaled text is dependent on the size of the window being rendered
//
// Example:
//		DWORD ariel14=STATE_font_create("Ariel", 14, FONT_CREATE_DEFAULT);
//		STATE_font_write_text_scaled(ariel14, "Test123", 10, 10, 2, 2, 255, 255, 255, 120, FONT_WRITE_DEFAULT, INT_MAX);
//
STATE_IMPORT DWORD STATE_WINAPI STATE_font_write_text_scaled(DWORD font_handle, char *text ,double x, double y, double scale_x, double scale_y ,BYTE alpha, BYTE red, BYTE green, BYTE blue, DWORD flags, int number_of_renderes); 

//Checks whether the given font handle is legal and represents a handle of a font.
//If the handle is not a legal font handle then a popup window will pop out (only if popup_title!=NULL).
//This function is used mainly for debugging (similar to the C++ function assert() )
//Return value: YES or NO.
//Arguments:
//	font_handle: The handle of the font to be examined.
//  popup_title: The title of the popup that will pop-out in case the given handle does not represent
//				a legal font handle. If popup_title==NULL then there wont be any popup
//				The best is to give the function name so if you get the error you know
//				what function should be fixed.
STATE_IMPORT int STATE_WINAPI STATE_font_is_font(DWORD font_handle, char *popup_title);

//-------------------------------------------------------//
//======== T H E       T E X T        A P I =============//
//-------------------------------------------------------//

// This API is currently under constructions.
// When it is completed it will offer some text writing effects


//Returns the number of text messages waiting to be rendered
STATE_IMPORT int STATE_WINAPI STATE_text_get_number_of_text_messages();

//Checks whether the given text handle is legal and represents a handle of a text.
//If the handle is not a legal text handle then a popup window will pop out (only if popup_title!=NULL).
//This function is used mainly for debugging (similar to the C++ function assert() )
//Return value: YES or NO.
//Arguments:
//	text_handle: The handle of the text to be examined.
//  popup_title: The title of the popup that will pop-out in case the given handle does not represent
//				a legal text handle. If popup_title==NULL then there wont be any popup
//				The best is to give the function name so if you get the error you know
//				what function should be fixed.
STATE_IMPORT int STATE_WINAPI STATE_text_is_text(DWORD text_handle, char *popup_title);


//-------------------------------------------------------//
//======== T H E     E N T I T Y      A P I =============//
//-------------------------------------------------------//

//This API contains functions that already exist in other STATE APIs.
//The idea is that one function can serve all API. For example the STATE_entity_get_name()
//can get the name of a camera, an object a group etc ... Some people use this API a lot and
//other dont use it at all. Both ways are fine.


//Gets the name of the entity.
//entity could be a handle to one of the following camera,group,object,track,background,bitmap
//The requested name is copied to the given buffer.
//buffer_size is the buffer size in bytes. If this value is shorter than the requested name
//then the name will be truncated.
//The return value is the length of the name
STATE_IMPORT int STATE_WINAPI STATE_entity_get_name(DWORD entity, char *buffer, int buffer_size);


//Entity could be a handle to a polygon an object or a group.
//Returns the number of bitmaps that were created and assigned to a polygon.
//For more details see STATE_polygon_create_lightmap(), STATE_group_create_light_map()
// Note this important function: STATE_engine_reduce_number_of_bitmaps()
STATE_IMPORT int STATE_WINAPI STATE_entity_create_lightmap(DWORD entity, int max_bm_side, int min_lightmap_size, double sample_rate, int force_ray_tracing, int bitmap_number);


//-------------------------------------------------------//
//======== T H E    3 D  C A R D      A P I =============//
//-------------------------------------------------------//

// This API include specific functions for working with the 3D accelerator.
// We tried to make the existence of the card transparent to the programmer
// so that things should work exactly the same with card or without.

//Note that some of the function exist also in the engine API
//those function do exactly the same. It is preferred if you use just the function from this
//API.

// return YES or NO
STATE_IMPORT int STATE_WINAPI STATE_3D_card_check_hardware_support(void);


// Obsolete. Does nothing defined only for competability with version 6.0
// In version 7.0 there is no software mode. the 3D card is always used
STATE_IMPORT int STATE_WINAPI STATE_3D_card_use(int YES_or_NO);

#define  CARD3D_SOFTWARE_EMULATION 1 //Use it to set DirectX software emulation.
									 //This is usually a lot slower than STATE Software rendering.
									 //The advantage is that it will look exactly like DirectX hardware rendering.
#define  CARD3D_NORMAL 2
#define  CARD3D_OFF 4//in this case it will use STATE software rendering
					 //which is equivalent to STATE_3D_card_use(NO)


// Obsolete. Does nothing defined only for competability with version 6.0
// In version 7.0 there is no software mode. the 3D card is always used
STATE_IMPORT int STATE_WINAPI STATE_3D_card_set_full_screen_mode(int resolution_x, int resolution_y);

#define GAMMA_RAMP_OFF 0
#define GAMMA_RAMP_MAKE_DARKER_COLORS_MORE_VISIBLE 1001
#define GAMMA_RAMP_MAKE_BRIGHTER_COLORS_MORE_VISIBLE 1002

// *** quite useful
// Gamma ramp is used in fullscreen mode to improve the image.
// Usually we use it to make a dark scene brighter
// works only in fullscreen mode !
//
// Examples:
//
//	A)  //making darker colors more visible
//		STATE_3D_card_set_gamma_correction(GAMMA_RAMP_MAKE_DARKER_COLORS_MORE_VISIBLE);
//
//	B)  //disabling gamma correction
//		STATE_3D_card_set_gamma_correction(GAMMA_RAMP_OFF);
//
//	C)  //Making all colors 10 steps brighter. for example rgb (70,80, 95)  will become (80, 90, 105)
//		// rgb (251,253,255) will be cliped to (255,255,255)
//		STATE_3D_card_set_gamma_correction(10);
//
//	D)  //Making all colors 10 steps darker. for example rgb (70,80, 95)  will become (60, 70, 85)
//		// rgb (1,2,3) will be clipped to rgb (0,0,0)
//		STATE_3D_card_set_gamma_correction(-10);
//
STATE_IMPORT void STATE_WINAPI STATE_3D_card_set_gamma_correction(int type);



//Returns YES if the 3D card is programed to use fullscreen.
STATE_IMPORT int STATE_WINAPI STATE_3D_card_is_full_screen_mode(void);


//Hides the driver selection popup
//The default is that this popup is visible.
// Example:
// STATE_3D_card_hide_driver_selection_window(YES);
STATE_IMPORT void STATE_WINAPI STATE_3D_card_hide_driver_selection_window(int YES_or_NO);

//Controls whether the driver selection popup will pop
//when the 3D card is put to work.
//Returns YES or NO.
// See also STATE_3D_card_hide_driver_selection_window()
STATE_IMPORT int STATE_WINAPI STATE_3D_card_is_driver_selection_window_visible(void);

//Calling this function with a YES allow the engine to use DirectX 
//software emulation when there is no 3D accellerator card.
//Use this function if you want to ensure a homogenous look to your
//application with or without a 3D card. The disadvantage of DirectX
//software emulation is that is is a lot slower then STATE software rendering (The default).
//In the case that your application uses a big 3D world then the DirectX
//emulation will probably be too slow. If it is just an object that
//is shown then using this function will probably give a good result.
//
//The default is NO.
//Example STATE_3D_card_allow_software_emulation(YES);
STATE_IMPORT void STATE_WINAPI STATE_3D_card_allow_software_emulation(int YES_or_NO);

#define BILINEAR_FOR_PRIMARY_TEXTURE_ONLY 8
#define BILINEAR_FOR_SECONDARY_TEXTURE_ONLY 16
#define BILINEAR_USE_3D_CARD_DEFAULTS 32

// Function level of importance = High.
//
// The function turns OFF or ON the bilinear filtering.
// Bilinear filtering is a technique for getting better looking textures. When bilinear 
// filtering is ON, it removes the pixelization effect from close textures.
// The default is ON. If preformance are very important you should consider
// shutting bilinear filltering off.
// Bilinear filtering could also be set separately for each polygon using
// STATE_polygon_set_bilinear_filtering() (this will overide any setting made by STATE_3D_card_set_bilinear_filtering() )
// parameters:
//		bilinear_mode:
//			ON  -		The default. Turns on bilinear filtering.
//			OFF -		Turns off bilinear filtering
//			BILINEAR_FOR_PRIMARY_TEXTURE_ONLY -		Turns on bilinear only for the primary texture (each polygon could have two textures, the primary and the secondary)
//			BILINEAR_FOR_PRIMARY_SECONDARY_ONLY -   Turns on bilinear only for the secondary texture (each polygon could have two textures, the primary and the secondary)
//Example:
// STATE_3D_card_set_bilinear_filtering(OFF);
//
// See also STATE_polygon_set_bilinear_filtering()
STATE_IMPORT void STATE_WINAPI STATE_3D_card_set_bilinear_filtering(int bilinear_mode);


#define MIPMAP_FOR_PRIMARY_TEXTURE_ONLY 8
#define MIPMAP_FOR_SECONDARY_TEXTURE_ONLY 16
#define MIPMAP_USE_3D_CARD_DEFAULTS 32


// Function level of importance = High.
//
// The function turns OFF or ON the mipmap filtering (also known as trilinear filtering).
// Try switching it off and see how it effect your world (example: STATE_3D_card_use_mipmaps(OFF);
// Read the rest for your general knowledge
//
// What are mipmaps
// -----------------
// Mipmaps are used for getting better looking textures. 
// The 3D engine creates several lower detailed bitmaps in decreasing order from each bitmap that is used
// for example: If you use a 64x64 bitmap then the engine will create 6 additional bitmaps:
// 32x32, 16x16, 8x8, 4x4, 2x2, 1x1. Using mipmaps use 33% more texture memory.
// 
// Why do we need mipmaps
// -----------------------
// Imagine that you have a 3D world and in this 3D world you have a house that uses
// a 256x256 bitmap size image for the house front wall. Now imagine that you are looking
// at your house from a very long distance. so far away that the house appear very small on your screen
// Now when the house is very far a way the front of the house is represented by only a few pixels (for 
// example 2x2=4 pixels). And now comes the big questions, which 4 lucky pixels of the bitmap should be selected ?
// You original bitmap has 256x256=65536 pixels. Your 3D card has to select only 4 pixels !
// The problem is that when the location of the camera (the player) is moved a little, the 4 chosen pixels will alos change
// In one frame a pixel from the door will be selected, in another frame a pixel from the window curtain etc ...
// The result will be 4 pixels that are flickering in random colors. In real life when we look at something from a distance
// we see only the main\big features of the object. We spot only the average color. This is exactly what mipmaps can achieve.
// When mipmaps are on (considering our example) the 3D card will use the 2x2 version of our original bitmaps.
//
// Inside the 3D card
// ------------------
// When calculating the color of a pixel the 3D card is actually able (most of them) to use two mipmap levels
// The pixel is calculated twice one for each bitmap that is used, then the 3D card makes an average (according
// to how fit is each of the mipmap level). If bilinear filtering is used then 3D card calculates for each mipmap level 
// a bilinear filtered pixel. Then when making a weighted average of the two pixels it is said that the result pixel
// is trilinear filtered. In sort: Bilinear filtering + mipmaps == Trilinear filtering

// Reasons for using Mipmaps
// -------------------------
// 1) In most cases the graphics will look better
// 2) The fact that far away polygons can be rendered using a very small bitmap can
//    improve rendering speed.
//
//
// Reasons for not using mipmaps
// -----------------------------
// 1) Take 33 percent more texture memory
// 2) In some rare cases the graphics will look better with no mipmaps
// 3) The extra caclulations might slow down the rendering.

// Mipmap could also be set separately for each polygon using
// STATE_polygon_use_mipmap() (this will overide any setting made by STATE_3D_card_use_mipmaps() )
// parameters:
//		mipmap_mode:
//			ON  -		The default. Use mipmaps (for both the primary texture and the secondary texture)
//			OFF -		Mipmaps are not used
//			MIPMAP_FOR_PRIMARY_TEXTURE_ONLY -		Use mipmaps only for the primary texture (each polygon could have two textures, the primary and the secondary)
//			MIPMAP_FOR_PRIMARY_SECONDARY_ONLY -    Use mipmaps only for the secondary texture (each polygon could have two textures, the primary and the secondary)
//Example:
// STATE_3D_card_use_mipmaps(OFF);
//
// See also STATE_polygon_use_mipmap()
STATE_IMPORT void STATE_WINAPI STATE_3D_card_use_mipmaps(int mipmap_mode);

// Function level of importance = low.
// Use this function to fine tune the way mipmaps work
// If this function is not use the 3D Engine will automatically decide wich mipmap level of the bitmap to use
// according to the distance between the surface containing the bitmap and the viewer (the camera)
// Sometimes the 3D engine is too quick in using lower resolution mipmap level in these cases one can use this function.
//
// Parameters:
//		bias_level:		
//				mipmap bias level (0,1,2 ... )
//
//		primary_or_secondary_texture:
//				0 for primary texture. 1 for secondary texture.
//
STATE_IMPORT void STATE_WINAPI STATE_3D_card_set_mipmap_bias_level(int bias_level, int primary_or_secondary_texture);



//Returns YES or NO. YES is returned if multiple texture blending is supported by the 3D card
STATE_IMPORT int STATE_WINAPI STATE_3D_card_is_multiple_texture_blending_supported(void);

// General
// --------
// Set the antialiasing mode ON or OFF.
// This is an important function that can enhance graphic quality.
// Usually turning antialiasing ON will slow down rendering though modern
// 3D cards such as GeForce 4 (Ti or MX) and newer cards have good support
// for antialiasing.
//
// What is antialiasing ?
// ----------------------
// Reduction of jagged edges by the use of intermediate shades, smoothing of aliasing.
// 
//
// Usage:
// ------
// Could be call before using the 3D card or afterwords.
// Has no effect when using software rendering.
// The default is OFF.
//
// Example: 
//	STATE_3D_card_set_antialiasing(ON);
//
STATE_IMPORT void STATE_WINAPI STATE_3D_card_set_antialiasing(int ON_or_OFF);

//Returns whether the 3D card supports antialiasing (SORTINDEPENDENT).
STATE_IMPORT int STATE_WINAPI STATE_3D_card_is_antialiasing_supported(void);


//Calling this function will cause the graphic mode popup window to appear.
//Through this window one can change the resolution, set full screen mode and
//control antialiasing.
STATE_IMPORT void STATE_WINAPI STATE_3D_card_show_graphic_mode_setup_window(void);

//Change from window mode to fullscreen mode or the other way arround.
STATE_IMPORT void STATE_WINAPI STATE_3D_card_toggle_fullscreen(void);

STATE_IMPORT void STATE_WINAPI STATE_3D_card_use_software_vertex_processing();

//-------------------------------------------------------//
//======== T H E    U T I L T I E S   A P I =============//
//-------------------------------------------------------//

// The function in this API have nothing to do with 
// 3DSTATE Engine ! You can think of them as an extension to the 
// WIN32 API.
// They are here because they are useful in many applications especially
// applications that use lots of graphics.


//returns the status of a keyboard key.
//If the returned value is negative then it means that the key is pressed
//This function is equivalent to WIN32 GetAsyncKeyState() so for more information 
//read microsoft documentation for the function GetAsyncKeyState()
// Parameters:
//		key_code:
//			Microsofot Virtual-Key Codes. See MSDN help for the values for each key.
//			Usually it is the ascii value but there are some special values
//
//	Example:
//		if(STATE_utilities_get_keyboard_key_state('G')<0)
//			do_something(); //the key is pressed so do something ...

STATE_IMPORT int STATE_WINAPI STATE_utilities_get_keyboard_key_state(int key_code);

//Loads a file into memory.
//returns a pointer to the allocated memory and the size of the file
//If the file could not be loaded a NULL is returned
//This file can be also treated as a string (there will always be a terminating NULL ).
//Example:
//		int file_size;
//		char *buff=(char *)STATE_utilities_file_to_memory("help.txt", &file_size);
//		MessageBox(NULL,buff,"Help", MB_OK);
//		STATE_utilities_free_file_memory(buff);
STATE_IMPORT BYTE *STATE_WINAPI STATE_utilities_file_to_memory(char *file_name, int *file_size);

//Used together with STATE_utilities_file_to_memory() to free memory
//that we don't need any more.
//note that "void *mem_ptr" is a pointer to void which means
//that it will accept any type of pointer.
STATE_IMPORT void STATE_WINAPI STATE_utilities_free_file_memory(void *mem_ptr);

//Returns an array of strings containing all the file names in the given folder and sub-folders  (sub-folders only if recursive_flag==YES)
//
// Parameters:
//	[IN] folder: 
//		The folder where the files reside
//
//	[IN] recursive_flag:
//		Whether we also want to retrieve the file names from the sub-folders
//
//  [OUT] num_of_files:
//			The number of files in the given folder (and sub flders if recursive_flag==YES)
//			This is also the size of the returned array
//
//
//	Example:
//		int num_of_skins;
//		char **all_skins_names=STATE_utilities_get_file_names("model\\skins\\", NO, &num_of_skins);
//		for(int i=0; i<num_of_skins; i++) {
//			DWORD bmp=STATE_bitmap_load(all_skins_names[i], -1);
//			// do something with 
//			...
//		}
//
STATE_IMPORT char ** STATE_WINAPI STATE_utilities_get_file_names(char *folder, int recursive_flag, int *num_of_files);

// stretch a bitmap on the given window
// border_to_leave is usually 0. We can give a
// different value if we want to keep the window
// frame lines visible.
// if border_to_leave==0 then all the window
// will be covered by the bitmap. if border_to_leave
// is equal to 2 (for example) then there will be 
// a border of two pixels left from each side.
STATE_IMPORT void  STATE_WINAPI STATE_utilities_paste_resource_bitmap(HWND hwnd, UINT bitmap_resource_id, int border);

// You can use STATE_utilities_load_bitmap_from_file() to get a handle to the bitmap and then to use this function
STATE_IMPORT void  STATE_WINAPI STATE_utilities_paste_bitmap(HWND hwnd, HBITMAP bitmap_handle, int border);

// This is a very useful function.
// It can be used for pasting 2D bitmaps on the rendered image
// For example an image denoting the health condition of the player
// or the existence of ammunition
// The function copies a bitmap on the given device context.
// Use STATE_utilities_load_bitmap_from_file() to get a handle to the bitmap (HBITMAP)
// Use STATE_engine_render_on_dc() to get the device context handle (HDC hdc)
//
// Parameters:
//
// hdc:				a handle to the device context. usually you would obtain this
//					handle from calling hdc=STATE_engine_render_on_dc()
//		
// hbm:				a HBITMAP handle of the bitmap. you can use 
//					STATE_utilities_load_bitmap_from_file() to get
//					the handle.
//
// x,y:				the coordinate where the bitmap should be placed
//
// bm_stretch_width,
// bm_stretch_height:	Using these params you can stretch the bitmap
//						If you dont want to stretch the bitmap give -1
//						
//
// Examples:
//
// A) 
//	//Copy the bitmap to the top left corner of the window
//	STATE_utilities_copy_bitmap_on_dc(hdc, hbm, 0,0, -1,-1);
//
// B) 
//	//Copy the bitmap to the top left corner of the window
//	//The bitmap will be stretched to a 100x100 square size (size in pixels)
//	STATE_utilities_copy_bitmap_on_dc(hdc, hbm, 0,0, 100, 100);
//
STATE_IMPORT void  STATE_WINAPI STATE_utilities_copy_bitmap_on_dc(HDC hdc, HBITMAP hbm, int x, int y, int bm_stretch_width, int bm_stretch_height);


// This function has nothing to do with STATE graphics engine
// Think of it as an extension to the Win32 API.
//
//It is also possible to load a jpeg file (*.jpg).
// If bitmap_file_name ends with ".jpg" and there is no file with the same name only with a .bmp extension 
// in the same directory then the function will load the jpg file
//
// Note the difference between this command and STATE_bitmap_load()
// STATE_bitmap_load() command loads a bitmap into the engine, while STATE_utilities_load_bitmap_from_file()
// is just an extension to the Win32 API making it easy to load a bitmap from
// the disk and to get its HBITMAP handle (a Win32 type)
//
STATE_IMPORT HBITMAP  STATE_WINAPI STATE_utilities_load_bitmap_from_file(char *bitmap_file_name);


// This function has nothing to do with 3DSTATE graphics engine
// Think of it as an extension to the Win32 API.
//
// bitmap_file_name should have a bmp extension.
// The file is always saved in 8 bit color depth.
// To save in 24 bit color use STATE_utilities_save_bitmap24()
//returns OK or VR_ERROR
STATE_IMPORT int STATE_WINAPI STATE_utilities_save_bitmap(HBITMAP hbm, char *bitmap_file_name);

//Exactly like STATE_utilities_save_bitmap(), only that it saves in 24 bit per pixel color depth (rather than 8)
STATE_IMPORT int STATE_WINAPI STATE_utilities_save_bitmap24(HBITMAP hbm, char *bitmap_file_name);


//Creates a new bitmap file
//The new bitmap file is a resampled version of the source bitmap file
//The given bitmap file should be in 8 bit per pixel or 24 bit per pixel file format.
// The return value is OK or VR_ERROR
// Example:
// STATE_utilities_resample_bitmap("C:\\STATE\\logos\\biglogo.bmp", "..\\small_logo.bmp", 32,32);
// This would create a 32x32 bitmap file in the name small_logo.
STATE_IMPORT int  STATE_WINAPI STATE_utilities_resample_bitmap(char *source_bitmap_file, char *new_bitmap_file_name, int new_width, int new_height);

//convert a jpg files into 256 colors bitmap
// returns OK or VR_ERROR;
//example:
// rc=STATE_utilities_jpg2bmp("bitmaps\jpeg\horse.jpg");
// This will create a 256 colors bitmap with the name horse.bmp
// the bitmap will be created in the same directory of horse.jpg
STATE_IMPORT int  STATE_WINAPI STATE_utilities_jpg2bmp(char *jpg_file_name);

//Like STATE_utilities_jpg2bmp() only the other way around.
// Note that the bitmap should be 256 colors
STATE_IMPORT int  STATE_WINAPI STATE_utilities_bmp2jpg(char *bmp_file_name);

// The output format is determined according to the extension of the output file
// The same with the input file format.
// example STATE_utilities_convert_3d_formats("test.x", "test.mft", "", SAVE_BITMP_AS_JPG);
// Note that this function just do STATE_engine_load_world() and STATE_engine_save()
// and that means that any world that is currently in the engine will be unloaded
// and the input_file_name world will be.
STATE_IMPORT int  STATE_WINAPI STATE_utilities_convert_3d_formats(char *input_file_name, char *world_directory_path, char *bitmaps_directory_path, char *output_file_name, int save_flags);

//Convert a 16bit rgb color to 24bit rgb. The result is return through red,green and blue.
STATE_IMPORT void  STATE_WINAPI STATE_utilities_rgb16_to_rgb24(WORD rgb16, BYTE *red, BYTE *green, BYTE *blue);

//Returns YES or NO.
//Returns YES if the operating system is Windows 98 or above (i.e Windows me , windows 2000 etc ...)
STATE_IMPORT int  STATE_WINAPI STATE_utilities_is_windows98_or_later(void);

//Saves the content of the given window as a bitmap on the disk
// Returns OK or VR_ERROR
// One shouldn't use this function when using the 3D card to render.
// Instead use this combination.
//		HBITMAP hbm=STATE_engine_render_on_bitmap(your_window_or_null, your_camera_or_null);
//		STATE_utilities_save_bitmap(hbm, "d:\\screenshots\\shot.bmp");
//
// Example:
// 
//		HWND hwnd=STATE_engine_get_default_rendering_window_hwnd()
//		STATE_utilities_save_screenshot(hwnd, "d:\\my_screenshots\\flight_sim1.bmp");
//
// Note that the above example is the same as writing STATE_utilities_save_screenshot(NULL, "d:\\my_screenshots\\flight_sim1.bmp");
//
STATE_IMPORT int  STATE_WINAPI STATE_utilities_save_screenshot(HWND rend_win, char *file_name);



// gets an address and display its content
// size is the size of the given buffer in bytes.
// You can use this function for debugging if you have problems interfacing the dll.
STATE_IMPORT void  STATE_WINAPI STATE_utilities_test_address(BYTE *mem_address, int size);

//Test receiving a string. the string that is received is "Testing string transfer"
//Returns "Testing string transfer"
STATE_IMPORT char *STATE_WINAPI STATE_utilities_test_string(void);

//popup a dialog with the given string
STATE_IMPORT void  STATE_WINAPI STATE_utilities_test_LPSTR(LPSTR str);

//popup a dialog with the given string
STATE_IMPORT void  STATE_WINAPI STATE_utilities_test_LPCSTR(LPCSTR str);

//popup a dialog box with the given pointer and the value it points at
STATE_IMPORT void  STATE_WINAPI STATE_utilities_test_PDWORD(DWORD *ptr);

//popup a dialog box with the given pointers and the value they point at
STATE_IMPORT void  STATE_WINAPI STATE_utilities_test_2PDWORD(DWORD *ptr1, DWORD *ptr2);

// Divides a bitmap into smaller pieces (image cutter)
//
// Parameters:
//
//		bitmap_to_cut: 
//			a bmp file name
//
//		fraction_width, fraction_width:
//			The size of the each small fraction cut from the big bitmap
//
//		name_to_save:
//			The path\filename where to save the fractions.
//			example: "D:\\my_folder\\filename" this will create the bitmaps inside the folder "my_folder"
//			and will name them filename1.bmp, filename2.bmp etc ...
//			
//		force_8bit_output
//			The fractions are saved in 8bit format.
//
//
// Return value:
//		Returns the number of bitmaps that were created.		
//
//
// Example:
//
//		//This will divide a big bitmap into 256x256 parts
//		STATE_utilities_cut_image("big_bitmap.bmp", 256, 256, "part", NO);
//
//	Remarks:
//		Almost all 3d cards cant handle bitmaps larger than 1024x1024
//		The Voodoo3 card can handle only 256x256 bitmaps (or smaller) !!!
//
//		Many 3d cards achieves best performance when bitmaps are at the size of 256x256
//		Though in some application 512x512 or 1024x1024 will get better results
//		depending on the application developed (you should test it to get suitable size for your application)
//
STATE_IMPORT int  STATE_WINAPI STATE_utilities_cut_image(char *bitmap_to_cut, int fraction_width, int fraction_length, char *name_to_save, int force_8bit_output);

STATE_IMPORT BYTE * STATE_WINAPI STATE_utilities_allocate_memory(int number_of_bytes_to_allocate);


STATE_IMPORT void STATE_WINAPI STATE_utilities_free_memory(BYTE *mem_to_free);


//-----------------------------------------------------//
//==+++====== T H E    M A T H   A P I ===========+++==//
//-----------------------------------------------------//

// The function in this API have nothing to do with the 
// STATE VR_Engine ! You can think of them as an extension to the 
// WIN32 API.
// They are here because they are useful in many applications especially
// applications that use 3D graphics.


// Constants for the math API
#define	ON_PLANE 				1//on the same plane
#define	IN_BACK_OF_PLANE        2
#define	IN_FRONT_OF_PLANE       3
#define	CUTS_PLANE     			4

#ifndef M_PI
#define M_PI        3.14159265358979323846
#endif


// returns the distance between two points in the power of two
// example: p1=(0,0,0) p2=(10,0,0)
// the distance is 10 so the return value will be 100 
// see also STATE_math_get_distance() which is the same only it
// calls sqrt() at the end.
STATE_IMPORT double  STATE_WINAPI STATE_math_get_square_distance(double p1[3], double p2[3]);

// returns the distance between two points
// example: p1=(0,0,0) p2=(10,0,0)
// the distance is 10 so the return value will be 10
// If speed is important it is better to use STATE_math_get_square_distance()
// which is much faster because it doesn't do the sqrt()
STATE_IMPORT double  STATE_WINAPI STATE_math_get_distance(double p1[3], double p2[3]);

// stores the result in "intersection_point" which is valid in all cases
// pt1 and pt2 are two points on a line
// return value:
// YES if pt1 and pt2 are in different sides of the plane
//		in this case intersection_point will be the expected intersection point
// NO if pt1 and pt2 are on the same side of the plane
//	 in this case intersection_point will be the intersection point if the infinite line that contains p1 and p2
//
// NO if the line is on the plane. 
//		In this case the intersection point will be pt1
//
// YES, If one edge of the segment is on the plane (and the other one is not).
//
STATE_IMPORT int STATE_WINAPI STATE_math_does_segment_intersect_plane(double pln[4],double pt1[3], double pt2[3], double intersection_point[3]);

// Return the distance between point P and the line going through points A and B.
STATE_IMPORT double STATE_WINAPI STATE_math_get_distance_between_point_and_line(double P[3], double A[3], double B[3]);

// P is a point
// AB is a line
// The result is returned through closest_point_on_line
//	 closest_point_on_line is the a point on the line defined by A and B that is closest to P
// Returns OK or VR_ERROR (error if point A is equal to point B)
// Example:
//		P[0]=1; P[1]=2; P[2]=30;
//		A[0]=10; A[1]=20; A[2]=30;
//		B[0]=100; B[1]=200; B[2]=300;
//		double  closest_point_on_line[3];
//		STATE_math_get_closest_point_on_line(P,A,B,closest_point_on_line);
//
// See also STATE_math_is_point_on_segment() which can be used to determine whether the point 
// between A and B
STATE_IMPORT int STATE_WINAPI STATE_math_get_closest_point_on_line(double P[3], double A[3], double B[3], double closest_point_on_line[3]);

// similar to get_closest_point_on_line()
// only that it returns the closest points among the point on the segment
// in other words if the closest point is not on the segment itself then
// the function will return one of the segment edges (the one which is closest)
//
//		Parameters:
//			[in] P:
//				a point
//
//			[in] A,B: 
//				Two pints that define a segment
//
//			[out] closest_point_on_segment:
//				The a point on the segment defined by A and B that is closest
//				to P. This parameter is calculated by the function (the result)
//
//		Returns 
//			OK or VR_ERROR (error if point A is equal to point B)
//
// Example:
//		P[0]=1; P[1]=2; P[2]=30;
//		A[0]=10; A[1]=20; A[2]=30;
//		B[0]=100; B[1]=200; B[2]=300;
//		double  closest_point_on_segment[3];
//		STATE_math_get_closest_point_on_segment(P,A,B,closest_point_on_segment);
//
STATE_IMPORT int STATE_WINAPI STATE_math_get_closest_point_on_segment(double P[3], double A[3], double B[3], double closest_point_on_segment[3]);



// return YES or NO
// Returns YES if point pt is on the given line segment 
STATE_IMPORT int STATE_WINAPI STATE_math_is_point_on_segment(double pt[3], double segment_start[3], double segment_end[3]);

// Get three points and calculate their plane equation (the plane equation: Ax+By+Cz+D=0 )
// Note that (pln[0],pln[1],pln[2]) are the normal of the plane.
// The direction of the normal (there are two possibilities) is determined according to
// the direction of the points. The Normal will point towards the side in which the points will be in clockwise order
// The return value is OK or VR_ERROR. note that we will get VR_ERROR if
// the points are on the same line or are all equal.
STATE_IMPORT int STATE_WINAPI STATE_math_points_to_plane(double pln[4], double p1[3], double p2[3], double p3[3]);

// Gets a plane equation (Ax+By+Cz+D=0 ) and a point
// and returns the following constants 
// ON_PLANE (if the point is on the plane)
// IN_BACK_OF_PLANE
// IN_FRONT_OF_PLANE
// The side which is called front is the side that the normal of the plane points towards)
// (the normal is: N=(pln[0], pln[1], pln[2]);
// the argument thickness is the thickness of the plane (actually half of the width of the plane).
// If for example thickness==6 than all the points that their distance from the plane
// is 6 or less will result a return value of ON_PLANE.
// Usually you will call this function with thickness==0
// But if you want to check whether a point is on the plane you
// should call this function with thickness bigger than 0 (for example 0.0001) 
// and thats to avoid floating point errors (if the plane was created using
// three points and you took only 6 decimal places after the decimal point then the
// plane equation wont be 100% accurate, there is also a limit to the accuracy
// of floating points numbers in a binary world)
// You can also call this function with a negative thickness though 
// it is not so useful, in this case the half of the space which is considered in front of the plane
// will get bigger ... In this case you will never get ON_PLANE)
// Note also that this function is extremely fast.
STATE_IMPORT int STATE_WINAPI STATE_math_get_plane_point_relations(double pln[4], double pt[3], double thickness);


// Takes three points and returns whether they are in clockwise order or not
// when looking from above (looking in direction 0,0,-1). If all the points
// have the same x or y values then one cant tell (when looking from above) if it is clockwise
// so in this case we look from front (looking in direction 0,0,-1) and if
// that doesn't succeed we look from the right side (looking in direction 0,-1,0)
// Returns YES or NO
// Note that the three points create a plane and from each side of the plane
// what is clockwise is exactly the other way around.
//
// If you want to check if the points are clockwise from a given point
// use STATE_math_get_plane_point_relations() like that
// create a plane from the three points and call the function when the point
// in which you look from is the given point.
 STATE_IMPORT int STATE_WINAPI STATE_math_is_clockwise(double p1[3], double p2[3], double p3[3]);

// return if points are in clockwise when
// looking on the three points from the given direction.
// The given direction doesn't have to be normalized.
// see also is_clockwise()
// Returns YES or NO.
 STATE_IMPORT int STATE_WINAPI STATE_math_is_clockwise_from_direction(double view_direction[3], double p1[3], double p2[3], double p3[3]);

// Calculates the ray that reflects from a surface according to the
// physic law: the hitting angle is equal to the reflected angle.
// Arguments:
// normal is the normal of the plane that the ray hits.
// hitting_ray is the ray that hits the plane
// the result is given in reflected_ray (the result is normalized)
// return value: OK, or VR_ERROR (in case of an error).
STATE_IMPORT int STATE_WINAPI STATE_math_get_reflected_direction(double normal[3], double hitting_ray[3], double reflecting_ray[3]);


//Returns the scalar product between two vectors.
//The return value is the length of the first vector multiply by
//the length of the second multiplied by the cosine of the angle between the
//two vectors.
//In a formula way it looks like that:
// return value= scalar_product= |vec1|*|vec2|*cos(alpha) . Alpha is the angle between the vectors
//For more information about scalar products see your math book (look for Linear Algebra and vectors)
STATE_IMPORT double STATE_WINAPI STATE_math_product(double vec1[3], double vec2[3]);

//Returns the cross product between vec1 and vec2.
//The result is returned in the argument "result"
//The returned vector is a vector that is perpendicular both to vec1 and vec2
//The length of the returned vector is equal to
//The length of vec1 multiplied by the length of vec2 multiplied by
//the sinus of the angle between the two vectors.
//For more information about scalar products see your math book (look for Linear Algebra and vectors)
STATE_IMPORT void STATE_WINAPI STATE_math_cross(double vec1[3], double vec2[3], double result[3]);

//returns OK or VR_ERROR
//normalize the vector. This means make it a unit vector.
//In other words the direction of the vector will stay the same
//but its length will become one
//example:
//		double vec[3]={6,8,0} //length==10
//		STATE_math_normalize_vec(vec);
//		//Now the vec will be equal to { 0.6, 0.8, 0} (length==1)
STATE_IMPORT int STATE_WINAPI STATE_math_normalize_vector(double vec[3]);

//Returns the length (magnitude) of the vector
STATE_IMPORT double STATE_WINAPI STATE_math_get_vector_length(double vec[3]);

// This is a very powerful function. Beginners will probably never use this function. 
// See also STATE_math_calculate_bitmap_coords_equations() which is less general but easier to use
// See the example below to understand what it is all about.
// It is used to interpolate values across a polygon. Please note that this function is used together with 
// STATE_math_get_interpolated_value_using_equation()
// With STATE_math_calculate_triangle_interpolation_equation() we build an equation
// and with STATE_math_get_interpolated_value_using_equation() we query for values using the equation we
// have already constructed.
// Return values.
// If an error occurred it returns VR_ERROR. On success it will return the type of the equation that 
// was created. We shouldn't care about this value though it is important that we keep it
// because later we will have to send it to STATE_math_get_interpolated_value_using_equation();
// This function also returns the resulted equation through the result_equation argument.
// Again we shouldnt care at all about what this means though we should save it for calling 
// STATE_math_get_interpolated_value_using_equation()
//
// Arguments:
//	double xyz_of_3_points[3][3] - Three points of the polygon that will be used for interpolating. For example:
//							xyz_of_3_points[0][0] X value of the first point
//							xyz_of_3_points[0][1] Y value of the first point
//							xyz_of_3_points[1][2] Z value of the SECOND point
//
// double value_to_interpolate[3]
//		The values to be interpolated.
//		value_to_interpolate[0] - The value from the first point (for example the point bitmap_x coord)
//		value_to_interpolate[1] - The value from the second point
//
// double polygon_plane[4] - the polygon plane. Use STATE_polygon_get_plane() to get it.
//
// double result_equation[4] - Through this argument the computed equation is returned.
//
//Example, 
//Lets say that we have a polygon and a point x,y,z on the plane of the polygon. We want
//to know what should be the point bm_x,bm_y values.
//Using this function we can build an equation that given any x,y,z it will return the 
//appropriate bm_x of that point. To build the equation we need to take three points of the polygon
//and their bm_x values. 
// xyz_of_3_points[0][0]=1 ; //X value of first point
// xyz_of_3_points[0][1]=2; //Y value of first point
// xyz_of_3_points[0][2]=3; //Z value of first point
//
// xyz_of_3_points[1][0]=4 // X value of second point
// and so on for the three points....
//
// value_to_interpolate[0]=0 ; // bm_x value of the first point
// value_to_interpolate[1]=1 ; // bm_x value of the second point
// value_to_interpolate[2]=0 ; // bm_x value of the third point
//
// double polygon_plane[4];
// STATE_polygon_get_plane(my_polygon, polygon_plane); //getting the polygon plane
//
// double result_equation[4];
// int equation_type;
// equation_type=STATE_math_calculate_polygon_interpolation_equation(xyz_of_3_points, value_to_interpolate, polygon_plane, result_equation);
// 
// //We have now created an equation for interpolating the bitmap_x coordinate over our polygon.
// //The equation_type and the result_equation are like a black box for us (we don't need to care about what it is exactly).
//
// Now lets use the equation and calculate the bitmap_x coordinate in the point x,y,z (that should be on the polygon plane)
// double xyz[3]= {1,2,3}  //a point on the polygon's plane
// double bitmap_x=STATE_math_get_interpolated_value_using_equation(xyz, result_equation, equation_type);
// 
// Note that for getting the bitmap_y coord of the same point we need to build another equation for the bitmap_y values.
STATE_IMPORT int STATE_WINAPI STATE_math_calculate_polygon_interpolation_equation(double xyz_of_3_points[3][3], double value_to_interpolate[3], double polygon_plane[4], double result_equation[4]);

//Returns the interpolated value according to the given equation.
//Before using this function one must construct an equation for the polygon using STATE_math_calculate_polygon_interpolation_equation
// double xyz[3]
//		A point on the polygon plane that we want get the interpolated value.
// double equation[4]
//		The equation as was calculated by STATE_math_calculate_polygon_interpolation_equation()
//		The meaning of this equation should not concern the user.
// int equation_type
//		The equation_type as was returned by STATE_math_calculate_polygon_interpolation_equation()
//		This is also used for internal purposes and should have no meaning for the user.
STATE_IMPORT double STATE_WINAPI STATE_math_get_interpolated_value_using_equation(double xyz[3], double equation[4], int equation_type);


// This function is intended to solve the following problem:
// You have a point x,y,z on a plane or a polygon and you want to know what is the correct bitmap coords u,v
// for this point according to the given polygon.
// This function is used in conjunction with STATE_math_get_interpolated_value_using_equation()
// Each bitmap coord has two values u,v. This function calculates the two equasions needed to 
// calcualte u,v. You don't need to know or care about what these equasions mean.
// you only need to pass them to the STATE_math_get_interpolated_value_using_equation() function
//
// Parameters:
//		U_equation [OUT]
//			The equasion for calculating the U equasion which is used to calculate the u coordinate
//			one doesn't need to understand what this equasion means.
//			one only needs to pass it to the STATE_math_get_interpolated_value_using_equation() function
//
//		V_equation [OUT]
//			The equasion for calculating the V equasion which is used to calculate the v coordinate
//			one doesn't need to understand what this equasion means.
//			one only needs to pass it to the STATE_math_get_interpolated_value_using_equation() function
//
//		polygon_handle [IN]
//			The equasions will be built according to the given polygon.
//			The bitmap coordinates of the poygon vertex define the connection between a point
//			X,Y,Z on the polygon plane and the corresponding u,v bitmap coordinate
//
//		coords_set_index [IN]
//			can be 0 or 1. Set it to 0 for the primary bitmap coords. Set it to 1 for the secondary bitmap coords
//			Remeber that each polygon could have two bitmaps and that each one of the bitmaps can be mapped separatelly
//
// Return value:
//		Returns the the equation type. Again we don't need to care about this value. Only to keep it and pass it later
//		to STATE_math_get_interpolated_value_using_equation()
//			
//
// Example:
//		double U_equation[4], V_equation[4];
//		int equation_set=STATE_math_calculate_bitmap_coords_equations(U_equation, V_equation, polygon_handle,  0);
//		double UV[2];
//		double point_on_polygon_plane[3]={1,2,3};
//		UV[0]=get_interpolated_value_using_equation(point_on_polygon_plane, U_equation, equation_set);
//		UV[1]=get_interpolated_value_using_equation(point_on_polygon_plane, V_equation, equation_set);
//		//
//		// UV now contains the bitmap coords of the given point
//
STATE_IMPORT int STATE_WINAPI STATE_math_calculate_bitmap_coords_equations(double U_equation[4], double V_equation[4], DWORD polygon_handle,  int coords_set_index);

//-----------------------------------------------------//
//======== T H E    P R O F I L E R   A P I =============//
//-----------------------------------------------------//

// One of the most important issue in many applications that use 3D graphics
// is Performances.
// This API makes it extremely easy to time any function (not just STATE's functions)
// using two macros.
// Here is an example of how to do it.
//
//		STATE_START_TAKING_TIME("2d point to 3D"); //The text that is given here will appear in the log file profiling summary
//		any_function_that_you_would_like_to_time(...); //calling the function that we want to time
//		STATE_STOP_TAKING_TIME();
//
// Note that the result of the profiling will be written automatically
// to the log file (look for the file log.log in your executable working directory)
// Note also that these macros will only work in C++, when writing in C
// do this instead:
//
//   static int time_id=-1;
//		time_id=STATE_profiler_start_measuring(time_id,"some text");
//		any_function_that_you_would_like_to_time(...); //calling the function that we want to time (you can any function not just functions of STATE)
//		STATE_profiler_end_measuring(time_id);


STATE_IMPORT int STATE_WINAPI STATE_profiler_start_measuring(int time_id, char *name);

STATE_IMPORT void STATE_WINAPI STATE_profiler_end_measuring(int time_id);

#define STATE_START_TAKING_TIME(name)\
	{\
		  static int time_id=-1;\
		  time_id=STATE_profiler_start_measuring(time_id, name);

#define STATE_STOP_TAKING_TIME() STATE_profiler_end_measuring(time_id); }

//----------------------------------------------//
//========= T H E    L I G H T   A P I =========//
//----------------------------------------------//

// This API was replaced by the STATE_light2_ API 
// This API can still be used for non dynamic light (light that is constant and not changing)
// In short: if you are now just starting adding light. Ignore this API and go straight to the light2 API
// If you have some code written using this API you can leave it but we suggest that you try the light2 which is much better.

//This API is using software light calcualtions while the light2 is using the 3D card hardware which is many times faster
//This is why the light API is good for one time light calculations but when it comes
//to updating the light every render we need to use the light2 API (hardware light rendering)


//This API deals with making light. It supports dynamic colored Gouraud shading light !
// while the light API is using Gouraud shading technique. Both methods
// could be mixed together to create great effects.
// Please also note that Gouraud shading is using vertex lighting.
// This means that the effects will be much better if there are more vertexes (this means more polygons)
// Note that in order to improve the graphic quality dramatically one need to create lightmaps
// (after defining the light sources) for more details please see STATE_polygon_create_lightmap()
// see also STATE_engine_create_shadow() which uses projection method techniques for creating light


//Use this function to create a new light.
// light_name is the name you want to call this light (give what ever name you want)
// The location is the x,y,z coordinates of the light location
// The location can be moved later using the function STATE_light_set_location()
// Note that in order to see any light one has to call STATE_light_activate()
// Example:
// double position[3]={10,20,30} ; //x=10, y=20, z=30
// DWORD light=STATE_light_create("my_light",position);
STATE_IMPORT DWORD STATE_WINAPI STATE_light_create(char *light_name, double location[3]);


#define LIGHT_SET	0
#define LIGHT_ADD	1
#define LIGHT_SUBTRACT 2

// This function is the function that actually creates light !
// You wont see any light till you call this function.
// For dynamic lighting this function should be call over and over
// each time there is some changes (for example the lit object changed position)
// For faster lighting in viewer mode use the function 
// STATE_light_set_entity_to_light(my_light,my_object);
// to light every object in the world separately.

// The light operation could be any one of the following
// LIGHT_SET, LIGHT_ADD, LIGHT_SUBTRACT
// Here is what each one will do:
// LIGHT_ADD:
//		The calculated light will be added to the existing light.
//		Each call will make the illuminated entity brighter
// LIGHT_SUBTRACT:
//		This will cause the light source to create darkness !!!
//		In the real world we dont have an equivalent for that.
//		Subsequent calls will make the illuminated entity becomes darker on each call 
// LIGHT_SET:
//		The entity will be illuminated while ignoring previous illuminations.
//		Normally you will use this option.
//		Subsequent calls wont make any different unless 
//		The location, direction or other properties of the light have changed between calls or
//		that we are illuminating a dynamic objects that has change it position
//		(and thats why we have to recalculate its illumination)
// Example:
//	STATE_light_activate(my_light, LIGHT_SET);
//
// The light is created with the following defaults:
//		* A non-directional light ( use STATE_light_set_type_of_light() to change it and then use STATE_light_set_direction() to set the direction).
//		* Infinite distance. The light could reach an infinite distance without getting less intense . Use STATE_light_set_distance_reach() to change it.
//		* The whole world is illuminated. Use STATE_light_set_entity_to_light() to change it. This will make this function much faster.
//		* White color. The default color is white (255,255,255) Use STATE_light_set_color() to change it.
STATE_IMPORT void STATE_WINAPI STATE_light_activate(DWORD light_handle, int light_operation);


//Returns a handle to the light according to the given name
STATE_IMPORT DWORD STATE_WINAPI STATE_light_get_using_name(char *light_name);


// This function, together with STATE_light_get_next_light()
// are used for going over all the defined lights
// Example
// for(DWORD light=STATE_light_get_first_light(); light!=NULL; light=STATE_light_get_next_light(light) {
//		//Write here the code to do for each light
// }
// Note that the function STATE_light_create() adds a new light
// to top of the queue. So STATE_light_get_first_light()
// will always return the last light that was created.
// The same rule is for all STATE_xxxx_get_first_xxxx() functions.
STATE_IMPORT DWORD STATE_WINAPI STATE_light_get_first_light(void);

STATE_IMPORT DWORD STATE_WINAPI STATE_light_get_next_light(DWORD light_handle);

#define FOREACH_LIGHT2(light) for(DWORD light=STATE_light_get_first_light(); light!=NULL; light=STATE_point_get_next_point(light))

//Sets the location of the light.
// Example
// double location[3]={10,20,30} //setting the X,Y,Z coordinates
// STATE_light_set_location(my_light,location);
STATE_IMPORT void STATE_WINAPI STATE_light_set_location(DWORD light_handle, double location[3]);

STATE_IMPORT void STATE_WINAPI STATE_light_get_location(DWORD light_handle, double location[3]);


// Sets the direction of the light. The direction of the light is relevant only if the 
// light type is set to be directional.
// The direction is set by giving a point in the world towards which the light should be pointed
// See also STATE_light_set_direction()
// Example:
// double location[3]={0,0,10000}; //we make a sun so let put it up in the sky
// DWORD my_light=STATE_light_create("the sun", location);
// double point={0,0,0}; //The sun should be pointing down
// STATE_light_point_at(my_light, point);
// STATE_light_set_type_of_light(my_light, LIGHT_DEFAULT_POINTED_LIGHT); //by default the light is not directional.
// STATE_light_activate(my_light, LIGHT_SET); //let there be light ...
STATE_IMPORT void STATE_WINAPI STATE_light_point_at(DWORD light_handle, double point[3]);


//Sets the direction of the light. The direction of the light is relevant only if the 
// light type is set to be directional.
// Note that a direction of {1,1,1} is exactly the same as {2,2,2}
// Example:
// double location[3]={0,0,0}
// DWORD my_light=STATE_light_create("my_light", location);
// double direction={1,1,0};
// STATE_light_set_direction(my_light, direction);
// STATE_light_set_type_of_light(my_light, LIGHT_DEFAULT_POINTED_LIGHT); //by default the light is not directional.
// STATE_light_activate(my_light, LIGHT_SET); //let there be light ...
STATE_IMPORT void STATE_WINAPI STATE_light_set_direction(DWORD light_handle, double direction[3]);

STATE_IMPORT void STATE_WINAPI STATE_light_get_direction(DWORD light_handle, double direction[3]);


//Set the color of the light. The default is {255,255,255} which is white.
STATE_IMPORT void STATE_WINAPI STATE_light_set_color(DWORD light_handle, BYTE color[3]);

STATE_IMPORT void STATE_WINAPI STATE_light_get_color(DWORD light_handle, BYTE color[3]);

//Remove light from the entity that is associated with the light source.
//Use STATE_light_set_entity_to_light() for choosing the entity that
//the light will be removed from.
//Note !! The function does not delete the light itself
//for deleting the light use STATE_light_delete()
//See also STATE_polygon_remove_light(), STATE_object_remove_light(), STATE_group_remove_light()
STATE_IMPORT void STATE_WINAPI STATE_light_remove_light(DWORD light_handle);


// Constant values for type_of_light (The values could be ored together e.g LIGHT_DIRECTIONAL | LIGHT_AMBIENT
#define LIGHT_DIRECTIONAL 1
#define LIGHT_EFFECTED_BY_DISTANCE_LINEAR 2
#define LIGHT_EFFECTED_BY_DISTANCE_SQUARE 4
#define LIGHT_AMBIENT 8
#define LIGHT_DIFFUSE 16
#define LIGHT_SPECULAR 32

//In most cases you will probably use one of the two combinations below
#define LIGHT_DEFAULT (LIGHT_EFFECTED_BY_DISTANCE_LINEAR | LIGHT_AMBIENT |LIGHT_DIFFUSE | LIGHT_SPECULAR)
#define LIGHT_DEFAULT_POINTED_LIGHT (LIGHT_DIRECTIONAL | LIGHT_EFFECTED_BY_DISTANCE_LINEAR | LIGHT_AMBIENT |LIGHT_DIFFUSE | LIGHT_SPECULAR)
#define LIGHT_DEFAULT_FAST (LIGHT_EFFECTED_BY_DISTANCE_LINEAR | LIGHT_AMBIENT |LIGHT_DIFFUSE )
#define LIGHT_DEFAULT_SUPER_FAST (LIGHT_EFFECTED_BY_DISTANCE_LINEAR | LIGHT_AMBIENT)
// The type of light can be combined from the following options 
// LIGHT_DIRECTIONAL
//		Denote that the light is directional. The direction could be set by STATE_light_set_direction()
// LIGHT_EFFECTED_BY_DISTANCE_LINEAR 
//		The light is reduced according to the distance from the light source location
//		The reduction is linear
//		This flag is included in the default light type though
//		NOTE that in order to feel its effect you have first to define the light source
//		maximum reach. Do that with STATE_light_set_distance_reach()
// LIGHT_EFFECTED_BY_DISTANCE_SQUARE 
//		The effect is that the light reduction is small near the light sourse and very strong
//		near the end of the light reach area
// LIGHT_AMBIENT 8
//		If this flag is not set then the ambient light component will not be used
// LIGHT_DIFFUSE 16
//		If this flag is not set then the diffuse light component will not be used
// LIGHT_SPECULAR 32
//		If this flag is not set then the ambient light component will not be used
//In most cases you will probably use one of the two combinations below
// LIGHT_DEFAULT (LIGHT_EFFECTED_BY_DISTANCE_LINEAR | LIGHT_AMBIENT |LIGHT_DIFFUSE | LIGHT_SPECULAR)
// LIGHT_DEFAULT_POINTED_LIGHT (LIGHT_DIRECTIONAL | LIGHT_EFFECTED_BY_DISTANCE_LINEAR | LIGHT_AMBIENT |LIGHT_DIFFUSE | LIGHT_SPECULAR)
//
// The default light type is LIGHT_DEFAULT
// Example A:
//		//Setting a directional light
//		STATE_light_set_type_of_light(my_light,LIGHT_DEFAULT_POINTED_LIGHT);
// Example B:
//		//using LIGHT_EFFECTED_BY_DISTANCE_SQUARE
//		STATE_light_set_type_of_light(my_light, LIGHT_DIRECTIONAL | LIGHT_EFFECTED_BY_DISTANCE_SQUARE | LIGHT_AMBIENT |LIGHT_DIFFUSE | LIGHT_SPECULAR);
STATE_IMPORT void STATE_WINAPI STATE_light_set_type_of_light(DWORD light_handle, int type_of_light);

STATE_IMPORT int STATE_WINAPI STATE_light_get_type_of_light(DWORD light_handle);

//Gives much more accurate lighting that takes into account obstacles between
// the lit surface and the source of light.
// The disadvantage is that lighting is much more slower when it is on.
// Dont use it for lighting every render.
// The default is NO (not activated)
// Example:
// STATE_light_set_ray_tracing(my_light, YES);
STATE_IMPORT void STATE_WINAPI STATE_light_set_ray_tracing(DWORD light_handle, int YES_or_NO);


STATE_IMPORT int STATE_WINAPI STATE_light_is_ray_tracing(DWORD light_handle);


//Set the distance that the light source can reach. The strength of the light is 
// decreasing as it gets further away from the light source. 
// A distance reach of -1 has a special meaning. It means an infinite reach.
// This is the default (infinite reach) so in order to see the effect of the distance
// one needs to call this function first with a value different than -1.
// Returns OK or VR_ERROR.
// Example:
// STATE_light_set_distance_reach(my_light, 100000);
STATE_IMPORT void STATE_WINAPI STATE_light_set_distance_reach(DWORD light_handle, double distance_reach);

STATE_IMPORT double STATE_WINAPI STATE_light_get_distance_reach(DWORD light_handle);


//Set the target object that will be lit by our light.
// entity_to_light could be a handle to a group an object or a polygon.
// The default entity is the whole world (NULL)
// To make the function STATE_light_activate() you should change it to illuminate just the 
// the entity that needs lighting.
// Examples:
// A)
//		STATE_light_set_entity_to_light(my_light, polyogn_handle);
//		In this case when the light will be activated it will only influence
//		the given polygon.
// B)
//		STATE_light_set_entity_to_light(my_light, NULL);
//		NULL is a valid group_handle. It means the whole world
//		In this case when the light will be activated it will influence
//		the whole world.
// C)
//		STATE_light_set_entity_to_light(my_light, chair_group_handle);
//		When the light will be activated it will only light the chair
//		This is a nice way to paint a specific item in the world.
//
// Returns OK or VR_ERROR
STATE_IMPORT int STATE_WINAPI STATE_light_set_entity_to_light(DWORD light_handle, DWORD entity_to_light);


STATE_IMPORT DWORD STATE_WINAPI STATE_light_get_entity_to_light(DWORD light_handle);


// Activates the light automatically before each render.
// The usage of this function is for dynamic lighting.
// If the light source moves every render (or other property of the light is changed)
// then the light has to be updated as well
// If however the light source properties do not change between every render then you
// should not use this method (it will waste CPU time).
// Another use for this method is for lighting dynamic objects.
// Again use it only if the dynamic object moves or rotates every render
// otherwise it is better to call STATE_light_activate() explicitly after
// each time that the object changes its position.
//
// Parameters:
//
// light_operation:
//		Determines how the light is going to be activated every render.
//		possible values are: LIGHT_ADD, LIGHT_SUBTRACT and LIGHT_SET
//		Normally you would give LIGHT_SET. The other two options can be 
//		used for creating special effects like gradually darkening the world
//		or a "get killed" effect when the screen turns into red (using a red colored light)
//		The light_operation argument is only relevant if YES_or_NO==YES
//
//	YES_or_NO:
//		YES to enable. Give No to stop activating the light before each render. The default is NO
//
//
//
// Important:
//		Dynamic lighting can be very slow if not done correctly. Note the following points:
//		1) Never light the whole world every render. Instead light only the moving objects that need to be re-lighted
//		2) Don't use ray tracing for dynamic lighting.
//		See example "C" below.
//
//Examples:
// A)
//		STATE_light_activate_before_each_render(my_light, YES, LIGHT_SET);
// B)
//		//Disable dynamic lighting.
//		STATE_light_activate_before_each_render(my_light, NO, 0); //doesnt matter what number is given (for the light operation)
//
// C)   //Creating fast dynamic lighting.
//		//In this example, light sources are used to illuminate dynmaic (moving) objects.
//
//		STATE_light_set_entity_to_light(light1, object1);
//		STATE_light_set_entity_to_light(light2, object2);
//
//		STATE_light_set_type_of_light(light1, LIGHT_DEFAULT_FAST);
//		STATE_light_set_type_of_light(light1, LIGHT_DEFAULT_FAST);

//		STATE_light_activate_before_each_render(light1, YES, LIGHT_SET);
//		STATE_light_activate_before_each_render(light2, YES, LIGHT_SET);
STATE_IMPORT void STATE_WINAPI STATE_light_activate_before_each_render(DWORD light_handle, int YES_or_NO, int light_operation);

//The function also returns in the light_operation param the type of operation.
// Example:
//		int light_operation;
//		STATE_light_is_activated_before_render(my_light, &light_operation);
STATE_IMPORT int STATE_WINAPI STATE_light_is_activated_before_render(DWORD light_handle, int *light_operation);

// Each source of light is combined of three different channels, they are
// called Ambient, Diffuse and specular. 
// Ambient light:
//		Ambient light is light that doesn't come from any particular direction
//		Objects illuminated by ambient light are evenly lit on all surfaces.
//		For example if we create a room with a few light sources then even
//		if all the light sources are turned off we will still have some light in the room.		
//		The ambient channel used to simulate this non-directional environment light
// Diffuse light:
//		Diffuse light comes from a particular direction but is reflected evenly of the surface		
//		A good example of a diffuse light source is fluorescent lighting.
//
// Every source light is actually a combination of the three type of lights
// Feel free to play with these values till you get the desired results.
// The rule is that if it looks good then it is correct..	
// The resulted lighting is a combination of the material characteristic and the type of light
// For controlling the material light properties see STATE_polygon_set_ambient(), STATE_group_set_ambient(), STATE_object_set_ambient()
//
// The given value should be a positive number or zero.
// The default is one. Common values are in the range of 0 to 10 
// Examples:
// A)
//		STATE_light_set_ambient(my_light, 1); //the default
// B)
//		STATE_light_set_ambient(my_light, 2.8); //A very strong ambient lighting.
// C)
//		STATE_light_set_ambient(my_light, 0); //Shutting off the ambient channel
//											   //It is better by not including the ambient light
//											   //flag when calling STATE_light_set_type_of_light()
//
// The same goes for the diffuse and specular lighting.
STATE_IMPORT void STATE_WINAPI STATE_light_set_ambient(DWORD light_handle, double value);

STATE_IMPORT void STATE_WINAPI STATE_light_set_diffuse(DWORD light_handle, double value);

STATE_IMPORT void STATE_WINAPI STATE_light_set_specular(DWORD light_handle, double value);


STATE_IMPORT double STATE_WINAPI STATE_light_get_ambient(DWORD light_handle);

STATE_IMPORT double STATE_WINAPI STATE_light_get_diffuse(DWORD light_handle);

STATE_IMPORT double STATE_WINAPI STATE_light_get_specular(DWORD light_handle);

// Use this function to control the size of the light spot that
// the specular lighting is creating.
// The default value is 1.
// The smaller the number is the bigger the light spot is.
// Values should be any positive number or zero.
// Normally values would be in the range of 0 to 10
STATE_IMPORT void STATE_WINAPI STATE_light_set_specular_shining(DWORD light_handle, double value);
STATE_IMPORT double STATE_WINAPI STATE_light_get_specular_shining(DWORD light_handle);

//Returns YES or NO. NO is returned if the given handle
//is not a handle of a light. YES is returned otherwise
STATE_IMPORT int STATE_WINAPI STATE_light_is_light(DWORD light_handle, char *function_asking);

//Deletes all the existing lights
STATE_IMPORT void STATE_WINAPI STATE_light_delete_all(void);

//Deletes the given light.
// Note that the object in the world will stay litted even after the light was deleted
// To remove the light effect from the objects use STATE_light_remove_light()
// or STATE_group_remove_light()
// Note not to use the handle after it the light was deleted
// Example:
// STATE_light_delete(my_light);
// my_light=NULL;
STATE_IMPORT void STATE_WINAPI STATE_light_delete(DWORD light_handle);

//Returns the light source name
// The function returns a pointer to the name which is stored internally inside the 3D Engine
// Visual Basic users should use STATE_entity_get_name() instead.
// See also STATE_entity_get_name()
STATE_IMPORT char *STATE_WINAPI STATE_light_get_name(DWORD light_handle);

STATE_IMPORT void STATE_WINAPI STATE_light_set_name(DWORD light_handle, char *name);

//Disables or enables a light source.
//This function is used before creating a lightmap (STATE_polygon_create_lightmap() )
//A light source that is disabled wont be included in the process of generating a lightmap
// See also STATE_light_is_disabled()
//
// Example:
//	STATE_light_disable(my_light, YES);
STATE_IMPORT void STATE_WINAPI STATE_light_disable(DWORD light_handle, int YES_or_NO);
STATE_IMPORT int STATE_WINAPI STATE_light_is_disabled(DWORD light_handle);

//Calling this function with NO will omit the given light source when saving the world.
// The default is YES.
//Example: STATE_light_set_save_flag(my_light, NO);
STATE_IMPORT void STATE_WINAPI STATE_light_set_save_flag(DWORD light_handle, int YES_or_NO);
STATE_IMPORT int STATE_WINAPI STATE_light_get_save_flag(DWORD light_handle);


#define SET_VERTEX_NORMAL_EQUAL_TO_AVERAGE_OF_ADJACENT_POLYGONS 1000
#define SET_VERTEX_NORMAL_EQUAL_TO_POLYGON_NORMAL 1001
#define DISABLE_AUTOMATIC_NORMAL_CALCULATION 1002
#define SET_VERTEX_NORMAL_USING_MIXED_METHOD 30 //Any number between 0 and 180 can be used. Uses a mixture of the other two method according to Chandan methos optimum normal normals setting (chandan@3dstate.com)
												  //since 2004 this is the default method

// By default the light engine ignores the normal values in the loaded file and
// calculate them automatically. The default normal calculatin mode is:
// SET_VERTEX_NORMAL_USING_MIXED_METHOD
// use this function to set one of the three modes. 
//
// SET_VERTEX_NORMAL_EQUAL_TO_AVERAGE_OF_ADJACENT_POLYGONS:
//		Use this mode to make 3d object look more round and less polygonal
//		This mode will set the normal of a vertex to be equal to the average of the normal of all
//		the adjacent polygons. For example the normal of points in a cube shape room
//		will all point to the center of the room.
//		Note that when using this mode, vertices that have the same x,y,z will
//		also have the same normal which menas that when lighting an object the edges where the polygons
//		connect each other wont be visible (smoother , rounder effect)
//
// SET_VERTEX_NORMAL_EQUAL_TO_POLYGON_NORMAL:
//		The points of a given polygon will have the same normal as the polygon.
//		Note that points with identical x,y,z coordinate might have a different normal
//		if the polygons to which they belong are not on the same plane.
//
// SET_VERTEX_NORMAL_USING_MIXED_METHOD:
//		The default. combines between the two other methods (SET_VERTEX_NORMAL_EQUAL_TO_AVERAGE_OF_ADJACENT_POLYGONS & SET_VERTEX_NORMAL_EQUAL_TO_POLYGON_NORMAL)
//		If the angle between two adjacent polygons is less then given threshold degrees than the average technique is used
//		if the angle between the polygons is more or equal to the given threshold (the default is 30) than the normals used for the points are the polygon normals
//		Actually any number between 0 and 180 can be used. The default is 30 but one can use other numbers as well
//		When 0 is given then it acts exactly like SET_VERTEX_NORMAL_EQUAL_TO_POLYGON_NORMAL
//		when 180 is given then it acts exactly as SET_VERTEX_NORMAL_EQUAL_TO_AVERAGE_OF_ADJACENT_POLYGONS
//		This methos was devised by Chandan Pawasker (chandan@3dstate.com)
//
// DISABLE_AUTOMATIC_NORMAL_CALCULATION:
//		Use the values as given in the world file.
//		Note that the file might not contain normal values
// 
// Example A:
//		STATE_light_calcualte_vertices_normals_automatically(SET_VERTEX_NORMAL_EQUAL_TO_POLYGON_NORMAL);
//
// Example B:
//		//the SET_VERTEX_NORMAL_USING_MIXED_METHOD will be used but with a smaller threshold the the default (30)
//		//This will will increase the SET_VERTEX_NORMAL_EQUAL_TO_POLYGON_NORMAL methos effect
//		STATE_light_calcualte_vertices_normals_automatically(20);
//
STATE_IMPORT void STATE_WINAPI STATE_light_calcualte_vertices_normals_automatically(int mode);




#ifdef __cplusplus
}
#endif



#endif 3DSTATE_H